﻿using System.Collections.Generic;
using CRM.Domain.Branch.Events;
using CRM.Domain.Branch.Models;
using EventFlow.Aggregates;
using EventFlow.MongoDB.ReadStores;
using EventFlow.ReadStores;
using MongoDB.Bson.Serialization.Attributes;

namespace CRM.Domain.Branch.ReadModels
{
    public class BranchReadModel : IMongoDbReadModel,
        IAmReadModelFor<BranchAggregate, BranchId, BranchCreatedEvent>,
        IAmReadModelFor<BranchAggregate, BranchId, BranchNameModifiedEvent>,
        IAmReadModelFor<BranchAggregate, BranchId, BranchAddressModifiedEvent>,
        IAmReadModelFor<BranchAggregate, BranchId, BranchDeletedEvent>,
        IAmReadModelFor<BranchAggregate, BranchId, BranchCustomerIdModifiedEvent>,
        IAmReadModelFor<BranchAggregate, BranchId, BranchContactDetailsModifiedEvent>
    {
        public string BranchName { get; private set; } = "";
        public AddressModel? Address { get; private set; }
        public string CustomerId { get; private set; } = "";
        public bool Deleted { get; private set; }
        [BsonIgnore] public bool IsHq { get; set; } = false;
        [BsonIgnore] public string? DossierId { get; set; } = null;
        public ICollection<ContactDetailModel> ContactDetails { get; private set; } = new List<ContactDetailModel>();

        public void Apply(IReadModelContext context,
            IDomainEvent<BranchAggregate, BranchId, BranchAddressModifiedEvent> domainEvent)
        {
            Address = domainEvent.AggregateEvent.Address;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<BranchAggregate, BranchId, BranchContactDetailsModifiedEvent> domainEvent)
        {
            ContactDetails = domainEvent.AggregateEvent.ContactDetails;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<BranchAggregate, BranchId, BranchCreatedEvent> domainEvent)
        {
            Id = domainEvent.AggregateIdentity.Value;
            BranchName = domainEvent.AggregateEvent.BranchName;
            Address = domainEvent.AggregateEvent.Address;
            CustomerId = domainEvent.AggregateEvent.CustomerId;
            ContactDetails = domainEvent.AggregateEvent.ContactDetails;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<BranchAggregate, BranchId, BranchCustomerIdModifiedEvent> domainEvent)
        {
            CustomerId = domainEvent.AggregateEvent.CompanyId;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<BranchAggregate, BranchId, BranchDeletedEvent> domainEvent)
        {
            Deleted = true;
            context.MarkForDeletion();
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<BranchAggregate, BranchId, BranchNameModifiedEvent> domainEvent)
        {
            BranchName = domainEvent.AggregateEvent.BranchName;
        }

        public string Id { get; private set; } = "";
        public long? Version { get; set; }
    }
}