﻿using System;
using System.Collections.Generic;
using System.Linq;
using CRM.Domain.CallScript.Events;
using CRM.Domain.CallScript.Models;
using EventFlow.Aggregates;
using EventFlow.MongoDB.ReadStores;
using EventFlow.ReadStores;

namespace CRM.Domain.CallScript.ReadModels
{
    public class CallScriptReadModel : IMongoDbReadModel,
        IAmReadModelFor<CallScriptAggregate, CallScriptId, CallScriptCreatedEvent>,
        IAmReadModelFor<CallScriptAggregate, CallScriptId, CallScriptModifiedEvent>,
        IAmReadModelFor<CallScriptAggregate, CallScriptId, CommentCreatedEvent>,
        IAmReadModelFor<CallScriptAggregate, CallScriptId, CommentModifiedEvent>,
        IAmReadModelFor<CallScriptAggregate, CallScriptId, CommentRemovedEvent>,
        IAmReadModelFor<CallScriptAggregate, CallScriptId, TagAddedEvent>,
        IAmReadModelFor<CallScriptAggregate, CallScriptId, CallScriptRemovedEvent>,
        IAmReadModelFor<CallScriptAggregate, CallScriptId, CallScriptDossierIdModifiedEvent>

    {
        public bool Deleted { get; private set; }
        public string CallScriptBody { get; private set; } = "";
        public string DossierId { get; private set; } = "";
        public ICollection<string> Tags { get; private set; } = new List<string>();
        public ICollection<CallScriptCommentModel> CommentList { get; private set; } = new List<CallScriptCommentModel>();
        public DateTime Updated { get; set; }

        public void Apply(IReadModelContext context,
            IDomainEvent<CallScriptAggregate, CallScriptId, CallScriptCreatedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Id = domainEvent.AggregateIdentity.Value;
            CallScriptBody = aggregateEvent.CallScriptBody;
            DossierId = aggregateEvent.DossierId;
            Tags = aggregateEvent.Tags;
            Updated = domainEvent.Timestamp.DateTime;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CallScriptAggregate, CallScriptId, CallScriptModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CallScriptBody = aggregateEvent.CallScriptBody;
            Updated = domainEvent.Timestamp.DateTime;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CallScriptAggregate, CallScriptId, CallScriptRemovedEvent> domainEvent)
        {
            Deleted = true;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CallScriptAggregate, CallScriptId, CommentCreatedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CommentList.Add(new CallScriptCommentModel(aggregateEvent.Id.Value, aggregateEvent.CommentBody,
                aggregateEvent.CommentQuote, aggregateEvent.UserId, domainEvent.Timestamp,
                aggregateEvent.CallscriptId));
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CallScriptAggregate, CallScriptId, CommentModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var comment = CommentList.Single(x => x.Id == aggregateEvent.CallscriptCommentId.Value);
            comment.CommentBody = aggregateEvent.CommentBody;
            comment.CommentQuote = aggregateEvent.CommentQuote;
            comment.LastModifiedDate = domainEvent.Timestamp.UtcDateTime;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CallScriptAggregate, CallScriptId, CommentRemovedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var comment = CommentList.Single(x => x.Id == aggregateEvent.CallscriptCommentId.Value);
            CommentList.Remove(comment);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CallScriptAggregate, CallScriptId, TagAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Tags.Add(aggregateEvent.Tag);
            Updated = domainEvent.Timestamp.DateTime;
        }

        public string Id { get; private set; } = "";
        public long? Version { get; set; }

        public void Apply(IReadModelContext context,
            IDomainEvent<CallScriptAggregate, CallScriptId, CallScriptDossierIdModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            DossierId = aggregateEvent.DossierId;
        }
    }
}