﻿using System.Collections.Generic;
using CRM.Domain.Customer.AbstractClasses;
using CRM.Domain.Customer.Events;
using EventFlow.Aggregates;
using EventFlow.ReadStores;

namespace CRM.Domain.Customer.ReadModels
{
    public class CustomerReadModel : ReadModel,
        IAmReadModelFor<CustomerAggregate, CustomerId, CustomerCreatedEvent>,
        IAmReadModelFor<CustomerAggregate, CustomerId, CustomerMainCustomerBranchModifiedEvent>,
        IAmReadModelFor<CustomerAggregate, CustomerId, CustomerWebPortalLicenseAmountModifiedEvent>,
        IAmReadModelFor<CustomerAggregate, CustomerId, CustomerNameModifiedEvent>,
        IAmReadModelFor<CustomerAggregate, CustomerId, CustomerDeletedEvent>,
        IAmReadModelFor<CustomerAggregate, CustomerId, CustomerSubscriptionsModifiedEvent>

    {
        public bool Deleted { get; private set; }
        public string CustomerName { get; private set; } = "";
        public string Tenant { get; private set; } = "";
        public int? CCMCLientId { get; private set; }
        public int WebPortalLicenseAmount { get; private set; }
        public string CustomerBranchId { get; private set; } = "";
        public ICollection<string> Subscriptions { get; private set; } = new List<string>();


        public void Apply(IReadModelContext context,
            IDomainEvent<CustomerAggregate, CustomerId, CustomerCreatedEvent> domainEvent)
        {
            Id = domainEvent.AggregateIdentity.Value;
            var aggregateEvent = domainEvent.AggregateEvent;
            CustomerName = aggregateEvent.CustomerName;
            Tenant = aggregateEvent.Tenant;
            CCMCLientId = aggregateEvent.CCMCLientId;
            WebPortalLicenseAmount = aggregateEvent.WebPortalLicenseAmount;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CustomerAggregate, CustomerId, CustomerDeletedEvent> domainEvent)
        {
            context.MarkForDeletion();
            Id = domainEvent.AggregateIdentity.Value;
            Deleted = true;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CustomerAggregate, CustomerId, CustomerMainCustomerBranchModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Id = domainEvent.AggregateIdentity.Value;
            CustomerBranchId = aggregateEvent.CustomerBranchId;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CustomerAggregate, CustomerId, CustomerNameModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Id = domainEvent.AggregateIdentity.Value;
            CustomerName = aggregateEvent.CustomerName;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CustomerAggregate, CustomerId, CustomerSubscriptionsModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Id = domainEvent.AggregateIdentity.Value;
            Subscriptions = aggregateEvent.Subscriptions;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<CustomerAggregate, CustomerId, CustomerWebPortalLicenseAmountModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Id = domainEvent.AggregateIdentity.Value;
            WebPortalLicenseAmount = aggregateEvent.WebPortalLicenseAmount;
        }
    }
}