﻿using System.Collections.Generic;
using CRM.Domain.Dossier.Events;
using EventFlow.Aggregates;
using EventFlow.MongoDB.ReadStores;
using EventFlow.ReadStores;

namespace CRM.Domain.Dossier.ReadModels
{
    public class DossierReadModel : IMongoDbReadModel,
        IAmReadModelFor<DossierAggregate, DossierId, DossierNameModifiedEvent>,
        IAmReadModelFor<DossierAggregate, DossierId, DossierDeletedEvent>,
        IAmReadModelFor<DossierAggregate, DossierId, DossierCreatedEvent>,
        IAmReadModelFor<DossierAggregate, DossierId, DossierBranchAddedEvent>,
        IAmReadModelFor<DossierAggregate, DossierId, DossierBranchRemovedEvent>,
        IAmReadModelFor<DossierAggregate, DossierId, DossierCustomerIdModifiedEvent>
    {
        public bool Deleted { get; private set; }

        public string DossierName { get; private set; } = "";
        public int? CCMId { get; private set; }
        public string CustomerId { get; private set; } = "";
        public ICollection<string> BranchIds { get; private set; } = new List<string>();

        public string Id { get; set; } = "";
        public long? Version { get; set; }


        public void Apply(IReadModelContext context,
            IDomainEvent<DossierAggregate, DossierId, DossierBranchAddedEvent> domainEvent)
        {
            BranchIds.Add(domainEvent.AggregateEvent.BranchId);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DossierAggregate, DossierId, DossierBranchRemovedEvent> domainEvent)
        {
            BranchIds.Remove(domainEvent.AggregateEvent.BranchId);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DossierAggregate, DossierId, DossierCreatedEvent> domainEvent)
        {
            Id = domainEvent.AggregateIdentity.Value;
            DossierName = domainEvent.AggregateEvent.DossierName;
            CCMId = domainEvent.AggregateEvent.CcmId;
            CustomerId = domainEvent.AggregateEvent.CustomerId;
        }


        public void Apply(IReadModelContext context,
            IDomainEvent<DossierAggregate, DossierId, DossierDeletedEvent> domainEvent)
        {
            context.MarkForDeletion();
            Deleted = true;
        }


        public void Apply(IReadModelContext context,
            IDomainEvent<DossierAggregate, DossierId, DossierNameModifiedEvent> domainEvent)
        {
            DossierName = domainEvent.AggregateEvent.DossierName;
        }


        public void Apply(IReadModelContext context,
            IDomainEvent<DossierAggregate, DossierId, DossierCustomerIdModifiedEvent> domainEvent)
        {
            CustomerId = domainEvent.AggregateEvent.CustomerId;
        }
    }
}