﻿using System;
using System.Collections.Generic;
using EventFlow.Aggregates;
using EventFlow.EventStores;

namespace CRM.Domain.Duty.Events
{
    [EventVersion("DutyEndModifiedEvent", 2)]
    public class DutyEndModifiedEvent : AggregateEvent<DutyAggregate, DutyId>
    {
        public DutyEndModifiedEvent(DateTime endDate,
            string endTime)
        {
            EndDate = endDate;
            EndTime = endTime;
        }

        public DateTime EndDate { get; }
        public string EndTime { get; }
    }

    public class UpdateDutyEndModifiedEventV1ToV2 : IEventUpgrader<DutyAggregate, DutyId>
    {
        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateDutyEndModifiedEventV1ToV2(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }

        public IEnumerable<IDomainEvent<DutyAggregate, DutyId>> Upgrade(IDomainEvent<DutyAggregate, DutyId> domainEvent)
        {
            var event1 = domainEvent as IDomainEvent<DutyAggregate, DutyId, DutyEndModifiedEventV1>;
            yield return event1 == null
                ? domainEvent
                : _domainEventFactory.Upgrade<DutyAggregate, DutyId>(domainEvent,
                    new DutyEndModifiedEvent
                    (
                        event1.AggregateEvent.EndDate,
                        event1.AggregateEvent.EndTime.ToString("HH:mm")
                    ));
        }
    }

    [EventVersion("DutyEndModifiedEvent", 1)]
    public class DutyEndModifiedEventV1 : AggregateEvent<DutyAggregate, DutyId>
    {
        public DutyEndModifiedEventV1(DateTime endDate,
            DateTime endTime)
        {
            EndDate = endDate;
            EndTime = endTime;
        }

        public DateTime EndDate { get; }
        public DateTime EndTime { get; }
    }
}