﻿using System;
using System.Collections.Generic;
using EventFlow.Aggregates;
using EventFlow.EventStores;

namespace CRM.Domain.Duty.Events
{
    [EventVersion("DutyStartModifiedEvent", 2)]
    public class DutyStartModifiedEvent : AggregateEvent<DutyAggregate, DutyId>
    {
        public DutyStartModifiedEvent(DateTime startDate,
            string startAt)
        {
            StartDate = startDate;
            StartAt = startAt;
        }

        public DateTime StartDate { get; }
        public string StartAt { get; }
    }

    public class UpdateDutyStartModifiedEventV1ToV2 : IEventUpgrader<DutyAggregate, DutyId>
    {
        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateDutyStartModifiedEventV1ToV2(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }

        public IEnumerable<IDomainEvent<DutyAggregate, DutyId>> Upgrade(IDomainEvent<DutyAggregate, DutyId> domainEvent)
        {
            var event1 = domainEvent as IDomainEvent<DutyAggregate, DutyId, DutyStartModifiedEventV1>;
            yield return event1 == null
                ? domainEvent
                : _domainEventFactory.Upgrade<DutyAggregate, DutyId>(domainEvent,
                    new DutyStartModifiedEvent
                    (
                        event1.AggregateEvent.StartDate,
                        event1.AggregateEvent.StartAt.ToString("HH:mm")
                    ));
        }
    }

    [EventVersion("DutyStartModifiedEvent", 1)]
    public class DutyStartModifiedEventV1 : AggregateEvent<DutyAggregate, DutyId>
    {
        public DutyStartModifiedEventV1(DateTime startDate,
            DateTime startAt)
        {
            StartDate = startDate;
            StartAt = startAt;
        }

        public DateTime StartDate { get; }
        public DateTime StartAt { get; }
    }
}