﻿using System;
using System.Collections.Generic;
using EventFlow.Aggregates;
using EventFlow.EventStores;

namespace CRM.Domain.Duty.Events
{
    [EventVersion("DutyUpdatedEvent", 2)]
    public class DutyUpdatedEvent : AggregateEvent<DutyAggregate, DutyId>
    {
        public DutyUpdatedEvent(string dossierId,
            string category,
            DateTime startDate,
            DateTime endDate,
            string actionDescription,
            string endAt,
            string startAt)
        {
            DossierId = dossierId;
            Category = category;
            StartDate = startDate;
            EndDate = endDate;
            ActionDescription = actionDescription;
            EndAt = endAt;
            StartAt = startAt;
        }

        public string DossierId { get; }
        public string Category { get; }
        public DateTime StartDate { get; }
        public DateTime EndDate { get; }
        public string ActionDescription { get; }
        public string EndAt { get; }
        public string StartAt { get; }
    }

    public class UpdateDutyUpdatedEventV1ToV2 : IEventUpgrader<DutyAggregate, DutyId>
    {
        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateDutyUpdatedEventV1ToV2(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }


        public IEnumerable<IDomainEvent<DutyAggregate, DutyId>> Upgrade(IDomainEvent<DutyAggregate, DutyId> domainEvent)
        {
            var event1 = domainEvent as IDomainEvent<DutyAggregate, DutyId, DutyUpdatedEventV1>;
            yield return event1 == null
                ? domainEvent
                : _domainEventFactory.Upgrade<DutyAggregate, DutyId>(domainEvent,
                    new DutyUpdatedEvent
                    (
                        event1.AggregateEvent.DossierId,
                        event1.AggregateEvent.Category,
                        event1.AggregateEvent.StartDate,
                        event1.AggregateEvent.EndDate,
                        event1.AggregateEvent.ActionDescription,
                        event1.AggregateEvent.EndAt.ToString("HH:mm"),
                        event1.AggregateEvent.StartAt.ToString("HH:mm")
                    ));
        }
    }

    [EventVersion("DutyUpdatedEvent", 1)]
    public class DutyUpdatedEventV1 : AggregateEvent<DutyAggregate, DutyId>
    {
        public DutyUpdatedEventV1(string dossierId,
            string category,
            DateTime startDate,
            DateTime endDate,
            string actionDescription,
            DateTime endAt,
            DateTime startAt)
        {
            DossierId = dossierId;
            Category = category;
            StartDate = startDate;
            EndDate = endDate;
            ActionDescription = actionDescription;
            EndAt = endAt;
            StartAt = startAt;
        }

        public string DossierId { get; }
        public string Category { get; }
        public DateTime StartDate { get; }
        public DateTime EndDate { get; }
        public string ActionDescription { get; }
        public DateTime EndAt { get; }
        public DateTime StartAt { get; }
    }
}