﻿using System;
using System.Collections.Generic;
using System.Linq;
using CRM.Domain.Duty.Enums;
using CRM.Domain.Duty.Events;
using CRM.Domain.Duty.Models;
using EventFlow.Aggregates;
using EventFlow.MongoDB.ReadStores;
using EventFlow.ReadStores;
using MongoDB.Bson.Serialization.Attributes;

namespace CRM.Domain.Duty.ReadModels
{
    public class DutyReadModelV2 : IMongoDbReadModel,
        IAmReadModelFor<DutyAggregate, DutyId, DutyAddedEvent>,
        IAmReadModelFor<DutyAggregate, DutyId, DutyRemovedEvent>,
        IAmReadModelFor<DutyAggregate, DutyId, DutyEmployeeAddedEvent>,
        IAmReadModelFor<DutyAggregate, DutyId, DutyEmployeeRemovedEvent>,
        IAmReadModelFor<DutyAggregate, DutyId, DutyCategoryModifiedEvent>,
        IAmReadModelFor<DutyAggregate, DutyId, DutyStartModifiedEvent>,
        IAmReadModelFor<DutyAggregate, DutyId, DutyEndModifiedEvent>,
        IAmReadModelFor<DutyAggregate, DutyId, DutyActionDescriptionModifiedEvent>,
        IAmReadModelFor<DutyAggregate, DutyId, DutyUpdatedEvent>,
        IAmReadModelFor<DutyAggregate, DutyId,DutyEmployeeUpdatedEvent>,
        IAmReadModelFor<DutyAggregate, DutyId, DutyModifiedDossierIdEvent>

    {
        public string Category { get; private set; }
        public DateTime StartDate { get; private set; }
        public DateTime EndDate { get; private set; }
        public ICollection<DutyEmployeeModel> DutyEmployees { get; private set; } = new List<DutyEmployeeModel>();
        public string DossierId { get; private set; }
        public bool Deleted { get; private set; }
        public string ActionDescription { get; private set; }
        public int StartAt { get; set; }

        public int EndAt { get; set; }
        public string Id { get; set; }
        public long? Version { get; set; }

        //Extra prop Status (string). Niet in DB

        [BsonIgnore] public DutyStatusEnum DutyStatus => CheckStatus();

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyActionDescriptionModifiedEvent> domainEvent)
        {
            ActionDescription = domainEvent.AggregateEvent.ActionDescription;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyAddedEvent> domainEvent)
        {
            Id = domainEvent.AggregateIdentity.Value;
            var aggregateEvent = domainEvent.AggregateEvent;
            Category = aggregateEvent.Category;
            StartDate = aggregateEvent.StartDate;
            EndDate = aggregateEvent.EndDate;
            DossierId = aggregateEvent.DossierId;
            ActionDescription = aggregateEvent.ActionDescription;

            var startTime = int.Parse(aggregateEvent.StartAt.Split(':')[0])*60;
            startTime += int.Parse(aggregateEvent.StartAt.Split(':')[1]);

            var endTime = int.Parse(aggregateEvent.EndAt.Split(':')[0])*60;
            endTime += int.Parse(aggregateEvent.EndAt.Split(':')[1]);

            EndAt = endTime;
            StartAt = startTime;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyCategoryModifiedEvent> domainEvent)
        {
            Category = domainEvent.AggregateEvent.Category;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyEmployeeAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            DutyEmployees.Add(new DutyEmployeeModel(aggregateEvent.Id.Value, aggregateEvent.Tags,
                aggregateEvent.OrderId, aggregateEvent.EmployeeId, aggregateEvent.ContactPoints));
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyEmployeeRemovedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;

            var d = DutyEmployees.Where(x => x.Id == aggregateEvent.Id.Value).ToList();
            foreach (var dutyEmployeeModel in d) DutyEmployees.Remove(dutyEmployeeModel);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyEndModifiedEvent> domainEvent)
        {
            EndDate = domainEvent.AggregateEvent.EndDate;
          
            var endTime = int.Parse(domainEvent.AggregateEvent.EndTime.Split(':')[0])*60;
            endTime += int.Parse(domainEvent.AggregateEvent.EndTime.Split(':')[1]);

            EndAt = endTime;
                   }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyRemovedEvent> domainEvent)
        {
            context.MarkForDeletion();
            Deleted = true;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyStartModifiedEvent> domainEvent)
        {
            StartDate = domainEvent.AggregateEvent.StartDate;
            var aggregateEvent = domainEvent.AggregateEvent;
            var startTime = int.Parse(aggregateEvent.StartAt.Split(':')[0])*60;
            startTime += int.Parse(aggregateEvent.StartAt.Split(':')[1]);

            StartAt = startTime;        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyUpdatedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Category = aggregateEvent.Category;
            StartDate = aggregateEvent.StartDate;
            EndDate = aggregateEvent.EndDate;
            DossierId = aggregateEvent.DossierId;
            ActionDescription = aggregateEvent.ActionDescription;
            var startTime = int.Parse(aggregateEvent.StartAt.Split(':')[0])*60;
            startTime += int.Parse(aggregateEvent.StartAt.Split(':')[1]);

            var endTime = int.Parse(aggregateEvent.EndAt.Split(':')[0])*60;
            endTime += int.Parse(aggregateEvent.EndAt.Split(':')[1]);

            EndAt = endTime;
            StartAt = startTime;
        }



        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyEmployeeUpdatedEvent> domainEvent)
        {
           var aggregateEvent = domainEvent.AggregateEvent;
           DutyEmployees.Clear();
           foreach (var dutyEmployee in aggregateEvent.DutyEmployees)
           {
               DutyEmployees.Add(new DutyEmployeeModel(dutyEmployee.Id, dutyEmployee.Tags,
                   dutyEmployee.OrderId, dutyEmployee.EmployeeId, dutyEmployee.ContactPoints));
           }
            
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyAggregate, DutyId, DutyModifiedDossierIdEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            DossierId = aggregateEvent.DossierId;
        }

        private DutyStatusEnum CheckStatus()
        {

            var startTime = TimeSpan.FromMinutes(StartAt);
            var endTime = TimeSpan.FromMinutes(EndAt);

            if (startTime == endTime)
            {
                var completeStartDate = StartDate.Date.Add(startTime);
                var completeEndDate = EndDate.Date.Add(endTime);
                var isStartActive = (DateTime.Now - completeStartDate).Ticks >= 0;

                var isEndActive = (completeEndDate - DateTime.Now).Ticks >= 0;

                if (isStartActive)
                {
                    if (isEndActive) return DutyStatusEnum.Active;

                    return DutyStatusEnum.Expired;
                }

                return DutyStatusEnum.Planned;
            }

            var isStartDateActive = (DateTime.Now.Date - StartDate.Date).Days >= 0;
            var isEndDateActive = (EndDate.Date - DateTime.Now.Date).Days >= 0;

            if (!isStartDateActive) return DutyStatusEnum.Planned;



            if (!isEndDateActive) return DutyStatusEnum.Expired;

            var completeStartTime = StartDate.Date.Add(startTime);

            if (completeStartTime > DateTime.Now) return DutyStatusEnum.Planned;


                var h24 = new TimeSpan(24, 0, 0);

            var totalTime = startTime < endTime
                ? endTime - startTime
                : endTime.Add(h24) - startTime;

            var timeNow = new TimeSpan(DateTime.Now.Hour, DateTime.Now.Minute, 0);

            var timeSinceStartTime = timeNow > startTime
                ? timeNow - startTime
                : timeNow.Add(h24) - startTime;

            var lastDay = (EndDate.Date - DateTime.Now.Date).Days == 0;

            if (lastDay && timeNow > endTime) return DutyStatusEnum.Expired;

            if (totalTime > timeSinceStartTime) return DutyStatusEnum.Active;

            return DutyStatusEnum.Planned;

        }

    }
}