﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Duty.Models;
using EventFlow.Snapshots;

namespace CRM.Domain.Duty.Snapshots
{
    [SnapshotVersion("Duty", 2)]
    public class DutySnapshot : ISnapshot
    {
        public DutySnapshot(string category,
            DateTime startDate,
            DateTime endDate,
            ICollection<DutyEmployeeModel> contacts,
            bool isCreated,
            bool deleted,
            string dossierId,
            string startAt,
            string endAt)
        {
            Category = category;
            StartDate = startDate;
            EndDate = endDate;
            Contacts = contacts;
            IsCreated = isCreated;
            Deleted = deleted;
            DossierId = dossierId;
            StartAt = startAt;
            EndAt = endAt;
        }

        public string DossierId { get; }
        public string Category { get; }
        public DateTime StartDate { get; }
        public DateTime EndDate { get; }
        public string StartAt { get; }
        public string EndAt { get; }
        public ICollection<DutyEmployeeModel> Contacts { get; }
        public bool IsCreated { get; }
        public bool Deleted { get; }
    }

    public class UpdateDutySnapshotV1ToV2 : ISnapshotUpgrader<DutySnapshotV1, DutySnapshot>
    {
        public Task<DutySnapshot> UpgradeAsync(DutySnapshotV1 fromVersionedType,
            CancellationToken cancellationToken)
        {
            return Task.Run(() => new DutySnapshot(
                fromVersionedType.Category,
                fromVersionedType.StartDate,
                fromVersionedType.EndDate,
                fromVersionedType.Contacts,
                fromVersionedType.IsCreated,
                fromVersionedType.Deleted,
                fromVersionedType.DossierId,
                fromVersionedType.StartAt.ToString("HH:mm"),
                fromVersionedType.EndAt.ToString("HH:mm")
            ), cancellationToken);
        }
    }

    [SnapshotVersion("Duty", 1)]
    public class DutySnapshotV1 : ISnapshot
    {
        public DutySnapshotV1(string category,
            DateTime startDate,
            DateTime endDate,
            ICollection<DutyEmployeeModel> contacts,
            bool isCreated,
            bool deleted,
            string dossierId,
            DateTime startAt,
            DateTime endAt)
        {
            Category = category;
            StartDate = startDate;
            EndDate = endDate;
            Contacts = contacts;
            IsCreated = isCreated;
            Deleted = deleted;
            DossierId = dossierId;
            StartAt = startAt;
            EndAt = endAt;
        }

        public string DossierId { get; }
        public string Category { get; }
        public DateTime StartDate { get; }
        public DateTime EndDate { get; }
        public DateTime StartAt { get; }
        public DateTime EndAt { get; }
        public ICollection<DutyEmployeeModel> Contacts { get; }
        public bool IsCreated { get; }
        public bool Deleted { get; }
    }
}