﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.Duties;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.Duties.TimeSlots;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.Duties.TimeSlots.DutyContacts;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.Duties.TimeSlots.DutyContacts.DutyContactPoints;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.Duties.TimeSlots.DutyContacts.Tags;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.DutyTemplate;
using CRM.Domain.DutyCategoryAndTemplate.Models;
using CRM.Domain.DutyCategoryAndTemplate.Snapshots;
using CRM.Domain.Shared.ValueObjects;
using EventFlow.Aggregates;
using EventFlow.Aggregates.ExecutionResults;
using EventFlow.Snapshots;
using EventFlow.Snapshots.Strategies;

namespace CRM.Domain.DutyCategoryAndTemplate.Aggregates
{
    public class DutyCategoryAggregate : SnapshotAggregateRoot<DutyCategoryAggregate, DutyCategoryId, DutyCategorySnapshot>,
    #region IEmit
        IEmit<DutyCategoyCreatedEvent>,
        IEmit<DutyCategoryDeletedEvent>,
        IEmit<DutyCategoryGlobalTemplateIdModifiedEvent>,
        IEmit<DutyCategoryNameModifiedEvent>,
        IEmit<DutyCategorySaveEvent>,
        IEmit<DutyCategoryCancelEvent>,
        IEmit<DutyCategoryDutyNoteModifiedEvent>,
        IEmit<DutyCategoryClearedEvent>,
        IEmit<DutyCategoryGlobalDutyIdRemovedEvent>,
        //TimeslotTemplates
        IEmit<DutyTemplateTimeSlotTemplateAddedEvent>,
        IEmit<DutyTemplateTimeSlotTemplateDeletedEvent>,
        IEmit<DutyTemplateTimeSlotTemplateEndTimeModifiedEvent>,
        IEmit<DutyTemplateTimeSlotTemplateNameModifiedEvent>,
        IEmit<DutyTemplateTimeSlotTemplateNoteModifiedEvent>,
        IEmit<DutyTemplateTimeSlotTemplateStartTimeModifiedEvent>,
        IEmit<DutyTemplateTimeSlotTemplateWeekdaysModifiedEvent>,
        //duties
        IEmit<DutyAddedEvent>,
        IEmit<DutyDeletedEvent>,
        IEmit<DutyModifiedEndEvent>,
        IEmit<DutyModifiedNoteEvent>,
        IEmit<DutyModifiedStartEvent>,
        //Duty Timeslots
        IEmit<DutyTimeSlotAddedEvent>,
        IEmit<DutyTimeSlotDeletedEvent>,
        IEmit<DutyTimeSlotEndTimeModifiedEvent>,
        IEmit<DutyTimeSlotNameModifiedEvent>,
        IEmit<DutyTimeSlotNoteModifiedEvent>,
        IEmit<DutyTimeSlotStartTimeModifiedEvent>,
        IEmit<DutyTimeSlotWeekdaysModifiedEvent>,
        //DutyContacts
        IEmit<DutyContactAddedEvent>,
        IEmit<DutyContactDeletedEvent>,
        IEmit<DutyContactModifiedEmployeeEvent>,
        IEmit<DutyContactModifiedOrderIdEvent>,
        //ContactPoints
        IEmit<DutyContactPointAddedEvent>,
        IEmit<DutyContactPointDeletedEvent>,
        //Tags
        IEmit<DutyContactTagAddedEvent>,
        IEmit<DutyContactTagDeletedEvent>
    #endregion


    {
        #region Props and Fields
        private bool _created;
        private bool _deleted;

        public string DossierId { get; private set; } = "";
        public string CategoryName { get; private set; } = "";
        public string Note { get; private set; } = "";
        //public DutyTemplate TemplateDuty { get; private set; }
        public ICollection<TimeSlotTemplate> TimeSlotTemplates { get; set; }

        public ICollection<Duty> Duties { get; private set; }
        public string GlobalDutyTemplateId { get; private set; } = "";
        public DutyCategoryModel Changes { get; private set; }
        #endregion
        public DutyCategoryAggregate(DutyCategoryId id) : base(id, SnapshotEveryFewVersionsStrategy.With(20))
        {
        }

        #region Snapshot
        protected override Task<DutyCategorySnapshot> CreateSnapshotAsync(CancellationToken cancellationToken)
        {
            return Task.Run(() => new DutyCategorySnapshot(_created, _deleted, DossierId, CategoryName, Note,
                TimeSlotTemplates == null
                    ? new List<TimeSlotTemplateModel>()
                    : TimeSlotTemplates.Select(x => new TimeSlotTemplateModel(x)).ToList(),
                Duties == null
                ? new List<DutyModel>()
                : Duties.Select(x => new DutyModel(x)).ToList(),
                GlobalDutyTemplateId,
                Changes == null ? null
                : new DutyCategoryChangesReadModel(Changes)), cancellationToken);
        }

        protected override Task LoadSnapshotAsync(DutyCategorySnapshot snapshot,
            ISnapshotMetadata metadata,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
            {
                _created = snapshot.Created;
                _deleted = snapshot.Deleted;
                DossierId = snapshot.DossierId;
                CategoryName = snapshot.CategoryName;
                Note = snapshot.Note;
                TimeSlotTemplates = snapshot.TimeSlotTemplates == null
                    ? new List<TimeSlotTemplate>()
                    : snapshot.TimeSlotTemplates.Select(x =>
                        new TimeSlotTemplate(TimeSlotTemplateId.With(x.Id), x)).ToList();
                //TemplateDuty = new DutyTemplate(snapshot.TemplateDuty,
                //    DutyTemplateId.With(snapshot.TemplateDuty.Id));
                Duties = snapshot.Duties == null
                    ? new List<Duty>()
                    : snapshot.Duties.Select(x => new Duty(DutyId.With(x.Id), x)).ToList();
                GlobalDutyTemplateId = snapshot.GlobalDutyTemplateId;
                Changes = snapshot.Changes == null ? null : new DutyCategoryModel(snapshot.Changes);
            }, cancellationToken);
        }


        #endregion

        #region IExecutionResult

        #region Category

        public IExecutionResult CreateDutyCategory(string dossierId)
        {
            if (_created) return ExecutionResult.Failed();
            var _event = new DutyCategoyCreatedEvent(dossierId);
            Emit(_event);
            return ExecutionResult.Success();
        }

        public IExecutionResult DeleteDutyCategory()
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            Emit(new DutyCategoryDeletedEvent());
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyDutyCategoryName(string categoryName)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            Emit(new DutyCategoryNameModifiedEvent(categoryName));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyDutyCategoryGlobalDutyId(string globalDutyTemplateId)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            Emit(new DutyCategoryGlobalTemplateIdModifiedEvent(globalDutyTemplateId));
            return ExecutionResult.Success();
        }


        public IExecutionResult SaveDutyCategory()
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            Emit(new DutyCategorySaveEvent());
            return ExecutionResult.Success();
        }

        public IExecutionResult CancelDutyCategory()
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            Emit(new DutyCategoryCancelEvent());
            return ExecutionResult.Success();
        }

        public IExecutionResult ClearDutyCategory()
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            Emit(new DutyCategoryClearedEvent());
            return ExecutionResult.Success();
        }

        public IExecutionResult RemoveGlobalDutyId()
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            Emit(new DutyCategoryGlobalDutyIdRemovedEvent());
            return ExecutionResult.Success();
        }


        #endregion

        #region DutyTemplate
        public IExecutionResult ModifyDutyCategoryDutyTemplateNote(string note)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            Emit(new DutyCategoryDutyNoteModifiedEvent(note));
            return ExecutionResult.Success();
        }

        public IExecutionResult DeleteTimeSlotInDutyTemplate(TimeSlotTemplateId id)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            CheckChanges();

            if (Changes.TimeSlotTemplates.All(x => x.Id != id))
                return ExecutionResult.Failed();

            Emit(new DutyTemplateTimeSlotTemplateDeletedEvent(id));
            return ExecutionResult.Success();
        }

        public IExecutionResult AddTimeSlotInDutyTemplate(TimeSlotTemplateId id)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            CheckChanges();

            //if (Changes.TimeSlotTemplates != null
            //    && Changes.TimeSlotTemplates.All(x => x.Id == id))
            //    return ExecutionResult.Failed();

            Emit(new DutyTemplateTimeSlotTemplateAddedEvent(id));

            if (TimeSlotTemplates != null && TimeSlotTemplates.Count == Changes.TimeSlotTemplates.Count)
            {
                return ExecutionResult.Failed("Timeslot was not just added to Changes-object.");
            }
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyEndTimeTimeSlotDutyTemplate(TimeSlotTemplateId id,
            TimeSpan endTime)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            CheckChanges();

            if (Changes.TimeSlotTemplates.All(x => x.Id != id))
                return ExecutionResult.Failed();

            Emit(new DutyTemplateTimeSlotTemplateEndTimeModifiedEvent(id, endTime));

            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyStartTimeTimeSlotDutyTemplate(TimeSlotTemplateId id,
            TimeSpan startTime)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            CheckChanges();

            if (Changes.TimeSlotTemplates.All(x => x.Id != id))
                return ExecutionResult.Failed();

            Emit(new DutyTemplateTimeSlotTemplateStartTimeModifiedEvent(id, startTime));

            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyTimeSlotNameDutyTemplate(TimeSlotTemplateId id,
            string name)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            CheckChanges();

            if (Changes.TimeSlotTemplates.All(x => x.Id != id))
                return ExecutionResult.Failed();

            Emit(new DutyTemplateTimeSlotTemplateNameModifiedEvent(id, name));

            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyNoteTimeSlotDutyTemplate(TimeSlotTemplateId id,
            string note)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            CheckChanges();

            if (Changes.TimeSlotTemplates.All(x => x.Id != id))
                return ExecutionResult.Failed();

            Emit(new DutyTemplateTimeSlotTemplateNoteModifiedEvent(id, note));

            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyWeekdaysTimeSlotDutyTemplate(TimeSlotTemplateId id,
            ICollection<WeekDay> weekDays)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();
            CheckChanges();

            if (Changes.TimeSlotTemplates.All(x => x.Id != id))
                return ExecutionResult.Failed();

            Emit(new DutyTemplateTimeSlotTemplateWeekdaysModifiedEvent(id, weekDays));

            return ExecutionResult.Success();
        }

        #endregion

        #region Duty

        public IExecutionResult AddDuty(DutyId id)
        {
            if (!_created || _deleted) return ExecutionResult.Failed();

            Emit(new DutyAddedEvent(id));
            return ExecutionResult.Success();
        }

        public IExecutionResult DeleteDuty(DutyId id)
        {
            if (!CheckChangesDuty(id)) return ExecutionResult.Failed();

            Emit(new DutyDeletedEvent(id));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyEndDuty(DutyId id,
            DateTime end)
        {
            if (!CheckChangesDuty(id)) return ExecutionResult.Failed();

            Emit(new DutyModifiedEndEvent(id, end));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyStartDuty(DutyId id,
            DateTime start)
        {
            if (!CheckChangesDuty(id)) return ExecutionResult.Failed();

            Emit(new DutyModifiedStartEvent(id, start));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyNoteDuty(DutyId id,
            string note)
        {
            if (!CheckChangesDuty(id)) return ExecutionResult.Failed();

            Emit(new DutyModifiedNoteEvent(id, note));
            return ExecutionResult.Success();
        }
        #endregion

        #region DutyTimeSlots

        public IExecutionResult AddTimeSlotToDuty(DutyId dutyId,
            TimeSlotId timeSlotId)
        {
            if (!CheckChangesDuty(dutyId)) return ExecutionResult.Failed();

            Emit(new DutyTimeSlotAddedEvent(dutyId, timeSlotId));
            return ExecutionResult.Success();
        }

        public IExecutionResult DeleteTimeSlotFromDuty(DutyId dutyId,
            TimeSlotId timeSlotId)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId)) return ExecutionResult.Failed();
            Emit(new DutyTimeSlotDeletedEvent(dutyId, timeSlotId));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyTimeSlotEndTimeInDuty(DutyId dutyId,
            TimeSlotId timeSlotId,
            TimeSpan endtime)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId)) return ExecutionResult.Failed();
            Emit(new DutyTimeSlotEndTimeModifiedEvent(dutyId, timeSlotId, endtime));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyTimeSlotStartTimeInDuty(DutyId dutyId,
            TimeSlotId timeSlotId,
            TimeSpan startTime)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId)) return ExecutionResult.Failed();
            Emit(new DutyTimeSlotStartTimeModifiedEvent(dutyId, timeSlotId, startTime));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyTimeSlotNoteInDuty(DutyId dutyId,
            TimeSlotId timeSlotId,
            string note)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId)) return ExecutionResult.Failed();
            Emit(new DutyTimeSlotNoteModifiedEvent(dutyId, timeSlotId, note));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyTimeSlotNameInDuty(DutyId dutyId,
            TimeSlotId timeSlotId,
            string name)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId)) return ExecutionResult.Failed();
            Emit(new DutyTimeSlotNameModifiedEvent(dutyId, timeSlotId, name));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyTimeSlotWeekdaysInDuty(DutyId dutyId,
            TimeSlotId timeSlotId,
            ICollection<WeekDay> weekDays)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId)) return ExecutionResult.Failed();
            Emit(new DutyTimeSlotWeekdaysModifiedEvent(dutyId, timeSlotId, weekDays));
            return ExecutionResult.Success();
        }


        #endregion

        #region DutyContacts

        public IExecutionResult AddDutyContact(DutyId dutyId,
            TimeSlotId timeSlotId, DutyContactId dutyContactId,
            string employeeId, int orderId)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId)) return ExecutionResult.Failed();
            Emit(new DutyContactAddedEvent(dutyId, timeSlotId, dutyContactId, employeeId, orderId));
            return ExecutionResult.Success();
        }

        public IExecutionResult DeleteDutyContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId contactId)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId, contactId)) return ExecutionResult.Failed();
            Emit(new DutyContactDeletedEvent(dutyId, timeSlotId, contactId));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyEmployeeDutyContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId contactId,
            string employeeId)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId, contactId)) return ExecutionResult.Failed();
            Emit(new DutyContactModifiedEmployeeEvent(dutyId, timeSlotId, contactId, employeeId));
            return ExecutionResult.Success();
        }

        public IExecutionResult ModifyOrderIdDutyContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId contactId,
            int orderId)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId, contactId)) return ExecutionResult.Failed();
            Emit(new DutyContactModifiedOrderIdEvent(dutyId, timeSlotId, contactId, orderId));
            return ExecutionResult.Success();
        }

        #endregion

        #region ContactPoints

        public IExecutionResult AddContactPointToDutyContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId dutyContactId,
            DutyContactPointId dutyContactPointId,
            string description,
            string contactDetailId)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId, dutyContactId)) return ExecutionResult.Failed();
            Emit(new DutyContactPointAddedEvent(dutyId, timeSlotId, dutyContactId, dutyContactPointId, contactDetailId, description));
            return ExecutionResult.Success();
        }

        public IExecutionResult DeleteContactPointFromDutyContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId dutyContactId,
            DutyContactPointId dutyContactPointId)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId, dutyContactId, dutyContactPointId)) return ExecutionResult.Failed();
            Emit(new DutyContactPointDeletedEvent(dutyId, timeSlotId, dutyContactId, dutyContactPointId));
            return ExecutionResult.Success();
        }

        #endregion

        #region Tags

        public IExecutionResult AddTagToDutyContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId dutyContactId,
            string tag)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId, dutyContactId)) return ExecutionResult.Failed();
            Emit(new DutyContactTagAddedEvent(dutyId, timeSlotId, dutyContactId, tag));
            return ExecutionResult.Success();
        }

        public IExecutionResult DeleteTagFromDutyContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId dutyContactId,
            string tag)
        {
            if (!CheckChangesDuty(dutyId, timeSlotId, dutyContactId, tag: tag)) return ExecutionResult.Failed();
            Emit(new DutyContactTagDeletedEvent(dutyId, timeSlotId, dutyContactId, tag));
            return ExecutionResult.Success();
        }

        #endregion

        #endregion

        #region Apply

        #region Category
        public void Apply(DutyCategoyCreatedEvent aggregateEvent)
        {
            DossierId = aggregateEvent.DossierId;
            Changes = new DutyCategoryModel();
            //Changes.TemplateDuty = new DutyTemplate(aggregateEvent.IdDutyTemplate);
            _created = true;
        }

        public void Apply(DutyCategoryDeletedEvent aggregateEvent)
        {
            _deleted = true;
        }


        public void Apply(DutyCategoryGlobalTemplateIdModifiedEvent aggregateEvent)
        {
            if (Changes == null)
                Changes = new DutyCategoryModel(CategoryName, Note, GlobalDutyTemplateId, TimeSlotTemplates);

            Changes.GlobalDutyTemplateId = aggregateEvent.GlobalDutyTemplateId;
        }

        public void Apply(DutyCategoryNameModifiedEvent aggregateEvent)
        {
            if (Changes == null)
                Changes = new DutyCategoryModel(CategoryName, Note, GlobalDutyTemplateId, TimeSlotTemplates);

            Changes.CategoryName = aggregateEvent.CategoryName;
        }


        public void Apply(DutyCategorySaveEvent aggregateEvent)
        {

            if (Changes != null)
            {
                CategoryName = Changes.CategoryName;
                Note = Changes.Note;
                //TemplateDuty = Changes.TemplateDuty;
                TimeSlotTemplates = Changes.TimeSlotTemplates;
                GlobalDutyTemplateId = Changes.GlobalDutyTemplateId;

                Changes = null;
            }

            if (Duties == null) return;
            if (Duties.All(x => x.Changes == null)) return;

            foreach (var duty in Duties.Where(x => x.Changes != null))
            {
                if (duty.Changes.MarkedForDeletion) continue;

                duty.Note = duty.Changes.Note;
                duty.End = duty.Changes.End;
                duty.Start = duty.Changes.Start;
                duty.TimeSlots = duty.Changes.TimeSlots
                    .Select(x => new TimeSlot(TimeSlotId.With(x.Id), x))
                    .ToList();
                duty.SavedOnce = true;
                duty.Changes = null;
            }

            if (Duties.All(x => x.Changes == null)) return;

            if (!Duties.Any(x => x.Changes.MarkedForDeletion)) return;

            var tempDuties = new List<Duty>(Duties.Where(x => x.Changes.MarkedForDeletion));

            foreach (var duty in tempDuties)
            {
                Duties.Remove(Duties.First(x => x.Id == duty.Id));
            }
        }

        public void Apply(DutyCategoryCancelEvent aggregateEvent)
        {
            if (Changes == null) return;

            Changes = null;

        }

        public void Apply(DutyCategoryClearedEvent aggregateEvent)
        {
            //oorspronkelijk werden ook CategoryName en GlobalDutyId gewist.
            //CN is niet nodig omdat je niet kan saven zonder CN.
            //GDT mag niet omdat je een 'savedGDT' moet hebben om de Usage-teller in GDT te kunnen verlagen.
            Changes = new DutyCategoryModel();
            TimeSlotTemplates = new List<TimeSlotTemplate>();
            Note = "";
            Changes.CategoryName = CategoryName;
            Changes.TimeSlotTemplates = new List<TimeSlotTemplate>();

        }
        public void Apply(DutyCategoryGlobalDutyIdRemovedEvent aggregateEvent)
        {
            GlobalDutyTemplateId = "";
        }

        #endregion

        //je changes zijn null wanneer je begint te editeren. Er zit op dat moment geen TemplateDuty in je Changes, dus die nemen we over van de aggregate.
        #region DutyTemplate en TimeslotTemplates
        public void Apply(DutyCategoryDutyNoteModifiedEvent aggregateEvent)
        {
            CheckChanges();

            Changes.Note = aggregateEvent.Note;
        }

        public void Apply(DutyTemplateTimeSlotTemplateAddedEvent aggregateEvent)
        {
            CheckChanges();
            if (Changes.TimeSlotTemplates == null)
            {
                Changes.TimeSlotTemplates = new List<TimeSlotTemplate>();
            }
            Changes.TimeSlotTemplates.Add(new TimeSlotTemplate(aggregateEvent.TemplateTimeSlotId));
            if (TimeSlotTemplates != null && TimeSlotTemplates.Count == Changes.TimeSlotTemplates.Count)
            {
            }

        }

        public void Apply(DutyTemplateTimeSlotTemplateDeletedEvent aggregateEvent)
        {
            var timeSlotToRemove = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);

            Changes.TimeSlotTemplates.Remove(timeSlotToRemove);
        }

        public void Apply(DutyTemplateTimeSlotTemplateEndTimeModifiedEvent aggregateEvent)
        {
            var timeSlotToEdit = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);
            timeSlotToEdit.EndTime = aggregateEvent.EndTime;
        }


        public void Apply(DutyTemplateTimeSlotTemplateNameModifiedEvent aggregateEvent)
        {
            var timeSlotToEdit = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);
            timeSlotToEdit.TimeSlotName = aggregateEvent.TimeSlotName;
        }

        public void Apply(DutyTemplateTimeSlotTemplateNoteModifiedEvent aggregateEvent)
        {
            var timeSlotToEdit = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);
            timeSlotToEdit.Note = aggregateEvent.Note;
        }

        public void Apply(DutyTemplateTimeSlotTemplateStartTimeModifiedEvent aggregateEvent)
        {
            var timeSlotToEdit = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);
            timeSlotToEdit.StartTime = aggregateEvent.StartTime;
        }

        public void Apply(DutyTemplateTimeSlotTemplateWeekdaysModifiedEvent aggregateEvent)
        {
            var timeSlotToEdit = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);
            timeSlotToEdit.WeekDays = aggregateEvent.WeekDays;
        }

        #endregion

        #region Duties

        public void Apply(DutyAddedEvent aggregateEvent)
        {
            var duty = new Duty(aggregateEvent.Id);
            duty.Changes = new DutyChangesModel(aggregateEvent.Id.Value);
            if (Duties == null)
                Duties = new List<Duty>();
            Duties.Add(duty);
        }
        public void Apply(DutyDeletedEvent aggregateEvent)
        {
            var duty = CheckDuty(aggregateEvent.Id);

            duty.Changes.MarkedForDeletion = true;
        }

        public void Apply(DutyModifiedEndEvent aggregateEvent)
        {
            var duty = CheckDuty(aggregateEvent.Id);
            duty.Changes.End = aggregateEvent.End;
        }

        public void Apply(DutyModifiedNoteEvent aggregateEvent)
        {
            var duty = CheckDuty(aggregateEvent.Id);
            duty.Changes.Note = aggregateEvent.Note;
        }

        public void Apply(DutyModifiedStartEvent aggregateEvent)
        {
            var duty = CheckDuty(aggregateEvent.Id);
            duty.Changes.Start = aggregateEvent.Start;
        }


        #endregion

        #region DutyTimeSlots
        public void Apply(DutyTimeSlotAddedEvent aggregateEvent)
        {
            var duty = CheckDuty(aggregateEvent.IdDuty);
            if (duty.Changes.TimeSlots == null)
            {
                duty.Changes.TimeSlots = new List<TimeSlotModelNoMore>();
            }
            duty.Changes.TimeSlots.Add(new TimeSlotModelNoMore(aggregateEvent.IdTimeSlot.Value));
        }
        public void Apply(DutyTimeSlotDeletedEvent aggregateEvent)
        {
            CheckDutyAndTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                out var duty, out var timeSlot);
            duty.Changes.TimeSlots.Remove(timeSlot);
        }

        public void Apply(DutyTimeSlotEndTimeModifiedEvent aggregateEvent)
        {
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            timeSlot.EndTime = aggregateEvent.EndTime;
        }

        public void Apply(DutyTimeSlotNameModifiedEvent aggregateEvent)
        {
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            timeSlot.TimeSlotName = aggregateEvent.Name;
        }

        public void Apply(DutyTimeSlotNoteModifiedEvent aggregateEvent)
        {
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            timeSlot.Note = aggregateEvent.Note;
        }

        public void Apply(DutyTimeSlotStartTimeModifiedEvent aggregateEvent)
        {
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            timeSlot.StartTime = aggregateEvent.StartTime;
        }

        public void Apply(DutyTimeSlotWeekdaysModifiedEvent aggregateEvent)
        {
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            timeSlot.WeekDays = aggregateEvent.WeekDays;
        }

        #endregion

        #region DutyContacts

        public void Apply(DutyContactAddedEvent aggregateEvent)
        {
            if (aggregateEvent.IdTimeSlot == null)
            {
                var duty = CheckDuty(aggregateEvent.IdDuty);
                foreach (var ts in duty.TimeSlots)
                {
                    var dc = new DutyContact(aggregateEvent.IdDutyContact, aggregateEvent.EmployeeId, aggregateEvent.OrderId);
                    if (ts.DutyContacts == null)
                    {
                        ts.DutyContacts = new List<DutyContact>();
                    }
                    ts.DutyContacts.Add(dc);
                }
            }
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            var duc = new DutyContact(aggregateEvent.IdDutyContact, aggregateEvent.EmployeeId, aggregateEvent.OrderId);
            if (timeSlot.DutyContacts == null)
            {
                timeSlot.DutyContacts = new List<DutyContactModel>();
            }
            timeSlot.DutyContacts.Add(new DutyContactModel(duc));

        }
        public void Apply(DutyContactDeletedEvent aggregateEvent)
        {
            CheckTimeSlotAndContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot, aggregateEvent.IdDutyContact, out var timeSlot, out var dutyContact);
            timeSlot.DutyContacts.Remove(dutyContact);
        }

        public void Apply(DutyContactModifiedEmployeeEvent aggregateEvent)
        {
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            dutyContact.EmployeeId = aggregateEvent.EmployeeId;
        }

        public void Apply(DutyContactModifiedOrderIdEvent aggregateEvent)
        {
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            dutyContact.OrderId = aggregateEvent.OrderId;
        }


        #endregion

        #region ContactPoints

        public void Apply(DutyContactPointAddedEvent aggregateEvent)
        {
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            if (dutyContact.ContactPoints == null)
            {
                dutyContact.ContactPoints = new List<DutyContactPointModel>();
            }
            dutyContact.ContactPoints.Add(
                new DutyContactPointModel(
                    new DutyContactPoint(aggregateEvent.IdDutyContactPoint, aggregateEvent.Description, aggregateEvent.ContactDetailId)));
        }

        public void Apply(DutyContactPointDeletedEvent aggregateEvent)
        {
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            var dutyContactPoint = dutyContact.ContactPoints.Single(x => x.Id == aggregateEvent.IdDutyContactPoint.Value);
            dutyContact.ContactPoints.Remove(dutyContactPoint);
        }

        #endregion

        #region Tags
        public void Apply(DutyContactTagAddedEvent aggregateEvent)
        {
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            if (dutyContact.Tags == null)
            {
                dutyContact.Tags = new List<string>();
            }
            dutyContact.Tags.Add(aggregateEvent.Tag);
        }

        public void Apply(DutyContactTagDeletedEvent aggregateEvent)
        {
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            dutyContact.Tags.Remove(aggregateEvent.Tag);
        }



        #endregion

        #endregion

        #region helpertjes
        private void CheckChanges()
        {
            if (Changes == null)
            {
                Changes = new DutyCategoryModel(CategoryName, Note, GlobalDutyTemplateId, TimeSlotTemplates);
                //Changes.TemplateDuty = TemplateDuty;
            }
        }

        private TimeSlotTemplate CheckChangesAndGetTimeSlotTemplate(
            TimeSlotTemplateId id)
        {
            CheckChanges();
            var timeSlot = Changes.TimeSlotTemplates
                .Single(x => x.Id == id);
            return timeSlot;
        }

        private Duty CheckDuty(DutyId id)
        {
            var duty = Duties.Single(x => x.Id == id);
            if (duty.Changes == null)
            {
                duty.Changes = new DutyChangesModel(duty);
            }

            return duty;
        }

        private void CheckDutyAndTimeSlot(DutyId dutyId,
            TimeSlotId timeSlotId, out Duty duty, out TimeSlotModelNoMore timeSlot)
        {
            duty = CheckDuty(dutyId);
            timeSlot = duty.Changes.TimeSlots.Single(x => x.Id == timeSlotId.Value);
        }

        private TimeSlotModelNoMore CheckTimeSlot(DutyId dutyId,
            TimeSlotId timeSlotId)
        {
            var duty = CheckDuty(dutyId);
            return duty.Changes.TimeSlots.Single(x => x.Id == timeSlotId.Value);
        }

        private void CheckTimeSlotAndContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId dutyContactId,
            out TimeSlotModelNoMore timeSlot,
            out DutyContactModel dutyContact)
        {
            CheckDutyAndTimeSlot(dutyId, timeSlotId, out var duty, out timeSlot);
            dutyContact = timeSlot.DutyContacts.Single(x => x.Id == dutyContactId.Value);
        }

        private DutyContactModel CheckDutyContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId dutyContactId)
        {
            CheckDutyAndTimeSlot(dutyId, timeSlotId, out var duty, out var timeSlot);
            return timeSlot.DutyContacts.Single(x => x.Id == dutyContactId.Value);
        }

        private bool CheckChangesDuty(DutyId id, TimeSlotId timeSlotId = null, DutyContactId dutyContactId = null, DutyContactPointId dutyContactPointId = null, string tag = null)
        {
            if (!_created || _deleted) return false;

            if (Duties.All(x => x.Id != id)) return false;

            if (timeSlotId == null) return true;

            if (Duties.All(x => x.Changes == null)) return true;

            if (Duties.Single(x => x.Id == id).Changes == null) return true;

            var timeSlot = Duties.Where(x => x.Changes != null)
                .Select(x => x.Changes)
                .SelectMany(x => x.TimeSlots)
                .SingleOrDefault(x => x.Id == timeSlotId.Value);

            if (timeSlot == null) return false;
            //if (Duties.Where(x => x.Changes.TimeSlots.Contains(timeSlotId)))

            //if (Duties.SelectMany(x => x.Changes.TimeSlots)
            //    .All(x => x.Id != timeSlotId)) return false;

            if (dutyContactId == null) return true;

            var dutyContacts = Duties
                .Single(x => x.Id == id).Changes.TimeSlots
                .Single(x => x.Id == timeSlotId.Value).DutyContacts;

            if (dutyContacts.All(x => x.Id != dutyContactId.Value)) return false;

            if (dutyContactPointId == null && tag == null) return true;

            if (dutyContactPointId != null && dutyContacts.SelectMany(x => x.ContactPoints)
                .All(x => x.Id != dutyContactPointId.Value)) return false;

            if (tag == null) return true;

            var onecontact = dutyContacts
                .SingleOrDefault(x => x.Id == dutyContactId.Value);

            if (onecontact == null) return false;

            if (onecontact.Tags.All(x => x != tag)) return false;

            return true;
        }



        #endregion

    }
}
