﻿using System;
using System.Collections.Generic;
using System.Linq;
using CRM.Domain.DutyCategoryAndTemplate.Abstract;
using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.Duties;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.Duties.TimeSlots;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.Duties.TimeSlots.DutyContacts;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.Duties.TimeSlots.DutyContacts.DutyContactPoints;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.Duties.TimeSlots.DutyContacts.Tags;
using CRM.Domain.DutyCategoryAndTemplate.Events.DutyCategory.DutyTemplate;
using CRM.Domain.DutyCategoryAndTemplate.Models;
using EventFlow.Aggregates;
using EventFlow.ReadStores;

namespace CRM.Domain.DutyCategoryAndTemplate.ReadModels
{
    public class DutyCategoryReadModel: ReadModel,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyCategoyCreatedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyCategoryDeletedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyCategoryGlobalTemplateIdModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyCategoryNameModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyCategorySaveEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyCategoryCancelEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyCategoryClearedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyCategoryGlobalDutyIdRemovedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyCategoryDutyNoteModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateAddedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateDeletedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateEndTimeModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateNameModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateNoteModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateStartTimeModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateWeekdaysModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyAddedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyDeletedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyModifiedEndEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyModifiedNoteEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyModifiedStartEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotAddedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotDeletedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotEndTimeModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotNameModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotNoteModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotStartTimeModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotWeekdaysModifiedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyContactAddedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyContactDeletedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyContactModifiedEmployeeEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyContactModifiedOrderIdEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyContactPointAddedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyContactPointDeletedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyContactTagAddedEvent>,
IAmReadModelFor<DutyCategoryAggregate, DutyCategoryId, DutyContactTagDeletedEvent>
    {
        public bool Deleted { get; private set; }
        public string DossierId { get; private set; } = "";
        public string CategoryName { get; private set; } = "--- No title saved ---";
        public string Note { get; private set; } = "";
        //public DutyTemplateModel TemplateDuty { get; private set; }
        public ICollection<TimeSlotTemplateModel> TimeSlotTemplates { get; set; }
        public ICollection<DutyModel> Duties { get; private set; }
        public string GlobalDutyTemplateId { get; private set; } = "";
        public string LastModifiedId { get; private set; } = "";
        public string CreatedId { get; private set; } = "";

        public string LastModifiedName { get; private set; } = "";
        public string CreatedName { get; set; } = "";
        public DateTime LastModifiedOn { get; private set; }
        public DateTime CreatedOn { get; private set; }

        public DutyCategoryChangesReadModel Changes { get; private set; }

        #region Category

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyCategoyCreatedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Id = domainEvent.AggregateIdentity.Value;
            DossierId = aggregateEvent.DossierId;
            Changes = new DutyCategoryChangesReadModel();
            //Changes.TemplateDuty = new DutyTemplateModel(new DutyTemplate(aggregateEvent.IdDutyTemplate));
            CreatedOn = DateTime.UtcNow;
            LastModifiedOn = DateTime.UtcNow;

            try
            {
                CreatedId = domainEvent.Metadata["userId"];
                LastModifiedId = domainEvent.Metadata["userId"];
                CreatedName = domainEvent.Metadata["userName"];
                LastModifiedName = domainEvent.Metadata["userName"];
            }
            catch (Exception e)
            {
                CreatedId = "TestId";
                LastModifiedId = "TestId";
                CreatedName = "TestName";
                LastModifiedName = "TestName";
            }

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyCategoryDeletedEvent> domainEvent)
        {
            context.MarkForDeletion();
            Deleted = true;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyCategoryGlobalTemplateIdModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            if (Changes == null)
                Changes = new DutyCategoryChangesReadModel(this);

            Changes.GlobalDutyTemplateId = aggregateEvent.GlobalDutyTemplateId;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyCategoryNameModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            if (Changes == null)
                Changes = new DutyCategoryChangesReadModel(this);

            Changes.CategoryName = aggregateEvent.CategoryName;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyCategorySaveEvent> domainEvent)
        {

            if (Changes != null)
            {
                CategoryName = Changes.CategoryName;
                Note = Changes.Note;
                TimeSlotTemplates = Changes.TimeSlotTemplates;
                //TemplateDuty = Changes.TemplateDuty;
                GlobalDutyTemplateId = Changes.GlobalDutyTemplateId;

                LastModifiedOn = DateTime.UtcNow;

                try
                {
                    LastModifiedId = domainEvent.Metadata["userId"];
                    LastModifiedName = domainEvent.Metadata["userName"];
                }
                catch (Exception e)
                {
                    LastModifiedId = "TestId";
                    LastModifiedName = "TestName";
                }
                Changes = null;
            }

            if (Duties == null) return;

            if (Duties.All(x => x.Changes == null)) return;

            foreach (var duty in Duties.Where(x => x.Changes != null))
            {
                if (duty.Changes.MarkedForDeletion) continue;

                duty.Note = duty.Changes.Note;
                duty.End = duty.Changes.End;
                duty.Start = duty.Changes.Start;
                duty.TimeSlots = duty.Changes.TimeSlots;
                duty.SavedOnce = true;
                duty.Changes = null;
            }

            if (Duties.All(x => x.Changes == null)) return;

            if (!Duties.Any(x => x.Changes.MarkedForDeletion)) return;

            var tempDuties = new List<DutyModel>(Duties.Where(x => x.Changes.MarkedForDeletion));

            foreach (var duty in tempDuties)
            {
                Duties.Remove(Duties.First(x => x.Id == duty.Id));
            }

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyCategoryCancelEvent> domainEvent)
        {
            //if (Duties.All(x => x.Changes == null) && Changes == null) return;

            //if (string.IsNullOrWhiteSpace(CategoryName))
            //{
            //    context.MarkForDeletion();
            //    Deleted = true;
            //    return;
            //}

            if (Changes == null) return;

            Changes = null;

            //if (Duties.All(x => x.Changes == null)) return;

            //var tempDuties = new List<DutyModel>(Duties.Where(x => x.Changes != null));

            //foreach (var duty in tempDuties)
            //{
            //    if (duty.SavedOnce)
            //        Duties.First(x => x.Id == duty.Id).Changes = null;
            //    else
            //    {
            //        Duties.Remove(Duties.First(x => x.Id == duty.Id));
            //    }
            //}
        }
        public void Apply(IReadModelContext context, IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyCategoryClearedEvent> domainEvent)
        {
            //oorspronkelijk werden ook CategoryName en GlobalDutyId gewist.
            //CN is niet nodig omdat je niet kan saven zonder CN.
            //GDT mag niet omdat je een 'savedGDT' moet hebben om de Usage-teller in GDT te kunnen verlagen.
            Changes = new DutyCategoryChangesReadModel();
            TimeSlotTemplates = new List<TimeSlotTemplateModel>();
            Note = "";
            Changes.CategoryName = CategoryName;
            Changes.TimeSlotTemplates = new List<TimeSlotTemplateModel>();

        }
        public void Apply(IReadModelContext context, IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyCategoryGlobalDutyIdRemovedEvent> domainEvent)
        {
            GlobalDutyTemplateId = "";
        }
        #endregion                                                       

        #region DutyTemplate en TimeSlotTemplates
        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyCategoryDutyNoteModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CheckChanges();

            Changes.Note = aggregateEvent.Note;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CheckChanges();

            if (Changes.TimeSlotTemplates == null)
            {
                Changes.TimeSlotTemplates = new List<TimeSlotTemplateModel>();
            }


            Changes.TimeSlotTemplates
                .Add(new TimeSlotTemplateModel(new TimeSlotTemplate(aggregateEvent.TemplateTimeSlotId)));

            if (TimeSlotTemplates != null && TimeSlotTemplates.Count == Changes.TimeSlotTemplates.Count)
            {

            }

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateDeletedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotToRemove = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);

            Changes.TimeSlotTemplates.Remove(timeSlotToRemove);

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateEndTimeModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent; 
            var timeSlotToEdit = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);
            timeSlotToEdit.EndTime = aggregateEvent.EndTime;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateNameModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotToEdit = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);
            timeSlotToEdit.TimeSlotName = aggregateEvent.TimeSlotName;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateNoteModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotToEdit = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);
            timeSlotToEdit.Note = aggregateEvent.Note;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateStartTimeModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotToEdit = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);
            timeSlotToEdit.StartTime = aggregateEvent.StartTime;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTemplateTimeSlotTemplateWeekdaysModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotToEdit = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.TemplateTimeSlotId);
            timeSlotToEdit.WeekDays = aggregateEvent.WeekDays;

        }

        #endregion

        #region Duties
        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var duty = new DutyModel( new Duty(aggregateEvent.Id));
            duty.Changes = new DutyChangesModel(new Duty(aggregateEvent.Id));
            if (Duties == null)
                Duties = new List<DutyModel>();
            Duties.Add(duty);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyDeletedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var duty = CheckDuty(aggregateEvent.Id);

            duty.Changes.MarkedForDeletion = true;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyModifiedEndEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var duty = CheckDuty(aggregateEvent.Id);
            duty.Changes.End = aggregateEvent.End;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyModifiedNoteEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent; 
            var duty = CheckDuty(aggregateEvent.Id);
            duty.Changes.Note = aggregateEvent.Note;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyModifiedStartEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var duty = CheckDuty(aggregateEvent.Id);
            duty.Changes.Start = aggregateEvent.Start;

        }

        #endregion

        #region TimeSlots

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var duty = CheckDuty(aggregateEvent.IdDuty);
            if (duty.Changes.TimeSlots == null)
            {
                duty.Changes.TimeSlots = new List<TimeSlotModelNoMore>();
            }

            duty.Changes.TimeSlots.Add(new TimeSlotModelNoMore(new TimeSlot(aggregateEvent.IdTimeSlot)));
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotDeletedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CheckDutyAndTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                out var duty, out var timeSlot);
            duty.Changes.TimeSlots.Remove(timeSlot);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotEndTimeModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            timeSlot.EndTime = aggregateEvent.EndTime;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotNameModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            timeSlot.TimeSlotName = aggregateEvent.Name;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotNoteModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            timeSlot.Note = aggregateEvent.Note;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotStartTimeModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            timeSlot.StartTime = aggregateEvent.StartTime;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyTimeSlotWeekdaysModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            timeSlot.WeekDays = aggregateEvent.WeekDays;
        }


        #endregion

        #region DutyContacts
        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyContactAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            if (aggregateEvent.IdTimeSlot == null)
            {
                var duty = CheckDuty(aggregateEvent.IdDuty);
                foreach (var ts in duty.TimeSlots)
                {
                    var dc = new DutyContactModel(
                        new DutyContact(aggregateEvent.IdDutyContact, aggregateEvent.EmployeeId, aggregateEvent.OrderId));
                    if (ts.DutyContacts == null)
                    {
                        ts.DutyContacts = new List<DutyContactModel>();
                    }

                    ts.DutyContacts.Add(dc);
                }
            }
            var timeSlot = CheckTimeSlot(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot);
            var duc = new DutyContactModel(
                new DutyContact(aggregateEvent.IdDutyContact, aggregateEvent.EmployeeId,
                aggregateEvent.OrderId));
            if (timeSlot.DutyContacts == null)
            {
                timeSlot.DutyContacts = new List<DutyContactModel>();
            }
            timeSlot.DutyContacts.Add(duc);

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyContactDeletedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CheckTimeSlotAndContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot, aggregateEvent.IdDutyContact, out var timeSlot, out var dutyContact);
            timeSlot.DutyContacts.Remove(dutyContact);

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyContactModifiedEmployeeEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            dutyContact.EmployeeId = aggregateEvent.EmployeeId;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyContactModifiedOrderIdEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            dutyContact.OrderId = aggregateEvent.OrderId;

        }

        #endregion

        #region ContactPoints en Tags
        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyContactPointAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            if (dutyContact.ContactPoints == null)
            {
                dutyContact.ContactPoints = new List<DutyContactPointModel>();
            }

            dutyContact.ContactPoints.Add(new DutyContactPointModel( 
                new DutyContactPoint(aggregateEvent.IdDutyContactPoint, aggregateEvent.Description, aggregateEvent.ContactDetailId)));

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyContactPointDeletedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            var dutyContactPoint = dutyContact.ContactPoints.Single(x => x.Id == aggregateEvent.IdDutyContactPoint.Value);
            dutyContact.ContactPoints.Remove(dutyContactPoint);

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyContactTagAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            if (dutyContact.Tags == null)
            {
                dutyContact.Tags = new List<string>();
            }

            dutyContact.Tags.Add(aggregateEvent.Tag);

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<DutyCategoryAggregate, DutyCategoryId, DutyContactTagDeletedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var dutyContact = CheckDutyContact(aggregateEvent.IdDuty, aggregateEvent.IdTimeSlot,
                aggregateEvent.IdDutyContact);
            dutyContact.Tags.Remove(aggregateEvent.Tag);

        }

        #endregion

        #region Helpertjes

        private void CheckChanges()
        {
            if (Changes == null)
            {
                Changes = new DutyCategoryChangesReadModel(this);
                //Changes.TemplateDuty = TemplateDuty;
            }
        }

        private TimeSlotTemplateModel CheckChangesAndGetTimeSlotTemplate(
            TimeSlotTemplateId id)
        {
            CheckChanges();
            var timeSlot = Changes.TimeSlotTemplates
                .Single(x => x.Id == id.Value);
            return timeSlot;
        }

        private DutyModel CheckDuty(DutyId id)
        {
            var duty = Duties.Single(x => x.Id == id.Value);
            if (duty.Changes == null)
            {
                duty.Changes = new DutyChangesModel(duty);
            }

            return duty;
        }

        private void CheckDutyAndTimeSlot(DutyId dutyId,
            TimeSlotId timeSlotId, out DutyModel duty, out TimeSlotModelNoMore timeSlot)
        {
            duty = CheckDuty(dutyId);
            timeSlot = duty.Changes.TimeSlots.Single(x => x.Id == timeSlotId.Value);
        }

        private TimeSlotModelNoMore CheckTimeSlot(DutyId dutyId,
            TimeSlotId timeSlotId)
        {
            var duty = CheckDuty(dutyId);
            return duty.Changes.TimeSlots.Single(x => x.Id == timeSlotId.Value);
        }

        private void CheckTimeSlotAndContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId dutyContactId,
            out TimeSlotModelNoMore timeSlot,
            out DutyContactModel dutyContact)
        {
            CheckDutyAndTimeSlot(dutyId, timeSlotId, out var duty, out timeSlot);
            dutyContact = timeSlot.DutyContacts.Single(x => x.Id == dutyContactId.Value);
        }


        private DutyContactModel CheckDutyContact(DutyId dutyId,
            TimeSlotId timeSlotId,
            DutyContactId dutyContactId)
        {
            CheckDutyAndTimeSlot(dutyId, timeSlotId, out var duty, out var timeSlot);
            return timeSlot.DutyContacts.Single(x => x.Id == dutyContactId.Value);
        }




        #endregion
    }
}
