﻿using System;
using System.Collections.Generic;
using System.Linq;
using CRM.Domain.DutyCategoryAndTemplate.Abstract;
using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using CRM.Domain.DutyCategoryAndTemplate.Events.GlobalDutyTemplate;
using CRM.Domain.DutyCategoryAndTemplate.Events.GlobalDutyTemplate.TimeSlots;
using CRM.Domain.DutyCategoryAndTemplate.Models;
using EventFlow.Aggregates;
using EventFlow.ReadStores;

namespace CRM.Domain.DutyCategoryAndTemplate.ReadModels
{
    public class GlobalDutyTemplateReadModel : ReadModel,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateCreatedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateDeletedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateTitleModifiedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateNoteModifiedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateTimeSlotAddedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateTimeSlotDeletedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId,
            GlobalDutyTemplateTimeSlotEndTimeModifiedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateTimeSlotNameModifiedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateTimeSlotNoteModifiedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId,
            GlobalDutyTemplateTimeSlotStartTimeModifiedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId,
            GlobalDutyTemplateTimeSlotWeekdaysModifiedEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateModifiedUsageEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateSaveEvent>,
        IAmReadModelFor<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateCancelEvent>
    {
        public string Title { get; private set; } = "---No title saved---";
        public string Note { get; private set; } = "";
        public int Usage { get; private set; } = 0;

        public string LastModifiedId { get; private set; }
        public string CreatedId { get; private set; }
        public string LastModifiedName { get; private set; }
        public string CreatedName { get; set; }
        public DateTime LastModifiedOn { get; private set; }
        public DateTime CreatedOn { get; private set; }

        public ICollection<TimeSlotTemplateModel> TimeSlotTemplates { get; private set; } =
            new List<TimeSlotTemplateModel>();

        public GlobalDutyTemplateChangesReadModel Changes { get; private set; }

        public bool Deleted { get; private set; }


        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateCreatedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Id = domainEvent.AggregateIdentity.Value;
            Changes = new GlobalDutyTemplateChangesReadModel();
            CreatedOn = DateTime.UtcNow;
            LastModifiedOn = DateTime.UtcNow;

            try
            {
                CreatedId = domainEvent.Metadata["userId"];
                LastModifiedId = domainEvent.Metadata["userId"];
                CreatedName = domainEvent.Metadata["userName"];
                LastModifiedName = domainEvent.Metadata["userName"];
            }
            catch (Exception e)
            {
                CreatedId = "TestId";
                LastModifiedId = "TestId";
                CreatedName = "TestName";
                LastModifiedName = "TestName";
            }
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateDeletedEvent> domainEvent)
        {
            context.MarkForDeletion();
            Deleted = true;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateTitleModifiedEvent>
                domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            if (Changes == null)
                Changes = new GlobalDutyTemplateChangesReadModel(Title, Note, TimeSlotTemplates);
            Changes.Title = aggregateEvent.Title;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateNoteModifiedEvent>
                domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            if (Changes == null)
                Changes = new GlobalDutyTemplateChangesReadModel(Title, Note, TimeSlotTemplates);

            Changes.Note = aggregateEvent.Note;

        }
        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateModifiedUsageEvent>
                domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var currentUsage = 0 + Usage;
            currentUsage = aggregateEvent.Increase ? currentUsage += 1 : currentUsage -= 1;
            if (currentUsage < 0)
            {
                Usage = 0;
                return;
            }
            Usage = currentUsage;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateSaveEvent> domainEvent)
        {
            if (Changes == null) return;
            Title = Changes.Title;
            Note = Changes.Note;
            TimeSlotTemplates = Changes.TimeSlotTemplates;

            LastModifiedOn = DateTime.UtcNow;

            try
            {
                LastModifiedId = domainEvent.Metadata["userId"];
                LastModifiedName = domainEvent.Metadata["userName"];
            }
            catch (Exception e)
            {
                LastModifiedId = "TestId";
                LastModifiedName = "TestName";
            }
            Changes = null;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateCancelEvent> domainEvent)
        {
            if (Changes == null) return;

            Changes = null;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateTimeSlotAddedEvent>
                domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            if (Changes == null)
                Changes = new GlobalDutyTemplateChangesReadModel(Title, Note, TimeSlotTemplates);

            if (Changes.TimeSlotTemplates == null)
            {
                Changes.TimeSlotTemplates = new List<TimeSlotTemplateModel>();
            }

            Changes.TimeSlotTemplates.Add(
                new TimeSlotTemplateModel(
                    new TimeSlotTemplate(aggregateEvent.IdTimeSlotTemplate)));

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateTimeSlotDeletedEvent>
                domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotTemplate = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.IdTimeSlotTemplate);
            Changes.TimeSlotTemplates.Remove(timeSlotTemplate);

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId,
                GlobalDutyTemplateTimeSlotEndTimeModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotTemplate = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.IdTimeSlotTemplate);
            timeSlotTemplate.EndTime = aggregateEvent.EndTime;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateTimeSlotNameModifiedEvent>
                domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotTemplate = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.IdTimeSlotTemplate);
            timeSlotTemplate.TimeSlotName = aggregateEvent.Name;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId, GlobalDutyTemplateTimeSlotNoteModifiedEvent>
                domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotTemplate = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.IdTimeSlotTemplate);
            timeSlotTemplate.Note = aggregateEvent.Note;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId,
                GlobalDutyTemplateTimeSlotStartTimeModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotTemplate = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.IdTimeSlotTemplate);
            timeSlotTemplate.StartTime = aggregateEvent.StartTime;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<GlobalDutyTemplateAggregate, GlobalDutyTemplateId,
                GlobalDutyTemplateTimeSlotWeekdaysModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var timeSlotTemplate = CheckChangesAndGetTimeSlotTemplate(aggregateEvent.IdTimeSlotTemplate);
            timeSlotTemplate.WeekDays = aggregateEvent.Weekdays;

        }

        private TimeSlotTemplateModel CheckChangesAndGetTimeSlotTemplate(
            TimeSlotTemplateId id)
        {
            if (Changes == null)
                Changes = new GlobalDutyTemplateChangesReadModel(Title, Note, TimeSlotTemplates);
            var timeSlot = Changes.TimeSlotTemplates
                .Single(x => x.Id == id.Value);
            return timeSlot;
        }

    }
}
