﻿using System.Collections.Generic;
using CRM.Domain.Employee.Entities;
using CRM.Domain.Shared.Enums;
using EventFlow.Aggregates;
using EventFlow.EventStores;

namespace CRM.Domain.Employee.Events
{
    [EventVersion("ContactDetailAddedEvent", 2)]
    public class ContactDetailAddedEvent : AggregateEvent<EmployeeAggregate, EmployeeId>
    {
        public ContactDetailAddedEvent(bool isPrivate,
            bool isInternal,
            bool isDefault,
            ContactDetailType contactDetailType,
            string contactDetailName,
            string contactDetailValue,
            EmployeeContactDetailId id)
        {
            Id = id;
            IsPrivate = isPrivate;
            IsInternal = isInternal;
            IsDefault = isDefault;
            ContactDetailType = contactDetailType;
            ContactDetailName = contactDetailName;
            ContactDetailValue = contactDetailValue;
        }

        public bool IsInternal { get; }

        public EmployeeContactDetailId Id { get; }
        public bool IsPrivate { get; }
        public bool IsDefault { get; }
        public ContactDetailType ContactDetailType { get; }
        public string ContactDetailName { get; }
        public string ContactDetailValue { get; }
    }

    public class UpdateContactDetailAddedEventV1ToV2 : IEventUpgrader<EmployeeAggregate, EmployeeId>
    {
        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateContactDetailAddedEventV1ToV2(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }

        public IEnumerable<IDomainEvent<EmployeeAggregate, EmployeeId>> Upgrade(IDomainEvent<EmployeeAggregate, EmployeeId> domainEvent)
        {
            var event1 = domainEvent as IDomainEvent<EmployeeAggregate, EmployeeId, ContactDetailAddedEventV1>;
            yield return event1 == null
                ? domainEvent
                : _domainEventFactory.Upgrade<EmployeeAggregate, EmployeeId>(domainEvent,
                    new ContactDetailAddedEvent(
                        event1.AggregateEvent.IsPrivate,
                        event1.AggregateEvent.IsInternal,
                        false,
                        event1.AggregateEvent.contactDetailType,
                        event1.AggregateEvent.ContactDetailName,
                        event1.AggregateEvent.ContactDetailValue,
                        event1.AggregateEvent.Id
                    ));
        }
    }

    [EventVersion("ContactDetailAddedEvent", 1)]
    public class ContactDetailAddedEventV1 : AggregateEvent<EmployeeAggregate, EmployeeId>
    {
        public ContactDetailAddedEventV1(bool isPrivate,
            bool isInternal,
            bool isDefault,
            ContactDetailType contactDetailType,
            string contactDetailName,
            string contactDetailValue,
            EmployeeContactDetailId id)
        {
            Id = id;
            IsPrivate = isPrivate;
            IsInternal = isInternal;
            this.contactDetailType = contactDetailType;
            ContactDetailName = contactDetailName;
            ContactDetailValue = contactDetailValue;
        }

        public bool IsInternal { get; }

        public EmployeeContactDetailId Id { get; }
        public bool IsPrivate { get; }

        public ContactDetailType contactDetailType { get; }
        public string ContactDetailName { get; }
        public string ContactDetailValue { get; }
    }

}