﻿using System.Collections.Generic;
using CRM.Domain.Employee.Entities;
using CRM.Domain.Shared.Enums;
using EventFlow.Aggregates;
using EventFlow.EventStores;

namespace CRM.Domain.Employee.Events
{
    [EventVersion("ContactDetailModifiedEvent", 3)]
    public class ContactDetailModifiedEvent : AggregateEvent<EmployeeAggregate, EmployeeId>
    {
        public ContactDetailModifiedEvent(EmployeeContactDetailId id,
            bool isPrivate,
            bool isInternal,
            bool isDefault,
            ContactDetailType contactDetailType,
            string contactDetailName,
            string contactDetailValue)
        {
            Id = id;
            IsPrivate = isPrivate;
            IsInternal = isInternal;
            IsDefault = isDefault;
            ContactDetailType = contactDetailType;
            ContactDetailName = contactDetailName;
            ContactDetailValue = contactDetailValue;
        }

        public EmployeeContactDetailId Id { get; }
        public bool IsPrivate { get; }
        public bool IsInternal { get; }
        public bool IsDefault { get; }
        public ContactDetailType ContactDetailType { get; }
        public string ContactDetailName { get; }
        public string ContactDetailValue { get; }
    }

    public class UpdateContactDetailModifiedEventv2Tov3 : IEventUpgrader<EmployeeAggregate, EmployeeId>
    {
        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateContactDetailModifiedEventv2Tov3(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }

        public IEnumerable<IDomainEvent<EmployeeAggregate, EmployeeId>> Upgrade(IDomainEvent<EmployeeAggregate, EmployeeId> domainEvent)
        {
            var event1 = domainEvent as IDomainEvent<EmployeeAggregate, EmployeeId, ContactDetailModifiedEventV2>;
            yield return event1 == null
                ? domainEvent
                : _domainEventFactory.Upgrade<EmployeeAggregate, EmployeeId>(domainEvent,
                    new ContactDetailModifiedEvent(
                        event1.AggregateEvent.Id,
                        event1.AggregateEvent.IsPrivate,
                        event1.AggregateEvent.IsInternal,
                        false,
                        event1.AggregateEvent.ContactDetailType,
                        event1.AggregateEvent.ContactDetailName,
                        event1.AggregateEvent.ContactDetailValue
                    ));
        }
    }


    [EventVersion("ContactDetailModifiedEvent", 2)]
    public class ContactDetailModifiedEventV2 : AggregateEvent<EmployeeAggregate, EmployeeId>
    {
        public ContactDetailModifiedEventV2(EmployeeContactDetailId id,
            bool isPrivate,
            bool isInternal,
            ContactDetailType contactDetailType,
            string contactDetailName,
            string contactDetailValue)
        {
            Id = id;
            IsPrivate = isPrivate;
            IsInternal = isInternal;
            this.ContactDetailType = contactDetailType;
            ContactDetailName = contactDetailName;
            ContactDetailValue = contactDetailValue;
        }

        public EmployeeContactDetailId Id { get; }
        public bool IsPrivate { get; }
        public bool IsInternal { get; }
        public ContactDetailType ContactDetailType { get; }
        public string ContactDetailName { get; }
        public string ContactDetailValue { get; }
    }

    public class UpdateContactDetailModifiedEventv1Tov2 : IEventUpgrader<EmployeeAggregate, EmployeeId>
    {
        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateContactDetailModifiedEventv1Tov2(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }

        public IEnumerable<IDomainEvent<EmployeeAggregate, EmployeeId>> Upgrade(IDomainEvent<EmployeeAggregate, EmployeeId> domainEvent)
        {
            var event1 = domainEvent as IDomainEvent<EmployeeAggregate, EmployeeId, ContactDetailModifiedEventV1>;
            yield return event1 == null
                ? domainEvent
                : _domainEventFactory.Upgrade<EmployeeAggregate, EmployeeId>(domainEvent,
                    new ContactDetailModifiedEventV2(
                        event1.AggregateEvent.Id,
                        event1.AggregateEvent.IsPrivate,
                        false,
                        event1.AggregateEvent.contactDetailType,
                        event1.AggregateEvent.ContactDetailName,
                        event1.AggregateEvent.ContactDetailValue
                    ));
        }
    }

    [EventVersion("ContactDetailModifiedEvent", 1)]
    public class ContactDetailModifiedEventV1 : AggregateEvent<EmployeeAggregate, EmployeeId>
    {
        public ContactDetailModifiedEventV1(EmployeeContactDetailId id,
            bool isPrivate,
            ContactDetailType contactDetailType,
            string contactDetailName,
            string contactDetailValue)
        {
            Id = id;
            IsPrivate = isPrivate;
            this.contactDetailType = contactDetailType;
            ContactDetailName = contactDetailName;
            ContactDetailValue = contactDetailValue;
        }

        public EmployeeContactDetailId Id { get; }
        public bool IsPrivate { get; }
        public ContactDetailType contactDetailType { get; }
        public string ContactDetailName { get; }
        public string ContactDetailValue { get; }
    }
}