﻿using System.Collections.Generic;
using CRM.Domain.Employee.Models;
using EventFlow.Aggregates;
using EventFlow.EventStores;

namespace CRM.Domain.Employee.Events
{
    [EventVersion("EmployeeCreatedEvent", 3)]
    public class EmployeeCreatedEvent : AggregateEvent<EmployeeAggregate, EmployeeId>
    {
        public EmployeeCreatedEvent(string firstName,
            string lastName,
            string cCEmail,
            string employeeComment,
            bool sendMailWithTicket,
            string customerId,
            bool setTicketAsRead,
            ICollection<EmployeeContactDetailModel> contactDetails,
            ICollection<string> tags,
            ICollection<string> defaultTags,
            bool canBeOnDuty,
            Address employeeAddress)
        {
            FirstName = firstName;
            LastName = lastName;
            CCEmail = cCEmail;
            EmployeeComment = employeeComment;
            SendMailWithTicket = sendMailWithTicket;
            CustomerId = customerId;
            SetTicketAsRead = setTicketAsRead;
            ContactDetails = contactDetails;
            Tags = tags;
            DefaultTags = defaultTags;
            CanBeOnDuty = canBeOnDuty;
            EmployeeAddress = employeeAddress;
        }

        public string FirstName { get; }
        public string LastName { get; }
        public string CCEmail { get; }
        public string EmployeeComment { get; }
        public bool SendMailWithTicket { get; }
        public string CustomerId { get; }
        public bool SetTicketAsRead { get; }
        public ICollection<EmployeeContactDetailModel> ContactDetails { get; }
        public ICollection<string> Tags { get; }
        public ICollection<string> DefaultTags { get; }
        public bool CanBeOnDuty { get; }
        public Address EmployeeAddress { get; }

        public static EmployeeCreatedEvent UpdateFromV2(EmployeeCreatedEventV2 v2)
        {
            return new EmployeeCreatedEvent(v2.FirstName, v2.LastName, v2.CCEmail, v2.EmployeeComment,
                v2.SendMailWithTicket, v2.CustomerId, v2.SetTicketAsRead, v2.ContactDetails, v2.Tags, new List<string>(),  v2.CanBeOnDuty,
                new Address());
        }
    }

    public class UpdateEmployeeCreatedEventV2ToV3 : IEventUpgrader<EmployeeAggregate, EmployeeId>
    {
        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateEmployeeCreatedEventV2ToV3(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }

        public IEnumerable<IDomainEvent<EmployeeAggregate, EmployeeId>> Upgrade(
            IDomainEvent<EmployeeAggregate, EmployeeId> domainEvent)
        {
            yield return !(domainEvent is IDomainEvent<EmployeeAggregate, EmployeeId, EmployeeCreatedEventV2> event2)
                ? domainEvent
                : _domainEventFactory.Upgrade<EmployeeAggregate, EmployeeId>(domainEvent,
                    EmployeeCreatedEvent.UpdateFromV2(event2.AggregateEvent));
        }
    }


    [EventVersion("EmployeeCreatedEvent", 2)]
    public class EmployeeCreatedEventV2 : AggregateEvent<EmployeeAggregate, EmployeeId>
    {
        public EmployeeCreatedEventV2(string firstName,
            string lastName,
            string cCEmail,
            string employeeComment,
            bool sendMailWithTicket,
            string customerId,
            bool setTicketAsRead,
            ICollection<EmployeeContactDetailModel> contactDetails,
            ICollection<string> tags,
            bool canBeOnDuty,
            Address employeeAddress)
        {
            FirstName = firstName;
            LastName = lastName;
            CCEmail = cCEmail;
            EmployeeComment = employeeComment;
            SendMailWithTicket = sendMailWithTicket;
            CustomerId = customerId;
            SetTicketAsRead = setTicketAsRead;
            ContactDetails = contactDetails;
            Tags = tags;
            CanBeOnDuty = canBeOnDuty;
            EmployeeAddress = employeeAddress;
        }

        public string FirstName { get; }
        public string LastName { get; }
        public string CCEmail { get; }
        public string EmployeeComment { get; }
        public bool SendMailWithTicket { get; }
        public string CustomerId { get; }
        public bool SetTicketAsRead { get; }
        public ICollection<EmployeeContactDetailModel> ContactDetails { get; }
        public ICollection<string> Tags { get; }
        public bool CanBeOnDuty { get; }
        public Address EmployeeAddress { get; }

        public static EmployeeCreatedEventV2 UpdateFromV1(EmployeeCreatedEventV1 v1)
        {
            return new EmployeeCreatedEventV2(v1.FirstName, v1.LastName, v1.CCEmail, v1.EmployeeComment,
                v1.SendMailWithTicket, v1.CustomerId, v1.SetTicketAsRead, v1.ContactDetails, v1.Tags, false,
                new Address());
        }
    }

    public class UpdateEmployeeCreatedEventV1ToV2 : IEventUpgrader<EmployeeAggregate, EmployeeId>
    {
        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateEmployeeCreatedEventV1ToV2(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }

        public IEnumerable<IDomainEvent<EmployeeAggregate, EmployeeId>> Upgrade(
            IDomainEvent<EmployeeAggregate, EmployeeId> domainEvent)
        {
            yield return !(domainEvent is IDomainEvent<EmployeeAggregate, EmployeeId, EmployeeCreatedEventV1> event1)
                ? domainEvent
                : _domainEventFactory.Upgrade<EmployeeAggregate, EmployeeId>(domainEvent,
                    EmployeeCreatedEventV2.UpdateFromV1(event1.AggregateEvent));
        }
    }

    [EventVersion("EmployeeCreatedEvent", 1)]
    public class EmployeeCreatedEventV1 : AggregateEvent<EmployeeAggregate, EmployeeId>
    {
        public EmployeeCreatedEventV1(string firstName,
            string lastName,
            string cCEmail,
            string employeeComment,
            bool sendMailWithTicket,
            string customerId,
            bool setTicketAsRead,
            ICollection<EmployeeContactDetailModel> contactDetails,
            ICollection<string> tags)
        {
            FirstName = firstName;
            LastName = lastName;
            CCEmail = cCEmail;
            EmployeeComment = employeeComment;
            SendMailWithTicket = sendMailWithTicket;
            CustomerId = customerId;
            SetTicketAsRead = setTicketAsRead;
            ContactDetails = contactDetails;
            Tags = tags;
        }

        public string FirstName { get; }
        public string LastName { get; }
        public string CCEmail { get; }
        public string EmployeeComment { get; }
        public bool SendMailWithTicket { get; }
        public string CustomerId { get; }
        public bool SetTicketAsRead { get; }
        public ICollection<EmployeeContactDetailModel> ContactDetails { get; }
        public ICollection<string> Tags { get; }
    }
}