﻿using System.Collections.Generic;
using CRM.Domain.Employee.Models;
using EventFlow.Aggregates;
using EventFlow.EventStores;

namespace CRM.Domain.Employee.Events
{
    [EventVersion("EmployeeModifiedEvent", 2)]
    public class EmployeeModifiedEvent : AggregateEvent<EmployeeAggregate, EmployeeId>
    {
        public EmployeeModifiedEvent(string firstName,
            string lastName,
            string cCEmail,
            string employeeComment,
            bool sendMailWithTicket,
            string customerId,
            bool setTicketAsRead,
            ICollection<EmployeeContactDetailModel> contactDetails,
            ICollection<string> tags,
            ICollection<string> defaultTags,
            bool canBeOnDuty,
            Address employeeAddress)
        {
            SendMailWithTicket = sendMailWithTicket;
            CCEmail = cCEmail;
            CustomerId = customerId;
            EmployeeComment = employeeComment;
            FirstName = firstName;
            LastName = lastName;
            SetTicketAsRead = setTicketAsRead;
            ContactDetails = contactDetails;
            Tags = tags;
            DefaultTags = defaultTags;
            CanBeOnDuty = canBeOnDuty;
            EmployeeAddress = employeeAddress;
        }

        public bool SendMailWithTicket { get; }
        public string CCEmail { get; }
        public string CustomerId { get; }
        public string EmployeeComment { get; }
        public string FirstName { get; }
        public string LastName { get; }
        public bool SetTicketAsRead { get; internal set; }
        public ICollection<EmployeeContactDetailModel> ContactDetails { get; }
        public ICollection<string> Tags { get; }
        public ICollection<string> DefaultTags { get; }

        public bool CanBeOnDuty { get; }
        public Address EmployeeAddress { get; }
    }

    public class UpdateEmployeeModifiedEventV1ToV2 : IEventUpgrader<EmployeeAggregate, EmployeeId>
    {
        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateEmployeeModifiedEventV1ToV2(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }

        public IEnumerable<IDomainEvent<EmployeeAggregate, EmployeeId>> Upgrade(IDomainEvent<EmployeeAggregate, EmployeeId> domainEvent)
        {
            var event1 = domainEvent as IDomainEvent<EmployeeAggregate, EmployeeId, EmployeeModifiedEventV1>;
            yield return event1 == null
                ? domainEvent
                : _domainEventFactory.Upgrade<EmployeeAggregate, EmployeeId>(domainEvent,
                    new EmployeeModifiedEvent(
                        event1.AggregateEvent.FirstName,
                        event1.AggregateEvent.LastName,
                        event1.AggregateEvent.CCEmail,
                        event1.AggregateEvent.EmployeeComment,
                        event1.AggregateEvent.SendMailWithTicket,
                        event1.AggregateEvent.CustomerId,
                        event1.AggregateEvent.SetTicketAsRead,
                        event1.AggregateEvent.ContactDetails,
                        event1.AggregateEvent.Tags,
                        new List<string>(),
                        event1.AggregateEvent.CanBeOnDuty,
                        event1.AggregateEvent.EmployeeAddress
                    ));
        }
    }

    [EventVersion("EmployeeModifiedEvent", 1)]
    public class EmployeeModifiedEventV1 : AggregateEvent<EmployeeAggregate, EmployeeId>
    {
        public EmployeeModifiedEventV1(string firstName,
            string lastName,
            string cCEmail,
            string employeeComment,
            bool sendMailWithTicket,
            string customerId,
            bool setTicketAsRead,
            ICollection<EmployeeContactDetailModel> contactDetails,
            ICollection<string> tags,
            bool canBeOnDuty,
            Address employeeAddress)
        {
            SendMailWithTicket = sendMailWithTicket;
            CCEmail = cCEmail;
            CustomerId = customerId;
            EmployeeComment = employeeComment;
            FirstName = firstName;
            LastName = lastName;
            SetTicketAsRead = setTicketAsRead;
            ContactDetails = contactDetails;
            Tags = tags;
            CanBeOnDuty = canBeOnDuty;
            EmployeeAddress = employeeAddress;
        }

        public bool SendMailWithTicket { get; }
        public string CCEmail { get; }
        public string CustomerId { get; }
        public string EmployeeComment { get; }
        public string FirstName { get; }
        public string LastName { get; }
        public bool SetTicketAsRead { get; internal set; }
        public ICollection<EmployeeContactDetailModel> ContactDetails { get; }
        public ICollection<string> Tags { get; }
        public bool CanBeOnDuty { get; }
        public Address EmployeeAddress { get; }
    }

}