﻿using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.Entities;
using CRM.Domain.Employee.Models;
using EventFlow.Snapshots;

namespace CRM.Domain.Employee.Snapshots
{
    [SnapshotVersion("Employee", 2)]
    public class EmployeeSnapshot : ISnapshot
    {
        public bool Created { get; set; }
        public bool Deleted { get; set; }
        public Address EmployeeAddress { get; set; }
        public bool SendMailWithTicket { get; set; }
        public ICollection<EmployeeContactDetail> ContactDetails { get; set; }
        public ICollection<string> Tags { get; set; }
        public ICollection<string> DefaultTags { get; set; }
        public string CCEmail { get; set; }
        public string CustomerId { get; set; }
        public string EmployeeComment { get; set; }
        public string FirstName { get; set; }
        public string LastName { get; set; }
        public bool SetTicketAsRead { get; set; }
        public bool CanBeOnDuty { get; set; }
    }

    public class UpdateEmployeeSnapshotV1ToV2 : ISnapshotUpgrader<EmployeeSnapshotV1, EmployeeSnapshot>
    {
        public Task<EmployeeSnapshot> UpgradeAsync(EmployeeSnapshotV1 fromVersionedType,
            CancellationToken cancellationToken)
        {
            return Task.Run(() => new EmployeeSnapshot()
            {
                CanBeOnDuty = fromVersionedType.CanBeOnDuty,
                CCEmail = fromVersionedType.CCEmail,
                ContactDetails = fromVersionedType.ContactDetails,
                Created = fromVersionedType.Created,
                CustomerId = fromVersionedType.CustomerId,
                DefaultTags = new List<string>(),
                Deleted = fromVersionedType.Deleted,
                EmployeeAddress = fromVersionedType.EmployeeAddress,
                EmployeeComment = fromVersionedType.EmployeeComment,
                FirstName = fromVersionedType.FirstName,
                LastName = fromVersionedType.LastName,
                SendMailWithTicket = fromVersionedType.SendMailWithTicket,
                SetTicketAsRead = fromVersionedType.SetTicketAsRead,
                Tags = fromVersionedType.Tags
            });
        }
    }

    [SnapshotVersion("Employee", 1)]
    public class EmployeeSnapshotV1 : ISnapshot
    {
        public bool Created { get; set; }
        public bool Deleted { get; set; }
        public Address EmployeeAddress { get; set; }
        public bool SendMailWithTicket { get; set; }
        public ICollection<EmployeeContactDetail> ContactDetails { get; set; }
        public ICollection<string> Tags { get; set; }
        public string CCEmail { get; set; }
        public string CustomerId { get; set; }
        public string EmployeeComment { get; set; }
        public string FirstName { get; set; }
        public string LastName { get; set; }
        public bool SetTicketAsRead { get; set; }
        public bool CanBeOnDuty { get; set; }
    }

}