﻿using System;
using System.Collections.Generic;
using CRM.Domain.InteractionStatistic.Enums;
using EventFlow.Aggregates;
using EventFlow.EventStores;

namespace CRM.Domain.InteractionStatistic.Events
{
    [EventVersion("InteractionStatisticCreatedEvent", 2)]
    public class
        InteractionStatisticCreatedEvent : AggregateEvent<InteractionStatisticAggregate, InteractionStatisticId>
    {
        public InteractionStatisticCreatedEvent(string interactionIdKey,
            string dossierId,
            int dnis,
            DateTime? established,
            DateTime connected,
            TimeSpan duration,
            DateTime disconnected,
            InteractionType interactionTypeEnum,
            int? originalCcmMessageId,
            string ticketId
            )
        {
            InteractionIdKey = interactionIdKey;
            DossierId = dossierId;
            DNIS = dnis;
            Established = established;
            Connected = connected;
            Duration = duration;
            Disconnected = disconnected;
            InteractionTypeEnum = interactionTypeEnum;
            OriginalCcmMessageId = originalCcmMessageId;
            TicketId = ticketId;
        }

        public string InteractionIdKey { get; }
        public string DossierId { get; }
        public int DNIS { get; }
        public DateTime? Established { get; }
        public DateTime Connected { get; }
        public TimeSpan Duration { get; }
        public DateTime Disconnected { get; }
        public InteractionType InteractionTypeEnum { get; }
        public int? OriginalCcmMessageId { get;  }
        public string TicketId { get; }
    }


    public class
        UpdateInteractionStatisticCreatedEventv1Tov2 : IEventUpgrader<InteractionStatisticAggregate,
            InteractionStatisticId>
    {

        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateInteractionStatisticCreatedEventv1Tov2(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }

        public IEnumerable<IDomainEvent<InteractionStatisticAggregate, InteractionStatisticId>> Upgrade(IDomainEvent<InteractionStatisticAggregate, InteractionStatisticId> domainEvent)
        {
            var event1 =
                domainEvent as IDomainEvent<InteractionStatisticAggregate, InteractionStatisticId,
                    InteractionStatisticCreatedEventV1>;
            yield return event1 == null
                ? domainEvent
                : _domainEventFactory.Upgrade<InteractionStatisticAggregate, InteractionStatisticId>(domainEvent,
                    new InteractionStatisticCreatedEvent(
                        event1.AggregateEvent.InteractionIdKey,
                        event1.AggregateEvent.DossierId,
                        event1.AggregateEvent.DNIS,
                        event1.AggregateEvent.Established,
                        event1.AggregateEvent.Connected,
                        event1.AggregateEvent.Duration,
                        event1.AggregateEvent.Disconnected,
                        event1.AggregateEvent.InteractionTypeEnum,
                        null,
                        null));
        }
    }


    [EventVersion("InteractionStatisticCreatedEvent", 1)]
    public class
        InteractionStatisticCreatedEventV1 : AggregateEvent<InteractionStatisticAggregate, InteractionStatisticId>
    {
        public InteractionStatisticCreatedEventV1(string interactionIdKey,
            string dossierId,
            int dNIS,
            DateTime? established,
            DateTime connected,
            TimeSpan duration,
            DateTime disconnected,
            InteractionType interactionTypeEnum)
        {
            InteractionIdKey = interactionIdKey;
            DossierId = dossierId;
            DNIS = dNIS;
            Established = established;
            Connected = connected;
            Duration = duration;
            Disconnected = disconnected;
            InteractionTypeEnum = interactionTypeEnum;
        }

        public string InteractionIdKey { get; }
        public string DossierId { get; }
        public int DNIS { get; }
        public DateTime? Established { get; }
        public DateTime Connected { get; }
        public TimeSpan Duration { get; }
        public DateTime Disconnected { get; }
        public InteractionType InteractionTypeEnum { get; }
    }
}