﻿using System;
using CRM.Domain.InteractionStatistic.Enums;
using CRM.Domain.InteractionStatistic.Events;
using EventFlow.Aggregates;
using EventFlow.MongoDB.ReadStores;
using EventFlow.ReadStores;

namespace CRM.Domain.InteractionStatistic.ReadModels
{
    public class InteractionStatisticReadModel : IMongoDbReadModel,
        IAmReadModelFor<InteractionStatisticAggregate, InteractionStatisticId, InteractionStatisticCreatedEvent>,
        IAmReadModelFor<InteractionStatisticAggregate, InteractionStatisticId, InteractionStatisticDeletedEvent>,
        IAmReadModelFor<InteractionStatisticAggregate, InteractionStatisticId, InteractionStatisticModifiedDossierIdEvent>,
        IAmReadModelFor<InteractionStatisticAggregate, InteractionStatisticId, InteractionStatisticModifiedTicketIdEvent>
    {
        public bool Deleted { get; private set; }

        public string InteractionIdKey { get; set; }
        public string DossierId { get; set; }
        public int DNIS { get; set; }
        public DateTime? Established { get; set; }
        public DateTime Connected { get; set; }
        public TimeSpan Duration { get; set; }
        public DateTime Disconnected { get; set; }
        public InteractionType InteractionTypeEnum { get; set; }
        public int? OriginalCcmMessageId { get; set; }
        public string TicketId { get; set; }

        public void Apply(IReadModelContext context,
            IDomainEvent<InteractionStatisticAggregate, InteractionStatisticId, InteractionStatisticCreatedEvent>
                domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Id = domainEvent.AggregateIdentity.Value;
            InteractionIdKey = aggregateEvent.InteractionIdKey;
            DossierId = aggregateEvent.DossierId;
            DNIS = aggregateEvent.DNIS;
            Established = aggregateEvent.Established;
            Connected = aggregateEvent.Connected;
            Duration = aggregateEvent.Duration;
            Disconnected = aggregateEvent.Disconnected;
            InteractionTypeEnum = aggregateEvent.InteractionTypeEnum;
            OriginalCcmMessageId = aggregateEvent.OriginalCcmMessageId;
            TicketId = aggregateEvent.TicketId;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<InteractionStatisticAggregate, InteractionStatisticId, InteractionStatisticDeletedEvent>
                domainEvent)
        {
            context.MarkForDeletion();
            Deleted = true;
        }

        public string Id { get; set; }
        public long? Version { get; set; }

        public void Apply(IReadModelContext context,
            IDomainEvent<InteractionStatisticAggregate, InteractionStatisticId, InteractionStatisticModifiedDossierIdEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            DossierId = aggregateEvent.DossierId;

        }

        public void Apply(IReadModelContext context,
            IDomainEvent<InteractionStatisticAggregate, InteractionStatisticId, InteractionStatisticModifiedTicketIdEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            TicketId = aggregateEvent.TicketId;
        }
    }
}