﻿using CRM.Domain.NeoDuty.Models;
using EventFlow.Entities;
using System.Collections.Generic;
using System.Linq;

namespace CRM.Domain.NeoDuty.Entities
{
    public class DutyContact : Entity<DutyContactId>
    {
        public DutyContact(DutyContactId id) : base(id)
        {
        }

        public DutyContact(DutyContactId id,
            string employeeId,
            ICollection<DutyContactPoint> contactPoints,
            ICollection<string> tags,
            int orderId) : base(id)
        {
            EmployeeId = employeeId;
            ContactPoints = contactPoints;
            Tags = tags;
            OrderId = orderId;
        }

        public DutyContact(DutyContact original) : base(original.Id)
        {
            EmployeeId = original.EmployeeId;
            ContactPoints = original.ContactPoints?
                .Select(x =>
                    new DutyContactPoint(x))
                .ToList();
            Tags = original.Tags;
            OrderId = original.OrderId;
        }

        public DutyContact(DutyContactId id, DutyContactModel model) : base(id)
        {
            EmployeeId = model.EmployeeId;
            ContactPoints = model.ContactPoints?
                .Select(x => 
                    new DutyContactPoint(DutyContactPointId.With( x.Id),x))
                .ToList();
            Tags = model.Tags;
            OrderId = model.OrderId;
        }

        public DutyContact(DutyContactId id,
            string employeeId, int orderId) : base(id)
        {
            EmployeeId = employeeId;
            OrderId = orderId;
        }

        public string EmployeeId { get; set; }
        public ICollection<DutyContactPoint> ContactPoints { get; set; }
        public ICollection<string> Tags { get; set; }
        public int OrderId { get; set; }
    }
}