﻿using CRM.Domain.NeoDuty.Models;
using CRM.Domain.Shared.ValueObjects;
using EventFlow.Entities;
using System;
using System.Collections.Generic;
using System.Linq;

namespace CRM.Domain.NeoDuty.Entities
{
    public class TimeSlot: Entity<TimeSlotId>
    {
        public TimeSlot(TimeSlotId id) : base(id)
        {
        }

        public TimeSlot(TimeSlot original): base(original.Id)
        {
            StartTime = original.StartTime;
            EndTime = original.EndTime;
            WeekDays = original.WeekDays;
            Note = original.Note;
            TimeSlotName = original.TimeSlotName;
            DutyContacts = original.DutyContacts == null
                           ? null
                : original.DutyContacts.Select(x
                => new DutyContact(x)).ToList();
        }

        public TimeSlot(TimeSlotId id, TimeSlotModel model) : base(id)
        {
            StartTime = model.StartTime;
            EndTime = model.EndTime;
            WeekDays = model.WeekDays;
            Note = model.Note;
            TimeSlotName = model.TimeSlotName;
            DutyContacts = model.DutyContacts == null
                           ? null
                : model.DutyContacts.Select(x 
                => new DutyContact(DutyContactId.With(x.Id), x)).ToList();
        }

        public TimeSlot(TimeSlotId id, AddTimeslotModel model) : base(id)
        {
            StartTime = model.StartTime;
            EndTime = model.EndTime;
            WeekDays = model.WeekDays;
            Note = model.Note;
            TimeSlotName = model.TimeSlotName;
        }

        public TimeSpan StartTime { get; set; }
        public TimeSpan EndTime { get; set; }
        public ICollection<WeekDay> WeekDays { get; set; }
        public string Note { get; set; }
        public string TimeSlotName { get; set; }
        public ICollection<DutyContact> DutyContacts { get; set; }

    }
}
