﻿using CRM.Domain.NeoDuty.Entities;
using CRM.Domain.Shared.Abstract;
using CRM.Domain.Shared.ValueObjects;
using System;
using System.Collections.Generic;
using System.Linq;

namespace CRM.Domain.NeoDuty.Models
{
    public class TimeSlotModel: TimeSlotModelAb
    {
        public TimeSlotModel(TimeSpan startTime,
            TimeSpan endTime,
            ICollection<WeekDay> weekDays,
            string note,
            string timeSlotName,
            string id, ICollection<DutyContactModel> dutyContacts) : base(startTime, endTime, weekDays, note, timeSlotName, id)
        {
            DutyContacts = dutyContacts;
        }
        public TimeSlotModel()
        {

        }

        public TimeSlotModel(TimeSlotModel model)
        {
            Id = model.Id;
            StartTime = model.StartTime;
            EndTime = model.EndTime;
            WeekDays = model.WeekDays;
            Note = model.Note;
            TimeSlotName = model.TimeSlotName;

            if (model.DutyContacts == null) return;
            DutyContacts = model.DutyContacts
                .Select(x => new DutyContactModel(x))
                .ToList();
        }


        public TimeSlotModel(TimeSlot timeSlot)
        {
            Id = timeSlot.Id.Value;
            StartTime = timeSlot.StartTime;
            EndTime = timeSlot.EndTime;
            WeekDays = timeSlot.WeekDays;
            Note = timeSlot.Note;
            TimeSlotName = timeSlot.TimeSlotName;

            if (timeSlot.DutyContacts == null) return;
            DutyContacts = timeSlot.DutyContacts
                .Select(x => new DutyContactModel(x))
                .ToList();
        }

        public TimeSlotModel(AddTimeslotModel timeSlot)
        {
            Id = timeSlot.Id;
            StartTime = timeSlot.StartTime;
            EndTime = timeSlot.EndTime;
            WeekDays = timeSlot.WeekDays;
            Note = timeSlot.Note;
            TimeSlotName = timeSlot.TimeSlotName;
        }


        public ICollection<DutyContactModel> DutyContacts { get; set; }

    }
}
