﻿using System;
using System.Collections.Generic;
using System.Linq;
using CRM.Domain.Ticket.Events;
using CRM.Domain.Ticket.Models;
using EventFlow.Aggregates;
using EventFlow.MongoDB.ReadStores;
using EventFlow.ReadStores;

namespace CRM.Domain.Ticket.ReadModels
{
    public class TicketReadModel : IMongoDbReadModel,
        IAmReadModelFor<TicketAggregate, TicketId, CommentAddedEvent>,
        IAmReadModelFor<TicketAggregate, TicketId, CommentModifiedEvent>,
        IAmReadModelFor<TicketAggregate, TicketId, CommentRemovedEvent>,
        IAmReadModelFor<TicketAggregate, TicketId, TagAddedEvent>,
        IAmReadModelFor<TicketAggregate, TicketId, TagRemovedEvent>,
        IAmReadModelFor<TicketAggregate, TicketId, TicketCreatedEvent>,
        IAmReadModelFor<TicketAggregate, TicketId, TicketModifiedEvent>,
        IAmReadModelFor<TicketAggregate, TicketId, TicketDeletedEvent>,
        IAmReadModelFor<TicketAggregate, TicketId, TicketReadEvent>,
        IAmReadModelFor<TicketAggregate, TicketId, TicketUnreadEvent>,
        IAmReadModelFor<TicketAggregate, TicketId, TicketModifiedDossierIdEvent>

    {
        public bool Deleted { get; private set; }
        public ICollection<string> TicketStatuses { get; private set; } = new List<string>();
        public ICollection<string> Tags { get; private set; }

        public ICollection<TicketCommentModel> Comments { get; private set; }
            = new List<TicketCommentModel>();

        public int? OriginalCCMMessageId { get; private set; }
        public string DossierId { get; private set; }
        public string TicketText { get; private set; }
        public DateTime CreatedInCRM { get; private set; }
        public DateTime LastModifiedInCRM { get; private set; }
        public string CreatedCRMUser { get; private set; }
        public string LastModifiedCRMUser { get; private set; }

        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, CommentAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;

            Comments.Add(new TicketCommentModel(aggregateEvent.Id.Value, aggregateEvent.CommentBody,
                aggregateEvent.CommentQuote, aggregateEvent.UserId, domainEvent.Timestamp));
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, CommentModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            var comment = Comments.Single(x => x.Id.Equals(aggregateEvent.CommentId.Value));
            comment.CommentBody = aggregateEvent.CommentBody;
            comment.CommentQuote = aggregateEvent.CommentQuote;
            comment.LastModifiedDate = domainEvent.Timestamp.UtcDateTime;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, CommentRemovedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Comments.Remove(Comments.Single(x => x.Id.Equals(aggregateEvent.TicketCommentId.Value)));
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, TagAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Tags.Add(aggregateEvent.TagId);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, TagRemovedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Tags.Remove(aggregateEvent.TagId);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, TicketCreatedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Id = domainEvent.AggregateIdentity.Value;
            DossierId = aggregateEvent.DossierId;
            TicketText = aggregateEvent.TicketText;
            OriginalCCMMessageId = aggregateEvent.OriginalCCMMessageId;
            CreatedCRMUser = aggregateEvent.CreatedCRMUser;
            CreatedInCRM = aggregateEvent.CreatedInCRM;
            LastModifiedCRMUser = aggregateEvent.LastModifiedCRMUser;
            LastModifiedInCRM = aggregateEvent.LastModifiedInCRM;
            Tags = aggregateEvent.Tags;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, TicketDeletedEvent> domainEvent)
        {
            Deleted = true;
        }


        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, TicketModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            LastModifiedCRMUser = aggregateEvent.LastModifiedCRMUser;
            LastModifiedInCRM = aggregateEvent.LastModifiedInCRM;
            TicketText = aggregateEvent.TicketText;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, TicketReadEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;

            TicketStatuses.Add(aggregateEvent.UserId);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, TicketUnreadEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;

            TicketStatuses.Remove(aggregateEvent.UserId);
        }

        public string Id { get; set; } = "";
        public long? Version { get; set; }

        public void Apply(IReadModelContext context,
            IDomainEvent<TicketAggregate, TicketId, TicketModifiedDossierIdEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            DossierId = aggregateEvent.DossierId;

        }
    }
}