﻿using System.Collections.Generic;
using EventFlow.Aggregates;
using EventFlow.EventStores;

namespace CRM.Domain.User.Events
{
    [EventVersion("UserCreatedEvent", 2)]
    public class UserCreatedEvent : AggregateEvent<UserAggregate, UserId>
    {
        public UserCreatedEvent(string email,
            string userName,
            string customerId,
            bool isCustomerAdmin,
            string employeeId, string tenant)
        {
            Email = email;
            UserName = userName;
            CustomerId = customerId;
            IsCustomerAdmin = isCustomerAdmin;
            EmployeeId = employeeId;
            Tenant = tenant;
        }

        public string Email { get; }
        public string UserName { get; }
        public string CustomerId { get; }
        public bool IsCustomerAdmin { get; }
        public string EmployeeId { get; }
        public string Tenant { get; }
    }



    public class
        UpdateInteractionStatisticCreatedEventv1Tov2 : IEventUpgrader<UserAggregate,
            UserId>
    {

        private readonly IDomainEventFactory _domainEventFactory;

        public UpdateInteractionStatisticCreatedEventv1Tov2(IDomainEventFactory domainEventFactory)
        {
            _domainEventFactory = domainEventFactory;
        }

        public IEnumerable<IDomainEvent<UserAggregate, UserId>> Upgrade(IDomainEvent<UserAggregate, UserId> domainEvent)
        {
            var event1 =
                domainEvent as IDomainEvent<UserAggregate, UserId,
                    UserCreatedEventV1>;
            yield return event1 == null
                ? domainEvent
                : _domainEventFactory.Upgrade<UserAggregate, UserId>(domainEvent,
                    new UserCreatedEvent(
                        event1.AggregateEvent.Email,
                        event1.AggregateEvent.UserName,
                        event1.AggregateEvent.CustomerId,
                        event1.AggregateEvent.IsCustomerAdmin,
                        event1.AggregateEvent.EmployeeId, ""
                        ));
        }
    }
    [EventVersion("UserCreatedEvent", 1)]
    public class UserCreatedEventV1 : AggregateEvent<UserAggregate, UserId>
    {
        public UserCreatedEventV1(string email,
            string userName,
            string customerId,
            bool isCustomerAdmin,
            string employeeId)
        {
            Email = email;
            UserName = userName;
            CustomerId = customerId;
            IsCustomerAdmin = isCustomerAdmin;
            EmployeeId = employeeId;
        }

        public string Email { get; }
        public string UserName { get; }
        public string CustomerId { get; }
        public bool IsCustomerAdmin { get; }
        public string EmployeeId { get; }
    }
}