﻿using System.Collections.Generic;
using System.Linq;
using CRM.Domain.User.AbstractClasses;
using CRM.Domain.User.Events;
using CRM.Domain.User.Models;
using EventFlow.Aggregates;
using EventFlow.ReadStores;

namespace CRM.Domain.User.ReadModels
{
    public class UserReadModel : ReadModel,
        IAmReadModelFor<UserAggregate, UserId, ModuleRightCustomerAddedEvent>,
        IAmReadModelFor<UserAggregate, UserId, ModuleRightCustomerGroupAddedEvent>,
        IAmReadModelFor<UserAggregate, UserId, ModuleRightCustomerGroupModifiedEvent>,
        IAmReadModelFor<UserAggregate, UserId, ModuleRightCustomerGroupRemovedEvent>,
        IAmReadModelFor<UserAggregate, UserId, ModuleRightCustomerModifiedEvent>,
        IAmReadModelFor<UserAggregate, UserId, ModuleRightCustomerRemovedEvent>,
        IAmReadModelFor<UserAggregate, UserId, ModuleRightDossierAddedEvent>,
        IAmReadModelFor<UserAggregate, UserId, ModuleRightDossierModifiedEvent>,
        IAmReadModelFor<UserAggregate, UserId, ModuleRightDossierRemovedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserCompanyAdminAssignedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserConnectedToEmployeeEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserCreatedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserDeletedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserNameModifiedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserCustomerAdminModifiedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserCustomerGroupModuleRightsModifiedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserCustomerModifiedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserCustomerModuleRightsModifiedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserDossierModuleRightsModifiedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserEmailModifiedEvent>,
        IAmReadModelFor<UserAggregate, UserId, UserTenantSetEvent>


    {
        public bool Deleted { get; private set; }

        public string Email { get; private set; } = "";
        public string UserName { get; private set; } = "";

        public string? CustomerId { get; private set; } = "";
        public bool IsCustomerAdmin { get; private set; }
        public string EmployeeId { get; private set; } = "";
        public string AuthenticationId { get; private set; } = "";
        public string? Tenant { get; set; }

        public Dictionary<string, ModuleRightModel> CustomerModuleRights { get; private set; } =
            new Dictionary<string, ModuleRightModel>();

        public Dictionary<string, ModuleRightModel> CustomerGroupModuleRights { get; private set; } =
            new Dictionary<string, ModuleRightModel>();

        public Dictionary<string, ModuleRightModel> DossierModuleRights { get; private set; } =
            new Dictionary<string, ModuleRightModel>();

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, ModuleRightCustomerAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CustomerModuleRights.Add(aggregateEvent.CustomerId,aggregateEvent.ModuleRight);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, ModuleRightCustomerGroupAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CustomerGroupModuleRights.Add(aggregateEvent.CustomerGroupId,
                aggregateEvent.ModuleRight);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, ModuleRightCustomerGroupModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CustomerGroupModuleRights[aggregateEvent.CustomerGroupId] =
                aggregateEvent.ModuleRight;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, ModuleRightCustomerGroupRemovedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CustomerGroupModuleRights.Remove(aggregateEvent.CustomerGroupId);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, ModuleRightCustomerModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;

            CustomerModuleRights[aggregateEvent.CustomerId] = aggregateEvent.ModuleRight;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, ModuleRightCustomerRemovedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CustomerModuleRights.Remove(aggregateEvent.CustomerId);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, ModuleRightDossierAddedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            DossierModuleRights.Add(aggregateEvent.DossierId, aggregateEvent.ModuleRight);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, ModuleRightDossierModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            DossierModuleRights[aggregateEvent.DossierId] = aggregateEvent.ModuleRight;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, ModuleRightDossierRemovedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            DossierModuleRights.Remove(aggregateEvent.DossierId);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserCompanyAdminAssignedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            IsCustomerAdmin = aggregateEvent.IsCompanyAdmin;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserConnectedToEmployeeEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            EmployeeId = aggregateEvent.EmployeeId;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserCreatedEvent> domainEvent)
        {
            Id = domainEvent.AggregateIdentity.Value;
            var aggregateEvent = domainEvent.AggregateEvent;
            Email = aggregateEvent.Email;
            CustomerId = aggregateEvent.CustomerId;
            IsCustomerAdmin = aggregateEvent.IsCustomerAdmin;
            UserName = aggregateEvent.UserName;
            EmployeeId = aggregateEvent.EmployeeId;
            Tenant = aggregateEvent.Tenant;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserDeletedEvent> domainEvent)
        {
            context.MarkForDeletion();

            Deleted = true;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserNameModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;

            UserName = aggregateEvent.UserName;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserCustomerAdminModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            IsCustomerAdmin = aggregateEvent.IsCustomerAdmin;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserCustomerGroupModuleRightsModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CustomerGroupModuleRights = aggregateEvent.ModuleRights.ToDictionary(pair => pair.Key, pair => pair.Value);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserCustomerModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            CustomerId = aggregateEvent.CustomerId;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserCustomerModuleRightsModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;

            CustomerModuleRights = aggregateEvent.ModuleRights.ToDictionary(pair => pair.Key, pair => pair.Value);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserDossierModuleRightsModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;

            DossierModuleRights = aggregateEvent.ModuleRights.ToDictionary(pair => pair.Key, pair => pair.Value);
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserEmailModifiedEvent> domainEvent)
        {
            var aggregateEvent = domainEvent.AggregateEvent;
            Email = aggregateEvent.Email;
        }

        public void Apply(IReadModelContext context,
            IDomainEvent<UserAggregate, UserId, UserTenantSetEvent> domainEvent)
        {
            Tenant = domainEvent.AggregateEvent.Tenant;
        }

    }
}