﻿using EventFlow;
using EventFlow.Queries;
using EventFlow.ReadStores;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.DependencyInjection;
using WebPortal.API.Helpers;
using WebPortal.Application.Authorization;

namespace WebPortal.API.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    [Authorize]
    public class BaseController : Controller
    {
        private ICommandBus? _commandBus;
        private IQueryProcessor? _queryProcessor;
        private IReadModelPopulator? _readModelPopulator;
        private IUserContext? _userContext;
        private AuthorizationService? _authorizationService;

        protected IQueryProcessor QueryProcessor =>
            _queryProcessor ??= HttpContext.RequestServices.GetService<IQueryProcessor>();

        protected ICommandBus CommandBus => _commandBus ??= HttpContext.RequestServices.GetService<ICommandBus>();

        protected IReadModelPopulator ReadModelPopulator =>
            _readModelPopulator ??= HttpContext.RequestServices.GetService<IReadModelPopulator>();

        protected AuthorizationService AuthorizationService =>
            _authorizationService ??= HttpContext.RequestServices.GetService<AuthorizationService>();

        protected IUserContext UserContext => _userContext ??= HttpContext.RequestServices.GetService<IUserContext>();
    }
}