﻿using System;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.CallScript;
using CRM.Domain.CallScript.Entities;
using CRM.Domain.CallScript.ReadModels;
using Microsoft.AspNetCore.Mvc;
using WebPortal.Application.CallScripts.Commands;
using WebPortal.Application.CallScripts.Models;
using WebPortal.Application.CallScripts.Queries;

namespace WebPortal.API.Controllers
{
    public class CallscriptController : ReprojectController<CallScriptReadModel>
    {
        [HttpGet]
        [Route("/api/callscripts")]
        public async Task<IActionResult> Get([FromQuery] GetCallScriptsWithCommentsQuery query,
            CancellationToken cancellationToken)
        {
            try
            {

          
            var a = await AuthorizationService.GetDossierAccess(UserContext.User, query.DossierId, 
                cancellationToken);
            if (!a.CallScript.Read) return Unauthorized();
            var result = await QueryProcessor.ProcessAsync(query, cancellationToken);

            return Ok(result);
            }
            catch (TaskCanceledException e)
            {
                return NoContent();
            }
        }

        [HttpGet]
        [Route("{id}")]
        public async Task<IActionResult> Getscript([FromQuery] GetCallScriptsWithCommentsQuery query,
            CancellationToken cancellationToken)
        {
            try
            {
                var a = await AuthorizationService.GetDossierAccess(UserContext.User, query.DossierId,
                    cancellationToken);
                if (!a.CallScript.Read) return Unauthorized();
                var result = await QueryProcessor.ProcessAsync(query, cancellationToken);


                return Ok(result);
            }
            catch (TaskCanceledException e)
            {
                return NoContent();
            }
        }

        [HttpGet]
        [Route("numbers")]
        public async Task<IActionResult> GetDashboardData([FromQuery] GetDataForDashboardCallscriptQuery query,
            CancellationToken cancellationToken)
        {
            try
            {
                var a = await AuthorizationService.GetDossierAccess(UserContext.User, query.DossierId,
                    cancellationToken);
                if (!a.CallScript.Read) return Unauthorized();

                var result = await QueryProcessor.ProcessAsync(query, cancellationToken);

                if (result != null)
                    return Ok(result);

                return NoContent();
            }
            catch (TaskCanceledException e)
            {
                return NoContent();
            }
        }

        [HttpPost]
        [Route("comment")]
        public async Task<IActionResult> PostComment([FromBody] CreateCommentModel model,
            CancellationToken cancellationToken)
        {

            var q = new GetCallScriptQuery() { Id = model.CallscriptId};
            var script = await QueryProcessor.ProcessAsync(q, cancellationToken);
            if (!(await AuthorizationService.GetDossierAccess(UserContext.User, script.DossierId,
                cancellationToken)).CallScript.Update) return Unauthorized();
            
            var command = new CreateCallScriptCommentCommand(new CallScriptId(model.CallscriptId), model.CreatedOn,
                model.LastModified, model.CommentBody, model.CommentQuote, model.UserId, model.Id);
            try
            {
                var result = await CommandBus.PublishAsync(command, cancellationToken);
                return result.IsSuccess ? Created("", result) : StatusCode(500, "command failed");
            }
            catch (Exception e)
            {
                return StatusCode(500, e.Message);
            }
        }

        [HttpDelete]
        [Route("comment/{callscriptid}/{id}")]
        public async Task<IActionResult> DeleteComment(string id,
            string callscriptId,
            CancellationToken cancellationToken)
        {
            var q = new GetCallScriptQuery() { Id = callscriptId};
            var script = await QueryProcessor.ProcessAsync(q, cancellationToken);
            if (!(await AuthorizationService.GetDossierAccess(UserContext.User, script.DossierId,
                cancellationToken)).CallScript.Update) return Unauthorized();
            var command =
                new RemoveCallScriptCommentCommand(new CallScriptId(callscriptId), new CallScriptCommentId(id));
            var result = await CommandBus.PublishAsync(command, cancellationToken);

            if (result.IsSuccess)
                return Ok();
            return StatusCode(500, "command failed");
        }
    }
}