﻿using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Customer.ReadModels;
using Microsoft.AspNetCore.Mvc;
using WebPortal.Application.Customers.Queries;

namespace WebPortal.API.Controllers
{
    public class CustomerController : ReprojectController<CustomerReadModel>
    {
        [HttpGet]
        [Route("{id}")]
        public async Task<IActionResult> GetCustomerById(string id,
            CancellationToken cancellationToken)
        {
            if (!string.IsNullOrEmpty(UserContext.User.CustomerId) || (UserContext.User.CustomerId != id))
            {
                var p = await AuthorizationService.HasCustomerKey(UserContext.User, id, cancellationToken);
                if (!p)
                    return Unauthorized();
            }
            var query = new GetCustomerByIdQuery(id);
            var result = await QueryProcessor.ProcessAsync(query, cancellationToken);
            if (result == null) return NoContent();
            return Ok(result);
        }

        [HttpGet]
        [Route("name")]
        public async Task<IActionResult> GetCustomersByName([FromQuery] string? customerName,
            CancellationToken cancellationToken)
        {
            if (!string.IsNullOrEmpty(UserContext.User.CustomerId))
            {
                var q = new GetCustomerByIdQuery(UserContext.User.CustomerId);
                var r = await QueryProcessor.ProcessAsync(q, cancellationToken);
                var l = new List<CustomerReadModel>
                {
                    r
                };
                return Ok(l);
            }
            var query = new GetAllCustomersQuery(true, "CustomerName", 0, 20, customerName ?? "");
            var result = await QueryProcessor.ProcessAsync(query, cancellationToken);
            if (result == null) return NoContent();
            return Ok(result.Results);
        }

        [HttpGet]
        [Route("getname")]
        public async Task<IActionResult> GetCustomerName([FromQuery] string? id,
            CancellationToken cancellationToken)
        {
            var query = new GetCustomerByIdQuery(id);
            var result = await QueryProcessor.ProcessAsync(query, cancellationToken);
            if (result == null) return NoContent();
            return Ok(result.CustomerName);
        }

        [HttpGet]
        [Route("license")]
        public async Task<IActionResult> GetLicenseInfo([FromQuery] GetLicenseInfoQuery q,
            CancellationToken cancellationToken)
        {
            if (!string.IsNullOrEmpty(UserContext.User.CustomerId) || (UserContext.User.CustomerId != q.CustomerId))
                return Unauthorized();
            var result = await QueryProcessor.ProcessAsync(q, cancellationToken);

            return Ok(result);
        }
    }
}