﻿using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Branch;
using CRM.Domain.Dossier.ReadModels;
using Microsoft.AspNetCore.Mvc;
using WebPortal.Application.Branches.Commands;
using WebPortal.Application.Branches.Models;
using WebPortal.Application.Branches.Queries;
using WebPortal.Application.Dossiers.Queries;
using WebPortal.Application.Extentions;

namespace WebPortal.API.Controllers
{
    public class DossierController : ReprojectController<DossierReadModel>
    {
        //GET
        [HttpGet]
        [Route("/api/dossiers")]
        public async Task<IActionResult> GetAllDossiers(
             CancellationToken cancellationToken,
             string? customerId,
             string? dossierName,
             string? clientId,
             string? sort,
             string? branchId = "",
             bool asc = true,
             int pageSize = 20,
             int page = 1)
        {
            try
            {
                Console.WriteLine(UserContext.User);
                if (string.IsNullOrEmpty(UserContext.User.CustomerId))
                {
                    customerId ??= UserContext.User.CustomerId;
                }
                else customerId = UserContext.User.CustomerId;

                var user = UserContext.User ?? null;
                var query = new GetAllDossiersQuery(asc, sort ?? "DossierName", page, pageSize, dossierName ?? "",
                    customerId ?? "", clientId ?? "", branchId ?? "", user);
                var resp = await QueryProcessor.ProcessAsync(query, cancellationToken);
                return Ok(resp.Results);
            }
            catch (TaskCanceledException e)
            {
                return NoContent();
            }
        }

        [HttpGet]
        [Route("name")]
        public async Task<IActionResult>? GetDossierByName(
            CancellationToken cancellationToken,
            string? dossierName
           )
        {
            try
            {
                string customerId = String.Empty;
                Console.WriteLine(UserContext.User);
                if (string.IsNullOrEmpty(UserContext.User.CustomerId))
                {
                    customerId ??= UserContext.User.CustomerId;
                }
                else customerId = UserContext.User.CustomerId;

                var user = UserContext.User ?? null;
                var query = new GetAllDossiersQuery(true, "DossierName", 1, 30, dossierName ?? "",
                    customerId ?? "", "", "", user);
                var resp = await QueryProcessor.ProcessAsync(query, cancellationToken);
                return Ok(resp.Results);
            }
            catch (TaskCanceledException e)
            {
                return NoContent();
            }
        }

        [HttpGet]
        [Route("{dossierId}")]
        public async Task<IActionResult> GetDossier(string dossierId,
            CancellationToken cancellationToken)
        {
            try
            {
                //var p = await AuthorizationService.HasAccessAtAll(UserContext.User, dossierId, cancellationToken);
                //if (!p) return Unauthorized();

                var query = new GetDossierQuery(dossierId);

                var result = await QueryProcessor.ProcessAsync(query, cancellationToken);

                if (result == null) return NoContent();

                var p = await AuthorizationService.HasCustomerKey(UserContext.User, result.Id,
                    cancellationToken);
                if (p) return Ok(result);

                var p2 = await AuthorizationService.HasDossierKey(UserContext.User, result.CustomerId,
                    cancellationToken);
                if (p2) return Ok(result);

                return Ok(result);
            }
            catch (TaskCanceledException e)
            {
                return NoContent();
            }
        }

        [HttpPost]
        [Route("{dossierId}/branch")]
        public async Task<IActionResult> AddBranch(string dossierId,
            CreateDossierBranchModel model,
            CancellationToken cancellationToken)
        {
            var command = new CreateBranchCommand(new BranchId(dossierId), model);
            var result = await CommandBus.PublishAsync(command, cancellationToken);
            return CreatedAtAction(nameof(GetBranch), new { dossierId, branchId = model.BranchId });
        }

        [HttpGet]
        [Route("{dossierId}/branch/{branchId}")]
        public async Task<IActionResult> GetBranch(string dossierId,
            string branchId,
            CancellationToken cancellationToken)
        {
            var query = new GetBranchByBranchIdQuery(branchId, dossierId);
            var result = await QueryProcessor.ProcessAsync(query, cancellationToken);
            if (result == null) return NotFound();

            return Ok(result);
        }

        [HttpPost]
        [Route("{dossierId}/branch/{branchId}")]
        public async Task<IActionResult> UpdateBranch(string dossierId,
            string branchId,
            UpdateBranchModel model,
            CancellationToken cancellationToken)
        {
            if (!string.IsNullOrEmpty(model.CompanyBranchName))
            {
                var command = new ModifyBranchNameCommand(new BranchId(branchId), model.CompanyBranchName);
                await CommandBus.PublishAsync(command, cancellationToken);
            }

            if (model.CompanyBranchAddress != null)
            {
                var command = new ModifyBranchAddressCommand(new BranchId(branchId), model.CompanyBranchAddress);
                await CommandBus.PublishAsync(command, cancellationToken);
            }

            if (model.ContactDetails != null)
            {
                var command = new ModifyBranchContactDetailsCommand(new BranchId(branchId), model.ContactDetails);
                await CommandBus.PublishAsync(command, cancellationToken);
            }

            if (model.ContactDetails != null)
            {
                var command = new ModifyBranchContactDetailsCommand(new BranchId(branchId), model.ContactDetails);
                await CommandBus.PublishAsync(command, cancellationToken);
            }

            return Ok();
        }
    }
}