﻿using CRM.Domain.NeoDuty;
using CRM.Domain.NeoDuty.Entities;
using CRM.Domain.NeoDuty.Models;
using CRM.Domain.NeoDuty.ReadModels;
using CRM.Domain.Shared.ValueObjects;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using WebPortal.API.Model;
using WebPortal.Application.DutyCategories.Queries;
using WebPortal.Application.Extentions;
using WebPortal.Application.NeoDuties.Commands;
using WebPortal.Application.NeoDuties.Models;
using WebPortal.Application.NeoDuties.Queries;
using WebPortal.Application.DutyCategories.Models;
using WebPortal.Application.Employees.Queries;
using WebPortal.Application.NeoDuties.Commands.TimeSlot.Contact;
using System.Linq;
using WebPortal.API.Model.DossierId;
using WebPortal.Application.Employees.Models;
using WebPortal.Application.NeoDuties.Queries.TimeSlot;
using WebPortal.Application.NeoDuties.Models.Detail;
using WebPortal.Application.NeoDuties.Queries.Detail;
using WebPortal.Application.NeoDuties.Commands.TimeSlot;
using WebPortal.Application.NeoDuties.Models.Employee;
using WebPortal.Application.NeoDuties.Queries.Employee;
using WebPortal.Application.NeoDuties.Models.Dashboard;

namespace WebPortal.API.Controllers
{
    public class NeoDutyController : ReprojectController<NeoDutyReadModel>
    {
        #region Queries
        [HttpGet]
        [ProducesDefaultResponseType(typeof(List<DutyCategoryNameModel>))]
        [Route("/api/neodutyoverview/categories")]
        public async Task<IActionResult> GetDutyCategories(CancellationToken token, string dossierId, string? filter = "")
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Read) return Unauthorized();

                var q = new GetDutyCategoryNamesQuery(dossierId, filter);
                var result = await QueryProcessor.ProcessAsync(q, token);

                return Ok(result);
            }
            catch (TaskCanceledException e)
            {
                return NoContent();
            }

        }

        [HttpGet]
        [ProducesDefaultResponseType(typeof(List<EmployeeNeoDutyOverviewModel>))]
        [Route("/api/neodutyoverview/employees")]
        public async Task<IActionResult> GetEmployeeOverview(CancellationToken token, string dossierId)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var q = new GetEmployeesNeoDutyOverviewQuery(dossierId);
                var result = await QueryProcessor.ProcessAsync(q, token);

                return Ok(result);
            }
            catch (Exception e)
            {
                return NoContent();
            }
        }


        [HttpGet]
        [ProducesDefaultResponseType(typeof(PageResult<NeoDutyOverviewModel>))]
        [Route("/api/neodutyoverview")]
        public async Task<IActionResult> GetNeoDuties(
            CancellationToken token,
             string dossierId, 
             string dutyCategoryId,
             string startDate,
             string endDate,
             bool showActive , 
             bool showPlanned , 
             bool showExpired , 
             bool showEmpty , 
             string? filter = "", 
             bool asc = true, 
             string? sort = "Start", 
             int page = 1, 
             int pageSize = 20
            )
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Read) return Unauthorized();
                var start = DateTime.Parse(startDate);
                var end = DateTime.Parse(endDate);
                var q = new GetOverviewNeoDutiesQuery(dossierId, dutyCategoryId, start, end, showActive, showPlanned, showExpired, showEmpty, filter, asc, sort, page, pageSize);
                var result = await QueryProcessor.ProcessAsync(q, token);
                Response.Headers.Add("X-pagination-total", result.Total.ToString());
                return Ok(result.Results);

            }
            catch (TaskCanceledException)
            {
                return NoContent();
            }
        }

        [HttpGet]
        [ProducesDefaultResponseType(typeof(NeoDutyDetailModel))]
        [Route("/api/neodutydetails")]
        public async Task<IActionResult> GetNeoDutyDetails(string id, string dossierId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Read) return Unauthorized();
                var q = new GetOneNeoDutyDetailsQuery(id);
                var result = await QueryProcessor.ProcessAsync(q, token);
                return Ok(result);

            }
            catch (Exception)
            {
                return NoContent(); 
            }
        }

        [HttpGet]
        [ProducesDefaultResponseType(typeof(List<EmployeeAndContactDetails>))]
        [Route("{id}/allcontactdetails")]
        public async Task<IActionResult> GetAllContactDetailsOneDuty(string id, string dossierId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();
                var q = new GetAllContactDetailsOneDutyQuery(id);
                var result = await QueryProcessor.ProcessAsync(q, token);
                return Ok(result);
            }
            catch (Exception)
            {
                return NoContent();
            }
        }

        [HttpGet]
        [ProducesDefaultResponseType(typeof(List<EmployeeAndContactDetails>))]
        [Route("{id}/alltags")]
        public async Task<IActionResult> GetAllTagsOneDuty(string id, string dossierId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();
                var q = new GetAllTagsOneDutyQuery(id);
                var result = await QueryProcessor.ProcessAsync(q, token);
                return Ok(result);
            }
            catch (Exception)
            {
                return NoContent();
            }
        }

        [HttpGet]
        [ProducesDefaultResponseType(typeof(List<DutyCategoryDbModel>))]
        [Route("/api/neodutydashboard")]
        public async Task<IActionResult> GetNeoDutyDashBoard(string dossierId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Read) return Unauthorized();

                var q = new GetNeoDutyDashBoardQuery(dossierId);
                var result = await QueryProcessor.ProcessAsync(q, token);
                return Ok(result);

            }
            catch (Exception)
            {
                return NoContent();
            }
        }
        #endregion

        #region NeoDuty
        [HttpDelete]
        [Route("{id}/dossier/{dossierId}")]
        public async Task<IActionResult> DeleteNeoDuty(string id, string dossierId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Delete) return Unauthorized();

                var c = new DeleteNeoDutyCommand(NeoDutyId.With(id));
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }
                return StatusCode(500, new { errorMessage = $"Could not delete duty: '{id}'." });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPost]
        public async Task<IActionResult> CreateNeoDuty([FromBody]OneStringModel dutyCategoryId, CancellationToken token)
        {
            try
            {

                var q = new GetOneDutyCategoryQuery(dutyCategoryId.ValOs);
                var result = await QueryProcessor.ProcessAsync(q, token);
                if (result == null)
                {
                    return StatusCode(404, new { errorMessage = $"Could not find Duty Category with Id '{dutyCategoryId.ValOs}'" });
                }
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, result.DossierId, token);
                if (!dutyRight.Duties.Add) return Unauthorized();

                var addTimeSlotsList = new List<AddTimeslotModel>();
                foreach (var ts in result.TimeSlotTemplates)
                {
                    var tsId = TimeSlotId.New;
                    var addTs = new AddTimeslotModel(ts, tsId.Value);
                    addTimeSlotsList.Add(addTs);
                }

                var id = NeoDutyId.New;

                var c = new CreateNeoDutyCommand(id, dutyCategoryId.ValOs, result.Note, addTimeSlotsList);
                var createResult = await CommandBus.PublishAsync(c, token);

                if (!createResult.IsSuccess)
                {
                    return StatusCode(500, new { errorMessage = $"Could not create Duty based on category '{result.CategoryName}'." });
                }

                var newNeoDuty = new GetNewNeoDutyQuery(id.Value);
                var newNeoResult = await QueryProcessor.ProcessAsync(newNeoDuty, token);

                if (newNeoDuty == null)
                {
                    return StatusCode(500, new { errorMessage = $"The Duty was succesfully created, but please refresh your screen. (Press F5)" });
                }
                return Ok(newNeoResult);
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPost]
        [Route("{id}/save")]
        public async Task<IActionResult> SaveNeoDuty(string id, [FromBody]OneStringModel dossierId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId.ValOs, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new SaveNeoDutyCommand(NeoDutyId.With(id));
                var result = await CommandBus.PublishAsync(c, token);
                if (!result.IsSuccess)
                {
                    return StatusCode(500, new { errorMessage = $"Could not Saved Duty with Id: '{id}'." });
                }

                var q = new GetOneNeoDutyOverviewQuery(id);
                var qResult = await QueryProcessor.ProcessAsync(q, token);

                if (qResult == null)
                {
                    return StatusCode(500, new { errorMessage = $"The Duty was succesfully saved, but please refresh your screen. (Press F5)" });
                }

                return Ok(qResult);
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPost]
        [Route("{id}/cancel")]
        public async Task<IActionResult> CancelNeoDuty(string id, [FromBody] OneStringModel dossierId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId.ValOs, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new CancelNeoDutyCommand(NeoDutyId.With(id));
                var result = await CommandBus.PublishAsync(c, token);
                if (!result.IsSuccess)
                {
                    return StatusCode(500, new { errorMessage = $"Could not cancel Duty with Id: '{id}'." });
                }

                var q = new GetOneNeoDutyOverviewQuery(id);
                var qResult = await QueryProcessor.ProcessAsync(q, token);

                if (qResult == null)
                {
                    return StatusCode(500, new { errorMessage = $"The Duty was succesfully canceled, but please refresh your screen. (Press F5)" });
                }

                return Ok(qResult);
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/note")]
        public async Task<IActionResult> ModifyNote(string id, [FromBody] AndStringModel note, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, note.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new ModifyNoteCommand(NeoDutyId.With(id), note.valOs);
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }

                return StatusCode(500, new { errorMessage = $"Could not modify note of duty: '{id}'." });
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/start")]
        public async Task<IActionResult> ModifyStart(string id, [FromBody] AndDateTimeModel start, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, start.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new ModifyStartCommand(NeoDutyId.With(id), start.ValDate);
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }

                return StatusCode(500, new { errorMessage = $"Could not modify start of duty: '{id}'." });
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/end")]
        public async Task<IActionResult> ModifyEnd(string id, [FromBody] AndDateTimeModel end, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, end.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new ModifyEndCommand(NeoDutyId.With(id), end.ValDate);
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }

                return StatusCode(500, new { errorMessage = $"Could not modify end of duty: '{id}'." });
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/startend")]
        public async Task<IActionResult> ModifyStartEnd(string id, [FromBody] AndTwoDateTimesModel startEnd, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, startEnd.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new ModifyStartEndCommand(NeoDutyId.With(id), startEnd.Start, startEnd.End);
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }

                return StatusCode(500, new { errorMessage = $"Could not modify Start and End of duty: '{id}'." });
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }

        }



        #endregion

        #region Timeslot

        [HttpPost]
        [Route("{id}/timeslots")]
        public async Task<IActionResult> AddTimeSlot(string id, [FromBody] AndStringModel timeSlot, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, timeSlot.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();
                var c = new AddTimeSlotCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlot.valOs));
                var result = await CommandBus.PublishAsync(c, token);
                if (result.IsSuccess)
                {
                    return Ok();
                }
                return StatusCode(500, new { errorMessage = $"Could not add a new timeslot to duty '{id}'" });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpDelete]
        [Route("{id}/dossier/{dossierId}/timeslot/{timeSlotId}/")]
        public async Task<IActionResult> RemoveTimeSlot(string id, string dossierId, string timeSlotId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();
                var c = new RemoveTimeSlotCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlotId));
                var result = await CommandBus.PublishAsync(c, token);
                if (result.IsSuccess)
                {
                    return Ok();
                }
                return StatusCode(500, new { errorMessage = $"Could not remove timeslot '{timeSlotId}' from duty '{id}'" });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/timeslot/{timeSlotId}/weekdays")]
        public async Task<IActionResult> ModifyWeekDays(string id, string timeSlotId, [FromBody] AndWeekdaysModel weekdays, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, weekdays.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();
                var c = new ModifyWeekdaysCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlotId), weekdays.WeekDays);
                var result = await CommandBus.PublishAsync(c, token);
                if (result.IsSuccess)
                {
                    return Ok();
                }

                return StatusCode(500, new { errorMessage = $"Could not modify weekdays of the timeslot '{timeSlotId}' in duty '{id}'" });
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/timeslot/{timeSlotId}/start")]
        public async Task<IActionResult> ModifyStartTimeTimeSlot(string id, string timeSlotId, [FromBody] AndTimeSpanModel start, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, start.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();
                var c = new ModifyStartTimeTimeSlotCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlotId), start.Time);
                var result = await CommandBus.PublishAsync(c, token);
                if (result.IsSuccess)
                {
                    return Ok();
                }

                return StatusCode(500, new { errorMessage = $"Could not modify the startTime of the timeslot '{timeSlotId}' in duty '{id}' to: '{start.Time.Hours}:{start.Time.Minutes}' " });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/timeslot/{timeSlotId}/end")]
        public async Task<IActionResult> ModifyEndTimeTimeSlot(string id, string timeSlotId, [FromBody] AndTimeSpanModel end, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, end.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();
                var c = new ModifyEndTimeTimeSlotCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlotId), end.Time);
                var result = await CommandBus.PublishAsync(c, token);
                if (result.IsSuccess)
                {
                    return Ok();
                }

                return StatusCode(500, new { errorMessage = $"Could not modify the endTime of the timeslot '{timeSlotId}' in duty '{id}' to: '{end.Time.Hours}:{end.Time.Minutes}' " });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/timeslot/{timeSlotId}/title")]
        public async Task<IActionResult> ModifyTitleTimeSlot(string id, string timeSlotId, [FromBody] AndStringModel title, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, title.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();
                var c = new ModifyTitleTimeSlotCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlotId), title.valOs);
                var result = await CommandBus.PublishAsync(c, token);
                if (result.IsSuccess)
                {
                    return Ok();
                }

                return StatusCode(500, new { errorMessage = $"Could not modify the title of the timeslot '{timeSlotId}' in duty '{id}' to: '{title.valOs}' " });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/timeslot/{timeSlotId}/note")]
        public async Task<IActionResult> ModifyNoteTimeSlot(string id, string timeSlotId, [FromBody] AndStringModel note, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, note.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();
                var c = new ModifyNoteTimeSlotCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlotId), note.valOs);
                var result = await CommandBus.PublishAsync(c, token);
                if (result.IsSuccess)
                {
                    return Ok();
                }

                return StatusCode(500, new { errorMessage = $"Could not modify the note of the timeslot '{timeSlotId}' in duty '{id}' to: '{note.valOs}' " });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }


        #endregion

        #region DutyContact
        [HttpPost]
        [Route("{id}/timeslot/{timeSlotId}/detaildutycontacts")]
        public async Task<IActionResult> AddContactToTimeSlotDetail(string id, string timeSlotId, [FromBody] AddDutyContactModel contactModel, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, contactModel.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var q = new GetEmployeeWithDefaultsQuery(contactModel.EmployeeId);
                var employeeWithDefaults = await QueryProcessor.ProcessAsync(q, token);

                if (employeeWithDefaults == null)
                {
                    return StatusCode(404, new { errorMessage = $"Could not find Employee with Id '{contactModel.EmployeeId}'" });
                }

                var defaultContactPoints = GetDefaultContactPoints(employeeWithDefaults);

                var contactId = DutyContactId.With(Guid.NewGuid());

                var c = new AddContactCommand(NeoDutyId.With(id), timeSlotId, contactId, contactModel.EmployeeId, employeeWithDefaults.DefaultTags, defaultContactPoints);
                var result = await CommandBus.PublishAsync(c, token);

                if (!result.IsSuccess)
                {
                    return StatusCode(500, new { errorMessage = $"Could not add contact with employeeId '{contactModel.EmployeeId}' to Duty with Id: '{id}'." });
                }

                var rtnQuery = new GetOneDutyContactQuery(id, timeSlotId, contactModel.EmployeeId);
                var rtn = await QueryProcessor.ProcessAsync(rtnQuery, token);
                return Ok(rtn);

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }

        }
        [HttpPost]
        [Route("{id}/timeslot/{timeSlotId}/dutycontacts")]
        public async Task<IActionResult> AddContactToTimeSlot(string id, string timeSlotId, [FromBody] AddDutyContactModel contactModel, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, contactModel.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var q = new GetEmployeeWithDefaultsQuery(contactModel.EmployeeId);
                var employeeWithDefaults = await QueryProcessor.ProcessAsync(q, token);

                if (employeeWithDefaults == null)
                {
                    return StatusCode(404, new { errorMessage = $"Could not find Employee with Id '{contactModel.EmployeeId}'" });
                }

                var defaultContactPoints = GetDefaultContactPoints(employeeWithDefaults);

                var contactId = DutyContactId.With(Guid.NewGuid());

                var c = new AddContactCommand(NeoDutyId.With(id), timeSlotId, contactId, contactModel.EmployeeId, employeeWithDefaults.DefaultTags, defaultContactPoints);
                var result = await CommandBus.PublishAsync(c, token);

                if (!result.IsSuccess)
                {
                    return StatusCode(500, new { errorMessage = $"Could not add contact with employeeId '{contactModel.EmployeeId}' to Duty with Id: '{id}'." });
                }

                var rtn = new AddContactToTimeSlotModel()
                {
                    EmployeeId = contactModel.EmployeeId,
                    HasContactPoints = defaultContactPoints.Any(),
                    TimeSlotId = timeSlotId
                };
                return Ok(rtn);

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }

        }

        [HttpPost]
        [Route("{id}/detaildutycontacts")]
        public async Task<IActionResult> AddContactToDutyDetail(string id, [FromBody] AddDutyContactModel contactModel, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, contactModel.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var q = new GetEmployeeWithDefaultsQuery(contactModel.EmployeeId);
                var employeeWithDefaults = await QueryProcessor.ProcessAsync(q, token);

                if (employeeWithDefaults == null)
                {
                    return StatusCode(404, new { errorMessage = $"Could not find Employee with Id '{contactModel.EmployeeId}'" });
                }

                var contactId = DutyContactId.With(Guid.NewGuid());

                var contactList = new List<AddContactToNeoDutyModel>();

                var tsQuery = new GetTimeSlotIdsQuery(id);
                var timeSlotIds = await QueryProcessor.ProcessAsync(tsQuery, token);

                foreach (var tsId in timeSlotIds)
                {
                    var defaultContactPoints = GetDefaultContactPoints(employeeWithDefaults);

                    var contact = new AddContactToNeoDutyModel()
                    {
                        ContactId = DutyContactId.With(Guid.NewGuid()),
                        TimeSlotId = TimeSlotId.With(tsId),
                        DefaultContactPoints = defaultContactPoints
                    };
                    contactList.Add(contact);
                }

                var command = new AddContactToDutyCommand(NeoDutyId.With(id), contactModel.EmployeeId, employeeWithDefaults.DefaultTags, contactList);

                var result = await CommandBus.PublishAsync(command, token);

                if (result.IsSuccess)
                {
                    var rtnQuery = new GetAllTimeSlotsQuery(id);
                    var rtnResult = await QueryProcessor.ProcessAsync(rtnQuery, token);
                    return Ok(rtnResult);
                }

                return StatusCode(500, new { errorMessage = $"Could not add contact with employeeId '{contactModel.EmployeeId}' to Duty with Id: '{id}'." });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }

        }

        [HttpPost]
        [Route("{id}/dutycontacts")]
        public async Task<IActionResult> AddContactToDuty(string id, [FromBody] AddDutyContactModel contactModel, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, contactModel.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var q = new GetEmployeeWithDefaultsQuery(contactModel.EmployeeId);
                var employeeWithDefaults = await QueryProcessor.ProcessAsync(q, token);

                if (employeeWithDefaults == null)
                {
                    return StatusCode(404, new { errorMessage = $"Could not find Employee with Id '{contactModel.EmployeeId}'" });
                }

                var contactId = DutyContactId.With(Guid.NewGuid());

                var contactList = new List<AddContactToNeoDutyModel>();

                var tsQuery = new GetTimeSlotIdsQuery(id);
                var timeSlotIds = await QueryProcessor.ProcessAsync(tsQuery, token);

                foreach (var tsId in timeSlotIds)
                {
                    var defaultContactPoints = GetDefaultContactPoints(employeeWithDefaults);

                    var contact = new AddContactToNeoDutyModel()
                    {
                        ContactId = DutyContactId.With(Guid.NewGuid()),
                        TimeSlotId = TimeSlotId.With(tsId),
                        DefaultContactPoints = defaultContactPoints
                    };
                    contactList.Add(contact);
                }

                var command = new AddContactToDutyCommand(NeoDutyId.With(id), contactModel.EmployeeId, employeeWithDefaults.DefaultTags, contactList);

                var result = await CommandBus.PublishAsync(command, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }

                return StatusCode(500, new { errorMessage = $"Could not add contact with employeeId '{contactModel.EmployeeId}' to Duty with Id: '{id}'." });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/reorder")]
        public async Task<IActionResult> ReOrderContactInTimeSlot(string id, [FromBody] ReOrderContactModel reOrder, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, reOrder.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new ModifyOrderIdContactCommand(NeoDutyId.With(id), TimeSlotId.With(reOrder.TimeSlotId ), reOrder.EmployeeId, reOrder.NewOrderId);
                var result = await CommandBus.PublishAsync(c, token);
                if (result.IsSuccess)
                {
                    return Ok();
                }
                return StatusCode(500, new { errorMessage = $"Could not move contact with employeeId '{reOrder.EmployeeId}' to position {reOrder.NewOrderId} in Duty with Id: '{id}'." });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpDelete]
        [Route("{id}/dossier/{dossierId}/timeslot/{timeSlotId}/employee/{employeeId}")]
        public async Task<IActionResult> RemoveEmployeeFromTimeslot(string id, string dossierId, string timeSlotId, string employeeId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new RemoveContactCommand(NeoDutyId.With(id), employeeId, timeSlotId);
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }
                return StatusCode(500, new { errorMessage = $"Could not remove contact with employeeId '{employeeId}' from Duty with Id: '{id}'." });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpDelete]
        [Route("{id}/dossier/{dossierId}/employee/{employeeId}")]
        public async Task<IActionResult> RemoveEmployeeFromDuty(string id, string dossierId, string employeeId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new RemoveContactCommand(NeoDutyId.With(id), employeeId);
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }
                return StatusCode(500, new { errorMessage = $"Could not remove contact with employeeId '{employeeId}' from Duty with Id: '{id}'." });

            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPost]
        [Route("{id}/timeslot/{timeSlotId}/dutycontact/{dutyContactId}/contactpoints")]
        public async Task<IActionResult> AddContactPoint(string id, string timeSlotId, string dutyContactId, AddContactPointModel contactPoint, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, contactPoint.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new AddContactPointCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlotId), DutyContactId.With(dutyContactId), DutyContactPointId.With(contactPoint.ContactPointId), contactPoint.ContactDetailId);
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }
                return StatusCode(500, new { errorMessage = $"Could not add contactpoint with contactDetailId '{contactPoint.ContactDetailId}' to timeslot '{timeSlotId}' in duty '{id}'. " });
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpDelete]
        [Route("{id}/dossier/{dossierId}/timeslot/{timeSlotId}/dutyContact/{dutyContactId}/contactpoint/{dutyContactPointId}")]
        public async Task<IActionResult> RemoveContactPoint(string dossierId, string id, string timeSlotId, string dutyContactId, string dutyContactPointId, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new RemoveContactPointCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlotId), DutyContactId.With(dutyContactId), DutyContactPointId.With(dutyContactPointId));
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }
                return StatusCode(500, new { errorMessage = $"Could not remove contactpoint with DutyContactPointId '{dutyContactPointId}' from timeslot '{timeSlotId}' in duty '{id}'. " });
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPost]
        [Route("{id}/timeslot/{timeSlotId}/dutyContact/{dutyContactId}/tags")]
        public async Task<IActionResult> AddTag(string id, string timeSlotId, string dutyContactId, AndStringModel tag, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, tag.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new AddTagCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlotId), DutyContactId.With(dutyContactId), tag.valOs);
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }
                return StatusCode(500, new { errorMessage = $"Could not add tag '{tag.valOs}' to timeslot '{timeSlotId}' in duty '{id}'. " });
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }

        [HttpPut]
        [Route("{id}/timeslot/{timeSlotId}/dutyContact/{dutyContactId}/tags")]
        public async Task<IActionResult> RemoveTag(string id, string timeSlotId, string dutyContactId, AndStringModel tag, CancellationToken token)
        {
            try
            {
                var dutyRight = await AuthorizationService.GetDossierAccess(UserContext.User, tag.DossierId, token);
                if (!dutyRight.Duties.Update) return Unauthorized();

                var c = new RemoveTagCommand(NeoDutyId.With(id), TimeSlotId.With(timeSlotId), DutyContactId.With(dutyContactId), tag.valOs);
                var result = await CommandBus.PublishAsync(c, token);

                if (result.IsSuccess)
                {
                    return Ok();
                }
                return StatusCode(500, new { errorMessage = $"Could not remove tag '{tag.valOs}' from timeslot '{timeSlotId}' in duty '{id}'. " });
            }
            catch (Exception e)
            {
                return StatusCode(500, new { errorMessage = e.Message });
            }
        }



        private static List<DutyContactPointModel> GetDefaultContactPoints(EmployeeWithDefaultsModel employeeWithDefaults)
        {
            var defaultContactPoints = new List<DutyContactPointModel>();

            foreach (var cpi in employeeWithDefaults.DefaultContactPointIds)
            {
                var dcpm = new DutyContactPointModel()
                {
                    ContactDetailId = cpi,
                    Id = DutyContactPointId.New.Value
                };
                defaultContactPoints.Add(dcpm);
            }

            return defaultContactPoints;
        }



        #endregion
    }
}
