﻿using System;
using System.Linq;
using System.Security.Claims;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.InteractionStatistic.ReadModels;
using Microsoft.AspNetCore.Mvc;
using WebPortal.API.Helpers;
using WebPortal.Application.Extentions;
using WebPortal.Application.Statistics.Queries;

namespace WebPortal.API.Controllers
{
    public class StatisticController : ReprojectController<InteractionStatisticReadModel>
    {
        [HttpGet]
        [Route("/api/statistics")]
        [ProducesResponseType(typeof(PageResult<InteractionStatisticReadModel>), 200)]
        public async Task<IActionResult> GetStats(
            string dossierId,
            bool asc,
            DateTime? startDate,
            DateTime? endDate,
            int? page,
            int? pageSize,
            string? sort, CancellationToken cancellationToken
        )
        {
            try
            {
                if (!(await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, cancellationToken))
                    .Statistics.Read) return Unauthorized();

                var start = startDate ?? DateTime.UtcNow.AddDays(-365);
                var end = endDate ?? DateTime.UtcNow;
                if (!DateHelper.CheckRangeIsLonger(start, end))
                    return BadRequest("Time range cannot be shorter then 7 days");

                var q = new GetStatisticsQuery(dossierId, start,
                    end, page ?? 1, asc, pageSize ?? 20, sort ?? "Established");
                var result = await QueryProcessor.ProcessAsync(q, cancellationToken);
                Response.Headers.Add("X-pagination-total", result.Total.ToString());
                return Ok(result.Results);
            }
            catch (TaskCanceledException e)
            {
                return NoContent();
            }
        }

        [HttpGet]
        [Route("numbers")]
        public async Task<IActionResult> DashboardData(
            CancellationToken cancellationToken,
            string? dossierId,
            DateTime? startDate,
            DateTime? endDate)
        {
            try
            {
                if (!(await AuthorizationService.GetDossierAccess(UserContext.User, dossierId, cancellationToken))
                    .Statistics.Read) return Unauthorized();
                var query = new GetStatisticNumbersQuery(dossierId, startDate ?? DateTime.UtcNow.AddDays(-7),
                    endDate ?? DateTime.UtcNow);
                var result = await QueryProcessor.ProcessAsync(query, cancellationToken);
                return Ok(result);
            }
            catch (TaskCanceledException e)
            {
                return NoContent();
            }
        }
    }
}