﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Extensions.Configuration;
using MongoDB.Driver;
using WebPortal.Application.Service.EventsReader.Models;

namespace WebPortal.Application.Service.EventsReader
{
    public class MongoEventReaderService
    {
        private readonly IQueryable<Event> _events;
        private readonly IMongoCollection<Event> _collection;

        public MongoEventReaderService(IConfiguration Configuration)
        {
            var client = new MongoClient(Configuration["Db:ConnectionString"]);
            var db = client.GetDatabase(Configuration["Db:DbName"]);
            _collection = db.GetCollection<Event>("eventflow.events");
            _events = _collection.AsQueryable();
        }

        public Task<List<Event>> GetLastEvents(int amount, int page, CancellationToken token)
        {
            return Task.Run(() =>
             {
                 if (page < 1) page = 1;
                 if (amount < 1) page = 20;

                 return _events.OrderByDescending(x => x.Id).Skip(amount * (page - 1)).Take(amount).ToList();
             }, token);
        }

        public Task<List<Event>> GetEventsContain(string filter, int amount, int page, CancellationToken token)
        {
            return Task.Run(() =>
            {
                if (page < 1) page = 1;
                if (amount < 1) page = 20;
                var filterBuilder = Builders<Event>.Filter.Where(x => x.Data.Contains(filter) || x.Metadata.Contains(filter));
                var containingEvents = new List<Event>();

                using (var cursor =  _collection.FindSync(filterBuilder))
                {
                    while (cursor.MoveNext())
                    {
                        foreach (var item in cursor.Current)
                        {
                            containingEvents.Add(item);
                        }
                    }
                }
                   

                return containingEvents.OrderByDescending(x => x.Id).Skip(amount * (page - 1)).Take(amount).ToList();
            }, token);
        }
    }
}