﻿using System;
using System.IO;
using System.Net;
using Newtonsoft.Json;

namespace WebPortal.API.Services.Geocoder
{
    public class GeoCoder
    {
        private readonly string _apiKey;

        public GeoCoder(string apiKey)
        {
            _apiKey = apiKey;
        }

        public GeoCodeResult GetGeographicInfo(string address,
            bool usesSensor = false)
        {
            if (string.IsNullOrEmpty(_apiKey))
                throw new NullReferenceException("No API key has been entered");

            string strUrl = string.Empty;

            strUrl = "https://maps.googleapis.com/maps/api/geocode/json?address=" + address;
            strUrl += $"&key={_apiKey}";
            strUrl += usesSensor ? "&sensor=true" : "&sensor=false";

            HttpWebRequest request = (HttpWebRequest) WebRequest.Create(strUrl);

            request.ServicePoint.Expect100Continue = false;

            request.Method = WebRequestMethods.Http.Get;
            request.Accept = "application/json";

            WebResponse response = request.GetResponse();

            string strResponse = string.Empty;

            using (var sr = new StreamReader(response.GetResponseStream()))
            {
                strResponse = sr.ReadToEnd();

                return JsonConvert.DeserializeObject<GeoCodeResult>(strResponse);
            }

            return null;
        }

        public GeoCodeResult GetGeographicInfo(double latitude,
            double longitude,
            bool usesSensor)
        {
            string strUrl = string.Empty;

            strUrl = "http://maps.googleapis.com/maps/api/geocode/json?latlng=" + latitude + "," + longitude;
            strUrl += usesSensor ? "&sensor=true" : "&sensor=false";

            HttpWebRequest request = (HttpWebRequest) WebRequest.Create(strUrl);

            request.ServicePoint.Expect100Continue = false;

            request.Method = WebRequestMethods.Http.Get;
            request.Accept = "application/json";

            WebResponse response = request.GetResponse();

            string strResponse = string.Empty;

            using (var sr = new StreamReader(response.GetResponseStream()))
            {
                strResponse = sr.ReadToEnd();

                return JsonConvert.DeserializeObject<GeoCodeResult>(strResponse);
            }

            return null;
        }

        public GeoCodeResult GetGeographicInfo(string street,
            string city,
            string state,
            string postalCode,
            string country)
        {
            return GetGeographicInfo(BuildAddress(street, city, state, postalCode, country));
        }

        private string BuildAddress(string street,
            string city,
            string state,
            string postalCode,
            string country)
        {
            return $"{street} {city}, {state} {postalCode}, {country}";
        }
    }
}