﻿using System;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.User;
using CRM.Domain.User.Entities;
using CRM.Domain.User.Models;
using CRM.Domain.User.ReadModels;
using CRM.Domain.User.ValueObjects;
using EventFlow.Queries;
using Google.Apis.Util;
using WebPortal.Application.Dossiers.Queries;
using WebPortal.Application.Users.Queries;

namespace WebPortal.Application.Authorization
{
    public class AuthorizationService
    {
        private readonly IQueryProcessor _queryProcessor;

        public AuthorizationService(IQueryProcessor queryProcessor)
        {
            _queryProcessor = queryProcessor;
        }

        public async Task<bool> HasDossierKey(UserReadModel user,
            string dossierId,
            CancellationToken token)
        {
            user.DossierModuleRights.TryGetValue(dossierId, out var dossierRight);
            if (dossierRight == null)
            {
                user.DossierModuleRights.TryGetValue("*", out dossierRight);
            }

            return (dossierRight != null);
        }

        public async Task<bool> HasCustomerKey(UserReadModel user,
            string customerId,
            CancellationToken token)
        {
            user.CustomerModuleRights.TryGetValue(customerId, out var customerRight);
            if (customerRight == null)
            {
                user.CustomerModuleRights.TryGetValue("*", out customerRight);
            }

            return (customerRight != null);
        }

        public async Task<ModuleRightModel> GetDossierAccess(UserReadModel user, string dossierId,
            CancellationToken token)
        {
            user.DossierModuleRights.TryGetValue(dossierId, out var dossierRight);
            if (dossierRight == null)
            {
                user.DossierModuleRights.TryGetValue("*", out dossierRight);
            }

            if (dossierRight != null) return dossierRight;
            var dossier = await _queryProcessor.ProcessAsync(new GetDossierQuery(dossierId), token);

            user.CustomerModuleRights.TryGetValue(dossier.CustomerId, out var customerRight);
            if (customerRight == null)
            {
                user.CustomerModuleRights.TryGetValue("*", out customerRight);
            }
            return customerRight ?? new ModuleRightModel() { Duties = ModuleAccessLevel.None, Statistics = ModuleAccessLevel.None, Tickets = ModuleAccessLevel.None, AutoTreeview = ModuleAccessLevel.None, CallScript = ModuleAccessLevel.None, UserManagement = ModuleAccessLevel.None, VirtualReception = ModuleAccessLevel.None };
        }

        public async Task<ModuleRightModel> GetCustomerAccess(UserReadModel user,
            string customerId, CancellationToken token)
        {
            user.CustomerModuleRights.TryGetValue(customerId, out var customerRight);
            if (customerRight == null)
            {
                user.CustomerModuleRights.TryGetValue("*", out customerRight);
            }
            return customerRight ?? new ModuleRightModel() { Duties = ModuleAccessLevel.None, Statistics = ModuleAccessLevel.None, Tickets = ModuleAccessLevel.None, AutoTreeview = ModuleAccessLevel.None, CallScript = ModuleAccessLevel.None, UserManagement = ModuleAccessLevel.None, VirtualReception = ModuleAccessLevel.None };
        }

        public async Task<bool> IsPortalAdmin(UserReadModel user,
            CancellationToken token)
        {
            return user.IsCustomerAdmin && string.IsNullOrEmpty(user.CustomerId);
        }
    }
}