﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Branch.ReadModels;
using CRM.Domain.Customer.ReadModels;
using CRM.Domain.Dossier.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;

namespace WebPortal.Application.Branches.Queries
{
    public class
        GetBranchesByCustomerIdQueryHandler : IQueryHandler<GetBranchesByCustomerIdQuery, List<BranchReadModel>>
    {
        private readonly IMongoDbReadModelStore<BranchReadModel> _branchStore;
        private readonly IMongoDbReadModelStore<CustomerReadModel> _customerStore;
        private readonly IMongoDbReadModelStore<DossierReadModel> _dossierStore;

        public GetBranchesByCustomerIdQueryHandler(IMongoDbReadModelStore<BranchReadModel> branchStore,
            IMongoDbReadModelStore<CustomerReadModel> customerStore,
            IMongoDbReadModelStore<DossierReadModel> dossierStore)
        {
            _branchStore = branchStore;
            _customerStore = customerStore;
            _dossierStore = dossierStore;
        }

        public Task<List<BranchReadModel>> ExecuteQueryAsync(GetBranchesByCustomerIdQuery query,
            CancellationToken cancellationToken)
        {
          var rtn = _branchStore
                    .AsQueryable()
                    .Where(x => x.CustomerId.Equals(query.CustomerId))
                    .ToList();
                rtn.ForEach(x =>
                {
                    x.IsHq = _customerStore.AsQueryable()
                                 .FirstOrDefault(y => y.CustomerBranchId == x.Id) != null;
                    x.DossierId = _dossierStore.AsQueryable()
                        .FirstOrDefault(y => y.BranchIds.Contains(x.Id))?.Id;
                });
                return Task.FromResult(rtn);
        }
    }
}