﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.CallScript.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.CallScripts.Models;

namespace WebPortal.Application.CallScripts.Queries
{
    public class
        GetCallScriptsWithCommentsQueryHandler : IQueryHandler<GetCallScriptsWithCommentsQuery,
            List<GetCallScriptWithCommentsModel>>
    {
        private readonly IMongoDbReadModelStore<CallScriptReadModel> _callScriptStore;

        public GetCallScriptsWithCommentsQueryHandler(IMongoDbReadModelStore<CallScriptReadModel> callScriptStore)
        {
            _callScriptStore = callScriptStore;
        }

        public Task<List<GetCallScriptWithCommentsModel>> ExecuteQueryAsync(GetCallScriptsWithCommentsQuery query,
            CancellationToken cancellationToken)
        {
            
                if (query.CommentAmount == null) query.CommentAmount = 0;
                var temp = _callScriptStore.AsQueryable()
                    .Where(x => !x.Deleted && x.DossierId == query.DossierId)
                    .Select(x => new GetCallScriptWithCommentsModel
                    {
                        DossierId = x.DossierId,
                        CallScriptBody = x.CallScriptBody,
                        Id = x.Id,
                        Comments = x.CommentList,
                        Tags = x.Tags,
                        TotalComments = x.CommentList.Count
                    }).ToList()
                    .Select(x =>
                    {
                        return new GetCallScriptWithCommentsModel
                        {
                            DossierId = x.DossierId,
                            CallScriptBody = x.CallScriptBody,
                            Id = x.Id,
                            TotalComments = x.TotalComments,
                            Comments = query.CommentOrderAscending != null && (bool) query.CommentOrderAscending
                                ? x.Comments.OrderBy(y => y.CreateDate).Take((int) query.CommentAmount).ToList()
                                : x.Comments.OrderByDescending(y => y.CreateDate).Take((int) query.CommentAmount)
                                    .ToList(),
                            Tags = x.Tags
                        };
                    }).ToList();
                //temp.ForEach(x => x.Tags = _tagStore.AsQueryable()
                //   .Where(y => x.Tags.Contains(y.Id)).ToList());
                return Task.FromResult(temp);
        }
    }
}