﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.CallScript.Models;
using CRM.Domain.CallScript.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;

namespace WebPortal.Application.CallScripts.Queries
{
    public class
        GetMoreCommentsForCallScriptQueryHandler : IQueryHandler<GetMoreCommentsForCallScriptQuery,
            List<CallScriptCommentModel>>
    {
        private readonly IMongoDbReadModelStore<CallScriptReadModel> _callScriptStore;

        public GetMoreCommentsForCallScriptQueryHandler(IMongoDbReadModelStore<CallScriptReadModel> callScriptStore)
        {
            _callScriptStore = callScriptStore;
        }

        public Task<List<CallScriptCommentModel>> ExecuteQueryAsync(GetMoreCommentsForCallScriptQuery query,
            CancellationToken cancellationToken)
        {
            
                var temp = _callScriptStore.AsQueryable()
                    .SingleOrDefault(x => x.Id == query.CallScriptId)
                    .CommentList.ToList();
                return Task.FromResult(query.CommentOrderAscending
                    ? temp.OrderBy(y => y.CreateDate)
                        .Skip(query.AmountAlreadyLoaded)
                        .Take(query.CommentAmount).ToList()
                    : temp.OrderByDescending(y => y.CreateDate)
                        .Skip(query.AmountAlreadyLoaded)
                        .Take(query.CommentAmount).ToList());
        }
    }
}