﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Customer.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Customers.Models;
using WebPortal.Application.Extentions;

namespace WebPortal.Application.Customers.Queries
{
    public class GetAllCustomersQueryHandler : IQueryHandler<GetAllCustomersQuery, PageResult<GetCustomerListModel>>
    {
        private readonly IMongoDbReadModelStore<CustomerReadModel> _store;

        public GetAllCustomersQueryHandler(IMongoDbReadModelStore<CustomerReadModel> store)
        {
            _store = store;
        }

        public Task<PageResult<GetCustomerListModel>> ExecuteQueryAsync(GetAllCustomersQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(
                _store.AsQueryable()
                    .Where(x => !x.Deleted)
                    .Where(x => string.IsNullOrWhiteSpace(query.CustomerName)
                                || x.CustomerName.ToLower().Contains(query.CustomerName))
                    .Select(x => new GetCustomerListModel
                    {
                        CustomerName = x.CustomerName,
                        Id = x.Id
                    })
                    .OrderBy(query.Sort, query.Asc)
                    .GetPage(query.Page, query.PageSize));
        }
    }
}