﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Customer.ReadModels;
using CRM.Domain.User.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Customers.Models;

namespace WebPortal.Application.Customers.Queries
{
    public class GetLicenseInfoQueryHandler : IQueryHandler<GetLicenseInfoQuery, GetLicenseInfoModel>
    {
        private readonly IMongoDbReadModelStore<CustomerReadModel> _customerStore;
        private readonly IMongoDbReadModelStore<UserReadModel> _userStore;

        public GetLicenseInfoQueryHandler(IMongoDbReadModelStore<CustomerReadModel> customerStore,
            IMongoDbReadModelStore<UserReadModel> userStore)
        {
            _customerStore = customerStore;
            _userStore = userStore;
        }

        public Task<GetLicenseInfoModel> ExecuteQueryAsync(GetLicenseInfoQuery query,
            CancellationToken cancellationToken)
        {

            var model = new GetLicenseInfoModel
            {
                TotalLicenses = _customerStore.AsQueryable().FirstOrDefault(x => x.Id.Equals(query.CustomerId))
                    ?.WebPortalLicenseAmount ?? 0,
                InUse = _userStore.AsQueryable().Count(x => x.CustomerId.Equals(query.CustomerId))
            };


            return Task.FromResult(model);
        }
    }
}