﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Dossier.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using MongoDB.Driver;
using WebPortal.Application.Extentions;

namespace WebPortal.Application.Dossiers.Queries
{
    public class GetAllDossiersQueryHandler : IQueryHandler<GetAllDossiersQuery, PageResult<DossierReadModel>>
    {
        private readonly IMongoDbReadModelStore<DossierReadModel> _store;

        public GetAllDossiersQueryHandler(IMongoDbReadModelStore<DossierReadModel> store)
        {
            _store = store;
        }

        public async Task<PageResult<DossierReadModel>> ExecuteQueryAsync(GetAllDossiersQuery query,
            CancellationToken cancellationToken)
        {

            IEnumerable<DossierReadModel> l = new List<DossierReadModel>();
            if ((query.User == null) || query.User.CustomerModuleRights.Any(y => y.Key == "*") ||
                query.User.DossierModuleRights.Any(y => y.Key == "*"))
            {
                var a = await _store.FindAsync(x =>

                        (string.IsNullOrEmpty(query.DossierName) ||
                         x.DossierName.ToLower().Contains(query.DossierName))
                    , new FindOptions<DossierReadModel>(){Skip = (query.Page - 1)*query.PageSize, Limit = query.PageSize, Sort = new JsonSortDefinition<DossierReadModel>("{DossierName:1}")}, cancellationToken);

                while (await a.MoveNextAsync(cancellationToken))
                {
                    l = l.Concat(a.Current);
                } 
                a.Dispose();
            }
            else
            {
                foreach (var customer in query.User.CustomerModuleRights)
                {
                    var a = await _store.FindAsync(x =>

                            (string.IsNullOrEmpty(query.DossierName) ||
                             x.DossierName.ToLower().Contains(query.DossierName))&&
                            customer.Key == x.CustomerId
                        , new FindOptions<DossierReadModel>(){Skip = (query.Page - 1)*query.PageSize, Limit = query.PageSize,Sort = new JsonSortDefinition<DossierReadModel>("{DossierName:1}")}, cancellationToken);

                    while (await a.MoveNextAsync(cancellationToken))
                    {
                        l = l.Concat(a.Current);
                    }
                    a.Dispose();
                }

                foreach (var doss in query.User.DossierModuleRights)
                {
                    var a = await _store.FindAsync(x =>

                            (string.IsNullOrEmpty(query.DossierName) ||
                             x.DossierName.ToLower().Contains(query.DossierName))&&
                            doss.Key == x.Id
                        , new FindOptions<DossierReadModel>(){Skip = (query.Page - 1)*query.PageSize, Limit = query.PageSize}, cancellationToken);

                    while (await a.MoveNextAsync(cancellationToken))
                    {
                        
                        l = l.Concat(a.Current);
                    }
                    a.Dispose();
                }
                
            }

            var rightList = new List<DossierReadModel>();

            l.ToList().ForEach(x =>
            {
                if (!rightList.Any(y => y.Id.Equals(x.Id)))
                {
                    rightList.Add(x);
                };
            });


            return rightList.AsQueryable().OrderBy(x=> x.DossierName).GetPage(query.Page, query.PageSize);


        }
    } 
}