﻿using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Duty.Enums;
using CRM.Domain.Duty.ReadModels;
using CRM.Domain.Employee.Models;
using CRM.Domain.Employee.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Duties.Models;
using WebPortal.Application.Extentions;

namespace WebPortal.Application.Duties.Queries
{
    public class GetActiveDutiesQueryHandler : IQueryHandler<GetActiveDutiesQuery, PageResult<GetOverviewDutyModel>>
    {
        private readonly IMongoDbReadModelStore<DutyReadModel> _dutyStore;
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _employeeStore;

        public GetActiveDutiesQueryHandler(IMongoDbReadModelStore<EmployeeReadModel> employeeStore,
            IMongoDbReadModelStore<DutyReadModel> dutyStore)
        {
            _employeeStore = employeeStore;
            _dutyStore = dutyStore;
        }

        public Task<PageResult<GetOverviewDutyModel>> ExecuteQueryAsync(GetActiveDutiesQuery query,
            CancellationToken cancellationToken)
        {
            
                
                var nowAndOne = DateTime.Now.AddDays(1);
                var nowAndMinOne = DateTime.Now.AddDays(-1);
                var activeDuties = _dutyStore.AsQueryable()
                    .Where(x => !x.Deleted && x.DossierId == query.DossierId)
                    .ToList()
                    .Where(x =>
                        x.StartDate.Date <= nowAndOne
                        && nowAndMinOne <= x.EndDate.Date
                        && x.DutyStatus == DutyStatusEnum.Active
                    )
                    .AsQueryable()
                    .OrderBy(query.Sort, query.Asc)
                    .ToList();
                var result = activeDuties.Select(x => new GetOverviewDutyModel
                {
                    Id = x.Id,
                    ActionDescription = x.ActionDescription,
                    DossierId = x.DossierId,
                    Category = x.Category,
                    EndAt = x.EndAt,
                    StartAt = x.StartAt,
                    DutyStatus = x.DutyStatus,
                    CompleteStartDate = x.CompleteStartDate,
                    CompleteEndDate = x.CompleteEndDate,
                    DutyEmployees = 
                        x.DutyEmployees.Select(em => new GetDutyEmployeeModel
                    {
                        Id = em.Id,
                        EmployeeId = em.EmployeeId,
                        FirstName = _employeeStore.AsQueryable().Single(y => y.Id == em.EmployeeId).FirstName,
                        LastName = _employeeStore.AsQueryable().Single(y => y.Id == em.EmployeeId).LastName,
                        OrderId = em.OrderId,
                        Tags = em.Tags,
                        ContactPoints = em.ContactPoints.Select(cp => new GetContactPointModel
                        {
                            ContactPointId = cp.Id,
                            Description = cp.Description,
                            ContactDetailEmployee = 
                                _employeeStore.AsQueryable()
                                .FirstOrDefault(cd => cd.Id == em.EmployeeId)?
                                .ContactDetails.FirstOrDefault(cd => cd.Id == cp.ContactDetailId)
                        }).ToList()
                    }).ToList()
                })
                    .AsQueryable()
                    .GetPage(query.Page, query.PageSize);
            return Task.FromResult( result);
        }
    }
}