﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Duty.ReadModels;
using CRM.Domain.Employee.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Duties.Models;

namespace WebPortal.Application.Duties.Queries
{
    public class GetDutyContainingEmployeeQueryHandler : IQueryHandler<GetDutyContainingEmployeeQuery, List<GetOverviewDutyModel>>
    {
        private readonly IMongoDbReadModelStore<DutyReadModel> _dutyStore;
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _employeeStore;

        public GetDutyContainingEmployeeQueryHandler(IMongoDbReadModelStore<DutyReadModel> dutyStore, IMongoDbReadModelStore<EmployeeReadModel> employeeStore)
        {
            _dutyStore = dutyStore;
            _employeeStore = employeeStore;
        }

        public Task<List<GetOverviewDutyModel>> ExecuteQueryAsync(GetDutyContainingEmployeeQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(
                _dutyStore.AsQueryable().Where(x => !x.Deleted)
                    .Where(d => d.DutyEmployees.Any(de => de.EmployeeId == query.EmployeeId)).ToList().Select(x => new GetOverviewDutyModel
                    {
                        Id = x.Id,
                        DossierId = x.DossierId,
                        ActionDescription = x.ActionDescription,
                        Category = x.Category,
                        EndDate = x.EndDate,
                        StartDate = x.StartDate,
                        StartAt = x.StartAt,
                        EndAt = x.EndAt,
                        DutyStatus = x.DutyStatus,
                        CompleteStartDate = x.CompleteStartDate,
                        CompleteEndDate = x.CompleteEndDate,
                        DutyEmployees = x.DutyEmployees.Select(em => new GetDutyEmployeeModel
                        {
                            Id = em.Id,
                            EmployeeId = em.EmployeeId,
                            FirstName = _employeeStore.AsQueryable().Single(y => y.Id == em.EmployeeId).FirstName,
                            LastName = _employeeStore.AsQueryable().Single(y => y.Id == em.EmployeeId).LastName,
                            OrderId = em.OrderId,
                            Tags = em.Tags,
                            ContactPoints = em.ContactPoints.Select(cp => new GetContactPointModel
                            {
                                ContactPointId = cp.Id,
                                Description = cp.Description,
                                ContactDetailEmployee = _employeeStore.AsQueryable()
                                    .FirstOrDefault(cd => cd.Id == em.EmployeeId)
                                    .ContactDetails.FirstOrDefault(cd => cd.Id == cp.ContactDetailId)
                            }).ToList()
                        }).ToList()
                    }).ToList()
            );
        }
    }
}