﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Duty.ReadModels;
using CRM.Domain.Employee.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Duties.Models;

namespace WebPortal.Application.Duties.Queries
{
    public class GetDutyQueryHandler : IQueryHandler<GetDutyQuery, GetOverviewDutyModel>
    {
        private readonly IMongoDbReadModelStore<DutyReadModel> _dutyStore;
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _employeeStore;

        public GetDutyQueryHandler(IMongoDbReadModelStore<EmployeeReadModel> employeeStore,
            IMongoDbReadModelStore<DutyReadModel> dutyStore)
        {
            _employeeStore = employeeStore;
            _dutyStore = dutyStore;
        }

        public Task<GetOverviewDutyModel> ExecuteQueryAsync(GetDutyQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(_dutyStore.AsQueryable().Where(x => x.Id.Equals(query.DutyId)).ToList().Select(x =>
                        new GetOverviewDutyModel
                        {
                            Id = x.Id,
                            ActionDescription = x.ActionDescription,
                            DossierId = x.DossierId,
                            Category = x.Category,
                            EndDate = x.EndDate,
                            StartDate = x.StartDate,
                            StartAt = x.StartAt,
                            EndAt = x.EndAt,
                            DutyStatus = x.DutyStatus,
                            CompleteStartDate = x.CompleteStartDate,
                            CompleteEndDate = x.CompleteEndDate,
                            DutyEmployees = x.DutyEmployees.Select(em => new GetDutyEmployeeModel
                            {
                                Id = em.Id,
                                EmployeeId = em.EmployeeId,
                                FirstName = _employeeStore.AsQueryable().Single(y => y.Id == em.EmployeeId).FirstName,
                                LastName = _employeeStore.AsQueryable().Single(y => y.Id == em.EmployeeId).LastName,
                                OrderId = em.OrderId,
                                Tags = em.Tags,
                                ContactPoints = em.ContactPoints.Select(cp => new GetContactPointModel
                                {
                                    ContactPointId = cp.Id,
                                    Description = cp.Description,
                                    ContactDetailEmployee = _employeeStore.AsQueryable()
                                        .FirstOrDefault(cd => cd.Id == em.EmployeeId)?
                                        .ContactDetails.FirstOrDefault(cd => cd.Id == cp.ContactDetailId)
                                }).ToList()
                            }).ToList()
                        }).FirstOrDefault());
                
        }
    }
}