﻿using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.Commands;
using System;
using System.Threading;
using System.Threading.Tasks;
using WebPortal.Application.DutyCategories.Models;

namespace WebPortal.Application.DutyCategories.Commands
{
    public class CopyDutyCategoryCommand : Command<DutyCategoryAggregate, DutyCategoryId>
    {
        public CopyDutyCategoryCommand(DutyCategoryId aggregateId, DutyCategoryDetailsModel readModel) : base(aggregateId)
        {
            ReadModel = readModel;
        }

        public DutyCategoryDetailsModel ReadModel { get; }
    }

    public class CopyDutyCategoryCommandHandler : CommandHandler<DutyCategoryAggregate, DutyCategoryId, CopyDutyCategoryCommand>
    {
        public override Task ExecuteAsync(DutyCategoryAggregate aggregate, CopyDutyCategoryCommand command, CancellationToken cancellationToken)
        {
            return Task.Run(() =>
            ExecuteAllOfThis(aggregate, command), cancellationToken);
        }

        private void ExecuteAllOfThis(DutyCategoryAggregate aggregate, CopyDutyCategoryCommand command)
        {
            aggregate.CreateDutyCategory(command.ReadModel.DossierId);
            aggregate.ModifyDutyCategoryName($"{command.ReadModel.CategoryName} - Copy");
            if (!string.IsNullOrEmpty(command.ReadModel.Note))
                aggregate.ModifyDutyCategoryDutyTemplateNote(command.ReadModel.Note);

            if (!string.IsNullOrEmpty(command.ReadModel.GlobalDutyTemplateId))
                aggregate.ModifyDutyCategoryGlobalDutyId(command.ReadModel.GlobalDutyTemplateId);

            if (command.ReadModel.TimeSlotTemplates == null || command.ReadModel.TimeSlotTemplates.Count == 0) return;

            foreach (var tst in command.ReadModel.TimeSlotTemplates)
            {
                var tstGuid = Guid.NewGuid();
                aggregate.AddTimeSlotInDutyTemplate(TimeSlotTemplateId.With(tstGuid));
                aggregate.ModifyStartTimeTimeSlotDutyTemplate(TimeSlotTemplateId.With(tstGuid), tst.StartTime);
                aggregate.ModifyEndTimeTimeSlotDutyTemplate(TimeSlotTemplateId.With(tstGuid), tst.EndTime);
                if (!string.IsNullOrEmpty(tst.TimeSlotName))
                    aggregate.ModifyTimeSlotNameDutyTemplate(TimeSlotTemplateId.With(tstGuid),tst.TimeSlotName);
                if (!string.IsNullOrEmpty(tst.Note))
                    aggregate.ModifyNoteTimeSlotDutyTemplate(TimeSlotTemplateId.With(tstGuid), tst.Note);

                if (tst.WeekDays == null || tst.WeekDays.Count == 0) continue;

                aggregate.ModifyWeekdaysTimeSlotDutyTemplate(TimeSlotTemplateId.With(tstGuid), tst.WeekDays);


            }
            aggregate.SaveDutyCategory();
        }
    }
}
