﻿using System;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using EventFlow.Commands;

namespace WebPortal.Application.DutyCategories.Commands.Duties.TimeSlots
{
    public class ModifyEndTimeTimeSlotDutyCommand : Command<DutyCategoryAggregate, DutyCategoryId>
    {
        public ModifyEndTimeTimeSlotDutyCommand(DutyCategoryId aggregateId,
            DutyId idDuty,
            TimeSlotId idTimeSlot,
            TimeSpan endTime) : base(aggregateId)
        {
            IdDuty = idDuty;
            IdTimeSlot = idTimeSlot;
            EndTime = endTime;
        }

        public DutyId IdDuty { get; }
        public TimeSlotId IdTimeSlot { get; }
        public TimeSpan EndTime { get; }
    }

    public class ModifyEndTimeTimeSlotDutyCommandHandler : CommandHandler<DutyCategoryAggregate,
        DutyCategoryId, ModifyEndTimeTimeSlotDutyCommand>
    {
        public override Task ExecuteAsync(DutyCategoryAggregate aggregate,
            ModifyEndTimeTimeSlotDutyCommand command,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
                aggregate.ModifyTimeSlotEndTimeInDuty(command.IdDuty, command.IdTimeSlot, command.EndTime), cancellationToken);
        }
    }
}
