﻿using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using EventFlow.Commands;

namespace WebPortal.Application.DutyCategories.Commands.Duties.TimeSlots
{
    public class ModifyNameTimeSlotDutyCommand : Command<DutyCategoryAggregate, DutyCategoryId>
    {
        public DutyId IdDuty { get; }
        public TimeSlotId IdTimeSlot { get; }
        public string Name { get; }

        public ModifyNameTimeSlotDutyCommand(DutyCategoryId aggregateId,
            DutyId idDuty,
            TimeSlotId idTimeSlot,
            string name) : base(aggregateId)
        {
            IdDuty = idDuty;
            IdTimeSlot = idTimeSlot;
            Name = name;
        }
    }

    public class ModifyNameTimeSlotDutyCommandHandler : CommandHandler<DutyCategoryAggregate,
        DutyCategoryId, ModifyNameTimeSlotDutyCommand>
    {
        public override Task ExecuteAsync(DutyCategoryAggregate aggregate,
            ModifyNameTimeSlotDutyCommand command,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
                aggregate.ModifyTimeSlotNameInDuty(command.IdDuty, command.IdTimeSlot, command.Name), cancellationToken);
        }
    }
}
