﻿using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using EventFlow.Commands;

namespace WebPortal.Application.DutyCategories.Commands.Duties.TimeSlots
{
    public class ModifyNoteTimeSlotDutyCommand : Command<DutyCategoryAggregate, DutyCategoryId>
    {
        public DutyId IdDuty { get; }
        public TimeSlotId IdTimeSlot { get; }
        public string Note { get; }

        public ModifyNoteTimeSlotDutyCommand(DutyCategoryId aggregateId,
            DutyId idDuty,
            TimeSlotId idTimeSlot,
            string note) : base(aggregateId)
        {
            IdDuty = idDuty;
            IdTimeSlot = idTimeSlot;
            Note = note;
        }
    }

    public class ModifyNoteTimeSlotDutyCommandHandler : CommandHandler<DutyCategoryAggregate,
        DutyCategoryId, ModifyNoteTimeSlotDutyCommand>
    {
        public override Task ExecuteAsync(DutyCategoryAggregate aggregate,
            ModifyNoteTimeSlotDutyCommand command,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
                aggregate.ModifyTimeSlotNoteInDuty(command.IdDuty, command.IdTimeSlot, command.Note), cancellationToken);
        }
    }
}
