﻿using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using CRM.Domain.Shared.ValueObjects;
using EventFlow.Commands;

namespace WebPortal.Application.DutyCategories.Commands.Duties.TimeSlots
{
    public class ModifyWeekdaysTimeSlotDutyCommand : Command<DutyCategoryAggregate, DutyCategoryId>
    {
        public ICollection<WeekDay> WeekDays { get; }
        public DutyId IdDuty { get; }
        public TimeSlotId IdTimeSlot { get; }

        public ModifyWeekdaysTimeSlotDutyCommand(DutyCategoryId aggregateId,
            DutyId idDuty,
            TimeSlotId idTimeSlot,
            ICollection<WeekDay> weekDays
            ) : base(aggregateId)
        {
            WeekDays = weekDays;
            IdDuty = idDuty;
            IdTimeSlot = idTimeSlot;
        }
    }

    public class ModifyWeekdaysTimeSlotDutyCommandHandler : CommandHandler<DutyCategoryAggregate,
        DutyCategoryId, ModifyWeekdaysTimeSlotDutyCommand>
    {
        public override Task ExecuteAsync(DutyCategoryAggregate aggregate,
            ModifyWeekdaysTimeSlotDutyCommand command,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
                aggregate.ModifyTimeSlotWeekdaysInDuty(command.IdDuty, command.IdTimeSlot, command.WeekDays), cancellationToken);
        }
    }
}
