﻿using System;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using EventFlow.Commands;

namespace WebPortal.Application.DutyCategories.Commands.DutyTemplate
{
    public class ModifyEndTimeTimeSlotTemplateCommand : Command<DutyCategoryAggregate, DutyCategoryId>
    {
        public TimeSlotTemplateId TemplateTimeSlotId { get; }
        public TimeSpan EndTime { get; }

        public ModifyEndTimeTimeSlotTemplateCommand(DutyCategoryId aggregateId,
            TimeSlotTemplateId templateTimeSlotId,
            TimeSpan endTime) : base(aggregateId)
        {
            TemplateTimeSlotId = templateTimeSlotId;
            EndTime = endTime;
        }
    }

    public class ModifyEndTimeTimeSlotTemplateCommandHandler : CommandHandler<DutyCategoryAggregate,
        DutyCategoryId, ModifyEndTimeTimeSlotTemplateCommand>
    {
        public override Task ExecuteAsync(DutyCategoryAggregate aggregate,
            ModifyEndTimeTimeSlotTemplateCommand command,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
                aggregate.ModifyEndTimeTimeSlotDutyTemplate(command.TemplateTimeSlotId, command.EndTime),
                cancellationToken);
        }
    }
}
