﻿using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using EventFlow.Commands;

namespace WebPortal.Application.DutyCategories.Commands.DutyTemplate
{
    public class ModifyNameTimeSlotTemplateCommand : Command<DutyCategoryAggregate, DutyCategoryId>
    {
        public ModifyNameTimeSlotTemplateCommand(DutyCategoryId aggregateId,
            TimeSlotTemplateId templateTimeSlotId,
            string timeSlotName) : base(aggregateId)
        {
            TemplateTimeSlotId = templateTimeSlotId;
            TimeSlotName = timeSlotName;
        }

        public TimeSlotTemplateId TemplateTimeSlotId { get; }
        public string TimeSlotName { get; }
    }

    public class ModifyNameTimeSlotTemplateCommandHandler : CommandHandler<DutyCategoryAggregate,
        DutyCategoryId, ModifyNameTimeSlotTemplateCommand>
    {
        public override Task ExecuteAsync(DutyCategoryAggregate aggregate,
            ModifyNameTimeSlotTemplateCommand command,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
                aggregate.ModifyTimeSlotNameDutyTemplate(command.TemplateTimeSlotId, command.TimeSlotName),
                cancellationToken);
        }
    }
}
