﻿using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using CRM.Domain.Shared.ValueObjects;
using EventFlow.Commands;

namespace WebPortal.Application.DutyCategories.Commands.DutyTemplate
{
    public class ModifyWeekdaysTimeSlotTemplateCommand : Command<DutyCategoryAggregate, DutyCategoryId>
    {
        public ModifyWeekdaysTimeSlotTemplateCommand(DutyCategoryId aggregateId,
            TimeSlotTemplateId templateTimeSlotId,
            ICollection<WeekDay> weekDays) : base(aggregateId)
        {
            TemplateTimeSlotId = templateTimeSlotId;
            WeekDays = weekDays;
        }

        public TimeSlotTemplateId TemplateTimeSlotId { get; }
        public ICollection<WeekDay> WeekDays { get; }

    }

    public class ModifyWeekdaysTimeSlotTemplateCommandHandler : CommandHandler<DutyCategoryAggregate,
        DutyCategoryId, ModifyWeekdaysTimeSlotTemplateCommand>
    {
        public override Task ExecuteAsync(DutyCategoryAggregate aggregate,
            ModifyWeekdaysTimeSlotTemplateCommand command,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
                aggregate.ModifyWeekdaysTimeSlotDutyTemplate(command.TemplateTimeSlotId, command.WeekDays), cancellationToken);
        }
    }
}
