﻿using CRM.Domain.DutyCategoryAndTemplate.Aggregates;
using CRM.Domain.DutyCategoryAndTemplate.Entities;
using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.Commands;
using System;
using System.Threading;
using System.Threading.Tasks;

namespace WebPortal.Application.DutyCategories.Commands
{
    public class InheritFromGlobalDutyTemplateCommand : Command<DutyCategoryAggregate, DutyCategoryId>
    {
        public InheritFromGlobalDutyTemplateCommand(DutyCategoryId aggregateId, GlobalDutyTemplateReadModel readModel, string globalDutyId) : base(aggregateId)
        {
            ReadModel = readModel;
            GlobalDutyId = globalDutyId;
        }

        public GlobalDutyTemplateReadModel ReadModel { get; }
        public string GlobalDutyId { get; }

    }

    public class InheritFromGlobalDutyTemplateCommandHandler : CommandHandler<DutyCategoryAggregate, DutyCategoryId, InheritFromGlobalDutyTemplateCommand>
    {
        public override Task ExecuteAsync(DutyCategoryAggregate aggregate, InheritFromGlobalDutyTemplateCommand command, CancellationToken cancellationToken)
        {
            return Task.Run(() =>
            ExecuteAllOfThis(aggregate, command), cancellationToken);
        }

        private void ExecuteAllOfThis(DutyCategoryAggregate aggregate, InheritFromGlobalDutyTemplateCommand command)
        {
            aggregate.ModifyDutyCategoryGlobalDutyId(command.GlobalDutyId);
            aggregate.ModifyDutyCategoryName($"{command.ReadModel.Title}");
            if (!string.IsNullOrEmpty(command.ReadModel.Note))
                aggregate.ModifyDutyCategoryDutyTemplateNote(command.ReadModel.Note);

            if (command.ReadModel.TimeSlotTemplates == null || command.ReadModel.TimeSlotTemplates.Count == 0) return;

            foreach (var tst in command.ReadModel.TimeSlotTemplates)
            {
                var tstGuid = Guid.NewGuid();
                aggregate.AddTimeSlotInDutyTemplate(TimeSlotTemplateId.With(tstGuid));
                aggregate.ModifyStartTimeTimeSlotDutyTemplate(TimeSlotTemplateId.With(tstGuid), tst.StartTime);
                aggregate.ModifyEndTimeTimeSlotDutyTemplate(TimeSlotTemplateId.With(tstGuid), tst.EndTime);
                if (!string.IsNullOrEmpty(tst.TimeSlotName))
                    aggregate.ModifyTimeSlotNameDutyTemplate(TimeSlotTemplateId.With(tstGuid), tst.TimeSlotName);
                if (!string.IsNullOrEmpty(tst.Note))
                    aggregate.ModifyNoteTimeSlotDutyTemplate(TimeSlotTemplateId.With(tstGuid), tst.Note);

                if (tst.WeekDays == null || tst.WeekDays.Count == 0) continue;

                aggregate.ModifyWeekdaysTimeSlotDutyTemplate(TimeSlotTemplateId.With(tstGuid), tst.WeekDays);
            }
        }
    }
}
