﻿using System;
using System.Collections.Generic;
using System.Linq;
using CRM.Domain.DutyCategoryAndTemplate.Models;

namespace WebPortal.Application.DutyCategories.Models
{
    public class DutyOverviewModel
    {
        public string Id { get; set; }
        public string DossierId { get; set; }
        public string CategoryName { get; set; }
        public string Note { get; set; }
        public ICollection<DutyModel> Duties { get; set; }

        public bool DoIMeetTheConditions(DateTime start, DateTime end, bool showActiveDuties,
            bool showPlannedDuties,
            bool showExpiredDuties,
            bool showEmptyCategories,
            string employeeId)
        {
            if (!showActiveDuties && !showExpiredDuties && !showPlannedDuties)
                Duties = null;

            if (Duties == null) return showEmptyCategories;



            List<DutyModel> filteredDuties = new List<DutyModel>();
            foreach (var duty in Duties)
            {
                if (duty.End < start) continue;
                if (duty.Start > end) continue;

                if (showActiveDuties && showExpiredDuties && showPlannedDuties)
                {
                    filteredDuties.Add(duty);
                    continue;
                }

                if (!showExpiredDuties && duty.End < DateTime.Now) continue;

                var filteredTimeSlots = new List<TimeSlotModelNoMore>();
                foreach (var timeSlot in duty.TimeSlots)
                {
                    if (showPlannedDuties)
                    {
                        filteredTimeSlots.Add(timeSlot);
                        continue;
                    }


                    if (!timeSlot.WeekDays
                        .Select(x => x.Value)
                        .Contains(DateTime.Now.DayOfWeek.ToString()))
                    {
                        continue;
                    }
                    if (timeSlot.EndTime < DateTime.Now.TimeOfDay) continue;

                    if (timeSlot.StartTime > DateTime.Now.TimeOfDay) continue;

                    filteredTimeSlots.Add(timeSlot);
                }

                filteredDuties.Add(duty);
                filteredDuties.Single(x => x.Id == duty.Id).TimeSlots = filteredTimeSlots;

            }

            if (filteredDuties.Count == 0)
                Duties = null;

            if (Duties == null) return showEmptyCategories;

            if (string.IsNullOrWhiteSpace(employeeId))
            {
                Duties = filteredDuties;
                return true;
            }

            var employeeDuties = new List<DutyModel>();

            foreach (var fd in filteredDuties)
            {
                if (!CheckEmployee(fd, employeeId)) continue;

                employeeDuties.Add(fd);
                employeeDuties.Single(x => x.Id == fd.Id).TimeSlots = GetEmployeeTimeslots(fd, employeeId);

            }

            Duties = employeeDuties;

            return true;

        }

        private bool CheckEmployee(DutyModel duty, string employeeId)
        {
            if (string.IsNullOrWhiteSpace(employeeId)) return true;

            var employee = duty.TimeSlots
                .SelectMany(x => x.DutyContacts
                    .Where(y => y.EmployeeId == employeeId))
                .ToList();
            return employee.Count > 0;
        }

        private List<TimeSlotModelNoMore> GetEmployeeTimeslots(DutyModel duty,
            string employeeId)
        {
            var rtn = new List<TimeSlotModelNoMore>();
            
            foreach (var timeSlot in duty.TimeSlots)
            {

                if (timeSlot.DutyContacts
                    .Select(x => x.EmployeeId)
                    .Contains(employeeId))
                    rtn.Add(timeSlot);
            }

            return rtn;
        }

    }
}
