﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.DutyCategories.Models;

namespace WebPortal.Application.DutyCategories.Queries
{
    public class GetDutyCategoriesQuery : IQuery<List<DutyOverviewModel>>
    {
        public GetDutyCategoriesQuery(string dossierId,
            DateTime start,
            DateTime end,
            bool showActiveDuties,
            bool showPlannedDuties,
            bool showExpiredDuties,
            bool showEmptyCategories,
            string filter,
            string employeeId)
        {
            DossierId = dossierId;
            Start = start;
            End = end;
            ShowActiveDuties = showActiveDuties;
            ShowPlannedDuties = showPlannedDuties;
            ShowExpiredDuties = showExpiredDuties;
            ShowEmptyCategories = showEmptyCategories;
            Filter = filter ?? "";
            EmployeeId = employeeId ?? "";
        }

        public string DossierId { get; set; }
        public DateTime Start { get; set; }
        public DateTime End { get; set; }
        public bool ShowActiveDuties { get; set; }
        public bool ShowPlannedDuties { get; set; }
        public bool ShowExpiredDuties { get; set; }
        public bool ShowEmptyCategories { get; set; }
        public string Filter { get; set; }
        public string EmployeeId { get; set; }
    }

    public class GetDutyCategoriesQueryHandler : IQueryHandler<GetDutyCategoriesQuery, List<DutyOverviewModel>>
    {
        public GetDutyCategoriesQueryHandler(IMongoDbReadModelStore<DutyCategoryReadModel> store)
        {
            _store = store;
        }

        private readonly IMongoDbReadModelStore<DutyCategoryReadModel> _store;

        public Task<List<DutyOverviewModel>> ExecuteQueryAsync(GetDutyCategoriesQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(
         FilterTheQuery(query)
            );
        }

        private List<DutyOverviewModel> FilterTheQuery(GetDutyCategoriesQuery query)
        {
            var rtn = _store.AsQueryable()
                .Where(x => !x.Deleted && query.DossierId == x.DossierId)
                .Where(x => string.IsNullOrEmpty(query.Filter) || x.CategoryName.Contains(query.Filter))
                .ToList()
                .Select(x => new DutyOverviewModel()
                {
                    Id = x.Id,
                    CategoryName = x.CategoryName,
                    DossierId = x.DossierId,
                    Note = x.Note,
                    Duties = x.Duties
                })

                .ToList();

            var temp = new List<DutyOverviewModel>(rtn);

            var rtn2 = new List<DutyOverviewModel>();

            foreach (var dcom in temp)
            {
                var dutyCom = rtn.Single(x => x.Id == dcom.Id);
                if (dutyCom.DoIMeetTheConditions(query.Start, query.End, query.ShowActiveDuties, query.ShowPlannedDuties,
                    query.ShowExpiredDuties, query.ShowEmptyCategories, query.EmployeeId))
                {
                    rtn2.Add(dcom);
                }
            }

            return rtn2;
        }
    }
}
