﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;

namespace WebPortal.Application.DutyCategories.Queries
{
    public class GetDutyCategoryIdsByGlobalDutyTemplateIdQuery : IQuery<List<string>>
    {
        public GetDutyCategoryIdsByGlobalDutyTemplateIdQuery(string globalDutyTemplateId)
        {
            GlobalDutyTemplateId = globalDutyTemplateId;
        }

        public string GlobalDutyTemplateId { get; set; }
    }

    public class GetDutyCategoryIdsByGlobalDutyTemplateIdQueryHandler : IQueryHandler<GetDutyCategoryIdsByGlobalDutyTemplateIdQuery, List<string>>
    {
        private readonly IMongoDbReadModelStore<DutyCategoryReadModel> _store;

        public GetDutyCategoryIdsByGlobalDutyTemplateIdQueryHandler(IMongoDbReadModelStore<DutyCategoryReadModel> store)
        {
            _store = store;
        }

        public Task<List<string>> ExecuteQueryAsync(GetDutyCategoryIdsByGlobalDutyTemplateIdQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                _store.AsQueryable()
                .Where(x => x.GlobalDutyTemplateId == query.GlobalDutyTemplateId)
                .Select(x => x.Id)
                .ToList());
                
        }
    }
}
