﻿using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using WebPortal.Application.DutyCategories.Models;

namespace WebPortal.Application.DutyCategories.Queries
{
    public class GetDutyCategoryNamesQuery : IQuery<List<DutyCategoryNameModel>>
    {
        public string DossierId { get; set; }
        public string Filter { get; set; }

        public GetDutyCategoryNamesQuery(string dossierId, string filter)
        {
            DossierId = dossierId;
            Filter = filter;
        }
    }

    public class GetDutyCategoryNamesQueryHandler : IQueryHandler<GetDutyCategoryNamesQuery, List<DutyCategoryNameModel>>
    {
        private readonly IMongoDbReadModelStore<DutyCategoryReadModel> _store;

        public GetDutyCategoryNamesQueryHandler(IMongoDbReadModelStore<DutyCategoryReadModel> store)
        {
            _store = store;
        }

        public Task<List<DutyCategoryNameModel>> ExecuteQueryAsync(GetDutyCategoryNamesQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                _store.AsQueryable()
                .Where(x => !x.Deleted && x.DossierId == query.DossierId && x.TimeSlotTemplates != null && x.TimeSlotTemplates.Count != 0)
                .Where(x => string.IsNullOrEmpty(query.Filter) 
                    || x.CategoryName.ToUpper().Contains(query.Filter.ToUpper()))
                .Select(x => new DutyCategoryNameModel()
                {
                    Id = x.Id,
                    CategoryName = x.CategoryName,
                })
                .OrderBy(x => x.CategoryName)
                .ToList());
        }
    }
}
