﻿using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using WebPortal.Application.DutyCategories.Models;
using WebPortal.Application.Extentions;

namespace WebPortal.Application.DutyCategories.Queries
{
    public class GetDutyCategoryOverviewQuery : PageQuery<DutyCategoryOverviewModel>
    {
        public GetDutyCategoryOverviewQuery(bool asc, string sort, int page, int pageSize, string dossierId, string filter) : base(asc, sort, page, pageSize)
        {
            Filter = filter;
            DossierId = dossierId;
        }

        public string Filter { get; }
        public string DossierId { get; }

    }

    public class GetDutyCategoryOverviewQueryHandler : IQueryHandler<GetDutyCategoryOverviewQuery, PageResult<DutyCategoryOverviewModel>>
    {
        public GetDutyCategoryOverviewQueryHandler(IMongoDbReadModelStore<DutyCategoryReadModel> dcStore, IMongoDbReadModelStore<GlobalDutyTemplateReadModel> gdStore)
        {
            _dcStore = dcStore;
            _gdStore = gdStore;
        }
        private readonly IMongoDbReadModelStore<GlobalDutyTemplateReadModel> _gdStore;
        private readonly IMongoDbReadModelStore<DutyCategoryReadModel> _dcStore;

        public Task<PageResult<DutyCategoryOverviewModel>> ExecuteQueryAsync(GetDutyCategoryOverviewQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                FilterTheQuery(query)
            );

        }

        private PageResult<DutyCategoryOverviewModel> FilterTheQuery(GetDutyCategoryOverviewQuery query)
        {
            List<DutyCategoryReadModel> temp;
            if (string.IsNullOrWhiteSpace(query.Filter))
            {
                temp = _dcStore.AsQueryable()
                    .Where(x => !x.Deleted && query.DossierId == x.DossierId)
                    .ToList();
            }
            else
            {
                temp = _dcStore.AsQueryable()
                    .Where(x => !x.Deleted && query.DossierId == x.DossierId)
                    .Where(x => x.CreatedName.ToUpper().Contains(query.Filter.ToUpper())
                                || x.LastModifiedName.ToUpper().Contains(query.Filter.ToUpper())
                                || x.CategoryName.ToUpper().Contains(query.Filter.ToUpper())
                                || x.Note.ToUpper().Contains(query.Filter.ToUpper()))
                    .ToList();
            }

            var rtn = temp.Select(x => new DutyCategoryOverviewModel()
            {
                CreatedName = x.CreatedName,
                LastModifiedName = x.LastModifiedName,
                LastModifiedOn = x.LastModifiedOn,
                CreatedOn = x.CreatedOn,
                GlobalDutyTemplateId = x.GlobalDutyTemplateId,
                HasChanges = x.Changes != null,
                Id = x.Id,
                Title = string.IsNullOrEmpty(x.CategoryName)
                ? "---No Title Saved---"
                : x.CategoryName,
                Note = x.Note,
                GlobalDutyTemplateName = string.IsNullOrEmpty(x.GlobalDutyTemplateId)
                    ? ""
                    : _gdStore.AsQueryable()
                        .FirstOrDefault(y => y.Id == x.GlobalDutyTemplateId).Title

            })
                    .AsQueryable()
                    .OrderBy(query.Sort, query.Asc)
                    .GetPage(query.Page, query.PageSize);

            return rtn;



        }
    }
}
