﻿using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using WebPortal.Application.DutyCategories.Models;

namespace WebPortal.Application.DutyCategories.Queries
{
    public class GetGlobalDutyIdsQuery: IQuery<GlobalDutyIdModel>
    {
        public GetGlobalDutyIdsQuery(string id)
        {
            Id = id;
        }

        public string Id { get; set; }
    }

    public class GetGlobalDutyIdsQueryHandler : IQueryHandler<GetGlobalDutyIdsQuery, GlobalDutyIdModel>
    {
        private readonly IMongoDbReadModelStore<DutyCategoryReadModel> _store;

        public GetGlobalDutyIdsQueryHandler(IMongoDbReadModelStore<DutyCategoryReadModel> store)
        {
            _store = store;
        }

        public Task<GlobalDutyIdModel> ExecuteQueryAsync(GetGlobalDutyIdsQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetResult(query)
            );
        }

        private GlobalDutyIdModel GetResult(GetGlobalDutyIdsQuery query)
        {
            var result = _store.AsQueryable()
                .SingleOrDefault(x => !x.Deleted && query.Id == x.Id);

            if (result == null) return null;

            var rtn = new GlobalDutyIdModel()
            {
                ChangesId = result.Changes?.GlobalDutyTemplateId,
                SavedId = result.GlobalDutyTemplateId
            };

            return rtn;
        }
    }
}
