﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.DutyCategories.Models;

namespace WebPortal.Application.DutyCategories.Queries
{
    public class GetOneDutyCategoryQuery: IQuery<DutyCategoryDetailsModel>
    {
        public GetOneDutyCategoryQuery(string id)
        {
            Id = id;
        }

        public string Id { get; set; }
    }

    //public class GetOneDutyCategoryQueryHandler : IQueryHandler<GetOneDutyCategoryQuery, DutyCategoryDetailsModel>
    //{
    //    private readonly IMongoDbReadModelStore<DutyCategoryReadModel> _dcStore;
    //    private readonly IMongoDbReadModelStore<GlobalDutyTemplateReadModel> __gdtStore;

    //    public GetOneDutyCategoryQueryHandler(IMongoDbReadModelStore<DutyCategoryReadModel> dcStore, IMongoDbReadModelStore<GlobalDutyTemplateReadModel> gdtStore)
    //    {
    //        _dcStore = dcStore;
    //        __gdtStore = gdtStore;
    //    }

    //    public Task<DutyCategoryReadModel> ExecuteQueryAsync(GetOneDutyCategoryQuery query,
    //        CancellationToken cancellationToken)
    //    {
    //        return Task.FromResult(
    //            _dcStore.AsQueryable()
    //                .SingleOrDefault(x => !x.Deleted && query.Id == x.Id));
    //    }
    //}

    public class GetOneDutyCategoryQueryHandler : IQueryHandler<GetOneDutyCategoryQuery, DutyCategoryDetailsModel>
    {
        private readonly IMongoDbReadModelStore<DutyCategoryReadModel> _dcStore;
        private readonly IMongoDbReadModelStore<GlobalDutyTemplateReadModel> __gdtStore;

        public GetOneDutyCategoryQueryHandler(IMongoDbReadModelStore<DutyCategoryReadModel> dcStore, IMongoDbReadModelStore<GlobalDutyTemplateReadModel> gdtStore)
        {
            _dcStore = dcStore;
            __gdtStore = gdtStore;
        }

        public Task<DutyCategoryDetailsModel> ExecuteQueryAsync(GetOneDutyCategoryQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetDetails(query)
            );        
        }

        private DutyCategoryDetailsModel GetDetails(GetOneDutyCategoryQuery query)
        {
            var readModel = _dcStore.AsQueryable()
                    .SingleOrDefault(x => !x.Deleted && query.Id == x.Id);
            if (readModel == null) return null;

            var rtn = new DutyCategoryDetailsModel()
            {
                Id = query.Id,
                DossierId = readModel.DossierId,
                CategoryName = readModel.CategoryName,
                Changes = readModel.Changes,
                GlobalDutyTemplateId = readModel.GlobalDutyTemplateId,
                GlobalDutyTemplateTitle = string.IsNullOrEmpty(readModel.GlobalDutyTemplateId)
                ? ""
                : __gdtStore.AsQueryable()
                            .SingleOrDefault(x => x.Id == readModel.GlobalDutyTemplateId)?.Title,
                Note = readModel.Note,
                TimeSlotTemplates = readModel.TimeSlotTemplates
            };
            if (rtn.Changes == null || string.IsNullOrEmpty(rtn.Changes.GlobalDutyTemplateId))
                return rtn;

            rtn.Changes.GlobalDutyTemplateTitle = __gdtStore.AsQueryable()
                .SingleOrDefault(x => x.Id == readModel.Changes.GlobalDutyTemplateId)?.Title;

            return rtn;
        }
    }
}
