﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.Shared.Enums;
using CsvHelper;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;

namespace WebPortal.Application.Employees.Queries
{
    public class ExportAllEmployeesQueryHandler : IQueryHandler<ExportAllEmployeesQuery, MemoryStream>
    {
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _store;

        public ExportAllEmployeesQueryHandler(IMongoDbReadModelStore<EmployeeReadModel> store)
        {
            _store = store;
        }

        public Task<MemoryStream> ExecuteQueryAsync(ExportAllEmployeesQuery query,
            CancellationToken cancellationToken)
        {
            var employees = _store.AsQueryable()
                .Where(x => !x.Deleted)
                .Where(x => string.IsNullOrEmpty(query.CustomerId) || x.CustomerId.Equals(query.CustomerId))
                .ToList();

            return Task.FromResult(CreateCsv(employees));
        }

        private MemoryStream CreateCsv(List<EmployeeReadModel> employees)
        {
            var emps = employees.Select(x => new
            {
                name = $"{x.FirstName} {x.LastName}",
                contactDetails = x.ContactDetails != null ? string.Join("|", x.ContactDetails.Select(x => $"{x.ContactDetailName} : {x.ContactDetailValue}")) : "",
                tags = x.Tags != null ? string.Join("|", x.Tags) : "",
            });

            var stream = new MemoryStream();
            var writer = new StreamWriter(stream);
            var csv = new CsvWriter(writer, CultureInfo.InvariantCulture);

            csv.WriteRecords(emps);
            writer.Flush();
            return stream;
        }
    }
}