﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.Shared.Enums;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Extentions;

namespace WebPortal.Application.Employees.Queries
{
    public class GetAllEmployeesQueryHandler : IQueryHandler<GetAllEmployeesQuery, PageResult<EmployeeReadModel>>
    {
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _store;

        public GetAllEmployeesQueryHandler(IMongoDbReadModelStore<EmployeeReadModel> store)
        {
            _store = store;
        }

        public Task<PageResult<EmployeeReadModel>> ExecuteQueryAsync(GetAllEmployeesQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(
                    _store.AsQueryable()
                        .Where(x => !x.Deleted)
                        .Where(x => string.IsNullOrEmpty(query.CustomerId) || x.CustomerId.Equals(query.CustomerId))
                        .Where(x => string.IsNullOrEmpty(query.Email) || x.ContactDetails.Any(y =>
                            y.ContactType.Equals(ContactDetailType.Email) &&
                            y.ContactDetailValue.Contains(query.Email)))
                        .Where(x => string.IsNullOrEmpty(query.FullName) ||
                                    x.FirstName.ToLower().Contains(query.FullName.ToLower()) ||
                                    x.LastName.ToLower().Contains(query.FullName.ToLower()))
                        .Where(x => string.IsNullOrEmpty(query.Phone) || x.ContactDetails.Any(y =>
                            y.ContactType.Equals(ContactDetailType.TelephoneNr) &&
                            y.ContactDetailValue.Contains(query.Phone)))
                        .Where(x => string.IsNullOrEmpty(query.Tag) || x.Tags.Any(y => y.ToLower().Contains(query.Tag.ToLower())))
                        .OrderBy(query.Sort, query.Asc)
                        .GetPage(query.Page, query.PageSize)
                );
        }
    }
}