﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Employees.Models;

namespace WebPortal.Application.Employees.Queries
{
    public class GetCustomerByContactDetailValueQuery: IQuery<string>
    {
        public GetCustomerByContactDetailValueQuery(string contactDetailValue)
        {
            ContactDetailValue = contactDetailValue;
        }

        public string ContactDetailValue { get; set; }
    }

    public class GetCustomerByContactDetailValueQueryHandler : IQueryHandler<GetCustomerByContactDetailValueQuery, string>
    {
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _store;

        public GetCustomerByContactDetailValueQueryHandler(IMongoDbReadModelStore<EmployeeReadModel> store)
        {
        }

        public Task<string> ExecuteQueryAsync(GetCustomerByContactDetailValueQuery query, CancellationToken cancellationToken)
        {
            return Task.Run(() => GetResult(query), cancellationToken);
        }

        private string GetResult(GetCustomerByContactDetailValueQuery query)
        {
            var emp = _store.AsQueryable()
                .Where(x => x.ContactDetails
                .Any(y => y.ContactDetailValue
                .Contains(query.ContactDetailValue)))
                .ToList();

            if (emp.Any())
            {
                return emp[0].CustomerId;
            }

            return "";
        }
    }
}
