﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Employees.Models;

namespace WebPortal.Application.Employees.Queries
{
    public class GetEmployeeNumbersQueryHandler : IQueryHandler<GetEmployeeNumbersQuery, GetEmployeeNumbersModel>
    {
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _store;

        public GetEmployeeNumbersQueryHandler(IMongoDbReadModelStore<EmployeeReadModel> store)
        {
            _store = store;
        }

        public Task<GetEmployeeNumbersModel> ExecuteQueryAsync(GetEmployeeNumbersQuery query,
            CancellationToken cancellationToken)
        {
            var q = _store.AsQueryable().Where(x => x.CustomerId == query.CustomerId && !x.Deleted);

            return Task.FromResult(new GetEmployeeNumbersModel
                {
                    TotalEmployees = q.Count(),
                    ForOnDuty = q.Count(x => x.CanBeOnDuty) 
                });
        }
    }
}