﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Employees.Models;

namespace WebPortal.Application.Employees.Queries
{
    public class GetEmployeeWithDefaultsQuery: IQuery<EmployeeWithDefaultsModel>
    {
        public GetEmployeeWithDefaultsQuery(string id)
        {
            Id = id;
        }

        public string Id { get; set; }
    }

    public class GetEmployeeWithDefaultsQueryHandler : IQueryHandler<GetEmployeeWithDefaultsQuery, EmployeeWithDefaultsModel>
    {
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _store;

        public GetEmployeeWithDefaultsQueryHandler(IMongoDbReadModelStore<EmployeeReadModel> store)
        {
            _store = store;
        }

        public Task<EmployeeWithDefaultsModel> ExecuteQueryAsync(GetEmployeeWithDefaultsQuery query, CancellationToken cancellationToken)
        {
            return Task.Run(() => GetResult(query), cancellationToken);
        }

        private EmployeeWithDefaultsModel GetResult(GetEmployeeWithDefaultsQuery query)
        {
            var rm = _store.AsQueryable()
                .SingleOrDefault(x => x.Id == query.Id);

            if (rm == null) return null;

            var rtn = new EmployeeWithDefaultsModel()
            {
                DefaultTags = rm.DefaultTags.ToList(),
                DefaultContactPointIds = rm.ContactDetails
                .Where(x => x.IsDefault)?
                .Select(x => x.Id)
                .ToList(),
                Id = query.Id,
            };
            return rtn;
        }


    }
}
