﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Dossier.ReadModels;
using CRM.Domain.Employee.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Employees.Models;

namespace WebPortal.Application.Employees.Queries
{
    public class GetEmployeesNeoDutyOverviewQuery: IQuery<List<EmployeeNeoDutyOverviewModel>>
    {
        public GetEmployeesNeoDutyOverviewQuery(string dossierId)
        {
            DossierId = dossierId;
        }

        public string DossierId { get; set; }
    }
    public class GetEmployeesNeoDutyOverviewQueryHandler : IQueryHandler<GetEmployeesNeoDutyOverviewQuery, List<EmployeeNeoDutyOverviewModel>>
    {
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _empStore;
        private readonly IMongoDbReadModelStore<DossierReadModel> _dosStore;

        public GetEmployeesNeoDutyOverviewQueryHandler(IMongoDbReadModelStore<EmployeeReadModel> empStore, IMongoDbReadModelStore<DossierReadModel> dosStore)
        {
            _empStore = empStore;
            _dosStore = dosStore;
        }

        public Task<List<EmployeeNeoDutyOverviewModel>> ExecuteQueryAsync(GetEmployeesNeoDutyOverviewQuery query, CancellationToken cancellationToken)
        {
            return Task.Run(() => GetResults(query), cancellationToken);
        }

        private List<EmployeeNeoDutyOverviewModel> GetResults(GetEmployeesNeoDutyOverviewQuery query)
        {
            var custId = _dosStore.AsQueryable()
                .SingleOrDefault(x => x.Id == query.DossierId)
                .CustomerId;

            var rtn = _empStore.AsQueryable()
                .Where(x => !x.Deleted && x.CanBeOnDuty && x.CustomerId == custId)
                .ToList()
                .Select(x => new EmployeeNeoDutyOverviewModel()
                {
                    EmployeeId = x.Id,
                    FullName = x.FullName,
                    HasContactPoints = x.ContactDetails.Where(y => y.IsDefault).Any(),
                })
                .OrderBy(x => x.FullName)
                .ToList();

            return rtn;

        }
    }
}
