﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Extentions;
using WebPortal.Application.GlobalDutyTemplates.Models;

namespace WebPortal.Application.GlobalDutyTemplates.Queries
{
    public class GetGlobalDutyTemplatesQuery: PageQuery<GlobalDutyTemplateOverviewModel>
    {
        public GetGlobalDutyTemplatesQuery(bool asc,
            string sort,
            int page,
            int pageSize,
            string filter) : base(asc, sort, page, pageSize)
        {
            Filter = filter;
        }

        public string Filter { get; }
    }

    public class
        GetGlobalDutyTemplatesQueryHandler : IQueryHandler<GetGlobalDutyTemplatesQuery,
            PageResult<GlobalDutyTemplateOverviewModel>>
    {
        public GetGlobalDutyTemplatesQueryHandler(IMongoDbReadModelStore<GlobalDutyTemplateReadModel> store)
        {
            _store = store;
        }

        private readonly IMongoDbReadModelStore<GlobalDutyTemplateReadModel> _store;

        public Task<PageResult<GlobalDutyTemplateOverviewModel>> ExecuteQueryAsync(GetGlobalDutyTemplatesQuery query,
            CancellationToken cancellationToken)
        {
            if (string.IsNullOrWhiteSpace(query.Filter))
            {
                return Task.FromResult(
                    _store.AsQueryable()
                        .Where(x => !x.Deleted)
                        .ToList()
                        .Select(x => new GlobalDutyTemplateOverviewModel()
                        {
                            Id = x.Id,
                            CreatedName = x.CreatedName,
                            CreatedOn = x.CreatedOn,
                            LastModifiedName = x.LastModifiedName,
                            LastModifiedOn = x.LastModifiedOn,
                            Note = x.Note,
                            Title = x.Title,
                            Usage = x.Usage,
                            HasChanges = x.Changes != null
                        })
                        .AsQueryable()
                        .OrderBy(query.Sort, query.Asc)
                        .GetPage(query.Page, query.PageSize)
                );
            }

            return Task.FromResult(
                _store.AsQueryable()
                    .Where(x => !x.Deleted)
                    .Where(x => x.CreatedName.ToUpper().Contains(query.Filter.ToUpper()) 
                                || x.LastModifiedName.ToUpper().Contains(query.Filter.ToUpper())
                                || x.Title.ToUpper().Contains(query.Filter.ToUpper())
                                || x.Note.ToUpper().Contains(query.Filter.ToUpper()))
                    .ToList()
                    .Select(x => new GlobalDutyTemplateOverviewModel()
                    {
                        Id = x.Id,
                        CreatedName = x.CreatedName,
                        CreatedOn = x.CreatedOn,
                        LastModifiedName = x.LastModifiedName,
                        LastModifiedOn = x.LastModifiedOn,
                        Note = x.Note,
                        Title = x.Title,
                        Usage = x.Usage,
                        HasChanges = x.Changes != null
                    })
                    .AsQueryable()

                    .OrderBy(query.Sort, query.Asc)
                    .GetPage(query.Page, query.PageSize)
            );
        }
    }
}
