﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.GlobalDutyTemplates.Models;

namespace WebPortal.Application.GlobalDutyTemplates.Queries
{
    public class GetTitlesGlobalDutyQuery : IQuery<List<IdAndTitleModel>>
    {
        public GetTitlesGlobalDutyQuery(string filter)
        {
            Filter = filter;
        }

        public string Filter { get; set; }
    }

    public class GetTitlesGlobalDutyQueryHandler : IQueryHandler<GetTitlesGlobalDutyQuery, List<IdAndTitleModel>>
    {
        private readonly IMongoDbReadModelStore<GlobalDutyTemplateReadModel> _store;

        public GetTitlesGlobalDutyQueryHandler(IMongoDbReadModelStore<GlobalDutyTemplateReadModel> store)
        {
            _store = store;
        }

        public Task<List<IdAndTitleModel>> ExecuteQueryAsync(GetTitlesGlobalDutyQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(
                _store.AsQueryable()
                    .Where(x => !x.Deleted && x.Title != null)
                    .Where(x => x.Title.ToLower().Contains(query.Filter.ToLower()))
                    .Select(x => new IdAndTitleModel()
                    {
                        Id = x.Id,
                        Title = x.Title
                    })
                    .OrderBy(x => x.Title)
                    .ToList()
            );
        }
    }
}
