﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.DutyCategoryAndTemplate.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.GlobalDutyTemplates.Models;

namespace WebPortal.Application.GlobalDutyTemplates.Queries
{
    public class GetUnsavedGlobalDutyTemplatesQuery: IQuery<List<IdAndTitleModel>>
    {
        
    }

    public class GetUnsavedGlobalDutyTemplatesQueryHandler : IQueryHandler<GetUnsavedGlobalDutyTemplatesQuery,
        List<IdAndTitleModel>>
    {
        private readonly IMongoDbReadModelStore<GlobalDutyTemplateReadModel> _store;

        public GetUnsavedGlobalDutyTemplatesQueryHandler(IMongoDbReadModelStore<GlobalDutyTemplateReadModel> store)
        {
            _store = store;
        }

        public Task<List<IdAndTitleModel>> ExecuteQueryAsync(GetUnsavedGlobalDutyTemplatesQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(
                _store.AsQueryable()
                    .Where(x => !x.Deleted
                                && x.Changes != null)
                    .ToList()
                    .Select(x => new IdAndTitleModel()
                    {
                        Id = x.Id,
                        Title = string.IsNullOrWhiteSpace(x.Changes.Title)
                            ? string.IsNullOrWhiteSpace(x.Title)
                                ? x.LastModifiedOn.ToString("G")
                                : x.Title
                            : x.Changes.Title
                    })
                    .ToList()
            );
        }
    }
}
