﻿using CRM.Domain.NeoDuty.Entities;
using CRM.Domain.NeoDuty;
using CRM.Domain.NeoDuty.Models;
using EventFlow.Commands;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;

namespace WebPortal.Application.NeoDuties.Commands.TimeSlot.Contact
{
    public class AddContactCommand : Command<NeoDutyAggregate, NeoDutyId>
    {
        public string TimeSlotId { get; set; }
        public DutyContactId ContactId { get; set; }
        public string EmployeeId { get; set; }
        public List<string> DefaultTags { get; set; }
        public List<DutyContactPointModel> DefaultContactPoints { get; }


        public AddContactCommand(NeoDutyId aggregateId, string timeSlotId, DutyContactId contactId, string employeeId,  List<string> defaultTags, List<DutyContactPointModel> defaultContactPoints): base (aggregateId)
        {
            TimeSlotId = timeSlotId;
            ContactId = contactId;
            EmployeeId = employeeId;
            DefaultTags = defaultTags;
            DefaultContactPoints = defaultContactPoints;
        }
    }

    public class AddContactCommandHandler : CommandHandler<NeoDutyAggregate, NeoDutyId, AddContactCommand>
    {
        public override Task ExecuteAsync(NeoDutyAggregate aggregate, AddContactCommand command, CancellationToken cancellationToken)
        {
            return Task.Run(() => aggregate.AddContactToTimeSlot
            (
                TimeSlotId.With(command.TimeSlotId), 
                (command.ContactId),
                command.EmployeeId,
                command.DefaultTags,
                command.DefaultContactPoints), cancellationToken);

        }
    }
}
