﻿using CRM.Domain.DutyCategoryAndTemplate.Models;
using CRM.Domain.NeoDuty.Models;
using CRM.Domain.Shared.ValueObjects;
using System;
using System.Collections.Generic;
using System.Linq;

namespace WebPortal.Application.NeoDuties.Models
{
    public class TimeSlotOverviewModel
    {

        public string Id { get; set; }
        public string TimeSlotName { get; set; }
        public TimeSpan StartTime { get; set; }
        public TimeSpan EndTime { get; set; }
        public List<WeekDay> WeekDays { get; set; }
        public List<ContactOverviewModel> Contacts { get; set; }
        public string WeekdayString => GetWeekDaysString();

        public TimeSlotOverviewModel(TimeSlotModel t)
        {
            Id = t.Id;
            TimeSlotName = t.TimeSlotName;
            StartTime = t.StartTime;
            EndTime = t.EndTime;
            WeekDays = t.WeekDays.ToList();
            Contacts = t.DutyContacts == null
                ? new List<ContactOverviewModel>()
                : t.DutyContacts.Select(x => new ContactOverviewModel(x))
                    .OrderBy(c => c.OrderId)
                    .ToList();
        }

        public TimeSlotOverviewModel(TimeSlotTemplateModel t)
        {
            Id = t.Id;
            TimeSlotName = t.TimeSlotName;
            StartTime = t.StartTime;
            EndTime = t.EndTime;
            WeekDays = t.WeekDays.ToList();
        }

        private string GetWeekDaysString()
        {
            var weekDaysInt = WeekDays.Select(x => Convert.ToInt32(x.Value)).ToList();
            var orderedWeekDays = weekDaysInt.OrderBy(x => x).ToList();

            string[] allDays = { "Sun","Mon", "Tue", "Wed", "Thu", "Fri", "Sat"  };

            string everyDay = "";
            string firstDay = "";
            string lastDay = "";
            bool subsequentOrder = true;
            var counter = 0;

            var previousWdInt = -1;

            foreach (var wd in orderedWeekDays)
            {
                everyDay += $"{allDays[wd]}, ";

                if (counter == 0)
                    firstDay = allDays[wd];

                if (counter > 0 && wd != previousWdInt + 1)
                {
                    subsequentOrder = false;
                }
                counter++;
                previousWdInt = wd;
                if (counter == orderedWeekDays.Count)
                {
                    lastDay = allDays[wd];
                }
            }

            if (!string.IsNullOrEmpty(firstDay) && !string.IsNullOrEmpty(lastDay) && subsequentOrder)
                return $"{firstDay} - {lastDay}";

            return everyDay.Substring(0, everyDay.Length - 2);
        }
    }
}
