﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.NeoDuty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.NeoDuties.Models.Detail;

namespace WebPortal.Application.NeoDuties.Queries.Detail
{
    public class GetAllTimeSlotsQuery: IQuery<List<TimeSlotDetailModel>>
    {
        public GetAllTimeSlotsQuery(string id)
        {
            Id = id;
        }

        public string Id { get; set; }
    }

    public class GetAllTimeSlotsQueryHandler: IQueryHandler<GetAllTimeSlotsQuery, List<TimeSlotDetailModel>>
    {
        private readonly IMongoDbReadModelStore<NeoDutyReadModel> _ndStore;
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _empStore;

        public GetAllTimeSlotsQueryHandler(IMongoDbReadModelStore<NeoDutyReadModel> ndStore, IMongoDbReadModelStore<EmployeeReadModel> empStore)
        {
            _ndStore = ndStore;
            _empStore = empStore;
        }

        public Task<List<TimeSlotDetailModel>> ExecuteQueryAsync(GetAllTimeSlotsQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetResults(query));
        }

        private List<TimeSlotDetailModel> GetResults(GetAllTimeSlotsQuery query)
        {
            var rtn =new List<TimeSlotDetailModel>();
            var readModel = _ndStore.AsQueryable()
.SingleOrDefault(x => x.Id == query.Id);
            if (readModel == null) return rtn;

            rtn = readModel.Changes.TimeSlots.Select(t => new TimeSlotDetailModel(t)).ToList();

            var allEmployeeIds = rtn
                .SelectMany(x => x.Contacts
                .Select(y => y.EmployeeId))
                .Distinct()
                .ToList();

            var allEmployees = _empStore.AsQueryable()
                .Where(x => !x.Deleted && allEmployeeIds.Contains(x.Id))
                .ToList()
                .Select(x => new
                {
                    x.Id,
                    x.FullName,
                    x.ContactDetails
                })
                .ToList();

            foreach (var t in rtn)
            {
                foreach (var c in t.Contacts)
                {
                    var emp = allEmployees.SingleOrDefault(x => x.Id == c.EmployeeId);
                    if (emp != null)
                        c.FullName = emp.FullName;
                    else
                    {
                        c.FullName = "Employee is missing!";
                        continue;
                    }
                    foreach (var p in c.ContactPoints)
                    {
                        var con = emp.ContactDetails.SingleOrDefault(x => x.Id == p.ContactDetailId);
                        if (con != null)
                        {
                            p.ContactPointTitleAndValue = $"{con.ContactDetailName}: {con.ContactDetailValue}";
                        }
                        else
                        {
                            p.ContactPointTitleAndValue = "Contactpoint is missing!";
                        }
                    }
                }
            }

            return rtn;
        }
    }
}
