﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.NeoDuty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.NeoDuties.Models.Detail;

namespace WebPortal.Application.NeoDuties.Queries.Detail
{
    public class GetOneDutyContactQuery : IQuery<ContactDetailModel>
    {
        public GetOneDutyContactQuery(string id, string timeSlotId, string employeeId)
        {
            Id = id;
            TimeSlotId = timeSlotId;
            EmployeeId = employeeId;
        }

        public string Id { get; set; }
        public string TimeSlotId { get; set; }
        public string EmployeeId { get; set; }
    }

    public class GetOneDutyContactQueryHandler: IQueryHandler<GetOneDutyContactQuery, ContactDetailModel>
    {
        private readonly IMongoDbReadModelStore<NeoDutyReadModel> _ndStore;
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _empStore;

        public GetOneDutyContactQueryHandler(IMongoDbReadModelStore<NeoDutyReadModel> ndStore, IMongoDbReadModelStore<EmployeeReadModel> empStore)
        {
            _ndStore = ndStore;
            _empStore = empStore;
        }

        public Task<ContactDetailModel> ExecuteQueryAsync(GetOneDutyContactQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetResult(query));
        }

        public ContactDetailModel GetResult(GetOneDutyContactQuery query)
        {
            var readModel = _ndStore.AsQueryable()
    .SingleOrDefault(x => x.Id == query.Id).Changes.TimeSlots.Single(x => x.Id == query.TimeSlotId).DutyContacts.Single(x => x.EmployeeId == query.EmployeeId);

            var rtn = new ContactDetailModel(readModel);
            var employee = _empStore.AsQueryable().SingleOrDefault(x => x.Id == query.EmployeeId);

            rtn.FullName = employee.FullName;

            foreach (var p in rtn.ContactPoints)
            {
                var con = employee.ContactDetails.SingleOrDefault(x => x.Id == p.ContactDetailId);
                if (con != null)
                {
                    p.ContactPointTitleAndValue = $"{con.ContactDetailName}: {con.ContactDetailValue}";
                }
                else
                {
                    p.ContactPointTitleAndValue = "Contactpoint is missing!";
                }
            }


            return rtn;
        }
    }
}
