﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.NeoDuty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.NeoDuties.Models.Detail;

namespace WebPortal.Application.NeoDuties.Queries.Detail
{
    public class GetOneNeoDutyDetailsQuery: IQuery<NeoDutyDetailModel>
    {
        public string Id { get; set; }

        public GetOneNeoDutyDetailsQuery(string id)
        {
            Id = id;
        }
    }

    public class GetOneNeoDutyDetailsQueryHandler : IQueryHandler<GetOneNeoDutyDetailsQuery, NeoDutyDetailModel>
    {
        private readonly IMongoDbReadModelStore<NeoDutyReadModel> _ndStore;
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _empStore;

        public GetOneNeoDutyDetailsQueryHandler(IMongoDbReadModelStore<NeoDutyReadModel> ndStore, IMongoDbReadModelStore<EmployeeReadModel> empStore)
        {
            _ndStore = ndStore;
            _empStore = empStore;
        }

        public Task<NeoDutyDetailModel> ExecuteQueryAsync(GetOneNeoDutyDetailsQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetResult(query));
        }

        private NeoDutyDetailModel GetResult(GetOneNeoDutyDetailsQuery query)
        {
            var readModel = _ndStore.AsQueryable()
    .SingleOrDefault(x => x.Id == query.Id);
            if (readModel == null) return null;

            var rtn = new NeoDutyDetailModel()
            {
                Id = query.Id,
                DutyCategoryId = readModel.DutyCategoryId,
                End = readModel.Changes == null ? readModel.End: readModel.Changes.End,
                Start = readModel.Changes == null ? readModel.Start: readModel.Changes.Start,
                Note = readModel.Changes == null ? readModel.Note: readModel.Changes.Note,
                TimeSlots = readModel.Changes == null 
                    ? readModel.TimeSlots.Select(t => new TimeSlotDetailModel(t)).ToList()
                    : readModel.Changes.TimeSlots.Select(t => new TimeSlotDetailModel(t)).ToList(),
            };

            var allEmployeeIds = rtn.TimeSlots
                .SelectMany(x => x.Contacts
                .Select(y => y.EmployeeId))
                .Distinct()
                .ToList();

            var allEmployees = _empStore.AsQueryable()
                .Where(x => !x.Deleted && allEmployeeIds.Contains(x.Id))
                .ToList()
                .Select(x => new
                {
                    x.Id,
                    x.FullName,
                    x.ContactDetails
                })
                .ToList();


            foreach (var t in rtn.TimeSlots)
            {
                foreach (var c in t.Contacts)
                {
                    var emp = allEmployees.SingleOrDefault(x => x.Id == c.EmployeeId);
                    if (emp != null)
                        c.FullName = emp.FullName;
                    else
                    {
                        c.FullName = "Employee is missing!";
                        continue;
                    }
                    foreach (var p in c.ContactPoints)
                    {
                        var con = emp.ContactDetails.SingleOrDefault(x => x.Id == p.ContactDetailId);
                        if (con != null)
                        {
                            p.ContactPointTitleAndValue = $"{con.ContactDetailName}: {con.ContactDetailValue}";
                        }
                        else
                        {
                            p.ContactPointTitleAndValue = "Contactpoint is missing!";
                        }
                    }
                }
            }

            return rtn;

        }
    }
}
