﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.NeoDuty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.NeoDuties.Models.Detail;

namespace WebPortal.Application.NeoDuties.Queries.Detail
{
    public class GetOneTimeSlotQuery: IQuery<TimeSlotDetailModel>
    {
        public GetOneTimeSlotQuery(string id, string timeSlotId)
        {
            Id = id;
            TimeSlotId = timeSlotId;
        }

        public string Id { get; set; }
        public string TimeSlotId { get; set; }
    }

    public class GetOneTimeSlotQueryHandler: IQueryHandler<GetOneTimeSlotQuery, TimeSlotDetailModel>
    {
        private readonly IMongoDbReadModelStore<NeoDutyReadModel> _ndStore;
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _empStore;

        public GetOneTimeSlotQueryHandler(IMongoDbReadModelStore<NeoDutyReadModel> ndStore, IMongoDbReadModelStore<EmployeeReadModel> empStore)
        {
            _ndStore = ndStore;
            _empStore = empStore;
        }

        public Task<TimeSlotDetailModel> ExecuteQueryAsync(GetOneTimeSlotQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetResult(query));
        }

        public TimeSlotDetailModel GetResult(GetOneTimeSlotQuery query)
        {
            var readModel = _ndStore.AsQueryable()
                .SingleOrDefault(x => x.Id == query.Id)
                .Changes.TimeSlots.Single(x => x.Id == query.TimeSlotId);

            var rtn = new TimeSlotDetailModel(readModel);

            var allEmployeeIds = rtn.Contacts.Select(x => x.EmployeeId).ToList();

            var allEmployees = _empStore.AsQueryable()
                .Where(x => !x.Deleted && allEmployeeIds.Contains(x.Id))
                .ToList()
                .Select(x => new
                {
                    x.Id,
                    x.FullName,
                    x.ContactDetails
                })
                .ToList();

            foreach (var c in rtn.Contacts)
            {
                var emp = allEmployees.SingleOrDefault(x => x.Id == c.EmployeeId);
                if (emp != null)
                    c.FullName = emp.FullName;
                else
                {
                    c.FullName = "Employee is missing!";
                    continue;
                }
                foreach (var p in c.ContactPoints)
                {
                    var con = emp.ContactDetails.SingleOrDefault(x => x.Id == p.ContactDetailId);
                    if (con != null)
                    {
                        p.ContactPointTitleAndValue = $"{con.ContactDetailName}: {con.ContactDetailValue}";
                    }
                    else
                    {
                        p.ContactPointTitleAndValue = "Contactpoint is missing!";
                    }
                }
            }

            return rtn;

        }
    }
}
