﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.NeoDuty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.NeoDuties.Models.Employee;

namespace WebPortal.Application.NeoDuties.Queries.Employee
{
    public class GetAllContactDetailsOneDutyQuery: IQuery<List<EmployeeAndContactDetails>>
    {
        public GetAllContactDetailsOneDutyQuery(string id)
        {
            Id = id;
        }

        public string Id { get; set; }
    }
    public class GetAllContactDetailsOneDutyQueryHandler : IQueryHandler<GetAllContactDetailsOneDutyQuery, List<EmployeeAndContactDetails>>
    {
        private readonly IMongoDbReadModelStore<NeoDutyReadModel> _ndStore;
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _empStore;

        public GetAllContactDetailsOneDutyQueryHandler(IMongoDbReadModelStore<NeoDutyReadModel> ndStore, IMongoDbReadModelStore<EmployeeReadModel> empStore)
        {
            _ndStore = ndStore;
            _empStore = empStore;
        }

        public Task<List<EmployeeAndContactDetails>> ExecuteQueryAsync(GetAllContactDetailsOneDutyQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetResults(query)
                );
        }

        private List<EmployeeAndContactDetails> GetResults(GetAllContactDetailsOneDutyQuery query)
        {
            var allEmployeeIds = _ndStore.AsQueryable()
                .SingleOrDefault(x => x.Id == query.Id)
                .TimeSlots.Where(x => x.DutyContacts != null).SelectMany(x => x.DutyContacts)
                .Select(x => x.EmployeeId)
                .Distinct()
                .ToList();

            if (!allEmployeeIds.Any())
            {
                return new List<EmployeeAndContactDetails>();
            }

            var rtn = _empStore.AsQueryable()
                .Where(x => allEmployeeIds.Contains(x.Id))
                .ToList()
                .Select(x => new EmployeeAndContactDetails()
                {
                    EmployeeId = x.Id,
                    ContactDetails = x.ContactDetails
                        .Select(y => new ContactAndIdModel()
                        {
                            ContactDetailId = y.Id,
                            TitleAndValue = $"{y.ContactDetailName}: {y.ContactDetailValue}"
                        })
                        .ToList()
                })
                .ToList();
            return rtn;
        }
    }
}
