﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.NeoDuty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.NeoDuties.Models.Employee;

namespace WebPortal.Application.NeoDuties.Queries.Employee
{
    public class GetAllTagsOneDutyQuery: IQuery<List<EmployeeAndTags>>
    {
        public GetAllTagsOneDutyQuery(string id)
        {
            Id = id;
        }

        public string Id { get; set; }
    }

    public class GetAllTagsOneDutyQueryHandler : IQueryHandler<GetAllTagsOneDutyQuery, List<EmployeeAndTags>>
    {
        private readonly IMongoDbReadModelStore<NeoDutyReadModel> _ndStore;
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _empStore;

        public GetAllTagsOneDutyQueryHandler(IMongoDbReadModelStore<NeoDutyReadModel> ndStore, IMongoDbReadModelStore<EmployeeReadModel> empStore)
        {
            _ndStore = ndStore;
            _empStore = empStore;
        }

        public Task<List<EmployeeAndTags>> ExecuteQueryAsync(GetAllTagsOneDutyQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetResults(query)
                );
        }

        private List<EmployeeAndTags> GetResults(GetAllTagsOneDutyQuery query)
        {
            var allEmployeeIds = _ndStore.AsQueryable()
                .SingleOrDefault(x => x.Id == query.Id)
                .TimeSlots.Where(x => x.DutyContacts != null).SelectMany(x => x.DutyContacts)
                .Select(x => x.EmployeeId)
                .Distinct()
                .ToList();

            if (!allEmployeeIds.Any())
            {
                return new List<EmployeeAndTags>();
            }


            var rtn = _empStore.AsQueryable()
                .Where(x => allEmployeeIds.Contains(x.Id))
                .ToList()
                .Select(x => new EmployeeAndTags()
                {
                    EmployeeId = x.Id,
                    Tags = x.Tags.ToList()
                })
                .ToList();

            return rtn;
        }
    }
}
