﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.NeoDuty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.NeoDuties.Models;

namespace WebPortal.Application.NeoDuties.Queries
{
    public class GetNewNeoDutyQuery: IQuery<NeoDutyOverviewModel>
    {
        public string Id { get; set; }

        public GetNewNeoDutyQuery(string id)
        {
            Id = id;
        }
    }

    public class GetNewNeoDutyQueryHandler : IQueryHandler<GetNewNeoDutyQuery, NeoDutyOverviewModel>
    {
        private readonly IMongoDbReadModelStore<NeoDutyReadModel> _ndStore;

        public GetNewNeoDutyQueryHandler(IMongoDbReadModelStore<NeoDutyReadModel> ndStore)
        {
            _ndStore = ndStore;
        }

        public Task<NeoDutyOverviewModel> ExecuteQueryAsync(GetNewNeoDutyQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetResult(query)
                );
        }

        private NeoDutyOverviewModel GetResult(GetNewNeoDutyQuery query)
        {
            var oneDuty = _ndStore.AsQueryable()
                .SingleOrDefault(x => x.Id == query.Id);

            if (oneDuty == null) return null;

            if (oneDuty.Changes == null) return null;

            var rtn = new NeoDutyOverviewModel()
            {
                Id = query.Id,
                DutyCategoryId = oneDuty.DutyCategoryId,
                End = oneDuty.Changes.End,
                Start = oneDuty.Changes.Start,
                HasChanges = true,
                IsDuty = true,
                IsFirstEnd = false,
                IsFirstStart = false,
                IsLastEnd = false,
                IsLastStart = false,
                Note = oneDuty.Changes.Note,
                TimeSlots = oneDuty.Changes.TimeSlots.Select(t => new TimeSlotOverviewModel(t)).ToList()
            };
            return rtn;

        }
    }
}
