﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.NeoDuty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.NeoDuties.Models;

namespace WebPortal.Application.NeoDuties.Queries
{
    public class GetOneNeoDutyOverviewQuery: IQuery<OneNeoDutyOverviewModel>
    {
        public string Id { get; set; }

        public GetOneNeoDutyOverviewQuery(string id)
        {
            Id = id;
        }

    }

    public class GetOneNeoDutyOverviewQueryHandler : IQueryHandler<GetOneNeoDutyOverviewQuery, OneNeoDutyOverviewModel>
    {
        private readonly IMongoDbReadModelStore<NeoDutyReadModel> _ndStore;
        private readonly IMongoDbReadModelStore<EmployeeReadModel> _empStore;

        public GetOneNeoDutyOverviewQueryHandler(IMongoDbReadModelStore<NeoDutyReadModel> ndStore, IMongoDbReadModelStore<EmployeeReadModel> empStore)
        {
            _ndStore = ndStore;
            _empStore = empStore;
        }

        public Task<OneNeoDutyOverviewModel> ExecuteQueryAsync(GetOneNeoDutyOverviewQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetResult(query));
        }

        private OneNeoDutyOverviewModel GetResult(GetOneNeoDutyOverviewQuery query)
        {
            var readModel = _ndStore.AsQueryable()
                .SingleOrDefault(x => x.Id == query.Id);    
            if (readModel == null) return null;

            var rtn = new OneNeoDutyOverviewModel()
            {
                Id = query.Id,
                DutyCategoryId = readModel.DutyCategoryId,
                End = readModel.End,
                Start = readModel.Start,
                Note = readModel.Note,
                TimeSlots = readModel.TimeSlots.Select(t => new TimeSlotOverviewModel(t)).ToList(),
            };

            var allEmployeeIds = rtn.TimeSlots
                .SelectMany(x => x.Contacts
                .Select(y => y.EmployeeId))
                .Distinct()
                .ToList();

            var allEmployees = _empStore.AsQueryable()
                .Where(x => !x.Deleted && allEmployeeIds.Contains(x.Id))
                .ToList()
                .Select(x => new
                {
                    x.Id,
                    x.FullName
                })
                .ToList();

            foreach (var t in rtn.TimeSlots)
            {
                foreach (var c in t.Contacts)
                {
                    var emp = allEmployees.SingleOrDefault(x => x.Id == c.EmployeeId);
                    if (emp != null)
                        c.FullName = emp.FullName;
                    else
                    {
                        c.FullName = "Employee is missing!";
                        continue;
                    }
                }
            }


            //rtn.TimeSlots.ForEach(t => t.Contacts
            //    .ForEach(c => c.FullName = allEmployees
            //        .Single(x => x.Id == c.EmployeeId).FullName));

            return rtn;
        }
    }
}
