﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.NeoDuty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;

namespace WebPortal.Application.NeoDuties.Queries.TimeSlot
{
    public class GetTimeSlotIdsQuery: IQuery<List<string>>
    {
        public GetTimeSlotIdsQuery(string id)
        {
            Id = id;
        }

        public string Id { get; set; }
    }

    public class GetTimeSlotIdsQueryHandler : IQueryHandler<GetTimeSlotIdsQuery, List<string>>
    {
        private readonly IMongoDbReadModelStore<NeoDutyReadModel> _ndStore;

        public GetTimeSlotIdsQueryHandler(IMongoDbReadModelStore<NeoDutyReadModel> ndStore)
        {
            _ndStore = ndStore;
        }

        public Task<List<string>> ExecuteQueryAsync(GetTimeSlotIdsQuery query, CancellationToken cancellationToken)
        {
            return Task.FromResult(
                GetResult(query)
                );
        }

        private List<string> GetResult(GetTimeSlotIdsQuery query)
        {
            var oneDuty = _ndStore.AsQueryable()
                .SingleOrDefault(x => x.Id == query.Id);

            if (oneDuty.Changes != null)
            {
                return oneDuty.Changes.TimeSlots.Select(x => x.Id).ToList();
            }

            return oneDuty.TimeSlots.Select(x => x.Id).ToList();
        }
    }
}
