﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.InteractionStatistic.Enums;
using CRM.Domain.InteractionStatistic.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Statistics.Models;

namespace WebPortal.Application.Statistics.Queries
{
    public class GetStatisticNumbersQueryHandler : IQueryHandler<GetStatisticNumbersQuery, StatisticNumbers>
    {
        private readonly IMongoDbReadModelStore<InteractionStatisticReadModel> _store;

        public GetStatisticNumbersQueryHandler(IMongoDbReadModelStore<InteractionStatisticReadModel> store)
        {
            _store = store;
        }

        public Task<StatisticNumbers> ExecuteQueryAsync(GetStatisticNumbersQuery query,
            CancellationToken cancellationToken)
        {
            var interactions = _store.AsQueryable()
                .Where(x => x.DossierId.Equals(query.DossierId))
                .Where(x => x.Connected >= query.Start)
                .Where(x => x.Connected <= query.End)
                .ToList();

            return Task.FromResult(new StatisticNumbers
            {
                Start = query.Start,
                End = query.End,
                TotalDuration = (int)interactions.Sum(x => x.Duration.TotalSeconds),
                AverageDuration = (int)(interactions.Sum(x => x.Duration.TotalSeconds) / interactions.Count()),
                Calls = interactions.Count(x => x.InteractionTypeEnum == InteractionType.Call)
            });
        }
    }
}