﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.InteractionStatistic.ReadModels;
using CRM.Domain.Ticket.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Extentions;
using WebPortal.Application.Statistics.Models;
using WebPortal.Application.Tickets.Models;

namespace WebPortal.Application.Statistics.Queries
{
    public class GetStatisticsQueryHandler : IQueryHandler<GetStatisticsQuery, PageResult<Statistic>>
    {
        private readonly IMongoDbReadModelStore<InteractionStatisticReadModel> _statStore;
        private readonly IMongoDbReadModelStore<TicketReadModel> _ticketStore;

        public GetStatisticsQueryHandler(IMongoDbReadModelStore<InteractionStatisticReadModel> statStore, IMongoDbReadModelStore<TicketReadModel> ticketStore)
        {
            _statStore = statStore;
            _ticketStore = ticketStore;
        }

        public Task<PageResult<Statistic>> ExecuteQueryAsync(GetStatisticsQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(_statStore.AsQueryable()
                .Where(x => !x.Deleted && x.DossierId == query.DossierId
                                       && x.Established >= query.StartDateTime
                                       && x.Established < query.EndDateTime)
                .ToList()
                .Select(x => new Statistic
                {
                    Id = x.Id,
                    DossierId = x.DossierId,
                    InteractionTypeEnum = x.InteractionTypeEnum,
                    Established = x.Established,
                    Connected = x.Connected,
                    DNIS = x.DNIS,
                    Disconnected = x.Disconnected,
                    Duration = x.Duration,
                    InteractionIdKey = x.InteractionIdKey,
                    TicketAvailable = _ticketStore.AsQueryable()
                        .Any(y => y.Id.Equals(x.TicketId)),
                    TicketId = x.TicketId ?? ""
                })
                .AsQueryable()
                .OrderBy(query.Sort, query.Asc)
                .GetPage(query.Page, query.PageSize));
        }
    }
}