﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Ticket.Models;
using CRM.Domain.Ticket.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;

namespace WebPortal.Application.Tickets.Queries
{
    public class
        GetMoreCommentsForTicketQueryHandler : IQueryHandler<GetMoreCommentsForTicketQuery, List<TicketCommentModel>>
    {
        private readonly IMongoDbReadModelStore<TicketReadModel> _store;

        public GetMoreCommentsForTicketQueryHandler(IMongoDbReadModelStore<TicketReadModel> store)
        {
            _store = store;
        }

        public Task<List<TicketCommentModel>> ExecuteQueryAsync(GetMoreCommentsForTicketQuery query,
            CancellationToken cancellationToken)
        {
            var temp = _store.AsQueryable()
                .SingleOrDefault(x => x.Id == query.TicketId)
                ?.Comments.ToList();

            var result = query.CommentOrderAscending
                ? (temp ?? throw new InvalidOperationException()).OrderBy(y => y.CreateDate)
                .Skip(query.AmountAlreadyLoaded)
                .Take(query.CommentAmount).ToList()
                : (temp ?? throw new InvalidOperationException()).OrderByDescending(y => y.CreateDate)
                .Skip(query.AmountAlreadyLoaded)
                .Take(query.CommentAmount).ToList();
            return Task.FromResult(result);
        }
    }
}