﻿using System;
using System.Collections.Generic;
using System.Linq;
using EventFlow.Queries;
using WebPortal.Application.Extentions;
using WebPortal.Application.Tickets.Models;

namespace WebPortal.Application.Tickets.Queries
{
    public class GetOverviewTicketsQuery : IQuery<PageResult<GetOverviewTicketsModel>>
    {
        /// <summary>
        /// </summary>
        /// <param name="dossierId"></param>
        /// <param name="userId"></param>
        /// <param name="startDateTime"></param>
        /// <param name="endDateTime"></param>
        /// <param name="unread"></param>
        /// <param name="page"></param>
        /// <param name="asc"></param>
        /// <param name="pageSize"></param>
        /// <param name="sort"></param>
        /// <param name="tagIds">Onderscheid verschillende tagIds door puntkomma</param>
        /// <param name="filterText">Mag niet null zijn.</param>
        public GetOverviewTicketsQuery(string dossierId,
            string userId,
            DateTime startDateTime,
            DateTime endDateTime,
            bool unread = true,
            int page = 1,
            bool asc = false,
            int pageSize = 20,
            string sort = "CreatedInCRM",
            string tagIds = null,
            string filterText = "")
        {
            Asc = asc;
            Sort = sort;
            Page = page;
            PageSize = pageSize;
            DossierId = dossierId;
            StartDateTime = startDateTime;
            EndDateTime = endDateTime;
            FilterText = filterText ?? "";
            UserId = userId;
            Unread = unread;
            TagIds = string.IsNullOrWhiteSpace(tagIds)
                ? new List<string>()
                : tagIds.Split(';').ToList();
        }

        public bool Asc { get; }
        public string Sort { get; }
        public int Page { get; }
        public int PageSize { get; }
        public string DossierId { get; }
        public DateTime StartDateTime { get; }
        public DateTime EndDateTime { get; }
        public string FilterText { get; }
        public string UserId { get; }
        public bool Unread { get; }

        public List<string> TagIds { get; }
    }
}