﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Ticket.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Extentions;
using WebPortal.Application.Tickets.Models;

namespace WebPortal.Application.Tickets.Queries
{
    public class
        GetOverviewTicketsQueryHandler : IQueryHandler<GetOverviewTicketsQuery, PageResult<GetOverviewTicketsModel>>
    {
        private readonly IMongoDbReadModelStore<TicketReadModel> _store;

        public GetOverviewTicketsQueryHandler(IMongoDbReadModelStore<TicketReadModel> store)
        {
            _store = store;
        }

        public Task<PageResult<GetOverviewTicketsModel>> ExecuteQueryAsync(GetOverviewTicketsQuery query,
            CancellationToken cancellationToken)
        {

            return Task.FromResult( _store.AsQueryable()
                .Where(x => !x.Deleted && x.DossierId == query.DossierId
                                       && x.CreatedInCRM >= query.StartDateTime
                                       && x.CreatedInCRM < query.EndDateTime)
                .Where(x => string.IsNullOrEmpty(query.FilterText) || x.TicketText.Contains(query.FilterText))
                .Where(x => query.TagIds.Count == 0 || x.Tags.Any(y => query.TagIds.Contains(y)))
                .Where(x => !query.Unread ||
                            x.TicketStatuses.Contains(query.UserId)
                )
                .Select(x => new GetOverviewTicketsModel
                {
                    TicketStatuses = x.TicketStatuses,
                    OriginalCCMMessageId = x.OriginalCCMMessageId,
                    Tags = x.Tags,
                    TicketText = x.TicketText,
                    CreatedCRMUser = x.CreatedCRMUser,
                    CreatedInCRM = x.CreatedInCRM, 
                    LastModifiedCRMUser = x.LastModifiedCRMUser,
                    LastModifiedInCRM = x.LastModifiedInCRM,
                    Id = x.Id,
                    DossierId = x.DossierId
                })
                
                .ToList()
                .Select(x => new GetOverviewTicketsModel
                {
                    OriginalCCMMessageId = x.OriginalCCMMessageId,
                    Tags = x.Tags,
                    IsRead = x.TicketStatuses.Any(y => y == query.UserId),
                    TicketText = x.TicketText,
                    CreatedCRMUser = x.CreatedCRMUser,
                    CreatedInCRM = x.CreatedInCRM,
                    LastModifiedCRMUser = x.LastModifiedCRMUser,
                    LastModifiedInCRM = x.LastModifiedInCRM,
                    Id = x.Id,
                    DossierId = x.DossierId
                })
                .AsQueryable()
                .OrderBy(query.Sort, query.Asc)
                .GetPage(query.Page, query.PageSize));
        }
    }
}