﻿using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Ticket.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Tickets.Models;

namespace WebPortal.Application.Tickets.Queries
{
    public class GetTicketQueryHandler: IQueryHandler<GetTicketQuery, GetTicketModel>
    {
        private readonly IMongoDbReadModelStore<TicketReadModel> _store;
        public GetTicketQueryHandler(IMongoDbReadModelStore<TicketReadModel> store)
        {
            _store = store;
        }
        public Task<GetTicketModel> ExecuteQueryAsync(GetTicketQuery query,
            CancellationToken cancellationToken)
        {
            var ticket =  _store.AsQueryable().FirstOrDefault(x => x.Id.Equals(query.Id));
            if(ticket == null)
                    throw new Exception($"No ticket found with id {query.Id}");


            var result = new GetTicketModel()
            {
                OriginalCCMMessageId = ticket.OriginalCCMMessageId,
                Tags = ticket.Tags,
                IsRead = ticket.TicketStatuses.Any(y => y == query.UserId),
                TicketText = ticket.TicketText,
                CreatedCRMUser = ticket.CreatedCRMUser,
                CreatedInCRM = ticket.CreatedInCRM,
                LastModifiedCRMUser = ticket.LastModifiedCRMUser,
                LastModifiedInCRM = ticket.LastModifiedInCRM,
                Id = ticket.Id,
                DossierId = ticket.DossierId
            };
            return Task.FromResult(result);
            
        }
    }
}