﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Ticket.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Tickets.Models;

namespace WebPortal.Application.Tickets.Queries
{
    public class GetTicketStatsQueryHandler : IQueryHandler<GetTicketStatsQuery, GetTicketStatsModel>
    {
        private readonly IMongoDbReadModelStore<TicketReadModel> _store;

        public GetTicketStatsQueryHandler(IMongoDbReadModelStore<TicketReadModel> store)
        {
            _store = store;
        }

        public Task<GetTicketStatsModel> ExecuteQueryAsync(GetTicketStatsQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(
                new GetTicketStatsModel
                {
                    TotalTickets = _store
                        .AsQueryable()
                        .Count(x => !x.Deleted && x.DossierId == query.DossierId),
                    Unread = _store.AsQueryable().Count(x =>
                        !x.Deleted && x.DossierId == query.DossierId && !x.TicketStatuses.Contains(query.UserId))
                });
        }
    }
}