﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Ticket.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Tickets.Models;

namespace WebPortal.Application.Tickets.Queries
{
    public class
        GetTicketWithCommentsQueryHandler : IQueryHandler<GetTicketWithCommentsQuery, GetTicketWithCommentsModel>
    {
        private readonly IMongoDbReadModelStore<TicketReadModel> _store;

        public GetTicketWithCommentsQueryHandler(IMongoDbReadModelStore<TicketReadModel> store
        )
        {
            _store = store;
        }

        public Task<GetTicketWithCommentsModel> ExecuteQueryAsync(GetTicketWithCommentsQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(
                _store.AsQueryable()
                    .Where(x => !x.Deleted && x.Id == query.Id)
                    .Select(x => new GetTicketWithCommentsModel
                    {
                        CreatedCRMUser = x.CreatedCRMUser,
                        CreatedInCRM = x.CreatedInCRM,
                        Id = x.Id,
                        LastModifiedCRMUser = x.LastModifiedCRMUser,
                        LastModifiedInCRM = x.LastModifiedInCRM,
                        OriginalCCMMessageId = x.OriginalCCMMessageId,
                        TicketStatuses = x.TicketStatuses,
                        TicketText = x.TicketText,
                        TotalComments = x.Comments.Count,
                        TagIds = x.Tags,
                        Comments = x.Comments,
                        DossierId = x.DossierId
                    }).ToList()
                    .Select(x => new GetTicketWithCommentsModel
                    {
                        CreatedCRMUser = x.CreatedCRMUser,
                        CreatedInCRM = x.CreatedInCRM,
                        Id = x.Id,
                        LastModifiedCRMUser = x.LastModifiedCRMUser,
                        LastModifiedInCRM = x.LastModifiedInCRM,
                        OriginalCCMMessageId = x.OriginalCCMMessageId,
                        IsRead = x.TicketStatuses.Any(y => y == query.UserId),
                        TicketText = x.TicketText,
                        TotalComments = x.TotalComments,
                        Comments = query.CommentOrderAscending
                            ? x.Comments.OrderBy(y => y.CreateDate).Take(query.CommentAmount).ToList()
                            : x.Comments.OrderByDescending(y => y.CreateDate).Take(query.CommentAmount).ToList(),
                        Tags = x.Tags,
                        DossierId = x.DossierId
                    })
                    .FirstOrDefault()
                );
        }
    }
}