﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Shared.Enums;
using CRM.Domain.Ticket.ReadModels;
using CRM.Domain.User;
using CsvHelper;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace WebPortal.Application.Tickets.Queries
{
    public class GetTicketsForExportQueryHandler : IQueryHandler<GetTicketsForExportQuery, MemoryStream>
    {
        private readonly IMongoDbReadModelStore<TicketReadModel> _ticketStore;

        public GetTicketsForExportQueryHandler(IMongoDbReadModelStore<TicketReadModel> _ticketStore)
        {
            this._ticketStore = _ticketStore;
        }

        public Task<MemoryStream> ExecuteQueryAsync(GetTicketsForExportQuery query,
            CancellationToken cancellationToken)
        {
            var tickets = _ticketStore.AsQueryable()
                .Where(x => x.Deleted == false)
                .Where(x => x.DossierId.Equals(query.DossierId))
                .Where(x => x.CreatedInCRM > query.StartDate && x.CreatedInCRM < query.EndDate)
                .Where(x => !query.IsRead || x.TicketStatuses.Contains(query.UserrId)).ToList();

            switch (Enum.Parse(typeof(FileEnum), query.FileType))
            {
                case FileEnum.csv:
                    return Task.FromResult(CreateCsv(tickets));

                case FileEnum.excel:
                    throw new NotImplementedException();
                default:
                    return Task.FromResult(CreateCsv(tickets));
            }
        }

        private MemoryStream CreateCsv(List<TicketReadModel> tickets)
        {
            var emps = tickets.Select(x => new
            {
                ticketText = x.TicketText,
                created = x.CreatedInCRM,
                createdBy = x.CreatedCRMUser
            });

            foreach (var emp in emps)
            {
                dynamic res = JObject.Parse(emp.ticketText);
            }
            string result;

            var stream = new MemoryStream();
            var writer = new StreamWriter(stream);
            var csv = new CsvWriter(writer, CultureInfo.InvariantCulture);

            csv.WriteRecords(emps);
            writer.Flush();
            return stream;
        }
    }
}