﻿using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.User;
using EventFlow.Aggregates.ExecutionResults;
using EventFlow.Commands;
using FirebaseAdmin.Auth;

namespace WebPortal.Application.Users.Commands
{
    public class CreateUserCommandHandler : CommandHandler<UserAggregate, UserId, IExecutionResult, CreateUserCommand>
    {
        private readonly FirebaseAuth _auth;

        public CreateUserCommandHandler(FirebaseAuth auth)
        {
            _auth = auth;
        }

        public override async Task<IExecutionResult> ExecuteCommandAsync(UserAggregate aggregate,
            CreateUserCommand command,
            CancellationToken cancellationToken)
        {
            var user = await _auth.CreateUserAsync(new UserRecordArgs
            {
                Uid = aggregate.Id.Value, Email = command.Email, DisplayName = command.UserName
            }, cancellationToken);
            
            var claims = new Dictionary<string, object>()
            {
                {"tenant", command.Tenant ?? ""}
            };

            await _auth.SetCustomUserClaimsAsync(aggregate.Id.Value, claims, cancellationToken);
            
            var result = aggregate.CreateUser(command.Email, command.UserName, command.CustomerId,
                command.IsCustomerAdmin, null, command.Tenant);

            if (!result.IsSuccess) await _auth.DeleteUserAsync(aggregate.Id.Value, cancellationToken);

            return result;
        }
    }
}