﻿using System;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.User;
using EventFlow.Aggregates.ExecutionResults;
using EventFlow.Commands;
using FirebaseAdmin.Auth;

namespace WebPortal.Application.Users.Commands
{
    public class ModifyEmailUsernameCommandHandler : CommandHandler<UserAggregate, UserId, ModifyEmailUsernameCommand>
    {
        private readonly FirebaseAuth _auth;

        public ModifyEmailUsernameCommandHandler(FirebaseAuth auth)
        {
            _auth = auth;
        }

        public override async Task ExecuteAsync(UserAggregate aggregate,
            ModifyEmailUsernameCommand command,
            CancellationToken cancellationToken)
        {
            var fireUser = await _auth.GetUserByEmailAsync(aggregate.Email, cancellationToken);

            if (fireUser != null)
            {
                var args = new UserRecordArgs()
                {
                    Uid = aggregate.Id.ToString(),
                    DisplayName = command.Username,
                    Email = command.Email
                };

                try
                {

                    await _auth.UpdateUserAsync(args, cancellationToken);
                }
                catch (Exception e)
                {
                    throw new Exception("Something went wrong with updating the user", e);
                }
                aggregate.Email = command.Email;
                aggregate.UserName = command.Username;
            }
        }
    }
}