﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Dossier.ReadModels;
using CRM.Domain.User;
using CRM.Domain.User.Entities;
using CRM.Domain.User.Models;
using EventFlow.Commands;
using EventFlow.MongoDB.ReadStores;

namespace WebPortal.Application.Users.Commands
{
    public class ModifyUserCommandHandler : CommandHandler<UserAggregate, UserId, ModifyUserCommand>
    {
        private readonly IMongoDbReadModelStore<DossierReadModel> _dossierStore;

        public ModifyUserCommandHandler(IMongoDbReadModelStore<DossierReadModel> dossierStore)
        {
            _dossierStore = dossierStore;
        }

        public override async Task ExecuteAsync(UserAggregate aggregate,
            ModifyUserCommand command,
            CancellationToken cancellationToken)
        {
            _ = command.Email.Equals(aggregate.Email) ? _ = command.Email : aggregate.Email = command.Email;
            _ = command.IsCustomerAdmin.Equals(aggregate.IsCustomerAdmin) ? _ = command.IsCustomerAdmin : aggregate.IsCustomerAdmin = command.IsCustomerAdmin;
            _ = command.CustomerId.Equals(aggregate.CustomerId) ? _ = command.CustomerId : aggregate.CustomerId = command.CustomerId;
            _ = command.EmployeeId == aggregate.EmployeeId ? _ = command.EmployeeId : aggregate.EmployeeId = command.EmployeeId;
            _ = command.UserName.Equals(aggregate.UserName) ? _ = command.UserName : aggregate.UserName = command.UserName;
            _ = command.Tenant != null && command.Tenant.Equals(aggregate.Tenant) ? _ = command.Tenant : aggregate.Tenant = command.Tenant;

            var customerRights = new Dictionary<string, ModuleRightModel>();
            foreach (var commandCustomerModuleRight in command.CustomerModuleRights)
            {
                customerRights.Add(commandCustomerModuleRight.Key, commandCustomerModuleRight.Value);
            }
            foreach (var right in command.DossierModuleRights)
            {
                if (right.Key.Equals("*")) break;

                var currentDossier = _dossierStore.AsQueryable().FirstOrDefault(x => x.Id == right.Key);

                if (currentDossier != null && !aggregate.CustomerModuleRights.ContainsKey(currentDossier.CustomerId) && !customerRights.ContainsKey(currentDossier.CustomerId))
                {
                    customerRights.Add(currentDossier.CustomerId, right.Value);
                }
            }

            // aggregate.CustomerModuleRights = customerRights.Distinct().ToDictionary(x => x.Key, x => x.Value);
            aggregate.CustomerModuleRights = command.CustomerModuleRights;
            aggregate.DossierModuleRights = command.DossierModuleRights;

            aggregate.CustomerGroupModuleRights = command.CustomerGroupModuleRights;
        }
    }
}