﻿using System;
using System.Globalization;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.User.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using FirebaseAdmin.Auth;

namespace WebPortal.Application.Users.Queries
{
    public class GetUserByEmailQueryHandler: IQueryHandler<GetUserByEmailQuery,bool>
    {
        private readonly IMongoDbReadModelStore<UserReadModel> _userStore;
        private readonly FirebaseAuth _fAuth;

        public GetUserByEmailQueryHandler(IMongoDbReadModelStore<UserReadModel> userStore, FirebaseAuth fAuth)
        {
            _userStore = userStore;
            _fAuth = fAuth;
        }
        public async Task<bool> ExecuteQueryAsync(GetUserByEmailQuery query,
            CancellationToken cancellationToken)
        {
            try
            {
                if (await _fAuth.GetUserByEmailAsync(query.Email.Trim(), cancellationToken) != null)
                    return true;

                var ourDb = _userStore.AsQueryable()
                    .Any(x => x.Email.Equals(query.Email));


                return ourDb;
            }
            catch (FirebaseAuthException e)
            {
                return false;
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                throw;
            }
      
        }
    }
}