﻿using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Customer.ReadModels;
using CRM.Domain.User.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Users.Models;

namespace WebPortal.Application.Users.Queries
{
    public class
        GetUserMgmtDashboardDataQueryHandler : IQueryHandler<GetUserMgmtDashboardDataQuery, GetUserMgmtDashboardData>
    {
        private readonly IMongoDbReadModelStore<CustomerReadModel> _customerStore;
        private readonly IMongoDbReadModelStore<UserReadModel> _userStore;

        public GetUserMgmtDashboardDataQueryHandler(IMongoDbReadModelStore<UserReadModel> userStore,
            IMongoDbReadModelStore<CustomerReadModel> customerStore)
        {
            _userStore = userStore;
            _customerStore = customerStore;
        }

        public Task<GetUserMgmtDashboardData> ExecuteQueryAsync(GetUserMgmtDashboardDataQuery query,
            CancellationToken cancellationToken)
        {
            var customer = _customerStore.AsQueryable().FirstOrDefault(x => x.Id.Equals(query.CustomerId)) ??
                           throw new Exception("No customer found");
            var totalUsers = _userStore.AsQueryable().Count(x => x.CustomerId.Equals(query.CustomerId));
            var result = new GetUserMgmtDashboardData
            {
                AvailableLicenses = customer.WebPortalLicenseAmount - totalUsers,
                TotalUsers = totalUsers
            };
            return Task.FromResult(result);
        }
    }
}