﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Xml.Schema;
using CRM.Domain.User.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Users.Models;

namespace WebPortal.Application.Users.Queries
{
    public class GetUsernameQueryHandler : IQueryHandler<GetUsernameQuery, GetUsernameModel>
    {
        private readonly IMongoDbReadModelStore<UserReadModel> _userStore;

        public GetUsernameQueryHandler(IMongoDbReadModelStore<UserReadModel> userStore)
        {
            _userStore = userStore;
        }

        public Task<GetUsernameModel> ExecuteQueryAsync(GetUsernameQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(Get(query));
        }

        private GetUsernameModel Get(GetUsernameQuery query)
        {
            var user = _userStore.AsQueryable().FirstOrDefault(x => x.Id.Equals(query.UserId));

            GetUsernameModel model;
            _ = user == null ? model = new GetUsernameModel
                {
                    Username = string.IsNullOrEmpty(user.UserName) ? user.UserName : user.Email
                } : model = new GetUsernameModel
                {
                    Username = "Anonymous"
                };

            return model;
        }
    }
}