﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.User.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.Application.Extentions;
using WebPortal.Application.Users.Models;

namespace WebPortal.Application.Users.Queries
{
    public class
        GetUsersByCustomerIdQueryHandler : IQueryHandler<GetUsersByCustomerIdQuery, PageResult<GetUserListModel>>
    {
        private readonly IMongoDbReadModelStore<UserReadModel> _store;

        public GetUsersByCustomerIdQueryHandler(IMongoDbReadModelStore<UserReadModel> store)
        {
            _store = store;
        }

        public Task<PageResult<GetUserListModel>> ExecuteQueryAsync(GetUsersByCustomerIdQuery query,
            CancellationToken cancellationToken)
        {
            return Task.FromResult(
                _store.AsQueryable()
                    .Where(x => !x.Deleted)
                    .Where(x => string.IsNullOrEmpty(query.CustomerId) || x.CustomerId == query.CustomerId)
                    .Where(x => string.IsNullOrEmpty(query.Filter) || x.Email.ToLower().Contains(query.Filter.ToLower()))
                    .OrderBy(query.Sort, query.Asc)
                    .Select(x =>
                        new GetUserListModel
                        {
                            CustomerId = x.CustomerId,
                            Email = x.Email,
                            IsCustomerAdmin = x.IsCustomerAdmin,
                            UserName = x.UserName,
                            Id = x.Id,
                            Tenant = x.Tenant
                        })
                    .GetPage(query.Page, query.PageSize));
        }
    }
}