﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.CallScript.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.CallScripts.Models;

namespace WebPortal.CCMConnector.Application.CallScripts.Queries
{
    public class GetDossierIdsAndTagsQueryHandler: IQueryHandler<GetDossierIdsAndTagsQuery, List<GetCallScriptIdModel>>
    {
        public GetDossierIdsAndTagsQueryHandler(IMongoDbReadModelStore<CallScriptReadModel> store)
        {
            _store = store;
        }

        private readonly IMongoDbReadModelStore<CallScriptReadModel> _store;
        public Task<List<GetCallScriptIdModel>> ExecuteQueryAsync(GetDossierIdsAndTagsQuery query,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
                _store.AsQueryable()
                    .Where(x => !x.Deleted
                                && query.DossierIds.Contains(x.DossierId))
                    .Select(x => new GetCallScriptIdModel
                    {
                        DossierId = x.DossierId,
                        Tags = x.Tags,
                        Id = x.Id
                    })
                    .ToList()
            );
        }
    }
}
