﻿using System;

namespace WebPortal.CCMConnector.Application.Customers.Models
{
    public class DutyAlarmModel
    {
        public int Id { get; set; }
        public string DossierId { get; set; }
        public bool SendMail { get; set; }
        public bool SendSms { get; set; }
        public DateTime WarningTime { get; set; }
        public string EmailAddresses { get; set; }
        public string SmsNumbers { get; set; }
        public string Category { get; set; }
        public DateTime? LastRun { get; set; }

        public  string Tenant { get; set; }
        public string CustomerName { get; set; }
        public string DossierName { get; set; }
        public string CustomerId { get; set; }

        public DateTime? NextRun { get; set; }
        public bool UseWeekDay { get; set; }
        public bool UseMonthDay { get; set; }
        public bool UseEveryDay { get; set; }
        public bool OnFixedDate { get; set; }



        public TimeSpan TimeWarning
        {
            get
            {
                DateTime dateToSubstract = new DateTime(2000, 1, 1);
                return WarningTime - dateToSubstract;
            }
        }

        public bool MustRun
        {
            get
            {
                if (OnFixedDate)
                {
                    if (!NextRun.HasValue) return false;

                    if (DateTime.UtcNow > NextRun && DateTime.UtcNow < NextRun.Value.AddMinutes(1))
                        return true;
                }
                if (!LastRun.HasValue) return true;

                DateTime dateToSubstract = new DateTime(2000, 1, 1);
                var time = WarningTime - dateToSubstract;

                var time2 = DateTime.UtcNow - LastRun.Value;

                return time < time2;
            }
        }

        public bool MustRun2(DateTime previousRun)
        {
            if (OnFixedDate)
            {
                if (!NextRun.HasValue) return false;

                var timeDif = DateTime.UtcNow - previousRun;
                if (previousRun < NextRun && previousRun.Add(timeDif) >= NextRun.Value)
                    return true;
                return false;
            }
            if (!LastRun.HasValue) return true;

            DateTime dateToSubstract = new DateTime(2000, 1, 1);
            var time = WarningTime - dateToSubstract;

            var time2 = DateTime.UtcNow - LastRun.Value;

            return time < time2;

        }

        public DateTime ExpireDateTime
        {
            get
            {
                if (OnFixedDate)
                {
                    return DateTime.Now.AddMinutes(10);
                }
                DateTime dateToSubstract = new DateTime(2000, 1, 1);
                var time = WarningTime - dateToSubstract;
                return DateTime.Now.Add(time);
            }
        }

        public void SetNextRun()
        {
            if (!OnFixedDate)
            {
                NextRun = null;
                return;
            }

            if (UseEveryDay)
            {
                var temp = DateTime.Now.Date
                    .AddHours(WarningTime.Hour)
                    .AddMinutes(WarningTime.Minute)
                    .ToUniversalTime();
                NextRun = DateTime.UtcNow > temp
                    ? temp.AddDays(1)
                    : temp;
            }
            else if (UseWeekDay)
            {
                var weekday = WarningTime.DayOfWeek;
                var daysToGo = weekday >= DateTime.Now.DayOfWeek
                    ? weekday - DateTime.Now.DayOfWeek
                    : (weekday + 7) - DateTime.Now.DayOfWeek;
                var temp = DateTime.Now.Date
                    .AddDays(daysToGo)
                    .AddHours(WarningTime.Hour)
                    .AddMinutes(WarningTime.Minute).ToUniversalTime();

                NextRun = DateTime.UtcNow > temp
                    ? temp.AddDays(7)
                    : temp;

            }
            else
            {
                var temp = new DateTime(DateTime.Now.Year, DateTime.Now.Month, WarningTime.Day, WarningTime.Hour, WarningTime.Minute, 0)
                    .ToUniversalTime();
                NextRun = DateTime.UtcNow > temp
                    ? temp.AddMonths(1)
                    : temp;
            }
        }


        public void CompleteTheDutyAlarm(DutyAlarmModel da)
        {
            Id = da.Id;
            SendMail = da.SendMail;
            SendSms = da.SendSms;
            WarningTime = da.WarningTime;
            EmailAddresses = da.EmailAddresses;
            SmsNumbers = da.SmsNumbers;
            Category = da.Category;
            LastRun = da.LastRun;

            NextRun = da.NextRun;
            UseWeekDay = da.UseWeekDay;
            UseMonthDay = da.UseMonthDay;
            UseEveryDay = da.UseEveryDay;
            OnFixedDate = da.OnFixedDate;
        }
}
}
