﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Customer.ReadModels;
using CRM.Domain.Dossier.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.Customers.Models;

namespace WebPortal.CCMConnector.Application.Customers.Queries
{
    public class GetCustomerNameTenantAndDossierNameQueryHandler: IQueryHandler<GetCustomerNameTenantAndDossierNameQuery, DutyAlarmModel>
    {
        public GetCustomerNameTenantAndDossierNameQueryHandler(IMongoDbReadModelStore<DossierReadModel> dossierStore,
            IMongoDbReadModelStore<CustomerReadModel> customerStore)
        {
            _dossierStore = dossierStore;
            _customerStore = customerStore;
        }

        private readonly IMongoDbReadModelStore<DossierReadModel> _dossierStore;
        private readonly IMongoDbReadModelStore<CustomerReadModel> _customerStore;

        public Task<DutyAlarmModel> ExecuteQueryAsync(GetCustomerNameTenantAndDossierNameQuery query,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
            {
                var temp = _dossierStore.AsQueryable()
                    .Where(x => x.Id == query.DossierId)
                    .Select(x => new DutyAlarmModel()
                    {
                        DossierId = x.Id,
                        DossierName = x.DossierName,
                        CustomerId = x.CustomerId
                    })
                    .FirstOrDefault();

                var rtn = _customerStore.AsQueryable()
                    .Where(x => x.Id == temp.CustomerId)
                    .Select(x => new DutyAlarmModel()
                    {
                        DossierId = temp.DossierId,
                        DossierName = temp.DossierName,
                        CustomerId = temp.CustomerId,
                        CustomerName = x.CustomerName,
                        Tenant = x.Tenant
                    })
                    .FirstOrDefault();

                return rtn;
            }, cancellationToken);

        }
    }
}



