﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Customer.ReadModels;
using CRM.Domain.Dossier.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.Customers.Models;
using WebPortal.CCMConnector.Application.Dossiers.Models;

namespace WebPortal.CCMConnector.Application.Customers.Queries
{
    public class GetCustomersWithDossiersQueryHandler : IQueryHandler<GetCustomersWithDossiersQuery, List<GetCustomerWithDossiersModel>>
    {
        private readonly IMongoDbReadModelStore<CustomerReadModel> _customerStore;
        private readonly IMongoDbReadModelStore<DossierReadModel> _dossierStore;

        public GetCustomersWithDossiersQueryHandler(IMongoDbReadModelStore<CustomerReadModel> customerStore, IMongoDbReadModelStore<DossierReadModel> dossierStore)
        {
            _customerStore = customerStore;
            _dossierStore = dossierStore;
        }

        public Task<List<GetCustomerWithDossiersModel>> ExecuteQueryAsync(GetCustomersWithDossiersQuery query,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
            _customerStore.AsQueryable()
            .Where(x => !x.Deleted && query.Ids.Contains(x.Id))
            .Select(x => new GetCustomerWithDossiersModel
            {
                Id = x.Id,
                CustomerName = x.CustomerName,
                MaxUsers = x.WebPortalLicenseAmount,


            })
            .ToList()
                .Select(x => new GetCustomerWithDossiersModel
                {
                    Id = x.Id,
                    CustomerName = x.CustomerName,
                    MaxUsers = x.MaxUsers,
                    Dossiers = _dossierStore.AsQueryable()
                    .Where(y => y.CustomerId == x.Id)
                    .Select(y => new GetDossierModel
                    {
                        Id = y.Id,
                        CCMId = y.CCMId,
                        DossierName = y.DossierName
                    })
                    .ToList()
                })
                .ToList());
        }
    }
}
