﻿using System.Collections.Generic;
using System.Linq;
using CRM.Domain.Dossier.ReadModels;
using EventFlow.Queries;
using System.Threading;
using System.Threading.Tasks;
using EventFlow.MongoDB.ReadStores;

namespace WebPortal.Application.Dossiers.Queries
{
    public class GetDossierIdsByCCMIdsQueryHandler: IQueryHandler<GetDossierIdsByCCMIdsQuery, Dictionary<string, int>>
    {
        public GetDossierIdsByCCMIdsQueryHandler(IMongoDbReadModelStore<DossierReadModel> store)
        {
            _store = store;
        }

        private readonly IMongoDbReadModelStore<DossierReadModel> _store;

        public Task<Dictionary<string, int>> ExecuteQueryAsync(GetDossierIdsByCCMIdsQuery query,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
                _store.AsQueryable()
                    .Where(x => query.CCMIds.Contains(x.CCMId.Value))
                    .Select(x => new {x.Id, x.CCMId.Value})
                    .ToDictionary(x => x.Id, x => x.Value));
        }
    }
}
