﻿using System.Collections.Generic;
using CRM.Domain.Duty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using WebPortal.CCMConnector.Application.Duties.Models;

namespace WebPortal.CCMConnector.Application.Duties.Queries
{
    public class GetDutiesQueryHandler : IQueryHandler<GetDutiesQuery, List<GetOverviewDutyModel>>
    {
        private readonly IMongoDbReadModelStore<DutyReadModel> _dutyStore;

        public GetDutiesQueryHandler( IMongoDbReadModelStore<DutyReadModel> dutyStore)
        {
            _dutyStore = dutyStore;
        }

        public Task<List<GetOverviewDutyModel>> ExecuteQueryAsync(GetDutiesQuery query, CancellationToken cancellationToken)
        {
            return Task.Run(() =>
            _dutyStore.AsQueryable()
            .Where(x => !x.Deleted && x.DossierId == query.DossierId
            && x.StartDate >= query.StartDateTime
            && x.StartDate < query.EndDateTime
            && x.EndDate >= query.StartDateTime
            && x.EndDate < query.EndDateTime)
                       .OrderBy(x => x.StartDate)
            .ToList()
            .Select(x => new GetOverviewDutyModel
            {
                Id = x.Id,
                DossierId = x.DossierId,
                ActionDescription = x.ActionDescription,
                Category = x.Category,
                EndDate = x.EndDate,
                StartDate = x.StartDate,
                StartAt = x.StartAt,
                EndAt = x.EndAt,
                DutyStatus = x.DutyStatus,
                CompleteStartDate = x.CompleteStartDate,
                CompleteEndDate = x.CompleteEndDate
            })
           .AsQueryable()
           .ToList(), cancellationToken);
        }
    }
}