﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Duty.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.Duties.Models;

namespace WebPortal.CCMConnector.Application.Duties.Queries
{
    public class GetDutiesToExpireQueryHandler: IQueryHandler<GetDutiesToExpireQuery, List<GetDutyToExpireModel>>
    {
        public GetDutiesToExpireQueryHandler(IMongoDbReadModelStore<DutyReadModel> dutyStore)
        {
            _dutyStore = dutyStore;
        }

        private readonly IMongoDbReadModelStore<DutyReadModel> _dutyStore;

        public Task<List<GetDutyToExpireModel>> ExecuteQueryAsync(GetDutiesToExpireQuery query,
            CancellationToken cancellationToken)
        {
            return Task.Run(() =>
            {
                var expiredDate = query.MinimumExpired.Date;
                var expiredAndOneDay = expiredDate.AddDays(1);
                var expiredMinusOneMonth = expiredDate.AddDays(-31);
                var today = DateTime.Now.Date;
                var rtn = _dutyStore.AsQueryable()
                    .Where(x => !x.Deleted
                                && x.DossierId == query.DossierId
                                && x.EndDate > today
                                && (query.OnFixedDate || (x.EndDate <= expiredAndOneDay || x.StartDate >= expiredMinusOneMonth))
                    )
                    .Select(x => new GetDutyToExpireModel()
                    {
                        Category = x.Category,
                        DossierId = x.DossierId,
                        EndDate = x.EndDate,
                        EndAt = x.EndAt,
                        StartDate = x.StartDate,
                        StartAt = x.StartAt,
                        AmountOfEmployees = x.DutyEmployees.Count
                    })
                    .ToList();
                return rtn;
            }
               , cancellationToken);
        }
    }
}
