﻿using System;
using System.Linq;
using System.Linq.Expressions;

namespace WebPortal.Application.Extentions
{
    public static class OrderByString
    {
        public static IQueryable<T> OrderBy<T>(this IQueryable<T> source,
            string property,
            bool asc = true) where T : class
        {
            var searchProperty = typeof(T).GetProperty(property);

            if (searchProperty == null)
                throw new ArgumentException("property");

            if (searchProperty.PropertyType.Namespace != null && !searchProperty.PropertyType.IsValueType &&
                !searchProperty.PropertyType.IsPrimitive &&
                !searchProperty.PropertyType.Namespace.StartsWith("System") && !searchProperty.PropertyType.IsEnum)
                throw new ArgumentException("property");

            if (searchProperty.GetMethod == null ||
                !searchProperty.GetMethod.IsPublic)
                throw new ArgumentException("property");

            var parameter = Expression.Parameter(typeof(T), "o");
            var selectorExpr = Expression.Lambda(
                Expression.Property(parameter, property), parameter);

            var queryExpr = source.Expression;
            queryExpr = Expression.Call(
                typeof(Queryable),
                asc ? "OrderBy" : "OrderByDescending",
                new[]
                {
                    source.ElementType,
                    searchProperty.PropertyType
                },
                queryExpr,
                selectorExpr);

            return source.Provider.CreateQuery<T>(queryExpr);
        }
    }
}