﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.InteractionStatistic.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;

namespace WebPortal.CCMConnector.Application.InteractionStats.Queries
{
    public class GetStatisticsWithMessageIdCcmQueryHandler: IQueryHandler<GetStatisticsWithMessageIdCcmQuery, Dictionary<string, int>>
    {
        private readonly IMongoDbReadModelStore<InteractionStatisticReadModel> _store;

        public GetStatisticsWithMessageIdCcmQueryHandler(IMongoDbReadModelStore<InteractionStatisticReadModel> store)
        {
            _store = store;
        }

        public Task<Dictionary<string, int>> ExecuteQueryAsync(GetStatisticsWithMessageIdCcmQuery query,
            CancellationToken cancellationToken)
        {
            var rtn = _store.AsQueryable()
                .Where(x =>
                    !x.Deleted
                    && x.TicketId == null
                    && query.MessageIdsCcm.Contains(x.OriginalCcmMessageId.Value))
                .Select(x => new
                {
                    x.OriginalCcmMessageId.Value,
                    x.Id
                })
                .ToDictionary(k => k.Id, v => v.Value);
            return Task.FromResult(rtn);
        }
    }
}
