﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Ticket.ReadModels;
using EventFlow.MongoDB.ReadStores;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.Tickets.Models;

namespace WebPortal.CCMConnector.Application.Tickets.Queries
{
    public class GetFaultyTicketsQueryHandler : IQueryHandler<GetFaultyTicketsQuery, List<GetFaultyTicketModel>>
    {
        public GetFaultyTicketsQueryHandler(IMongoDbReadModelStore<TicketReadModel> store)
        {
            _store = store;
        }

        private readonly IMongoDbReadModelStore<TicketReadModel> _store;
        public Task<List<GetFaultyTicketModel>> ExecuteQueryAsync(GetFaultyTicketsQuery query, CancellationToken cancellationToken)
        {
            return Task.Run(() =>
            {
                var multipleCcmIds = _store.AsQueryable()
                    .Where(x => !x.Deleted 
                                && x.OriginalCCMMessageId.HasValue)
                    .GroupBy(x => x.OriginalCCMMessageId)
                    .Select(x => new
                    {
                        CcmId = x.Key.Value,
                        Counter = x.Count()
                    })
                    .Where(x => x.Counter > 1)
                    .Select(x => x.CcmId)
                    .ToList();

                var rtn = new List<GetFaultyTicketModel>();
                rtn = _store.AsQueryable()
                    .Where(x => !x.Deleted
                                && multipleCcmIds.Contains(x.OriginalCCMMessageId.Value))
                    .Select(x => new GetFaultyTicketModel()
                    {
                        CcmMessageId = x.OriginalCCMMessageId.Value,
                        LastModified = x.LastModifiedInCRM,
                        TicketId = x.Id
                    })
                    .OrderBy(x => x.CcmMessageId)
                    .ToList();
                return rtn;
            });
        }
    }
}
