﻿using System;
using CRM.Domain.Branch;
using CRM.Domain.Branch.ReadModels;
using CRM.Domain.CallScript;
using EventFlow;
using EventFlow.Configuration;
using EventFlow.Extensions;
using EventFlow.MongoDB.Extensions;
using CRM.Domain.Ticket.ReadModels;
using CRM.Domain.Customer;
using CRM.Domain.CallScript.ReadModels;
using CRM.Domain.Customer.ReadModels;
using CRM.Domain.Dossier;
using CRM.Domain.Dossier.ReadModels;
using CRM.Domain.Duty;
using CRM.Domain.Duty.ReadModels;
using CRM.Domain.Employee;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.InteractionStatistic;
using CRM.Domain.InteractionStatistic.ReadModels;
using CRM.Domain.Ticket;
using WebPortal.Application.Dossiers.Queries;
using WebPortal.Application.Duties.Commands;
using WebPortal.CCMConnector.Application.Branches.Commands;
using WebPortal.CCMConnector.Application.CallScripts.Commands;
using WebPortal.CCMConnector.Application.CallScripts.Queries;
using WebPortal.CCMConnector.Application.Customers.Commands;
using WebPortal.CCMConnector.Application.Customers.Queries;
using WebPortal.CCMConnector.Application.InteractionStats.Commands;
using WebPortal.CCMConnector.Application.Tickets.Commands;
using WebPortal.CCMConnector.Application.Dossiers.Commands;
using WebPortal.CCMConnector.Application.Dossiers.Queries;
using WebPortal.CCMConnector.Application.Duties.Commands;
using WebPortal.CCMConnector.Application.Duties.Queries;
using WebPortal.CCMConnector.Application.Employees.Commands;
using WebPortal.CCMConnector.Application.InteractionStats.Queries;
using WebPortal.CCMConnector.Application.Tickets.Queries;

namespace WebPortal.CCMConnector.ApplicationTest.Configuration
{
    public class ConfigureServices
    {
        public static IRootResolver ConfigureServicesWithHandlersMC(string connectionString, string databaseName)


        {


            return EventFlowOptions.New
                //.AddDefaults(typeof(CreateCustomerCommand).Assembly)
                .AddDefaults(typeof(CustomerAggregate).Assembly)
                .AddDefaults(typeof(BranchAggregate).Assembly)
                .AddDefaults(typeof(DossierAggregate).Assembly)
                .AddDefaults(typeof(InteractionStatisticAggregate).Assembly)
                .AddDefaults(typeof(CallScriptAggregate).Assembly)
                .AddDefaults(typeof(TicketAggregate).Assembly)
                .AddDefaults(typeof(EmployeeAggregate).Assembly)
                .AddDefaults(typeof(DutyAggregate).Assembly)
                .AddCommands(new Type[]
                {
                        typeof(CreateCustomerCommand),
                        typeof(ModifyCustomerMainCustomerBranchCommand),
                        typeof(CreateInteractionStatCommand),
                        typeof(CreateTicketCommand),
                        typeof(CreateDossierCommand),
                        typeof(CreateBranchCommand),
                        typeof(ModifyTicketCommand),
                        typeof(CreateEmployeeCommand),
                        typeof(CreateCallScriptCommand),
                        typeof(ModifyCallScriptCommand),
                        typeof(ModifyDossierNameCommand),
                        typeof(ModifyCustomerNameCommand),
                        typeof(ModifiyCustomerWebPortalLicenseAmountCommand),
                        typeof(ModifyCustomerSubscriptionsCommand),
                        typeof(DeleteDossierCommand),
                        typeof(DeleteTicketCommand),
                        typeof(DeleteInteractionStatCommand),
                        typeof(RemoveCallScriptCommand),
                        typeof(DeleteDutyCommand),
                        typeof(ModifyDutyDossierIdCommand),
                        typeof(ModifyTicketDossierIdCommand),
                        typeof(ModifyTicketDossierIdCommand),
                        typeof(AddDutyCommand),
                        typeof(RemoveAllEmployeesFromDutyCommand),
                        typeof(ModifyDutyEmployeesCommand)
                })
                .AddCommandHandlers(new Type[]
                {
                        typeof(CreateCustomerCommandHandler),
                        typeof(ModifyCustomerMainCustomerBranchCommandHandler),
                        typeof(CreateInteractionStatCommandHandler),
                        typeof(CreateTicketCommandHandler),
                        typeof(CreateDossierCommandHandler),
                        typeof(CreateBranchCommandHandler),
                        typeof(ModifyTicketCommandHandler),
                        typeof(CreateEmployeeCommandHandler),
                        typeof(CreateCallScriptCommandHandler),
                        typeof(ModifyCallScriptCommandHandler),
                        typeof(ModifyDossierNameCommandHandler),
                        typeof(ModifyCustomerNameCommandHandler),
                        typeof(ModifiyCustomerWebPortalLicenseAmountCommandHandler),
                        typeof(ModifyCustomerSubscriptionsCommandHandler),
                        typeof(DeleteDossierCommandHandler),
                        typeof(DeleteTicketCommandHandler),
                        typeof(DeleteInteractionStatCommandHandler),
                        typeof(RemoveCallScriptCommandHandler),
                        typeof(DeleteDutyCommandHandler),
                        typeof(ModifyDutyDossierIdCommandHandler),
                        typeof(ModifyTicketDossierIdCommandHandler),
                        typeof(ModifyTicketDossierIdCommandHandler),
                        typeof(AddDutyCommandHandler),
                        typeof(RemoveAllEmployeesFromDutyCommandHandler),
                        typeof(ModifyDutyEmployeesCommandHandler)

                })
                .AddQueryHandlers(new Type[]
                {
                        typeof(GetDossierIdsByCCMIdsQueryHandler),
                        typeof(GetTicketIdsByCCMMessageIdsQueryHandler),
                        typeof(GetDossierIdsAndTagsQueryHandler),
                        typeof(GetCustomersWithDossiersQueryHandler),

                        typeof(GetCallScriptIdsByDossierIdQueryHandler),
                        typeof(GetTicketIdsByDossierIdQueryHandler),
                        typeof(GetDutyIdsByDossierIdQueryHandler),
                        typeof(GetDutiesQueryHandler),
                        typeof(GetDutiesToExpireQueryHandler),
                        typeof(GetStatisticIdsByDossierIdQueryHandler),
                        typeof(GetFaultyDossiersQueryHandler),
                        typeof(GetFaultyTicketsQueryHandler)

                })


                .ConfigureMongoDb(connectionString, databaseName)
                .UseMongoDbEventStore()
                .UseMongoDbSnapshotStore()
                .UseMongoDbReadModel<CallScriptReadModel>()
                .UseMongoDbReadModel<CustomerReadModel>()
                .UseMongoDbReadModel<DossierReadModel>()
                .UseMongoDbReadModel<InteractionStatisticReadModel>()
                .UseMongoDbReadModel<TicketReadModel>()
                .UseMongoDbReadModel<BranchReadModel>()
                .UseMongoDbReadModel<EmployeeReadModel>()
                .UseMongoDbReadModel<DutyReadModel>()

                .CreateResolver();
        }


    }

}

