﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Duty;
using CRM.Domain.Duty.Entities;
using CRM.Domain.Duty.Models;
using EventFlow;
using EventFlow.Queries;
using WebPortal.Application.Duties.Commands;
using WebPortal.CCMConnector.Application.Duties.Commands;
using WebPortal.CCMConnector.Application.Duties.Queries;
using WebPortal.CCMConnector.ApplicationTest.Configuration;
using Xunit;

namespace WebPortal.CCMConnector.ApplicationTest
{
    public class DutyTest
    {
        string conString = "mongodb://connector:connector123456789@192.168.121.11";
        //string dbName = "wp_dev";
        //string dbName = "wp_staging";
        //string conString = "mongodb://localhost";
        //string conString = "mongodb://aac-dev01.cisicc.local";
        //string dbName = "WPMongoTest";
        string dbName = "wp_prod";



        [Fact]
        public async Task GetDutiesTest()
        {
            var startDate = DateTime.UtcNow.AddDays(-30);
            var endDate = DateTime.UtcNow.AddDays(365);
            var dossierId = "dossier-56d080ef-362f-4105-a7d4-42603ef9a50b";
            dossierId = "dossier-3bb7edb5-f532-4342-939e-d1705bcd5a58";

            var services = ConfigureServices.ConfigureServicesWithHandlersMC(conString, dbName);

            //var commandBus = services.Resolve<ICommandBus>();
            var queryProcessor = services.Resolve<IQueryProcessor>();

            var q = new GetDutiesQuery(dossierId, startDate, endDate);

            var duties = await queryProcessor.ProcessAsync(q, new CancellationToken());

            Assert.True(duties.Count > 0);
        }

        [Fact]
        public async Task MakeNewDutyTest()
        {
            //var in5minutes = DateTime.Now.AddMinutes(5);
            //var sundDay = DateTime.Now.AddDays(2).AddMinutes(10);

            //var services = ConfigureServices.ConfigureServicesWithHandlersMC(conString, dbName);

            //var commandBus = services.Resolve<ICommandBus>();
            //var c = new AddDutyCommand(DutyId.New,
            //    dossierId,
            //    "Vijf minuten",
            //    in5minutes.Date.AddHours(12),
            //    sundDay.Date.AddHours(12),
            //    "Testing the duties upcoming en planned",
            //    in5minutes.ToString("HH:mm"),
            //    sundDay.ToString("HH:mm"));
            //var result = await commandBus.PublishAsync(c, new CancellationToken());
            //Assert.True(result.IsSuccess);

            var dossierId = "dossier-07d05eca-3fa0-4228-9834-2db6a3a8195e"; //Gilead


            var today = DateTime.Now.Date.AddHours(10);

            DateTime[] startDates = {today.AddDays(-1), today, today.AddDays(1)};
            DateTime[] endDates = {today.AddDays(-1), today, today.AddDays(1), today.AddDays(2)};

            var theTime = DateTime.Now.Date;

            DateTime[] startTimes = {theTime.AddHours(8), theTime.AddHours(22)};
            DateTime[] endTimes = {theTime.AddHours(17), theTime.AddHours(6)};

            string category = "";
            int i = 1;
            for (int sd = 0; sd < startDates.Length; sd++)
            {
                for (int ed = 0; ed < endDates.Length; ed++)
                {
                    for (int st = 0; st < startTimes.Length; st++)
                    {
                        for (int et = 0; et < endTimes.Length; et++)
                        {
                            if (endDates[ed] < startDates[sd]) continue;
                            if ((endDates[ed] == startDates[sd])&& endTimes[et] < startTimes[st]) continue;

                            category = "";
                            switch (sd)
                            {
                                case 0: category += "Gisteren "; break;
                                case 1: category += "Vandaag "; break;
                                case 2: category += "Morgen "; break;
                            }
                            
                            category += startTimes[st] < endTimes[et]? "Dag": "Nacht";

                            await MakeOneDuty(startDates[sd], endDates[ed], 
                                startTimes[st].ToString("HH:mm"),
                                endTimes[et].ToString("HH:mm"),
                                category, dossierId, i);
                            if (i == 3)
                                i = 1;
                            else
                            {
                                i++;
                            }
                        }
                    }
                }
            }

        }

        private async  Task MakeOneDuty (DateTime startDate,
            DateTime endDate,
            string startat,
            string endat,
            string category,
            string dossierId, int i)
        {
            var services = ConfigureServices.ConfigureServicesWithHandlersMC(conString, dbName);

            var commandBus = services.Resolve<ICommandBus>();
            var dutyId = DutyId.New;
            var c = new AddDutyCommand(dutyId,
                dossierId,
                category,
                startDate,
                endDate,
                "Test",
                startat,
                endat);
            await commandBus.PublishAsync(c, new CancellationToken());
            await AddEmployeeToDuty(dutyId, commandBus, i);


        }

        private async Task AddEmployeeToDuty(DutyId dutyId, ICommandBus commandBus, int i)
        {
            DutyContactPointId contactId = DutyContactPointId.New;
            DutyContactId dutyContactId = DutyContactId.New;

            //Claire Albano

            ContactPointModel contact1 = new ContactPointModel(contactId.Value,
                "employeecontactdetail-185fa2c7-2263-429a-b0fe-6a019c03f5a9", "Cell Phone");

            DutyEmployeeModel employee1 = new DutyEmployeeModel(dutyContactId.Value, null, 1,
                "employee-14738e1b-9e07-4bff-bab3-85a6507f5fe2", new List<ContactPointModel>(){contact1});

            contactId = DutyContactPointId.New;
            dutyContactId = DutyContactId.New;

            //Naziha Ameziani
            var contact2 = new ContactPointModel(contactId.Value, "employeecontactdetail-21e067fd-5fbc-4164-9ad3-25a3a7fde367", "Office Phone");

            var employee2 = new DutyEmployeeModel(dutyContactId.Value, null, i == 3 ? 2: 1, "employee-ece551bf-9d2a-4137-ae6c-e6d987ae9be4", new List<ContactPointModel>() { contact2 });


            var employees = i == 1
                ? new List<DutyEmployeeModel>() {employee1}
                : i == 2
                    ? new List<DutyEmployeeModel>() {employee2}
                    : new List<DutyEmployeeModel>() {employee1, employee2}; 

            var c = new ModifyDutyEmployeesCommand(dutyId, employees);
            await commandBus.PublishAsync(c, new CancellationToken());

        }
    }
}
