﻿using System;
using WebPortal.CCMConnector.ConfigConsole.Events;
using WebPortal.CCMConnector.ConfigConsole.UI;

namespace WebPortal.CCMConnector.ConfigConsole.Main
{
    public class CommandCenter
    {
        private RepairCommands _repairCommand;
        private ReplaceCommands _replaceCommand;

        public CommandCenter()
        {
            _repairCommand = new RepairCommands();
            _repairCommand.Completed += Completed;
            _repairCommand.AksToConfirm += ConfirmAction;
        }

        private void Completed(object sender,
            EventArgs e)
        {
            ExecuteCommand();
        }

        internal  void ExecuteCommand()
        {
            WriteText.WriteInstruction("Awaiting your input. Type '/h' to see a list of valid commands.");
            string input = Console.ReadLine();
            string[] commandString = input.Split();
            switch (commandString[0].ToUpper())
            {
                case "REPAIR": _repairCommand.Repair(input); break;
                case "REPLACE":
                    if (_replaceCommand == null)
                    {
                        _replaceCommand = new ReplaceCommands();
                        _replaceCommand.AksToConfirm += ConfirmAction;
                        _replaceCommand.Completed += Completed;
                        _replaceCommand.GoToNexStep += GoToNextStep;
                    }
                    _replaceCommand.Replace(input);
                    break;
                case "/H":
                    var cmdList = CommandList.GetCommandList();
                    WriteText.WriteInfo("You can execute the following commands:");
                    foreach (var cmd in cmdList)
                    {
                        WriteText.WriteInfo(cmd);
                    }
                    break;
                default:
                    WriteText.WriteError($"'{input}' is not recognized as a valid command. To see a list of valid commands type '/h'");
                    break;
            }
            ExecuteCommand();
        }

        private void GoToNextStep(object sender,
            NextStepEventArgs e)
        {
            string input = Console.ReadLine();
            _replaceCommand.Replace(input, e.Step, e.EntityType);
        }

        internal  void ConfirmAction(object sender, NextStepEventArgs e)
        {

            WriteText.WriteInstruction("Do you want to proceed? (Press 'Y' or 'N')");
            var confirm = Console.ReadLine();

            var working = sender.GetType() == typeof(RepairCommands)
                ? "repairing"
                : "replacing";

            var work = sender.GetType() == typeof(RepairCommands)
                ? "Repair"
                : "Replace";


            switch (confirm.ToUpper())
            {
                case "Y": WriteText.WriteInfo($"Please wait, {working} {e.EntityType.ToLower()}...");
                    if (sender.GetType() == typeof(RepairCommands))
                        _repairCommand.ConfirmRepair(e.EntityType);
                    if (sender.GetType() == typeof(ReplaceCommands))
                        _replaceCommand.ConfirmReplace();
                    break;
                case "N":
                    WriteText.WriteInfo($"{work} {e.EntityType.ToLower()} canceled!"); ExecuteCommand();
                    ; return;
                default: WriteText.WriteError($"'{confirm}' is not valid."); ConfirmAction(sender, e); break;
            }

        }
    }
}
