﻿using System;
using WebPortal.CCMConnector.ConfigConsole.Events;
using WebPortal.CCMConnector.ConfigConsole.StartUp;
using WebPortal.CCMConnector.ConfigConsole.UI;
using WebPortal.CCMConnector.Data.Configuration;

namespace WebPortal.CCMConnector.ConfigConsole.Main
{
    public class RepairCommands
    {
        private RepairAndFix _raf;
        public EventHandler<NextStepEventArgs> AksToConfirm;
        public EventHandler<EventArgs> Completed;
        public RepairCommands()
        {
            _raf = new RepairAndFix(MakeConnection.ConString, MakeConnection.DbName);
        }
        internal void Repair(string input)
        {
            string[] repairString = input.Split();
            if (repairString.Length == 1)
            {
                WriteText.WriteInfo("What do you want to repair? To see a list of valid entities type '/h'");
                return;
            }

            var entity = repairString[1];

            switch (entity.ToUpper())
            {
                case "DOSSIERS":
                    Console.WriteLine("Checking if there are any dossiers to repair...");
                    var s = _raf.CheckDossiersWithoutCustomerId().ConfigureAwait(false).GetAwaiter().GetResult();
                    var amount = _raf.GetAmountToClean("dossier");
                    WriteText.WriteResultInfo(s);
                    if (amount == 0)
                    {
                        OnCompleted();
                        return;
                    }
                    OnAskForConfirmation(entity);
                    break;
                case "/H":
                    WriteText.WriteInfo("You can repair the following entities:");
                    var cmdList = CommandList.GetCommandList("REPAIR", 1);
                    foreach (var cmd in cmdList)
                    {
                        WriteText.WriteInfo(cmd);
                    }
                    break;
                case "TICKETS":
                    Console.WriteLine("Checking if there are any tickets to repair...");
                    var t = _raf.CheckDuplicateTickets()
                        .ConfigureAwait(false)
                        .GetAwaiter()
                        .GetResult();
                    var amountTickets = _raf.GetAmountToClean("tickets");
                    WriteText.WriteResultInfo(t);
                    if (amountTickets == 0)
                    {
                        OnCompleted();
                        return;
                    }
                    OnAskForConfirmation(entity);
                    break;
                default:
                    WriteText.WriteError($"'{entity}' is not recognized as a valid entity. To see a list of valid entities, type '/h'");
                    break;

            }
        OnCompleted();

        }

        internal void ConfirmRepair(string entityType)
        {
            string s = "";
            switch (entityType.ToUpper())
            {
                case "DOSSIERS": s = _raf.RepairDossiers(); break;
                case "TICKETS": s = _raf.RepairTickets(); break;
                    
            }
            WriteText.WriteResultInfo(s);
            OnCompleted();
        }

        /// <summary>
        /// Sender = string eventType
        /// </summary>
        /// <param name="entityType"></param>
        protected virtual void OnAskForConfirmation(string entityType)
        {
            (AksToConfirm as EventHandler<NextStepEventArgs>)?
                .Invoke(this, new NextStepEventArgs(0, entityType));

        }

        protected virtual void OnCompleted()
        {
            (Completed as EventHandler<EventArgs>)?
                .Invoke(this, new EventArgs());
        }


    }
}
