﻿using System;
using System.Collections.Generic;
using System.Linq;
using CRM.Domain.Shared.Enums;
using WebPortal.CCMConnector.ConfigConsole.Events;
using WebPortal.CCMConnector.ConfigConsole.StartUp;
using WebPortal.CCMConnector.ConfigConsole.UI;
using WebPortal.CCMConnector.Data.Configuration;

namespace WebPortal.CCMConnector.ConfigConsole.Main
{
    public class ReplaceCommands
    {
        private ReplaceAndFix _raf;
        public EventHandler<NextStepEventArgs> AksToConfirm;
        public EventHandler<EventArgs> Completed;
        public EventHandler<NextStepEventArgs> GoToNexStep;
        private List<ContactDetailType> _allContactDetailTypes;
        private ContactDetailType _cdtFrom;
        private ContactDetailType _cdtto;

        public ReplaceCommands()
        {
            _raf = new ReplaceAndFix(MakeConnection.ConString, MakeConnection.DbName);
        }

        internal void Replace(string input, int step = 0, string entityType = "")
        {
            if (input.ToLower() == "q")
            {
                OnCompleted();
                return;
            }
            if (step == 0)
            {

                string[] replaceString = input.Split();
                if (replaceString.Length == 1)
                {
                    WriteText.WriteInfo("What do you want to replace? To see a list of valid entities type '/h'");
                    return;
                }

                var entity = replaceString[1];

                switch (entity.ToUpper())
                {
                    case "CONTACTDETAILTYPES":
                        WriteText.WriteInstruction("Please type the number of the type you want to replace: ");
                        WriteText.WriteInstruction("(To go back to main menu, type 'q'.)");
                        _allContactDetailTypes =
                            Enum.GetValues(typeof(ContactDetailType)).Cast<ContactDetailType>().ToList();
                        var s = "";
                        _allContactDetailTypes.ForEach(x => s += $"- {(int)x}: {x.ToString()} \n");
                        WriteText.WriteInstruction(s);
                        OnGoToNextStep(entity.ToUpper(), 1);
                        break;
                    case "/H":
                        WriteText.WriteInfo("You can replace the following entities:");
                        var cmdList = CommandList.GetCommandList("REPLACE", 1);
                        foreach (var cmd in cmdList)
                        {
                            WriteText.WriteInfo(cmd);
                        }
                        OnCompleted();
                        break;
                }
            }
            else
            {
                switch (entityType.ToUpper())
                {
                    case "CONTACTDETAILTYPES":
                        switch (step)
                        {
                            case 1:
                                if (!ValidateContactDetailTypeChoice(input, entityType, out var cdt, step)) return;
                                _cdtFrom = cdt;
                                WriteText.WriteInfo(
                                    $"Checking if we can find contactDetails in employees of type '{cdt.ToString()}'...");
                                var s = _raf.CheckContactDetailsForType(cdt)
                                    .ConfigureAwait(false)
                                    .GetAwaiter()
                                    .GetResult();
                                var amount = _raf.GetAmountToReplace(entityType);
                                WriteText.WriteResultInfo(s);
                                if (amount == 0)
                                {
                                    OnCompleted();
                                    return;
                                }
                                WriteText.WriteInstruction($"Please type the number of the type you want to replace '{cdt.ToString()}' with: ");
                                WriteText.WriteInstruction("(To go back to main menu, type 'q'.)");
                                 s = "";
                                _allContactDetailTypes.ForEach(x => s += $"- {(int)x}: {x.ToString()} \n");
                                WriteText.WriteInstruction(s);
                                OnGoToNextStep(entityType, 2);

                                break;
                            case 2:
                                if (!ValidateContactDetailTypeChoice(input, entityType, out var cdtto, step))
                                    return;
                                _cdtto = cdtto;
                                WriteText.WriteInfo($"Replace contactDetails of type '{_cdtFrom.ToString()}' by type '{cdtto.ToString()}'? " );
                                OnAskForConfirmation(entityType);
                                break;

                        }

                        break;

                }
            }
        }

        private bool ValidateContactDetailTypeChoice(string input,
            string entityType,
            out ContactDetailType cdt, int step)
        {
            cdt = ContactDetailType.Invalid;
            if (!int.TryParse(input, out var choice))
            {
                WriteText.WriteError($"'{input}' is not a valid choice.");
                OnGoToNextStep(entityType.ToUpper(), step);
                return false;
            }

            if (!Enum.IsDefined(typeof(ContactDetailType), choice))
            {
                WriteText.WriteError($"'{input}' is not a valid choice.");
                OnGoToNextStep(entityType.ToUpper(), step);
                return false;
            }

            cdt = (ContactDetailType) choice;
            return true;
        }

        public void ConfirmReplace()
        {
            var s = _raf.ReplaceContactDetailTypess(_cdtFrom, _cdtto);
            WriteText.WriteResultInfo(s);
            OnCompleted();
        }

        protected virtual void OnAskForConfirmation(string entityType)
        {
            (AksToConfirm as EventHandler<NextStepEventArgs>)?
                .Invoke(this, new NextStepEventArgs(0, entityType));

        }

        protected virtual void OnCompleted()
        {
            (Completed as EventHandler<EventArgs>)?
                .Invoke(this, new EventArgs());
        }

        protected virtual void OnGoToNextStep(string entityType,
            int step)
        {
            (GoToNexStep as EventHandler<NextStepEventArgs>)?
                .Invoke(this, new NextStepEventArgs(step, entityType));
        }

    }
}
