﻿using System;
using System.Collections.Generic;
using AutoMapper;
using WebPortal.CCMConnector.Data.ConnectorLogic.Tickets.CommandsCCM;
using WebPortal.CCMConnector.Data.EDMX;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.CcmTicketMaker
{
    public class TicketMaker
    {
        public static void PostNewCcmTickets(List<CcmTicketModel> ccmTickets, IMapper mapper)
        {
            List<Message> messageList = new List<Message>();
            foreach (var ct in ccmTickets)
            {
                Message m = new Message()
                {
                    CreateUser = "CNCTR",
                    LastModifiedUser = "CNCTR",
                    LastModifiedDate = DateTime.Now,
                    CustomerId = ct.CustomerId,
                    MessageBody = MakeRichText(ct),
                    MessageText = MakePlainText(ct),
                    MessageDateTime = DateTime.Now,
                    TypeMessage = "0",
                    ImportanceId = 1,
                    SensitivityId = 1,
                    ToTransfer = "0",
                    ReportOff = false
                };
                messageList.Add(m);
            }

            if (messageList.Count == 0) return;

            UploadCcmTicketsCommand.UploadTicketsInCcm(messageList, mapper);

        }
        private static string MakePlainText(CcmTicketModel ccmTicket)
        {
            string newLine = "\n";
            var dutyAlarmText = MakeDefaultTexts(ccmTicket, out var emails, out var sms);

            string plainText = dutyAlarmText;
            plainText += MakeEmailCharge(ccmTicket)
            //plainText += !string.IsNullOrWhiteSpace(emails)
                ? $"{newLine}{emails}{MakeFunctionText("E-MAIL", false)}"
                : !string.IsNullOrWhiteSpace(emails) ? $"{newLine}{emails}" :"";

            if (string.IsNullOrWhiteSpace(sms)) return plainText;

            plainText += $"{newLine}{sms}";
            foreach (var s in ccmTicket.SmsNumbers.Split(';'))
            {
                plainText += MakeFunctionText("SMS", false);
            }

            return plainText;
        }
        private static string MakeRichText(CcmTicketModel ccmTicket)
        {

            string newLine = @"\par ";
            var dutyAlarmText = MakeDefaultTexts(ccmTicket, out var emails, out var sms);

            string richtText = @"{\rtf1\ansi\ansicpg1252\deff0{\fonttbl{\f0\fnil\fcharset0 Courier New; } }\viewkind4\uc1\pard\lang1033\f0\fs17 " + dutyAlarmText;
            richtText += MakeEmailCharge(ccmTicket)
            //richtText += !string.IsNullOrWhiteSpace(emails)
                ? $"{newLine}{emails}{MakeFunctionText("E-MAIL", true)}"
            : !string.IsNullOrWhiteSpace(emails) ? $"{newLine}{emails}" : "";

            if (string.IsNullOrWhiteSpace(sms)) return richtText + @"\fs17 \par }";

            richtText += $"{newLine}{sms}";
            foreach (var s in ccmTicket.SmsNumbers.Split(';'))
            {
                richtText += MakeFunctionText("SMS", true);
            }
            return richtText + @"\fs17 \par }";
        }

        private static bool MakeEmailCharge(CcmTicketModel ccmTicket)
        {
            if (string.IsNullOrWhiteSpace(ccmTicket.EmailAddresses)) return false;

            var emails = ccmTicket.EmailAddresses.Split(';');
            bool otherDomain = false;
            foreach (var m in emails)
            {
                if (string.IsNullOrWhiteSpace(m)) continue;

                var domain = m.Split('@')[1];
                if (domain.ToLower() != "aac.be" && domain.ToLower() != "actiphone.be")
                {
                    otherDomain = true;
                    break;
                }
            }

            return otherDomain;
        }

        private static string MakeDefaultTexts(CcmTicketModel ccmTicket,
            out string emails,
            out string sms)
        {
            string dutyAlarmText = $"Duty-Alarm {ccmTicket.DutyAlarmId} went off.";
            emails = string.IsNullOrWhiteSpace(ccmTicket.EmailAddresses)
                ? ""
                : $"Emails were send to: {ccmTicket.EmailAddresses.Replace(";", "; ")}";
            sms = string.IsNullOrWhiteSpace(ccmTicket.SmsNumbers)
                ? ""
                : $"Text-Messages were send to: {ccmTicket.SmsNumbers.Replace(";", "; ")}";
            return dutyAlarmText;
        }

        private static string MakeFunctionText(string functionDescr, bool isRich)
        {
            return  isRich ? @"\par \{" + functionDescr + @"\} " 
                             + DateTime.Now.ToString("dd/MM/yyyy HH:mm")
                             : "\n{" + functionDescr + "} " + DateTime.Now.ToString("dd/MM/yyyy HH:mm");
        }
    }
}
