﻿using System;
using System.Collections.Generic;
using System.Linq;
using EventFlow;
using EventFlow.Configuration;
using EventFlow.Queries;
using WebPortal.CCMConnector.Data.ConnectorLogic.CallScripts.CommandsCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.CallScripts.CommandsWP;
using WebPortal.CCMConnector.Data.ConnectorLogic.CallScripts.QueriesCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.CallScripts.QueriesWP;
using WebPortal.CCMConnector.Data.ConnectorLogic.General.QueriesCCM;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.CallScripts
{
    public class CallScriptLogic
    {
        private readonly ICommandBus _commandBus;
        private readonly IQueryProcessor _queryProcessor;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public CallScriptLogic(IRootResolver serviceResolver)
        {
            _commandBus = serviceResolver.Resolve<ICommandBus>();
            _queryProcessor = serviceResolver.Resolve<IQueryProcessor>();
        }

        internal int UploadCallScripts(List<WPModules> callScriptKeys)
        {
            var clientIds = callScriptKeys
                .Where(x => x.CCMLevel == "CLIENT")
                .Select(x => x.CCMId).ToList();

            var mainCustIds = callScriptKeys
                .Where(x => x.CCMLevel == "MAIN")
                .Select(x => x.CCMId).ToList();

            var subCustIds = callScriptKeys
                .Where(x => x.CCMLevel == "SUB")
                .Select(x => x.CCMId).ToList();

            var allCustIds = GetCustomerIdsQuery
                .GetCustomerIds(false, clientIds, mainCustIds, subCustIds);

            //new GetDossierIdsQuery(_queryProcessor).AddDossierIdsToMainAndSubfiches(allCustIds);

            var useNewScriptLogic = true;

            var tempCallScripts = useNewScriptLogic
                ? GetCallscriptsQuery.GetCallScripts2(allCustIds)
                : GetCallscriptsQuery.GetCallScripts(allCustIds);

            if (tempCallScripts.Count == 0)
                return tempCallScripts.Count;

            Logger.Info($"{tempCallScripts.Count} callscripts to upload.");

            var dossierIds = tempCallScripts
                .Where(x => !string.IsNullOrWhiteSpace( x.MainAndSubficheIdAndDossierId.DossierId))
                .Select(x => x.MainAndSubficheIdAndDossierId.DossierId)
                .ToList();

            if (dossierIds == null || dossierIds.Count == 0)
            {
                string custIdsNoDossier = "";
                tempCallScripts.ForEach(x => custIdsNoDossier += x.MainAndSubficheIdAndDossierId.CustomerIdCCM.ToString() + "; ");

                Logger.Error($"The following CCM-fiches have no DossierId: {custIdsNoDossier} - Please provide those fiches with a DossierId in order for the CCM-Connector to work!");
                throw new Exception($"No dossierIds for customers: {custIdsNoDossier}");
            }

            var dossierIdsOfCallScriptsToModify =
                new GetCallScriptsWPQuery(_queryProcessor).GetAlreadyUploaded(dossierIds);

            new UploadCallscripts(_commandBus).InsertOfModifyCallscripts(tempCallScripts,
                dossierIdsOfCallScriptsToModify);

            var custIds = tempCallScripts.Select(x => x.MainAndSubficheIdAndDossierId.CustomerIdCCM).ToList();

            if (useNewScriptLogic) MarkCallScriptsAsUploadedCommand.MarkCallScriptsAsUploaded2(custIds);
            else MarkCallScriptsAsUploadedCommand.MarkCallScriptsAsUploaded(custIds);

            return tempCallScripts.Count;
        }
    }
}
