﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using CRM.Domain.CallScript;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using WebPortal.CCMConnector.Application.CallScripts.Commands;
using WebPortal.CCMConnector.Application.CallScripts.Models;
using WebPortal.CCMConnector.Application.InteractionStats.Commands;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.CallScripts.CommandsWP
{
    public class UploadCallscripts
    {
        private ICommandBus _commandBus;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public UploadCallscripts(ICommandBus commandBus)
        {
            _commandBus = commandBus;
        }

        public void InsertOfModifyCallscripts(List<TempCallScriptItem> tempCallScripts,
            List<GetCallScriptIdModel> dossierIdsOfCallScriptsToModify)
        {
            foreach (var script in tempCallScripts)
            {
                try
                {
                    var alreadyThere = dossierIdsOfCallScriptsToModify
                        .FirstOrDefault(x =>
                            x.DossierId == script.MainAndSubficheIdAndDossierId.DossierId
                            && (string.IsNullOrWhiteSpace(script.SubficheTagName)
                                ? x.Tags == null
                                : x.Tags.Contains(script.SubficheTagName)));
                    if (alreadyThere == null)
                        CreateCallScript(script);
                    else
                    {
                        ModifyCallScript(script, alreadyThere);
                    }

                }
                catch (Exception e)
                {
                    if (script.MainAndSubficheIdAndDossierId.DossierId == null)
                    {
                        Logger.Error(e, $"CCM-fiche {script.MainAndSubficheIdAndDossierId.CustomerIdCCM} has no dossierId! Provide fiche {script.MainAndSubficheIdAndDossierId.CustomerIdCCM} with a dossierId so that the Connector can continue working!");
                    }
                    else
                    {
                        Logger.Error(e, e.Message);
                    }
                    throw;
                }
            }
        }

        private IExecutionResult CreateCallScript(TempCallScriptItem callScript)
        {
            try
            {
                List<string> tags = null;
                if (!string.IsNullOrWhiteSpace(callScript.SubficheTagName))
                {
                    tags = new List<string>{callScript.SubficheTagName};
                }

                var command = new CreateCallScriptCommand(CallScriptId.New, callScript.CallScriptBody,
                    callScript.MainAndSubficheIdAndDossierId.DossierId, tags);
                var result = _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                return result;
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't create callscript for dossier '{callScript.MainAndSubficheIdAndDossierId.DossierId} with CCMId {callScript.MainAndSubficheIdAndDossierId.CustomerIdCCM}." );
                throw;
            }

        }

        private IExecutionResult ModifyCallScript(TempCallScriptItem callScript,
            GetCallScriptIdModel callScriptId)
        {
            try
            {
                var command = new ModifyCallScriptCommand(CallScriptId.With(callScriptId.Id), callScript.CallScriptBody);
                var result = _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                return result;
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't modify callscript for dossier '{callScript.MainAndSubficheIdAndDossierId.DossierId} with CCMId {callScript.MainAndSubficheIdAndDossierId.CustomerIdCCM}.");
                throw;
            }

        }
    }
}
