﻿using System;
using System.Collections.Generic;
using System.Linq;
using EventFlow.Logs;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.CallScripts.QueriesCCM
{
    public class GetCallscriptsQuery: ACCMCommandAndQuery
    {
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        internal static List<TempCallScriptItem> GetCallScripts(List<IdsFromCCMandWP> mainAndSubficheIds)
        {
            List<int> custIds = mainAndSubficheIds
                .Select(x => x.CustomerIdCCM)
                .Distinct().ToList();

            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                try
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;
                    var rtn = ent.CallScriptsHTML
                        .Join(ent.Customer,
                            s => s.CustomerId,
                            c => c.CustomerId,
                            (s,
                                c) => new {s, c})
                        .Where(x => custIds.Contains(x.s.CustomerId) && x.s.UploadedInWP != true)
                        .Select(x => new
                        {
                            x.s.CustomerId,
                            x.c.Mainfiche,
                            x.s.NotesHTML,
                            x.c.SearchName
                        })
                        .ToList()
                        .Select(x => new TempCallScriptItem
                        {
                            MainAndSubficheIdAndDossierId = mainAndSubficheIds.FirstOrDefault(y => y.CustomerIdCCM == x.CustomerId),
                            CallScriptBody = x.NotesHTML,
                            SubficheTagName = x.Mainfiche.HasValue ? x.SearchName : null
                        }).ToList();
                    return rtn;
                }
                catch (Exception e)
                {
                    Logger.Error(e, "Couldn't get CallScripts from CCM");
                    throw;
                }
            }
        }

        internal static List<TempCallScriptItem> GetCallScripts2(List<IdsFromCCMandWP> mainAndSubficheIds)
        {
            List<int> custIds = mainAndSubficheIds
                .Select(x => x.CustomerIdCCM)
                .Distinct().ToList();

            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                try
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;

                    var rtn = ent.CallScript
                        .Join(ent.Customer,
                            s => s.CustomerId,
                            c => c.CustomerId,
                            (s,
                                c) => new { s, c })
                        .Where(x => custIds.Contains(x.s.CustomerId) && !x.s.UploadedInWp && x.s.Active)
                        .Select(x => new
                        {
                            x.s.CustomerId,
                            x.c.Mainfiche,
                            x.s.HtmlScript,
                            x.c.SearchName
                        })
                        .ToList()
                        .Select(x => new TempCallScriptItem
                        {
                            MainAndSubficheIdAndDossierId = mainAndSubficheIds.FirstOrDefault(y => y.CustomerIdCCM == x.CustomerId),
                            CallScriptBody = x.HtmlScript,
                            SubficheTagName = x.Mainfiche.HasValue ? x.SearchName : null
                        }).ToList();
                    return rtn;
                }
                catch (Exception e)
                {
                    Logger.Error(e, "Couldn't get CallScripts from CCM");
                    throw;
                }
            }
        }
    }
}
