﻿using System;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.ConnectorLogic.General.QueriesCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.Tickets;
using System.Linq;
using System.Threading.Tasks;
using CRM.Domain.Shared.Enums;
using WebPortal.CCMConnector.Data.ConnectorLogic.CallScripts;
using WebPortal.CCMConnector.Data.ConnectorLogic.Customers;
using WebPortal.CCMConnector.Data.ConnectorLogic.Customers.CommandsWP;
using WebPortal.CCMConnector.Data.ConnectorLogic.Duties;
using WebPortal.CCMConnector.Data.ConnectorLogic.Employees;
using WebPortal.CCMConnector.Data.ConnectorLogic.General.CommandsCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.InteractionStats;
using WebPortal.CCMConnector.Data.ConnectorLogic.Users;
using WebPortal.CCMConnector.MailSmtp;
using System.Threading;

namespace WebPortal.CCMConnector.Data.ConnectorLogic
{
    public class ConnectorLogic: SharedLogic
    {
        //volgorde:
        //éénmalig: ophalen alle CCM-customers met treeview
        //CCMCommmand: Subscriptions toevoegen met module 'AutoTreeview'

        //Per minuut:
        //CCMQuery: nieuwe subscriptions (subscriptions zonder DossierId)
        //WPcommand: voor alle nieuwe subscription:
        //  Eerst nieuwe companies en companybranches maken (WP-Command en query)
        //  Dan admin-user voor company aanmaken (WP-Command en query)
        //  Dan admin-user relevante module-rechten toewijzen (CCMQuery en WP-Command)

        //CCMquery: Alle subscriptions met module afgaan.
        //per module nieuwe data afgaan in CCM
        //WP-Query: moet data aangepast worden.
        //WP-command: data importeren (insert of update)

        //WPQuery en CCM-query: nagaan welke subscriptions verlopen zijn.
        //Users voor bovenstaande deactiveren.


        private readonly TicketLogic _ticketLogic;
        private readonly CustomerLogic _customerLogic;
        private EmployeeLogic _employeeLogic;
        private InteractionStatLogic _interactionStatLogic;
        private CallScriptLogic _callScriptLogic;
        private DutyLogic _dutyLogic;
        private ModificationLogic _modificationLogic;
        private UserLogic _userLogic;

        private string _dbName;


        private DateTime _previousRun;
        private DateTime _currentRun;

        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public ConnectorLogic(string conString, string dbName, DateTime? previousRun = null)
        {
            _dbName = dbName;
            var serviceResolver = ConfigureServices.ConfigureServicesWithHandlersMC(conString, dbName);
            _customerLogic = new CustomerLogic(serviceResolver);
            _ticketLogic = new TicketLogic(serviceResolver);
            _employeeLogic = new EmployeeLogic(serviceResolver);
            _interactionStatLogic = new InteractionStatLogic(serviceResolver);
            _callScriptLogic = new CallScriptLogic(serviceResolver);
            _dutyLogic = new DutyLogic(serviceResolver);
            _modificationLogic = new ModificationLogic(serviceResolver);

            CheckPreviousAndSetCurrentRun(previousRun);
            SendWarningMail();
        }

        public ConnectorLogic(string conString, string dbName, bool testingAddUser)
        {
            _dbName = dbName;
            var serviceResolver = ConfigureServices.ConfigureServicesWithHandlersMC(conString, dbName);
            _userLogic = new UserLogic(serviceResolver);

        }

        //public ConnectorLogic(string conString,
        //    string dbName,
        //    bool testDeleteDossier)
        //{
        //    _dbName = dbName;
        //    var serviceResolver = ConfigureServices.ConfigureServicesWithHandlersMC(conString, dbName);
        //    _modificationLogic = new ModificationLogic(serviceResolver);
        //}

        public void DeleteDossierTest(string dossierId)
        {
            _modificationLogic.TestDeleteDossier(dossierId);
        }


        private void CheckPreviousAndSetCurrentRun(DateTime? previousRun)
        {
            try
            {
                _previousRun = previousRun ?? GetPreviousRunQuery.GetPreviousRun();
            }
            catch (Exception e)
            {
                Logger.Error(e, "Couldn't get Previous Run");
                throw;
            }

            try
            {
                _currentRun = AddCurrentRunCommand.AddCurrentRun();
            }
            catch (Exception e)
            {
                Logger.Error(e, "Couldn't add Current Run");
                throw;
            }
        }


        public void ManualUpload()
        {
            DateTime d2500 = new DateTime(2500,1,1);
            _customerLogic.MakeNewTreeviewSubscriptions();
            var allKeys = GetAllKeysQuery.GetAllTheKeysForGettingTheData(_mapper.GetMapper(), d2500);

            var employeeKeys = allKeys
                .SelectMany(x => x.Modules)
                .Where(x => x.ModuleId == ModuleName.AutoTreeview).ToList();

            _employeeLogic.UploadEmployees(employeeKeys);
            //_moduleLogic.UploadModules();
        }

        public void ModificationDeleteTest()
        {
            Logger.Info("Starting Test Delete Staging");
            var allKeys = GetAllKeysQuery.GetAllTheKeysForGettingTheData(_mapper.GetMapper(), _currentRun);
            var modifiedSubs = _modificationLogic.ModifyCustomersDossiersAndBranches();
            string uploadComment =
                $"Test Delete! Modified: {modifiedSubs} subscriptions/dossiers.";
            AddCurrentRunCommand.AddRemarksToCurrentRun(uploadComment);
            Logger.Info("Upload successful. " + uploadComment);

        }


        public void RepeatedUpload()
        {
            Logger.Info("Starting Upload");
            //var wpSubscriptions = new GetSubscriptionOverview().GetSubscriptionOverviewFromCCM();
            var allKeys =  GetAllKeysQuery.GetAllTheKeysForGettingTheData(_mapper.GetMapper(), _currentRun);

            var noCustIdInWP = allKeys
                .Where(x => string.IsNullOrWhiteSpace(x.CustomerIdWP)).ToList();

            var newSubs = _customerLogic.MakeNewCustomersForSubscriptionsMadeInCCM(noCustIdInWP);

            var modifiedSubs = _modificationLogic.ModifyCustomersDossiersAndBranches();

            var callScriptKeys = allKeys
                .SelectMany(x => x.Modules)
                .Where(x => 
                    !x.Deleted 
                    &&  x.ModuleId == ModuleName.Script 
                    && x.StartDate <= DateTime.UtcNow 
                    && (x.EndDate == null || x.EndDate.Value > DateTime.UtcNow))
                .ToList();
            var callScripts = _callScriptLogic.UploadCallScripts(callScriptKeys);



            var ticketKeys = allKeys
                .SelectMany(x => x.Modules)
                .Where(x => 
                    !x.Deleted 
                    && x.ModuleId == ModuleName.Tickets
                    && x.StartDate <= DateTime.UtcNow
                    && (x.EndDate == null || x.EndDate.Value > DateTime.UtcNow))
                .ToList();

            var tickets = _ticketLogic.UploadTickets(ticketKeys, _previousRun, out var allCustIdsForTickets);


            var statisticKeys = allKeys
                .SelectMany(x => x.Modules)
                .Where(x => 
                    !x.Deleted 
                    && x.ModuleId == ModuleName.Statistics
                    && x.StartDate <= DateTime.UtcNow
                    && (x.EndDate == null || x.EndDate.Value > DateTime.UtcNow))
                .ToList();
            var statistics = _interactionStatLogic.UploadInteractionStats(statisticKeys, allCustIdsForTickets, _previousRun, _currentRun);

            var dutyAlarmAmount = _dutyLogic.CheckDutiesToExpire(_previousRun);

            string uploadComment =
                $"Uploaded: {newSubs} subscriptions/dossiers; {tickets} tickets; {statistics} statistics; {callScripts} callscripts. Modified: {modifiedSubs} subscriptions/dossiers. {dutyAlarmAmount} duty-alarms went off.";
            if (_dbName == "wp_staging")
            {
                uploadComment = "Staging-Test! Verwijder deze lijn! " + uploadComment;
            }
            AddCurrentRunCommand.AddRemarksToCurrentRun(uploadComment);
            Logger.Info("Upload successful. " + uploadComment);
        }

        private void SendWarningMail()
        {
            if (_previousRun > DateTime.UtcNow.AddMinutes(-5)) return;

            if (_previousRun < DateTime.UtcNow.AddMinutes(-5) && _previousRun > DateTime.UtcNow.AddMinutes(-7))
            {
                Mailer.SendNotRunMail(_previousRun).GetAwaiter().GetResult();
            }
            else if (_previousRun < DateTime.UtcNow.AddMinutes(-15) && _previousRun > DateTime.UtcNow.AddMinutes(-7))
            {
                Mailer.SendNotRunMail(_previousRun).GetAwaiter().GetResult();
            }
            else if (_previousRun < DateTime.UtcNow.AddMinutes(-30) && _previousRun > DateTime.UtcNow.AddMinutes(-32))
            {
                Mailer.SendNotRunMail(_previousRun).GetAwaiter().GetResult();
            }
            else if (_previousRun < DateTime.UtcNow.AddHours(-1) && _previousRun > DateTime.UtcNow.AddHours(-1).AddMinutes(-2))
            {
                Mailer.SendNotRunMail(_previousRun).GetAwaiter().GetResult();
            }
            else if (_previousRun < DateTime.UtcNow.AddHours(-2) && _previousRun > DateTime.UtcNow.AddHours(-2).AddMinutes(-2))
            {
                Mailer.SendNotRunMail(_previousRun).GetAwaiter().GetResult();
            }
            else if (_previousRun < DateTime.UtcNow.AddHours(-4) && _previousRun > DateTime.UtcNow.AddHours(-4).AddMinutes(-2))
            {
                Mailer.SendNotRunMail(_previousRun).GetAwaiter().GetResult();
            }
            else if (_previousRun < DateTime.UtcNow.AddHours(-8) && _previousRun > DateTime.UtcNow.AddHours(-8).AddMinutes(-2))
            {
                Mailer.SendNotRunMail(_previousRun).GetAwaiter().GetResult();
            }
            else if (_previousRun < DateTime.UtcNow.AddHours(-16) && _previousRun > DateTime.UtcNow.AddHours(-16).AddMinutes(-2))
            {
                Mailer.SendNotRunMail(_previousRun).GetAwaiter().GetResult();
            }
            else
            {
                Mailer.SendNotRunMail(_previousRun).GetAwaiter().GetResult();
            }


        }


        #region TestMethods
        public void TestDutyAlarm()
        {
            var dutyAlarmAmount = _dutyLogic.CheckDutiesToExpire(_previousRun);
            Logger.Info($"{dutyAlarmAmount} duty-alarms went off!");
        }

        public void TestDeleteDossier()
        {
            var modifiedSubs = _modificationLogic.ModifyCustomersDossiersAndBranches();

        }

        public async Task TestCreateUser(string email, string userName, string? tenant, string? customerId, CancellationToken cancellationToken)
        {
            await _userLogic.AddUser(email, userName, tenant, customerId, cancellationToken);
        }
        #endregion

    }
}
