﻿using System.Collections.Generic;
using System.Linq;
using CRM.Domain.Branch.Entities;
using CRM.Domain.Branch.Models;
using CRM.Domain.Shared.Enums;
using WebPortal.CCMConnector.Data.Models;
using WebPortal.CCMConnector.Data.Models.AbstractModels;
using WebPortal.CCMConnector.Data.Validation;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Customers.CommandsWP
{
    public class AddBranchContactDetailsCommand
    {
        internal static List<ContactDetailModel> MakeContactDetailList(ABranchModel bm)
        {
            List<ContactDetailModel> rtn = new List<ContactDetailModel>();
            if (!string.IsNullOrWhiteSpace(bm.Phone1))
                rtn.Add(MakeContactDetail("Office phone", bm.Phone1, ContactDetailType.TelephoneNr));

            if (!string.IsNullOrWhiteSpace(bm.Phone2))
                rtn.Add(MakeContactDetail("Office phone 2", bm.Phone2, ContactDetailType.TelephoneNr));

            var allEmails = new TempEmail()
            {
                EmailAddress = $"{bm.Email};{bm.EmailAddress};{bm.DefaultEmail}"
            };
            SplitAndValidate.ValidateAndSplitEmails(allEmails);
            int emailcounter = 0;
            foreach (var m in allEmails.EmailList)
            {
                string email = emailcounter > 0 ? $"Email {emailcounter}" : "Email";
                rtn.Add(MakeContactDetail(email, m, ContactDetailType.Email));
                emailcounter++;
            }

            if (!string.IsNullOrWhiteSpace(bm.HomePage) && RegexUtilities.IsValidUrl(bm.HomePage))
                rtn.Add(MakeContactDetail("Website", bm.HomePage, ContactDetailType.Website));

            return rtn;
        }

        private static ContactDetailModel MakeContactDetail(string contactDetailName, string contactDetailValue, ContactDetailType contactType, bool isPrivate = false)
        {
            ContactDetailModel detail = new ContactDetailModel(ContactDetailId.New.Value, contactDetailName,
                contactDetailValue, isPrivate, contactType);
            return detail;
        }
    }
}