﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Branch;
using CRM.Domain.Employee;
using CRM.Domain.Shared.Enums;
using CRM.Domain.User;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using FirebaseAdmin;
using FirebaseAdmin.Auth;
using Google.Apis.Auth.OAuth2;
using WebPortal.CCMConnector.Application.Branches.Commands;
using WebPortal.CCMConnector.Application.Employees.Commands;
using WebPortal.CCMConnector.Application.Users.Commands;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Customers.CommandsWP
{
    public class DeleteCustomerAndConnectedToCustomerCommand
    {
        private ICommandBus _commandBus;
        private readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();
        private readonly FirebaseAuth _auth;
        private string _customerId;
        private FirebaseApp _firebaseApp;

        public DeleteCustomerAndConnectedToCustomerCommand(ICommandBus commandBus)
        {
            _commandBus = commandBus;
            //FirebaseApp app = FirebaseApp.GetInstance();
            _firebaseApp = FirebaseApp.Create(new AppOptions()
            {
                Credential = GoogleCredential.FromFile(@"C:\WebPortal\key.json"),
                ProjectId = "webportal-productie",
                ServiceAccountId = "webportal-productie@appspot.gserviceaccount.com"
            });
            
            _auth = FirebaseAuth.GetAuth(_firebaseApp);
        }

        internal async Task DeleteCustomerInBranchesEmployeesAndUsers(
            Dictionary<AggregateNameEnum, List<string>> typeAndIds,
            string customerId)
        {
            _customerId = customerId;
            foreach (var tai in typeAndIds)
            {
                switch (tai.Key)
                {
                    case AggregateNameEnum.User:
                        foreach (var u in tai.Value)
                        {
                            var uId = UserId.With(u);
                            await DeleteUser(uId);
                        }
                        break;
                    case AggregateNameEnum.Branch:
                        foreach (var b in tai.Value)
                        {
                            var bId = BranchId.With(b);
                            await DeleteBranch(bId);
                        }
                        break;
                    case AggregateNameEnum.Employee:
                        foreach (var e in tai.Value)
                        {
                            var eId = EmployeeId.With(e);
                            await DeleteEmployee(eId);
                        }
                        break;
                }
            }

            new DeleteOneCustomerCommand(_commandBus).DeleteCustomer(customerId);
        }

        private async Task<IExecutionResult> DeleteBranch(BranchId bId)
        {
            try
            {
                //var command = new ModifyBranchCustomerIdCommand(bId, customerIdDestination);
                var command = new DeleteBranchCommand(bId);
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't delete the branch '{bId.Value}'.");
                throw;
            }
        }

        private async Task<IExecutionResult> DeleteEmployee(EmployeeId eId)
        {
            try
            {
                var command = new DeleteEmployeeCommand(eId);
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't delete employee '{eId.Value}' from customer '{_customerId}'."); 
                throw;
            }
        }

        private async Task<IExecutionResult> DeleteUser(UserId uId)
        {
            try
            {
                await _auth.DeleteUserAsync(uId.Value, new CancellationToken());
                var command = new DeleteUserCommand(uId);
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't delete user '{uId.Value}' from customer '{_customerId}'.");
                throw;
            }
        }

        public void DeleteFireBaseApp()
        {
            _firebaseApp.Delete();
        }
    }
}
