﻿using System;
using System.Threading;
using CRM.Domain.Branch;
using CRM.Domain.Branch.Models;
using CRM.Domain.Branch.ReadModels;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.Branches.Commands;
using WebPortal.CCMConnector.Application.Branches.Queries;
using WebPortal.CCMConnector.Data.ConnectorLogic.Customers.QueriesCCM;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Customers.CommandsWP
{
    public class ModifyBranchHqAddressAndContactDetailsCommand
    {
        private readonly ICommandBus _commandBus;
        private readonly IQueryProcessor _queryProcessor;

        private readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public ModifyBranchHqAddressAndContactDetailsCommand(ICommandBus commandBus,
            IQueryProcessor queryProcessor)
        {
            _commandBus = commandBus;
            _queryProcessor = queryProcessor;
        }

        public void ModifyBranchHq(int clientId)
        {
            var branchModel = GetModifyBranchModelsQuery.GetBranchModels(clientId);
            if (branchModel == null) return;

                var readModel = GetBranchReadModel(branchModel);

                if (readModel == null)return;
                if (branchModel.NeedToUpdateAddress(readModel))
                {
                    ModifyAddress(branchModel, readModel.Id);
                }

                if (branchModel.NeedToUpdateContactDetails(readModel))
                {
                    ModifyContactDetails(branchModel, readModel.Id);
                }
        }

        private BranchReadModel GetBranchReadModel(ModifyBranchModel bm)
        {
            var q = new GetHqBranchByCustomerIdQuery(bm.CustomerIdWp);
            var branchId = _queryProcessor.ProcessAsync(q, new CancellationToken())
                .GetAwaiter()
                .GetResult();
            var qBranch = new GetBranchByBranchIdQuery(branchId);
            var readModel = _queryProcessor.ProcessAsync(qBranch, new CancellationToken())
                .GetAwaiter()
                .GetResult();
            return readModel;
        }

        private IExecutionResult ModifyAddress(ModifyBranchModel mbm, string branchId)
        {
            try
            {
                AddressModel address = new AddressModel
                {
                    City = mbm.City,
                    Country = mbm.Country,
                    Number = mbm.HouseNumber,
                    PostalBoxNr = mbm.BoxNumber,
                    PostalCode = mbm.PostalCode,
                    Street = mbm.Street,
                    Longitude = null,
                    Latitude = null
                };

                var command = new ModifyBranchAddressCommand(BranchId.With(branchId), address);
                var result = _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                return result;


            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not modify the address of branch '{branchId}' of CCM-fiche: {mbm.CustomerIdCcm}.");
                throw;
            }
        }

        private IExecutionResult ModifyContactDetails(ModifyBranchModel mbm,
            string branchId)
        {
            try
            {
                var contactDetails = AddBranchContactDetailsCommand
                    .MakeContactDetailList(mbm);

                var command = new ModifyBranchContactDetailsCommand(BranchId.With(branchId), contactDetails);
                var result = _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                return result;

            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not modify the contactDetails of branch '{branchId}' of CCM-fiche: {mbm.CustomerIdCcm}.");
                throw;
            }
        }
    }
}
