﻿using System;
using System.Collections.Generic;
using System.Linq;
using CRM.Domain.Shared.Enums;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Customers.CommandsWP
{
    public class GetModuleStringsQuery
    {
        public static List<string> GetModuleStrings(WPSubscriptionOverview subscription)
        {
            if (subscription == null) return null;


            var allModules = subscription.Modules
                .Where(x => !x.Deleted
                            && (x.EndDate == null || (x.EndDate.HasValue && x.EndDate > DateTime.UtcNow)))
                .Select(x => x.ModuleId)
                .Distinct()
                .ToList();

            if (allModules.Count == 0) return null;

            List<string> moduleStrings = new List<string>();
            allModules.ForEach(x => moduleStrings.Add(GiveModuleName(x)));

            return moduleStrings.Where(x => !string.IsNullOrWhiteSpace(x)).ToList();

        }

        private static string GiveModuleName(ModuleName moduleEnum)
        {
            switch (moduleEnum)
            {
                case ModuleName.Duties: return "duties";
                case ModuleName.VirtualReception: return "vr";
                case ModuleName.Tickets: return "tickets";
                case ModuleName.Statistics: return "statistics";
                case ModuleName.Script: return "script";
            }

            return null;
        }

    }
}
