﻿using System;
using System.Threading;
using CRM.Domain.Customer;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using WebPortal.CCMConnector.Application.Customers.Commands;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Customers.CommandsWP
{
    public class ModifyWpCustomerNameCommand
    {
        private ICommandBus _commandBus;

        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public ModifyWpCustomerNameCommand(ICommandBus commandBus)
        {
            _commandBus = commandBus;
        }

        public IExecutionResult ModifyCustomerName(string customerId, string customerName)
        {
            try
            {
                var aggregateId = CustomerId.With(customerId);
                var command = new ModifyCustomerNameCommand(aggregateId, customerName);
                var result = _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                Logger.Info($"Customer '{customerId}' now goes by the name: '{customerName}'.");
                return result;
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't modify customerName for customer '{customerId}' into '{customerName}'");
                throw;
            }
        }

    }
}
