﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Branch;
using CRM.Domain.Dossier;
using CRM.Domain.Employee;
using CRM.Domain.Shared.Enums;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using WebPortal.CCMConnector.Application.Branches.Commands;
using WebPortal.CCMConnector.Application.Dossiers.Commands;
using WebPortal.CCMConnector.Application.Employees.Commands;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Customers.CommandsWP
{
    public class ReplaceCustomerIdCommand
    {
        public ReplaceCustomerIdCommand(ICommandBus commandBus)
        {
            _commandBus = commandBus;
        }

        private ICommandBus _commandBus;
        private readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        internal async Task ReplaceTheCustomerIdInDossiersBranchesAndEmployees(
            Dictionary<AggregateNameEnum, List<string>> typeAndIds,
            string customerIdDestination,
            string customerIdSource)
        {
            foreach (var tai in typeAndIds)
            {
                switch (tai.Key)
                {
                    case AggregateNameEnum.Dossier:
                        foreach (var s in tai.Value)
                        {
                            var dId = DossierId.With(s);
                            await ReplaceDossierCustomerId(dId, customerIdDestination, customerIdSource);
                        }
                        break;
                    case AggregateNameEnum.Branch:
                        foreach (var s in tai.Value)
                        {
                            var bId = BranchId.With(s);
                            await DeleteBranch(bId);
                        }
                        break;
                    case AggregateNameEnum.Employee:
                        foreach (var s in tai.Value)
                        {
                            var eid = EmployeeId.With(s);
                            await ReplaceEmployeeCustomerId(eid, customerIdDestination, customerIdSource);
                        }
                        break;
                        
                }
            }
        }

        private async Task<IExecutionResult> ReplaceEmployeeCustomerId(EmployeeId eId,
            string customerIdDestination,
            string customerIdSource)
        {
            try
            {
                var command = new ModifyEmployeeCustomerIdCommand(eId, customerIdDestination);
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't modify the CustomerId in employee '{eId.Value}' from '{customerIdSource}' to '{customerIdDestination}'.");
                throw;
            }
        }

        private async Task<IExecutionResult> DeleteBranch(BranchId bId)
        {
            try
            {
                //var command = new ModifyBranchCustomerIdCommand(bId, customerIdDestination);
                var command = new DeleteBranchCommand(bId);
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't delete the branch '{bId.Value}'.");
                throw;
            }
        }

        private async Task<IExecutionResult> ReplaceDossierCustomerId(DossierId dId,
            string customerIdDestination,
            string customerIdSource)
        {
            try
            {
                var command = new ModifyDossierCustomerIdCommand(dId, customerIdDestination);
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't modify the CustomerId in dossier '{dId.Value}' from '{customerIdSource}' to '{customerIdDestination}'.");
                throw;
            }
        }
    }
}
