﻿using System;
using System.Collections.Generic;
using System.Linq;
using EventFlow;
using EventFlow.Configuration;
using EventFlow.Queries;
using WebPortal.CCMConnector.Data.ConnectorLogic.Customers.CommandsCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.Customers.CommandsWP;
using WebPortal.CCMConnector.Data.ConnectorLogic.Customers.QueriesCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.Customers.QueriesWP;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Customers
{
    public class CustomerLogic : SharedLogic
    {
        private readonly ICommandBus _commandBus;
        private readonly IQueryProcessor _queryProcessor;

        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public CustomerLogic(IRootResolver serviceResolver)
        {
            _commandBus = serviceResolver.Resolve<ICommandBus>();
            _queryProcessor = serviceResolver.Resolve<IQueryProcessor>();
        }

        internal void MakeNewTreeviewSubscriptions()
        {
            var custIdsWithTreeview = new GetAllCustomersWithTreeviewQuery().GetAllCustomerIdsWithTreeview(out var rawList);

            int zero = 0;
            var custIdsMaxusersDict = custIdsWithTreeview.Select(x => new { x, zero })
                .ToDictionary(t => t.x, t => t.zero);

            var tempCompanyItems = new GetTempCompanyItemsWithsubfichesQuery().GetTempCompanyItemsWithSubfiches(custIdsWithTreeview, custIdsMaxusersDict);
            var tempListToGo = new List<TempCompanyItem>(tempCompanyItems);
            foreach (var tci in tempListToGo.Where(x => x.MainFicheId == null && x.ClientId > 0))
            {
                var subFiches = tempCompanyItems.Where(x => x.MainFicheId == tci.CustomerId);
                tci.CompanyBranches = subFiches.ToList();
            }

            var tempWithSubs = tempListToGo
                .Where(x => x.MainFicheId == null && x.ClientId > 0).ToList();
            new CreateCustomersAndBranchesCommand(_commandBus).CreateCustomersAndBranches(tempWithSubs);

            var subscriptions = tempWithSubs
                .Where(x => x.TempSubscription != null)
                .Select(x => x.TempSubscription).ToList();

            var custModules = tempWithSubs
                .Where(x => x.TempCustomerModule != null)
                .Select(x => x.TempCustomerModule).ToList();

            var custSubs =
                tempWithSubs.SelectMany(x => x.CompanyBranches.Select(y => y.TempCustomerModule).ToList()).ToList();

            custModules.AddRange(custSubs);

            UpdateCustomersWithDossierIdsCommand.UpdateCustomersWithDossierIds(tempWithSubs);
            UpdateCustomersWithDossierIdsCommand.UpdateCustomersSetViewWpButton(rawList);
            CreateSubscriptionsAndModules.CreateAndSaveSubscriptions(subscriptions, _mapper.GetMapper());

            CreateSubscriptionsAndModules.CreateAndSaveCustomerModules(custModules, _mapper.GetMapper());

        }

        internal int MakeNewCustomersForSubscriptionsMadeInCCM(List<WPSubscriptionOverview> wpSubscriptions)
        {
            if (wpSubscriptions == null || wpSubscriptions.Count == 0)
            {
                return 0;
            }
            Logger.Info($"{wpSubscriptions.Count} new subscriptions to add.");
            var tempCompanyItems =
                new GetTempCompanyItemsWithsubfichesQuery().GetTempCompanyItemsByNewSubscriptions(wpSubscriptions);
            new CreateCustomersAndBranchesCommand(_commandBus).CreateCustomersAndBranches(tempCompanyItems, wpSubscriptions);
            UpdateCustomersWithDossierIdsCommand.UpdateCustomersWithDossierIds(tempCompanyItems);

            try
            {
                new UpdateSubscriptionsInCCM().UpdateSubscriptionsWithCustomerIdWP(wpSubscriptions);
            }
            catch (Exception e)
            {
                Logger.Error(e,"Couldn't update Subscriptions in ProdTelesec-DB.");
                throw;
            }

            return tempCompanyItems.Count;


        }
        /// <summary>
        /// enkel aantal users + naamwijzigingen aan Dossier of Customer worden voorlopig aanvaard
        /// </summary>


    }
}
