﻿using System;
using System.Collections.Generic;
using System.Linq;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.EDMX;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Customers.QueriesCCM
{
    public class GetTempCompanyItemsWithsubfichesQuery : ACCMCommandAndQuery
    {
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        /// <summary>
        /// De custidsWithMaxUsers mag een lege dictionary zijn.
        /// </summary>
        /// <param name="custIds"></param>
        /// <param name="clientIdsWithMaxUsers">Mag leeg zijn, maar niet null</param>
        /// <returns></returns>
        public List<TempCompanyItem> GetTempCompanyItemsWithSubfiches(List<int> custIds,
            Dictionary<int, int> clientIdsWithMaxUsers)
        {
            int? noSubfiche = null;
            try
            {
                using (ProdtelesecEntities ent = new ProdtelesecEntities())
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;
                    var rtn = ent.Customer
                        .Where(x => custIds.Contains(x.CustomerId))
                        .Select(x => new TempCompanyItem()
                        {
                            ClientId = x.ClientId ?? 0,
                            TempId = Guid.NewGuid(),
                            MainFicheId = x.Mainfiche,
                            CustomerId = x.CustomerId,
                            DossierName = x.Name,
                            BranchName = x.SearchName,
                            Tenant = x.Client.Tenant,
                            CustomerName = x.Client.ClientName,
                            SubFicheId = x.Mainfiche == null ? noSubfiche : x.CustomerId,
                            Street = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Street.Trim(),
                            City = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().City.Trim(),
                            PostalCode = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Zipcode.Trim(),
                            Country = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Country.Trim(),
                            HouseNumber = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Number.Trim(),
                            BoxNumber = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Box.Trim(),
                            FaxNumber = x.FaxNumber.Trim(),
                            EmailAddress = x.EmailAddress.Trim(),
                            Fax = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Fax.Trim(),
                            DefaultFax = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().DefaultFax.Trim(),
                            Email = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Email.Trim(),
                            DefaultEmail = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().DefaultEmail.Trim(),
                            Phone1 = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Phone1.Trim(),
                            Phone2 = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Phone2.Trim(),
                            HomePage = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().HomePage.Trim(),

                        }).ToList();
                    if (clientIdsWithMaxUsers.Count > 0)
                    {
                        rtn.ForEach(x => x.MaxUsers = clientIdsWithMaxUsers
                            .Where(y => y.Key == x.CustomerId).Any()
                            ? clientIdsWithMaxUsers.SingleOrDefault(y => y.Key == x.CustomerId).Value
                            : 0);
                    }

                    return rtn;
                }
            }
            catch (Exception e)
            {
                Logger.Error(e,"Couldn't create TempCompanies with custIds.");
                throw;
            }
        }

        public List<TempCompanyItem> GetTempCompanyItemsByNewSubscriptions(
            List<WPSubscriptionOverview> newSubscriptions)
        {
            int? noSubfiche = null;
            foreach (var ns in newSubscriptions)
            {
                if (ns.Modules == null || ns.Modules.Count == 0)
                {
                    Logger.Error($"No modules selected for subscription {ns.Id}");
                    throw new Exception($"No modules selected for subscription {ns.Id}");
                }
            }
            try
            {
                var clientIds =
                    newSubscriptions.SelectMany(x => 
                            x.Modules.Where(y => 
                                y.CCMLevel == "CLIENT").Select(y => 
                                y.CCMId))
                        .ToList();
                var mainCustIds = newSubscriptions.SelectMany(x => 
                        x.Modules.Where(y 
                            => y.CCMLevel == "MAIN").Select(y =>
                            y.CCMId))
                    .ToList();
                var subCustIds = newSubscriptions.SelectMany(x =>
                        x.Modules.Where(y
                            => y.CCMLevel == "SUB").Select(y =>
                            y.CCMId))
                    .ToList();

            //Per client moeten we een nieuwe Customer aanmaken in de WP.
            //Per mainfiche moeten we een nieuw dossier aanmaken in WP.

            //deze methode moet een lijst terugsturen van mainfiches geordend op ClientId
            //hiervoor hebben we allereerst een lijst nodig van allemaal unieke mainficheIds

            var custIdsToGet = new List<int>();



            //newSubscriptions.Select(x => x.ClientIdCCM)
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;

                    if (clientIds.Count > 0)
                    {
                        var custIdsFromClient = ent.Customer.Where(x => clientIds.Contains(x.ClientId.Value))
                            .Select(x => x.CustomerId)
                            .ToList();
                        custIdsToGet.AddRange(custIdsFromClient);
                    }

                    //ent.Client.Where(x => 
                    //    clientIds.Contains(x.ClientId))
                    //.SelectMany(x => 
                    //    x.Customer.Select(y => 
                    //        y.CustomerId)).ToList();

                    if (subCustIds.Count > 0)
                    {
                        var mainCustsBasedOnSubCusts = ent.Customer.Where(x => subCustIds.Contains(x.CustomerId))
                            .Select(x => x.Mainfiche.Value).ToList();

                        custIdsToGet.AddRange(mainCustsBasedOnSubCusts);

                    }

                    if (mainCustIds.Count > 0)
                    {
                        custIdsToGet.AddRange(mainCustIds);
                    }

                    if (custIdsToGet.Count == 0)
                    {
                        return null;
                    }
                    custIdsToGet = custIdsToGet.Distinct().ToList();



                    var rtn = ent.Customer
                        .Where(x => custIdsToGet.Contains(x.CustomerId))
                        .Select(x => new TempCompanyItem()
                        {
                            ClientId = x.ClientId ?? 0,
                            TempId = Guid.NewGuid(),
                            MainFicheId = x.Mainfiche,
                            CustomerId = x.CustomerId,
                            DossierName = x.Name,
                            BranchName = x.SearchName,
                            Tenant = x.Client.Tenant,
                            CustomerName = x.Client.ClientName,
                            SubFicheId = x.Mainfiche == null ? noSubfiche : x.CustomerId,
                            Street = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Street.Trim(),
                            City = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().City.Trim(),
                            PostalCode = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Zipcode.Trim(),
                            Country = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Country.Trim(),
                            HouseNumber = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Number.Trim(),
                            BoxNumber = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Box.Trim(),
                            FaxNumber = x.FaxNumber.Trim(),
                            EmailAddress = x.EmailAddress.Trim(),
                            Fax = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Fax.Trim(),
                            DefaultFax = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().DefaultFax.Trim(),
                            Email = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Email.Trim(),
                            DefaultEmail = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().DefaultEmail.Trim(),
                            Phone1 = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Phone1.Trim(),
                            Phone2 = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().Phone2.Trim(),
                            HomePage = x.Address.FirstOrDefault() == null
                                ? null
                                : x.Address.FirstOrDefault().HomePage.Trim(),

                        }).ToList();
                    return rtn;
            }
            }
            catch (Exception e)
            {
                Logger.Error(e,"Couldn't create TempCompanies with newSubscriptions.");
                throw;
            }

        }
    }
}
