﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Shared.Enums;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.Branches.Queries;
using WebPortal.CCMConnector.Application.Dossiers.Queries;
using WebPortal.CCMConnector.Application.Employees.Queries;
using WebPortal.CCMConnector.Application.Users.Queries;
using WebPortal.CCMConnector.Data.ConnectorLogic.General.QueriesWP;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Customers.QueriesWP
{
    public class GetIdsForMultipleTypesByCustomerIdQuery: AGetIdsForMultipleTypes
    {
        public GetIdsForMultipleTypesByCustomerIdQuery(IQueryProcessor queryProcessor) : base(queryProcessor)
        {
        }

        internal override async Task <Dictionary<AggregateNameEnum, List<string>>> GetIdsBySharedId(string sharedId)
        {
            await base.GetIdsBySharedId(sharedId);
            await GetDossierIds(sharedId);
            await GetBranchIds(sharedId);
            await GetEmployeeIds(sharedId);
            await GetUserIds(sharedId);
            return _replaceDictionary;
        }

        private async Task GetDossierIds(string customerId)
        {
            try
            {
                var ids = await _queryProcessor.ProcessAsync(new GetDossierIdsByCustomerIdQuery(customerId),
                    new CancellationToken());
                AddToDictionary(AggregateNameEnum.Dossier, ids);
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not get DossierIds for customer: '{customerId}'");
                throw;
            }
        }

        private async Task GetBranchIds(string customerId)
        {
            try
            {
                var ids = await _queryProcessor.ProcessAsync(new GetBranchIdsByCustomerIdQuery(customerId),
                    new CancellationToken());
                AddToDictionary(AggregateNameEnum.Branch, ids);
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not get BranchIds for customer: '{customerId}'");
                throw;
            }
        }

        private async Task GetEmployeeIds(string customerId)
        {
            try
            {
                var ids = await _queryProcessor.ProcessAsync(new GetEmployeeIdsByCustomerIdQuery(customerId),
                    new CancellationToken());
                AddToDictionary(AggregateNameEnum.Employee, ids);
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not get EmployeeIds for customer: '{customerId}'");
                throw;
            }
        }

        private async Task GetUserIds(string customerId)
        {
            try
            {
                var ids = await _queryProcessor.ProcessAsync(new GetUserIdsByCustomerIdQuery(customerId),
                    new CancellationToken());
                AddToDictionary(AggregateNameEnum.User, ids);
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not get UserIds for customer: '{customerId}'");
                throw;
            }
        }


    }
}
