﻿using System;
using System.Collections.Generic;
using System.Threading;
using CRM.Domain.CallScript;
using CRM.Domain.Dossier;
using CRM.Domain.Duty;
using CRM.Domain.InteractionStatistic;
using CRM.Domain.Shared.Enums;
using CRM.Domain.Ticket;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.CallScripts.Commands;
using WebPortal.CCMConnector.Application.Dossiers.Commands;
using WebPortal.CCMConnector.Application.Duties.Commands;
using WebPortal.CCMConnector.Application.InteractionStats.Commands;
using WebPortal.CCMConnector.Application.Tickets.Commands;
using WebPortal.CCMConnector.Data.ConnectorLogic.Dossiers.QueriesWP;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Dossiers.Commands
{
    public class DeleteWPDossierCommand
    {
        private ICommandBus _commandBus;
        private readonly IQueryProcessor _queryProcessor;
        private readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();
        private string _dossierId;

        public DeleteWPDossierCommand(ICommandBus commandBus,
            IQueryProcessor queryProcessor)
        {
            _commandBus = commandBus;
            _queryProcessor = queryProcessor;
        }

        internal void DeleteDossierWP(string dossierId)
        {
            _dossierId = dossierId;
            List<string> dutyIds;
            List<string> interactionStatIds;
            List<string> callScriptIds;
            List<string> ticketIds;
            var allIds = new GetIdsForMultipleTypesByDossierIdQuery(_queryProcessor).GetIdsBySharedId(dossierId)
                .ConfigureAwait(false)
                .GetAwaiter().GetResult();
             
            DeleteEverythingConnectedToTheDossier(allIds);

            DeleteDossier(DossierId.With(dossierId));

            string deleteInfo = "Deleted: ";
            foreach (var ai in allIds)
            {
                deleteInfo += $"{ai.Value.Count} {ai.Key}s, ";
            }

            deleteInfo = deleteInfo.Substring(0, deleteInfo.Length - 2)
                         + $". They belonged to dossier '{dossierId}' which is deleted as well.";
            Logger.Info(deleteInfo);
        }

        private void DeleteEverythingConnectedToTheDossier(Dictionary<AggregateNameEnum, List<string>> typeAndIds)
        {
            foreach (var tai in typeAndIds)
            {
                switch (tai.Key)
                {
                    case AggregateNameEnum.Duty:
                        foreach (var s in tai.Value)
                        {
                            var dId = DutyId.With(s);
                            DeleteDuty(dId);
                        }
                        break;
                    case AggregateNameEnum.Statistic:
                        foreach (var s in tai.Value)
                        {
                            var sId = InteractionStatisticId.With(s);
                            DeleteStatistic(sId);
                        }
                        break;
                    case AggregateNameEnum.CallScript:
                        foreach (var s in tai.Value)
                        {
                            var scrId = CallScriptId.With(s);
                            DeleteCallScript(scrId);
                        }
                        break;
                    case AggregateNameEnum.Ticket:
                        foreach (var s in tai.Value)
                        {
                            var tId = TicketId.With(s);
                            DeleteTicket(tId);
                        }
                        break;
                        
                }
            }
        }

        private IExecutionResult DeleteDuty(DutyId dutyId)
        {
                try
                {
                    var command = new DeleteDutyCommand(dutyId);
                    return _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                }
                catch (Exception e)
                {
                    Logger.Error(e, $"Couldn't delete Duty '{dutyId.Value}' for dossier '{_dossierId}'. ");
                    throw;
                }
        }

        private IExecutionResult DeleteStatistic(InteractionStatisticId statId)
        {
            try
            {
                var command = new DeleteInteractionStatCommand(statId);
                return _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't delete statistic '{statId.Value}' for dossier '{_dossierId}'. ");
                throw;
            }
        }

        private IExecutionResult DeleteCallScript(CallScriptId scriptId)
        {
            try
            {
                var command = new RemoveCallScriptCommand(scriptId);
                return _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't delete script '{scriptId.Value}' for dossier '{_dossierId}'. ");
                throw;
            }
        }

        private IExecutionResult DeleteTicket(TicketId ticketId)
        {
            try
            {
                var command = new DeleteTicketCommand(ticketId);
                return _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't delete ticket '{ticketId.Value}' for dossier '{_dossierId}'. ");
                throw;
            }
        }

        private IExecutionResult DeleteDossier(DossierId dossierId)
        {
            try
            {
                var command = new DeleteDossierCommand(dossierId);
                var result = _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                return result;
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't delete Dossier '{dossierId}'.");
                throw;
            }
        }

    }
}
