﻿using System;
using System.Threading;
using CRM.Domain.Dossier;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using WebPortal.CCMConnector.Application.Dossiers.Commands;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Dossiers.Commands
{
    public class ModifyWpDossierNameCommand
    {
        private ICommandBus _commandBus;

        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public ModifyWpDossierNameCommand(ICommandBus commandBus)
        {
            _commandBus = commandBus;
        }

        public IExecutionResult ModifyDossierName(string dossierId, string dossierName)
        {
            try
            {
                var aggregateId = DossierId.With(dossierId);
                var command = new ModifyDossierNameCommand(aggregateId, dossierName);
                var result = _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                Logger.Info($"Dossier '{dossierId}' now goes by the name: '{dossierName}'.");
                return result;
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't modify dossierName for dossier '{dossierId}' into '{dossierName}'");
                throw;
            }
        }


    }
}
