﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.CallScript;
using CRM.Domain.Duty;
using CRM.Domain.InteractionStatistic;
using CRM.Domain.Shared.Enums;
using CRM.Domain.Ticket;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using WebPortal.CCMConnector.Application.CallScripts.Commands;
using WebPortal.CCMConnector.Application.Duties.Commands;
using WebPortal.CCMConnector.Application.InteractionStats.Commands;
using WebPortal.CCMConnector.Application.Tickets.Commands;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Dossiers.Commands
{
    public class ReplaceDossierIdCommand
    {
        public ReplaceDossierIdCommand(ICommandBus commandBus)
        {
            _commandBus = commandBus;
        }

        private ICommandBus _commandBus;
        private readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        internal async Task ReplaceTheDossierIdsInDutiesTicketsAndStats (Dictionary<AggregateNameEnum, List<string>> typeAndIds, string dossierIdSource, string dossierIdDestination)
        {
            foreach (var tai in typeAndIds)
            {
                switch (tai.Key)
                {
                    case AggregateNameEnum.Duty:
                        foreach (var s in tai.Value)
                        {
                            var dId = DutyId.With(s);
                            await ReplaceDutyDossierId(dId, dossierIdDestination, dossierIdSource);
                        }
                        break;
                    case AggregateNameEnum.Statistic:
                        foreach (var s in tai.Value)
                        {
                            var sId = InteractionStatisticId.With(s);
                            await ReplaceStatDossierId(sId, dossierIdDestination, dossierIdSource);
                        }
                        break;
                    case AggregateNameEnum.Ticket:
                        foreach (var s in tai.Value)
                        {
                            var tId = TicketId.With(s);
                            await ReplaceTicketDossierId(tId, dossierIdDestination, dossierIdSource);
                        }
                        break;
                    case AggregateNameEnum.CallScript:
                        foreach (var s in tai.Value)
                        {
                            var cId = CallScriptId.With(s);
                            await ReplaceCallScriptDossierId(cId, dossierIdDestination, dossierIdSource);
                        }
                        break;
                }
            }

        }

        private async Task<IExecutionResult> ReplaceTicketDossierId(TicketId tId,
            string dossierIdDestination, string dossierIdSource)
        {
            try
            {
                var command = new ModifyTicketDossierIdCommand(tId,dossierIdDestination);
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't modify the DossierId in ticket '{tId.Value}' from '{dossierIdSource}' to '{dossierIdDestination}'.");
                throw;
            }
        }

        private async Task<IExecutionResult> ReplaceStatDossierId(InteractionStatisticId sId,
            string dossierIdDestination, string dossierIdSource)
        {
            try
            {
                var command = new ModifyInteractionStatDossierIdCommand(sId, dossierIdDestination);
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't modify the DossierId in statistic '{sId.Value}' from '{dossierIdSource}' to '{dossierIdDestination}'.");
                throw;
            }
        }

        private async Task<IExecutionResult> ReplaceDutyDossierId(DutyId dId,
            string dossierIdDestination, string dossierIdSource)
        {
            try
            {
                var command = new ModifyDutyDossierIdCommand(dId, dossierIdDestination);
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't modify the DossierId in duty '{dId.Value}' from '{dossierIdSource}' to '{dossierIdDestination}'.");
                throw;
            }
        }

        private async Task<IExecutionResult> ReplaceCallScriptDossierId(CallScriptId cId,
            string dossierIdDestination,
            string dossierIdSource)
        {
            try
            {
                var command = new ModifyCallScriptDossierIdCommand(cId, dossierIdDestination);
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't modify the DossierId in callscript '{cId.Value}' from '{dossierIdSource}' to '{dossierIdDestination}'.");
                throw;
            }
        }
    }
}
