﻿using System;
using System.Collections.Generic;
using System.Linq;
using WebPortal.CCMConnector.Application.Customers.Models;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Dossiers.QueriesCCM
{
    public class GetCustomerIdsInCcmTicketsQuery : ACCMCommandAndQuery
    {
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public static List<CcmTicketModel> GetCustomerIdsInCcmTickets(List<DutyAlarmModel> alarms)
        {
            try
            {
                var dossierIds = alarms
                    .Select(x => x.DossierId)
                    .Distinct()
                    .ToList();
                using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;
                    var dossiersAndCusts = ent.Customer
                        .Where(x => 
                            x.Mainfiche == null
                            && dossierIds.Contains(x.DossierIdWP))
                        .ToDictionary(v => v.DossierIdWP, x => x.CustomerId);

                    var rtn = alarms.Select(x => new CcmTicketModel()
                        {
                            CustomerId = dossiersAndCusts.Single(d => d.Key == x.DossierId).Value,
                            DutyAlarmId = x.Id,
                            EmailAddresses = x.SendMail ? x.EmailAddresses : null,
                            SmsNumbers = x.SendSms ? x.SmsNumbers : null
                        }).ToList()
                        ;
                    return rtn;
                }
            }
            catch (Exception e)
            {
                var dossierIds = alarms
                    .Select(x => x.DossierId)
                    .Distinct()
                    .ToList();
                var dossierIdString = "";
                dossierIds.ForEach(x => dossierIdString += $"{x}; ");
                Logger.Error(e, $"Couldn't get customerIds for the following dossierIds: {dossierIdString}");
                throw;
            }
        }
    }
}
