﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using EventFlow.Queries;
using WebPortal.Application.Dossiers.Queries;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Dossiers.QueriesWP
{
    public class GetDossierIdsQuery
    {
        private readonly IQueryProcessor _queryProcessor;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public GetDossierIdsQuery(IQueryProcessor queryProcessor)
        {
            _queryProcessor = queryProcessor;
        }

        internal void AddDossierIdsToMainAndSubfiches(List<IdsFromCCMandWP> mainAndSubficheIds)
        {
            try
            {
                var onlyMain = mainAndSubficheIds.Select(x => x.MainIdCCM).Distinct().ToList();
                var mainAndDossiers =
                    _queryProcessor.ProcessAsync(new GetDossierIdsByCCMIdsQuery(onlyMain), new CancellationToken())
                        .ConfigureAwait(false)
                        .GetAwaiter()
                        .GetResult();

                mainAndSubficheIds.ForEach(x => 
                    x.DossierId = mainAndDossiers
                        .SingleOrDefault(y => y.Value == x.MainIdCCM).Key);
            }
            catch (Exception e)
            {
               Logger.Error(e, $"Couldn't add DossierIds to main- and subficheIds.");
                throw;
            }

        }

        internal void AddDossierIdsToCustomerInfoForStats(List<IdsWithININVars> isdIdsWithIninVars)
        {
            var onlyMain = isdIdsWithIninVars.Select(x => x.MainIdCCM).Distinct().ToList();
            var mainAndDossiers =
                _queryProcessor.ProcessAsync(new GetDossierIdsByCCMIdsQuery(onlyMain), new CancellationToken())
                    .ConfigureAwait(false)
                    .GetAwaiter()
                    .GetResult();
            isdIdsWithIninVars.ForEach(x =>
                x.DossierId = mainAndDossiers
                    .SingleOrDefault(y => y.Value == x.MainIdCCM).Key);
        }
    }
}
