﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Shared.Enums;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.CallScripts.Queries;
using WebPortal.CCMConnector.Application.Dossiers.Models;
using WebPortal.CCMConnector.Application.Duties.Queries;
using WebPortal.CCMConnector.Application.InteractionStats.Queries;
using WebPortal.CCMConnector.Application.Tickets.Queries;
using WebPortal.CCMConnector.Data.ConnectorLogic.General.QueriesWP;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Dossiers.QueriesWP
{
    public class GetIdsForMultipleTypesByDossierIdQuery: AGetIdsForMultipleTypes
    {
        public GetIdsForMultipleTypesByDossierIdQuery(IQueryProcessor queryProcessor) : base(queryProcessor)
        {
        }

        internal override async Task<Dictionary<AggregateNameEnum, List<string>>> GetIdsBySharedId(string sharedId)
        {
            await base.GetIdsBySharedId(sharedId);

            await GetDutyIds(sharedId);
            await GetInteractionStatIds(sharedId);
            await GetCallscriptIds(sharedId);
            await GetTicketIds(sharedId);

            return _replaceDictionary;

        }

        //internal Dictionary<string, List<string>> GetIdsByDossierId(string dossierId)
        //{
        //    _replaceDictionary = new Dictionary<AggregateNameEnum, List<string>>();

        //    GetDutyIds(dossierId);
        //    GetInteractionStatIds(dossierId);
        //    GetCallscriptIds(dossierId);
        //    GetTicketIds(dossierId);

        //    return _replaceDictionary;
        //}

        private async Task GetDutyIds(
            string dossierId)
        {
            try
            {
                var ids = await _queryProcessor.ProcessAsync(new GetDutyIdsByDossierIdQuery(dossierId),
                    new CancellationToken());
                AddToDictionary(AggregateNameEnum.Duty, ids);
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not get DutyIds for dossier: '{dossierId}'");
                throw;
            }
        }

        private async Task GetInteractionStatIds(
            string dossierId)
        {
            try
            {
                var ids = await _queryProcessor.ProcessAsync(new GetStatisticIdsByDossierIdQuery(dossierId),
                    new CancellationToken());
                AddToDictionary(AggregateNameEnum.Statistic, ids);

            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not get InteractionStatIds for dossier: '{dossierId}'");
                throw;
            }
        }

        private async Task GetCallscriptIds(
            string dossierId)
        {
            try
            {
                var ids = await _queryProcessor.ProcessAsync(new GetCallScriptIdsByDossierIdQuery(dossierId),
                    new CancellationToken());
                AddToDictionary(AggregateNameEnum.CallScript, ids);

            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not get CallscriptIds for dossier: '{dossierId}'");
                throw;
            }
        }

        private async Task GetTicketIds(
            string dossierId)
        {
            try
            {
                var ids = await _queryProcessor.ProcessAsync(new GetTicketIdsByDossierIdQuery(dossierId),
                    new CancellationToken());
                AddToDictionary(AggregateNameEnum.Ticket, ids);

            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not get TicketIds for dossier: '{dossierId}'");
                throw;
            }
        }

    }
}
