﻿using System;
using System.Collections.Generic;
using System.Linq;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Duties.CommandsCCM
{
    public class ModifyDutyAlarmCommand: ACCMCommandAndQuery
    {
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public static void SetLastRunTimes(List<int> dutyAlarmIds)
        {
            try
            {
                using (var ent = new EDMX.ProdtelesecEntities())
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;
                    var toUpdate = ent.DutyAlarm
                        .Where(x => dutyAlarmIds.Contains(x.Id))
                        .ToList();

                    toUpdate.ForEach(x =>
                    {
                        x.LastRun = DateTime.UtcNow;
                        x.LastModifiedBy = "CCMConnector";
                        x.LastModifiedOn = DateTime.UtcNow;
                    });
                    ent.SaveChanges();
                }

            }
            catch (Exception e)
            {
                Logger.Error(e, "Couldn't set Lastrun for completed duty-alarms");
                throw;
            }
        }

        public static void SetNextRunTimes(Dictionary<int, DateTime> idsWithNextRun)
        {
            try
            {
                var dutyAlarmIds = idsWithNextRun.Keys;
                using (var ent = new EDMX.ProdtelesecEntities())
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;
                    var toUpdate = ent.DutyAlarm
                        .Where(x => dutyAlarmIds.Contains(x.Id))
                        .ToList();

                    toUpdate.ForEach(x =>
                    {
                        x.NextRun = idsWithNextRun.Single(y => y.Key == x.Id).Value;
                        x.LastModifiedBy = "CCMConnector";
                        x.LastModifiedOn = DateTime.UtcNow;
                    });
                    ent.SaveChanges();
                }

            }
            catch (Exception e)
            {
                Logger.Error(e, "Couldn't set NextRun for static date duty-alarms");
                throw;
            }
        }
    }
}
