﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Duty;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.Duties.Commands;
using WebPortal.CCMConnector.Application.Duties.Queries;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Duties.CommandsWP
{
    public class RemoveAllEmployeesFromDutiesByDossierIdCommand
    {
        public RemoveAllEmployeesFromDutiesByDossierIdCommand(ICommandBus commandBus,
            IQueryProcessor queryProcessor)
        {
            _commandBus = commandBus;
            _queryProcessor = queryProcessor;
        }

        private ICommandBus _commandBus;
        private readonly IQueryProcessor _queryProcessor;
        private readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        internal async Task RemoveAllEmployeesFromDutiesByDossierId(string dossierId)
        {
            var dutyIds = await GetDutiesByDossierId(dossierId);
            foreach (var dutyId in dutyIds)
            {
                await RemoveAllEmployeesFromDuty(dutyId);
            }
        }


        private async Task<List<string>> GetDutiesByDossierId(string dossierId)
        {
            try
            {
                var q = new GetDutyIdsByDossierIdQuery(dossierId);
                return await _queryProcessor.ProcessAsync(q, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't get duties from dossier: '{dossierId}'");
                throw;
            }
        }

        private async Task<IExecutionResult> RemoveAllEmployeesFromDuty(string dutyId)
        {
            try
            {
                var command = new RemoveAllEmployeesFromDutyCommand(DutyId.With(dutyId));
                return await _commandBus.PublishAsync(command, new CancellationToken());
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't remove employees from Duty '{dutyId}'");
                throw;
            }
        }
    }
}
