﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using EventFlow.Configuration;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.Customers.Models;
using WebPortal.CCMConnector.Application.Duties.Models;
using WebPortal.CCMConnector.Data.CcmTicketMaker;
using WebPortal.CCMConnector.Data.ConnectorLogic.Dossiers.QueriesCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.Duties.CommandsCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.Duties.QueriesCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.Duties.QueriesWP;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.IO;
using WebPortal.CCMConnector.MailSmtp;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Duties
{
    public class DutyLogic : SharedLogic
    {
        private readonly IQueryProcessor _queryProcessor;
        private SmsLogic _smsLogic;
        private List<DutyAlarmModel> _completedDutyAlarms;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public DutyLogic(IRootResolver serviceResolver)
        {
            _queryProcessor = serviceResolver.Resolve<IQueryProcessor>();
        }

        internal int CheckDutiesToExpire(DateTime previousRun)
        {
            _completedDutyAlarms = null;
            var dutyAlarms = GetDutyAlarmsQuery.GetDutyAlarms(_mapper.GetMapper(), previousRun);

            Logger.Info($"Applicable Duty-Alarms to check: {dutyAlarms.Count}");
            if (dutyAlarms.Count == 0) return 0;


            GoThroughDuties(dutyAlarms).ConfigureAwait(false).GetAwaiter().GetResult();

            SetNextDatesForDutyAlarmsWithFixedDate(dutyAlarms);

            if (_completedDutyAlarms == null || _completedDutyAlarms.Count == 0) return 0;

            return _completedDutyAlarms.Count;
        }

        private async Task GoThroughDuties(List<DutyAlarmModel> dutyAlarms)
        {
            var q = new GetDutyListToExpireQuery(_queryProcessor);
            List<Task> taskList = new List<Task>();

            foreach (var da in dutyAlarms)
            {

                var minimumDate = da.ExpireDateTime;
                var dutiesToExpire = q.GetDutiesToExpire(minimumDate, da.DossierId, da.OnFixedDate);

                if (dutiesToExpire.Count == 0 && !da.OnFixedDate) continue;

                taskList.Add(DetermineExpirationAndSendAlarms(da, dutiesToExpire, q, da.DossierId));

                if (_completedDutyAlarms == null) continue;

                var ccmTickets = GetCustomerIdsInCcmTicketsQuery
                    .GetCustomerIdsInCcmTickets(_completedDutyAlarms);

                TicketMaker.PostNewCcmTickets(ccmTickets, _mapper.GetMapper());

                ModifyDutyAlarmCommand.SetLastRunTimes(_completedDutyAlarms
                    .Select(x => x.Id).ToList());


            }

            #region Foreach obv dossierId (verwijderen van zodra bovenstaande werkt)
            //var dossierIds = dutyAlarms
            //    .Select(x => x.DossierId)
            //    .Distinct()
            //    .ToList();

            //foreach (var dossierId in dossierIds)
            //{
            //    var minimumDate = dutyAlarms
            //        .Where(x => x.DossierId == dossierId)
            //        .OrderBy(x => x.ExpireDateTime)
            //        .First().ExpireDateTime;

            //    var dutiesToExpire = q
            //        .GetDutiesToExpire(minimumDate, dossierId);

            //    if (dutiesToExpire.Count == 0
            //        && dutyAlarms.FirstOrDefault(x => x.OnFixedDate) == null) continue;

            //    taskList.Add(DetermineExpirationAndSendAlarms(dutyAlarms, dutiesToExpire, q, dossierId));

            //    if (_completedDutyAlarms == null) continue;

            //    var ccmTickets = GetCustomerIdsInCcmTicketsQuery
            //        .GetCustomerIdsInCcmTickets(_completedDutyAlarms);

            //    TicketMaker.PostNewCcmTickets(ccmTickets, _mapper.GetMapper());



            //    ModifyDutyAlarmCommand.SetLastRunTimes(_completedDutyAlarms
            //        .Select(x => x.Id).ToList());

            //}

            #endregion

            await Task.WhenAll(taskList);

        }

        private void SetNextDatesForDutyAlarmsWithFixedDate(List<DutyAlarmModel> dutyAlarms)
        {
            var dutyAlarmsWithFixedDate = dutyAlarms.Where(x => x.OnFixedDate)
                .ToList();
            if (dutyAlarmsWithFixedDate.Count == 0) return;

            dutyAlarmsWithFixedDate.ForEach(x => x.SetNextRun());
            var dict = dutyAlarmsWithFixedDate
                .Where(x => x.NextRun.HasValue)
                .ToDictionary(x => x.Id, y => y.NextRun.Value);
            ModifyDutyAlarmCommand.SetNextRunTimes(dict);

        }

        private async Task DetermineExpirationAndSendAlarms(DutyAlarmModel da,
            List<GetDutyToExpireModel> dutiesToExpire,
            GetDutyListToExpireQuery q, string dossierId)
        {
            List<Task> taskList = new List<Task>();
            if (dutiesToExpire.Count == 0 && da.OnFixedDate)
            {
                CompleteTheDutyAlarmAndSend(q, da, taskList, dutiesToExpire);
                return;
            }
            var duties = dutiesToExpire
                .Where(x =>
                             x.ExpireDateTime >= DateTime.Now
                            && (string.IsNullOrWhiteSpace(da.Category)
                                || x.Category == da.Category))
                .ToList();

            if (duties.Count == 0)
            {
                if (da.OnFixedDate) CompleteTheDutyAlarmAndSend(q, da, taskList, duties);

                return;
            }


            GetDutyToExpireModel firstInTheFutureNoEmployees = null;
            GetDutyToExpireModel firstInTheFutureWithEmployees = null;
            if (duties.Any(x => x.FullStartDateTime > DateTime.Now))
            {
                var temp = duties
                    .Where(x => x.FullStartDateTime > DateTime.Now)
                    .OrderBy(x => x.FullStartDateTime)
                    .First();
                if (temp.AmountOfEmployees == 0)
                    firstInTheFutureNoEmployees = temp;
                else
                {
                    firstInTheFutureWithEmployees = temp;
                }
            }


            if (da.OnFixedDate)
            {
                if (duties.Any(x => x.FullStartDateTime < DateTime.Now
                                    && x.ExpireDateTime > DateTime.Now.AddDays(2)))
                {
                    return;
                }
                //Alle duties vervallen binnen twee dagen.
                //Is er echter een duty die in de toekomst begint?

                if (duties.Any(x => x.FullStartDateTime > DateTime.Now) && firstInTheFutureNoEmployees == null)
                {
                    return;
                }

                duties = duties.Where(x 
                    => x.ExpireDateTime <= DateTime.Now.AddDays(2)).ToList();

                if (firstInTheFutureNoEmployees != null)
                    duties.Add(firstInTheFutureNoEmployees);

                //haal alle dutiesWithinCat uit duties.
                //duties.ForEach(x => duties.Remove(x));
            }
            else
            {
                if (firstInTheFutureWithEmployees != null) return;

                var notExpired = duties.Where(x => x.FullStartDateTime < DateTime.Now && x.ExpireDateTime > da.ExpireDateTime).ToList();
                if (notExpired.Count > 0) return;

                duties = duties.Where(x => x.FullStartDateTime < DateTime.Now && x.ExpireDateTime <= da.ExpireDateTime).ToList();


                if (firstInTheFutureNoEmployees != null)
                {

                    if (duties.Count == 0)
                    {
                        duties = new List<GetDutyToExpireModel>(){ firstInTheFutureNoEmployees };
                    }
                    else
                    {
                        duties.Add(firstInTheFutureNoEmployees);
                    }
                }
                else
                {
                    if (duties.Count == 0) return;

                }

                #region Verwijder dit later
                //var lastExpireDate = duties.OrderByDescending(x => x.ExpireDateTime).First().ExpireDateTime;


                //var nextDuties = dutiesToExpire
                //    .Where(x => x.DossierId == da.DossierId
                //                && x.ExpireDateTime > lastExpireDate
                //    )
                //    .OrderBy(x => x.FullStartDateTime)
                //    .ToList();
                //bool doWeFindAnEmployee = false;
                //if (nextDuties.Count > 0)
                //{
                //    //heeft de eerst volgende een employee?
                //    doWeFindAnEmployee = nextDuties.First().AmountOfEmployees > 0;
                //}

                //if (nextDuties.Count > 0 && doWeFindAnEmployee)
                //{
                //    //is er een duty met dezelde categorie? Zo ja, haal deze dan uit de duties.
                //    dutiesWithinCat.ForEach(x => duties.Remove(x));
                //}
                #endregion
            }

            #region Checken per categorie (houd ik bij indien ooit terug van toepassing)
            ////ga nu na of er binnen de categorieën van de gevonden duties toch opvolgers zijn die binnen de tijd vallen.
            //var categories = new List<string>(duties
            //    .Select(x => x.Category)
            //    .Distinct()
            //    .ToList());

            //foreach (var cat in categories)
            //{
            //    var dutiesWithinCat = duties
            //        .Where(x => x.Category == cat).ToList();

            //    if (dutiesWithinCat.Count == 0) continue;

            //    if (da.OnFixedDate)
            //    {
            //        if (dutiesWithinCat.Any(x => x.ExpireDateTime > DateTime.Now.AddDays(2)))
            //        {
            //            dutiesWithinCat.ForEach(x => duties.Remove(x));
            //            continue;
            //        }
            //        if (!dutiesWithinCat.Any(x => x.FullStartDateTime > DateTime.Now))
            //        {
            //            continue;
            //        }

            //        if (dutiesWithinCat.Any(x => x.AmountOfEmployees == 0
            //                                     & x.FullStartDateTime > DateTime.Now))
            //        {
            //            var firstInTheFutureNoEmployees = dutiesWithinCat
            //                                                  .Where(x => x.FullStartDateTime > DateTime.Now)
            //                                                  .OrderBy(x => x.FullStartDateTime)
            //                                                  .First()
            //                                                  .AmountOfEmployees == 0;

            //            if (firstInTheFutureNoEmployees) continue;


            //            //De eerste in de toekomst heeft wel employees.
            //        }
            //        //haal alle dutiesWithinCat uit duties.
            //        dutiesWithinCat.ForEach(x => duties.Remove(x));

            //    }
            //    else
            //    {

            //        var lastExpireDate = dutiesWithinCat.OrderByDescending(x => x.ExpireDateTime).First().ExpireDateTime;


            //        var nextDuties = dutiesToExpire
            //            .Where(x => x.DossierId == da.DossierId
            //                        && x.Category == cat
            //                        && x.ExpireDateTime > lastExpireDate
            //                        )
            //            .OrderBy(x => x.FullStartDateTime)
            //            .ToList();
            //        bool doWeFindAnEmployee = false;
            //        if (nextDuties.Count > 0)
            //        {
            //            //heeft de eerst volgende een employee?
            //            doWeFindAnEmployee = nextDuties.First().AmountOfEmployees > 0;
            //        }

            //        if (nextDuties.Count > 0 && doWeFindAnEmployee)
            //        {
            //            //is er een duty met dezelde categorie? Zo ja, haal deze dan uit de duties.
            //            dutiesWithinCat.ForEach(x => duties.Remove(x));
            //        }
            //    }
            //}


            #endregion
            if (duties.Count == 0) return;

            CompleteTheDutyAlarmAndSend(q, da, taskList, duties);

            await Task.WhenAll(taskList);
        }

        private void CompleteTheDutyAlarmAndSend(GetDutyListToExpireQuery q,
            DutyAlarmModel da,
            List<Task> taskList,
            List<GetDutyToExpireModel> duties)
        {
            var fullDutyAlarm = q.GetDossierAndcustomerInfoForDutyAlarm(da.DossierId);
            fullDutyAlarm.CompleteTheDutyAlarm(da);

            //if (_completedDutyAlarms == null) _completedDutyAlarms = new List<DutyAlarmModel>();

            _completedDutyAlarms = new List<DutyAlarmModel>();

            _completedDutyAlarms.Add(fullDutyAlarm);

            taskList.Add(SendEmailOrSms(fullDutyAlarm, duties));
        }

        private async Task SendEmailOrSms(DutyAlarmModel alarm,
            List<GetDutyToExpireModel> duties)
        {
            string timeString = "";
            if (!alarm.OnFixedDate)
            {
                timeString += alarm.TimeWarning.Days > 0 
                    ? alarm.TimeWarning.Days == 1 
                        ? $" {alarm.TimeWarning.Days} day" 
                        : $" {alarm.TimeWarning.Days} days"
                    : "";
                timeString += alarm.TimeWarning.Hours > 0 
                    ? alarm.TimeWarning.Hours == 1 
                        ? $" {alarm.TimeWarning.Hours} hour" 
                        : $" {alarm.TimeWarning.Hours} hours"
                    : "";
                timeString += alarm.TimeWarning.Minutes > 0 
                    ? alarm.TimeWarning.Minutes == 1 
                        ? $" {alarm.TimeWarning.Minutes} minute" 
                        : $" {alarm.TimeWarning.Minutes} minutes"
                    : "";
            }

            if (alarm.SendSms)
            {
                var smsText = $"Attention from {alarm.Tenant}-portal for {alarm.CustomerName}: ";

                if (alarm.OnFixedDate)
                {
                    smsText += $"There is no future duty planned";
                    smsText += string.IsNullOrWhiteSpace(alarm.Category) ? "." : $"for category {alarm.Category}.";
                }
                else
                {
                    string dutyString = duties.Count > 1 ? "duties are" : "duty is";

                    smsText += alarm.TimeWarning == new TimeSpan()
                        ? $"{duties.Count} {dutyString} now expired."
                        : $"{duties.Count} {dutyString} to expire within{timeString}.";
                }

                string receivers = alarm.SmsNumbers.Replace(';', ',');
                if (_smsLogic == null) _smsLogic = new SmsLogic();

                _smsLogic.SendSms(receivers, smsText);

            }

            if (alarm.SendMail)
            {
                var txtBody = MailLogic.MakeTxtBody(alarm, duties, timeString);
                var html = MailLogic.MakeHtmlBody(alarm, duties, timeString);

                var subject = $"Duty-Alarm from {alarm.Tenant}-portal for {alarm.CustomerName}";
                await Mailer.SendMail(alarm.Tenant, subject, html, txtBody, alarm.EmailAddresses);
                Logger.Info($"Successfully send email to: {alarm.EmailAddresses}");
            }
        }
    }
}
