﻿using System;
using System.Collections.Generic;
using System.Threading;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.Customers.Models;
using WebPortal.CCMConnector.Application.Customers.Queries;
using WebPortal.CCMConnector.Application.Duties.Models;
using WebPortal.CCMConnector.Application.Duties.Queries;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Duties.QueriesWP
{
    public class GetDutyListToExpireQuery
    {
        private readonly IQueryProcessor _queryProcessor;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public GetDutyListToExpireQuery(
            IQueryProcessor queryProcessor)
        {
            _queryProcessor = queryProcessor;
        }

        internal List<GetDutyToExpireModel> GetDutiesToExpire(
            DateTime minimumExpired, string dossierId, bool onFixedDate)
        {
            try
            {
                return _queryProcessor.ProcessAsync(new GetDutiesToExpireQuery(minimumExpired, dossierId, onFixedDate),
                    new CancellationToken())
                    .ConfigureAwait(false)
                    .GetAwaiter()
                    .GetResult();

            }
            catch (Exception e)
            {
                
                Logger.Error(e, $"Couldn't get about to expire duties for this dossierId: {dossierId}");
                throw;
            }
        }

        internal DutyAlarmModel GetDossierAndcustomerInfoForDutyAlarm(string dossierId)
        {
            try
            {
                return _queryProcessor.ProcessAsync(new GetCustomerNameTenantAndDossierNameQuery(dossierId),
                    new CancellationToken())
                    .ConfigureAwait(false)
                    .GetAwaiter()
                    .GetResult();

            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't get dossier and customer info for Dutyalarm with dossierId: {dossierId} ");
                throw;
            }
        }
    }
}
