﻿using System.Collections.Generic;
using CRM.Domain.Employee.Entities;
using CRM.Domain.Employee.Models;
using CRM.Domain.Shared.Enums;
using WebPortal.CCMConnector.Data.Models;
using WebPortal.CCMConnector.Data.Validation;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Employees.CommandsWP
{
    public class AddEmployeeContactDetailsCommand
    {
        internal static List<EmployeeContactDetailModel> MakeContactDetailList(TempEmployeeItem tei)
        {
            List<EmployeeContactDetailModel> rtn = new List<EmployeeContactDetailModel>();
            if (!string.IsNullOrWhiteSpace(tei.OfficePhone))
                rtn.Add(MakeContactDetail("Office Phone", tei.OfficePhone, ContactDetailType.TelephoneNr, tei.IsOfficePhonePrivate));

            if (!string.IsNullOrWhiteSpace(tei.PrivatePhone))
                rtn.Add(MakeContactDetail("Private Phone", tei.PrivatePhone, ContactDetailType.TelephoneNr, tei.IsPrivatePhonePrivate));

            if (!string.IsNullOrWhiteSpace(tei.Email))
            {
                var allEmails = new TempEmail()
                {
                    EmailAddress = $"{tei.Email}"
                };
                SplitAndValidate.ValidateAndSplitEmails(allEmails);
                int emailcounter = 0;
                foreach (var m in allEmails.EmailList)
                {
                    string email = emailcounter > 0 ? $"Email {emailcounter}" : "Email";
                    rtn.Add(MakeContactDetail(email, m, ContactDetailType.Email, tei.IsEmailPrivate));
                    emailcounter++;
                }
            }

            if (!string.IsNullOrWhiteSpace(tei.GSM))
                rtn.Add(MakeContactDetail("Cell Phone", tei.GSM, ContactDetailType.TelephoneNr, tei.IsGSMPrivate));

            return rtn;
        }

        private static EmployeeContactDetailModel MakeContactDetail(string contactDetailName, string contactDetailValue, ContactDetailType contactType, bool isPrivate = false)
        {
            EmployeeContactDetailModel detail = new EmployeeContactDetailModel(EmployeeContactDetailId.New.Value,
                contactDetailName,
                contactDetailValue, isPrivate, false, false, contactType);
            return detail;
        }


    }
}
