﻿using System;
using System.Collections.Generic;
using System.Threading;
using CRM.Domain.Employee;
using CRM.Domain.Shared.Enums;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using WebPortal.CCMConnector.Application.Employees.Commands;
using WebPortal.CCMConnector.Application.Employees.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Employees.CommandsWP
{
    public class ModifyContactDetailsTypeCommand
    {
        private readonly ICommandBus _commandBus;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();
        public ModifyContactDetailsTypeCommand(ICommandBus commandBus)
        {
            _commandBus = commandBus;
        }

        public void ModifyContactDetails(List<GetContactDetailsByTypeModel> contactDetails, ContactDetailType to)
        {
            foreach (var cd in contactDetails)
            {
                ModifyContactDetailType(cd, to);
            }
        }

        private IExecutionResult ModifyContactDetailType(GetContactDetailsByTypeModel contactDetail, ContactDetailType to)
        {
            try
            {
                var command = new ModifyContactDetailTypeCommand(EmployeeId.With(contactDetail.EmployeeId),
                    contactDetail.ContactDetailId, to);
                var result = _commandBus.PublishAsync(command, new CancellationToken())
                    .ConfigureAwait(false)
                    .GetAwaiter()
                    .GetResult();
                return result;
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't set ContactDetailtype with Id '{contactDetail.ContactDetailId}' from '{contactDetail.DetailType}' to '{to}'" );
                throw;
            }
        }
    }
}
