﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Shared.Enums;
using EventFlow;
using EventFlow.Configuration;
using EventFlow.Queries;
using WebPortal.CCMConnector.Application.Employees.Models;
using WebPortal.CCMConnector.Application.Employees.Queries;
using WebPortal.CCMConnector.Data.ConnectorLogic.Employees.CommandsCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.Employees.CommandsWP;
using WebPortal.CCMConnector.Data.ConnectorLogic.Employees.QueriesCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.General.QueriesCCM;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Employees
{
    public class EmployeeLogic
    {
        private readonly ICommandBus _commandBus;
        private readonly IQueryProcessor _queryProcessor;
        private List<GetContactDetailsByTypeModel> _contactDetailsToChangeTypeFor;

        public int AmountOfContactDetailsToModifyTypeFor { get; set; }

        public EmployeeLogic(IRootResolver serviceResolver)
        {
            _commandBus = serviceResolver.Resolve<ICommandBus>();
            _queryProcessor = serviceResolver.Resolve<IQueryProcessor>();
        }

        internal void UploadEmployees(List<WPModules> treeviewKeys)
        {
            var clientIds = treeviewKeys
                .Where(x => x.CCMLevel == "CLIENT")
                .Select(x => x.CCMId).ToList();

            var mainCustIds = treeviewKeys
                .Where(x => x.CCMLevel == "MAIN")
                .Select(x => x.CCMId).ToList();

            var subCustIds = treeviewKeys
                .Where(x => x.CCMLevel == "SUB")
                .Select(x => x.CCMId).ToList();

            var allCustIds = GetCustomerIdsQuery
                .GetCustomerIds(true, clientIds, mainCustIds, subCustIds);

            var tempEmployees = new GetEmployeesFromCCMQuery().GetEmployeesFromCCM(allCustIds);

            tempEmployees.ForEach(x =>
                x.MainAndSubficheIdAndCustomerId.CustomerId =
                    treeviewKeys.FirstOrDefault(y => y.CCMId == x.CustomerId).CustomerIdWP);

            foreach (var employee in tempEmployees)
            {
                employee.ContactDetails = AddEmployeeContactDetailsCommand.MakeContactDetailList(employee);

            }

            var filteredEmployees = RemoveAndCombineDuplicateEmployees(tempEmployees);
            FilterLastNames(tempEmployees);
            new CreateEmployeesCommand(_commandBus).CreateEmployees(filteredEmployees);
            SetEmployeesAsUploadedCommand.SetEmployeesAsUploaded(tempEmployees.Select(x => x.MainAndSubficheIdAndCustomerId.CustomerIdCCM).ToList());
        }

        private void FilterLastNames(List<TempEmployeeItem> emps)
        {
            foreach (var emp in emps)
            {
                FilterLastName(emp);
            }
        }

        private static void FilterLastName(TempEmployeeItem emp)
        {
            var firstNameWords = emp.Firstname.Split();

            if (firstNameWords.Length == 0)
                return;

            var lastNameWords = emp.LastName.Split();

            if (firstNameWords.Length >= lastNameWords.Length)
                return;

            //eerst eenvoudig. firstNameWords == 1 woord.
            //kijk of het laatste woord van lastname overeenstemt
            if (firstNameWords.Length == 1)
            {
                if (emp.Firstname.ToUpper() == lastNameWords[lastNameWords.Length - 1].ToUpper())
                {
                    //laatste woord verwijderen van lastname
                    emp.LastName = emp.LastName.Substring(0, emp.LastName.Length - emp.Firstname.Length - 1);
                }
                return;

            }
            //FirstName bestaat uit meerdere woorden.
            int l = firstNameWords.Length;

            //Veronderstel FirstName = 'Jan Bart' == 8 length
            //LastName = 'Aerts Jan Bart' == 14 length
            //StartIndex = 6 (dus 14 - 8)
            //Lenght = 8

            //Veronderstel LastName == 'Van Der Aa' == 11 length
            //Substring(6, 8) == error

            //maar subsrting = 11 - 8. (3,8)

            int dif = emp.LastName.Length - emp.Firstname.Length;

            if (dif <= 0)
            {
                return;
            }

            var subLastName = emp.LastName
                    .Substring(dif, emp.Firstname.Length).ToUpper();
            if (emp.Firstname.ToUpper() == subLastName)
            {
                emp.LastName = emp.LastName.Substring(0, emp.LastName.Length - emp.Firstname.Length - 1);

            }

        }

        private List<TempEmployeeItem> RemoveAndCombineDuplicateEmployees(List<TempEmployeeItem> emps)
        {
            List<TempEmployeeItem> rtn = new List<TempEmployeeItem>();
            emps = emps.OrderBy(x => x.LastName).ToList();
            string lastName = "";
            string firstName = "";
            IdsFromCCMandWP mainAndSubandCust = null;
            int original = 0;
            for (int i = 1; i < emps.Count; i++)
            {
                if (emps[original].Equals(emps[i]))
                {
                    //We moeten checken of de original toch verschilt in sommige elementen en daarop zowel Tags als ContactDetails toevoegen.
                    emps[original].CombineWithOther(emps[i]);
                    //Blijkt dat een contactDetail op private staat, dan wordt deze aangepast.
                    if (i == emps.Count - 1)
                    {
                        rtn.Add(emps[original]);
                    }
                }
                else
                {
                    rtn.Add(emps[original]);
                    //Voeg de original toe aan de rtn-lijst en pas daarna de int original aan.
                    original = i;
                    if (i == emps.Count - 1)
                    {
                        rtn.Add(emps[i]);
                    }

                }
            }

            return rtn;
        }


        public async Task<string> GetContactDetailsByType(ContactDetailType from)
        {
            var q = new GetContactDetailsByTypeQuery(from);
            _contactDetailsToChangeTypeFor = await _queryProcessor
                .ProcessAsync(q, new CancellationToken());

            AmountOfContactDetailsToModifyTypeFor = _contactDetailsToChangeTypeFor.Count;


            return $"We have found {AmountOfContactDetailsToModifyTypeFor} of type {from}";
        }

        public string ReplaceContactDetailTypes(ContactDetailType from,
            ContactDetailType to)
        {
            new ModifyContactDetailsTypeCommand(_commandBus).ModifyContactDetails(_contactDetailsToChangeTypeFor, to);
            return $"{AmountOfContactDetailsToModifyTypeFor} ContactDetailTypes are replaced from '{from}' to '{to}'";
        }
    }
}
