﻿using System.Collections.Generic;
using System.Linq;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Employees.QueriesCCM
{
    public class GetEmployeesFromCCMQuery : ACCMCommandAndQuery
    {
        internal List<TempEmployeeItem> GetEmployeesFromCCM(List<IdsFromCCMandWP> mainAndSubficheIds)
        {
            List<int> custIds = mainAndSubficheIds
                .Select(x => x.CustomerIdCCM)
                .Distinct().ToList();
            string nullString = null;
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var employees = ent.Treeview
                    .Where(x => custIds.Contains(x.CustomerId)
                    && x.UploadedInWP != true)
                    .Select(x => new TempEmployeeItem()
                    {
                        CC = x.CC,
                        Comments = x.CommentText,
                        CustomerId = x.CustomerId,
                        Email = x.Email,
                        Firstname = x.Firstname,
                        LastName = x.LastName,
                        GSM = x.GSM,
                        IsEmailPrivate = x.IsEmailPrivate,
                        IsFirstNamePrivate = x.IsFirstNamePrivate,
                        IsGSMPrivate = x.IsGSMPrivate,
                        IsLastNamePrivate = x.IsLastNamePrivate,
                        IsOfficePhonePrivate = x.IsOfficePhonePrivate,
                        IsPositionPrivate = x.IsPositionPrivate,
                        IsPrivatePhonePrivate = x.IsPrivatePhonePrivate,
                        IsServicePrivate = x.IsServicePrivate,
                        MainFicheId = x.Customer.Mainfiche,
                        TagServiceName = x.Service_Organisation.SERVICE_NAME == "Services"
                            ? nullString
                            : x.Service_Organisation.SERVICE_NAME,
                        Position = x.Position,
                        PrivatePhone = x.PrivatePhone,
                        OfficePhone = x.OfficePhone,
                        SendMail = x.SendMail,
                        SetMessageToRead = x.SetMessageToRead
                    })
                    .ToList();
                employees.ForEach(x =>
                {
                    x.MainAndSubficheIdAndCustomerId = mainAndSubficheIds
                        .FirstOrDefault(y => y.CustomerIdCCM == x.CustomerId);
                    x.Comments = string.IsNullOrWhiteSpace(x.Comments) ? "":  x.Comments.Trim();
                    x.Email = x.Email.Trim();
                    x.Firstname = x.Firstname.Trim();
                    x.LastName = x.LastName.Trim();
                    x.GSM = x.GSM.Trim();
                    x.Position = string.IsNullOrWhiteSpace(x.Position)
                        ? x.Position
                        : $"#{x.Position.Trim()}";
                    x.PrivatePhone = x.PrivatePhone.Trim();
                    x.OfficePhone = x.OfficePhone.Trim();
                    x.TagServiceName = string.IsNullOrWhiteSpace(x.TagServiceName)
                        ? null
                        : $"*{x.TagServiceName.Trim()}";
                });

                return employees;
            }
        }
    }
}
