﻿using System;
using CRM.Domain.Branch;
using CRM.Domain.Branch.ReadModels;
using CRM.Domain.CallScript;
using EventFlow;
using EventFlow.Configuration;
using EventFlow.Extensions;
using EventFlow.MongoDB.Extensions;
using CRM.Domain.Ticket.ReadModels;
using CRM.Domain.Customer;
using CRM.Domain.CallScript.ReadModels;
using CRM.Domain.Customer.ReadModels;
using CRM.Domain.Dossier;
using CRM.Domain.Dossier.ReadModels;
using CRM.Domain.Duty;
using CRM.Domain.Duty.ReadModels;
using CRM.Domain.Employee;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.InteractionStatistic;
using CRM.Domain.InteractionStatistic.ReadModels;
using CRM.Domain.Ticket;
using CRM.Domain.User;
using CRM.Domain.User.ReadModels;
using WebPortal.Application.Dossiers.Queries;
using WebPortal.CCMConnector.Application.Branches.Commands;
using WebPortal.CCMConnector.Application.Branches.Queries;
using WebPortal.CCMConnector.Application.CallScripts.Commands;
using WebPortal.CCMConnector.Application.CallScripts.Queries;
using WebPortal.CCMConnector.Application.Customers.Commands;
using WebPortal.CCMConnector.Application.Customers.Queries;
using WebPortal.CCMConnector.Application.InteractionStats.Commands;
using WebPortal.CCMConnector.Application.Tickets.Commands;
using WebPortal.CCMConnector.Application.Dossiers.Commands;
using WebPortal.CCMConnector.Application.Dossiers.Queries;
using WebPortal.CCMConnector.Application.Duties.Commands;
using WebPortal.CCMConnector.Application.Duties.Queries;
using WebPortal.CCMConnector.Application.Employees.Commands;
using WebPortal.CCMConnector.Application.Employees.Queries;
using WebPortal.CCMConnector.Application.InteractionStats.Queries;
using WebPortal.CCMConnector.Application.Tickets.Queries;
using WebPortal.CCMConnector.Application.Users.Commands;
using WebPortal.CCMConnector.Application.Users.Queries;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.General
{
    public class ConfigureServices
    {
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();
        public static IRootResolver ConfigureServicesWithHandlersMC(string connectionString, string databaseName)
        {
            try
            {
                return EventFlowOptions.New
                    //.AddDefaults(typeof(CreateCustomerCommand).Assembly)
                    .AddDefaults(typeof(CustomerAggregate).Assembly)
                    .AddDefaults(typeof(BranchAggregate).Assembly)
                    .AddDefaults(typeof(DossierAggregate).Assembly)
                    .AddDefaults(typeof(InteractionStatisticAggregate).Assembly)
                    .AddDefaults(typeof(CallScriptAggregate).Assembly)
                    .AddDefaults(typeof(TicketAggregate).Assembly)
                    .AddDefaults(typeof(EmployeeAggregate).Assembly)
                    .AddDefaults(typeof(DutyAggregate).Assembly)
                    .AddDefaults(typeof(UserAggregate).Assembly)
                    .AddCommands(new Type[]
                    {
                        typeof(CreateCustomerCommand),
                        typeof(ModifyCustomerMainCustomerBranchCommand),
                        typeof(CreateInteractionStatCommand),
                        typeof(CreateTicketCommand),
                        typeof(CreateDossierCommand),
                        typeof(CreateBranchCommand),
                        typeof(ModifyTicketCommand),
                        typeof(CreateEmployeeCommand),
                        typeof(CreateCallScriptCommand),
                        typeof(ModifyCallScriptCommand),
                        typeof(ModifyDossierNameCommand),
                        typeof(ModifyCustomerNameCommand),
                        typeof(ModifiyCustomerWebPortalLicenseAmountCommand),
                        typeof(ModifyCustomerSubscriptionsCommand),
                        typeof(DeleteDossierCommand),
                        typeof(DeleteTicketCommand),
                        typeof(DeleteInteractionStatCommand),
                        typeof(RemoveCallScriptCommand),
                        typeof(DeleteDutyCommand),
                        typeof(ModifyDutyDossierIdCommand),
                        typeof(ModifyTicketDossierIdCommand),
                        typeof(ModifyContactDetailTypeCommand),

                        typeof(ModifyBranchCustomerIdCommand),
                        typeof(DeleteBranchCommand),
                        typeof(ModifyCallScriptDossierIdCommand),
                        typeof(DeleteCustomerCommand),
                        typeof(ModifyDossierCustomerIdCommand),
                        typeof(ModifyEmployeeCustomerIdCommand),
                        typeof(ModifyInteractionStatDossierIdCommand),
                        typeof(RemoveAllEmployeesFromDutyCommand),

                        typeof(ModifyInteractionStatTicketIdCommand),
                        typeof(ModifyBranchAddressCommand),
                        typeof(ModifyBranchContactDetailsCommand),
                        typeof(DeleteUserCommand),
                        typeof(CreateUserCommand),
                        typeof(DeleteEmployeeCommand)


                    })
                    .AddCommandHandlers(new Type[]
                    {
                        typeof(CreateCustomerCommandHandler),
                        typeof(ModifyCustomerMainCustomerBranchCommandHandler),
                        typeof(CreateInteractionStatCommandHandler),
                        typeof(CreateTicketCommandHandler),
                        typeof(CreateDossierCommandHandler),
                        typeof(CreateBranchCommandHandler),
                        typeof(ModifyTicketCommandHandler),
                        typeof(CreateEmployeeCommandHandler),
                        typeof(CreateCallScriptCommandHandler),
                        typeof(ModifyCallScriptCommandHandler),
                        typeof(ModifyDossierNameCommandHandler),
                        typeof(ModifyCustomerNameCommandHandler),
                        typeof(ModifiyCustomerWebPortalLicenseAmountCommandHandler),
                        typeof(ModifyCustomerSubscriptionsCommandHandler),
                        typeof(DeleteDossierCommandHandler),
                        typeof(DeleteTicketCommandHandler),
                        typeof(DeleteInteractionStatCommandHandler),
                        typeof(RemoveCallScriptCommandHandler),
                        typeof(DeleteDutyCommandHandler),
                        typeof(ModifyDutyDossierIdCommandHandler),
                        typeof(ModifyTicketDossierIdCommandHandler),
                        typeof(ModifyContactDetailTypeCommandHandler),

                        typeof(ModifyBranchCustomerIdCommandHandler),
                        typeof(DeleteBranchCommandHandler),
                        typeof(ModifyCallScriptDossierIdCommandHandler),
                        typeof(DeleteCustomerCommandHandler),
                        typeof(ModifyDossierCustomerIdCommandHandler),
                        typeof(ModifyEmployeeCustomerIdCommandHandler),
                        typeof(ModifyInteractionStatDossierIdCommandHandler),
                        typeof(RemoveAllEmployeesFromDutyCommandHandler),

                        typeof(ModifyInteractionStatTicketIdCommandHandler),
                        typeof(ModifyBranchAddressCommandHandler),
                        typeof(ModifyBranchContactDetailsCommandHandler),
                        typeof(DeleteUserCommandHandler),
                        typeof(CreateUserCommandHandler),
                        typeof(DeleteEmployeeCommandHandler)
                    })
                    .AddQueryHandlers(new Type[]
                    {
                        typeof(GetDossierIdsByCCMIdsQueryHandler),
                        typeof(GetTicketIdsByCCMMessageIdsQueryHandler), 
                        typeof(GetDossierIdsAndTagsQueryHandler),
                        typeof(GetCustomersWithDossiersQueryHandler),

                        typeof(GetCallScriptIdsByDossierIdQueryHandler),
                        typeof(GetTicketIdsByDossierIdQueryHandler),
                        typeof(GetDutyIdsByDossierIdQueryHandler),
                        typeof(GetDutiesToExpireQueryHandler),
                        typeof(GetStatisticIdsByDossierIdQueryHandler),
                        typeof(GetFaultyDossiersQueryHandler),
                        typeof(GetCustomerNameTenantAndDossierNameQueryHandler),
                        typeof(GetFaultyTicketsQueryHandler),
                        typeof(GetContactDetailsByTypeQueryHandler),

                        typeof(GetDossierIdsByCustomerIdQueryHandler),
                        typeof(GetBranchIdsByCustomerIdQueryHandler),
                        typeof(GetEmployeeIdsByCustomerIdQueryHandler),

                        typeof(GetStatisticsWithMessageIdCcmQueryHandler),
                        typeof(GetHqBranchByCustomerIdQueryHandler),
                        typeof(GetBranchByBranchIdQueryHandler),
                        typeof(GetUserIdsByCustomerIdQueryHandler)

                    })


                    .ConfigureMongoDb(connectionString, databaseName)
                    .UseMongoDbEventStore()
                    .UseMongoDbSnapshotStore()
                    .UseMongoDbReadModel<CallScriptReadModel>()
                    .UseMongoDbReadModel<CustomerReadModel>()
                    .UseMongoDbReadModel<DossierReadModel>()
                    .UseMongoDbReadModel<InteractionStatisticReadModel>()
                    .UseMongoDbReadModel<TicketReadModel>()
                    .UseMongoDbReadModel<BranchReadModel>()
                    .UseMongoDbReadModel<EmployeeReadModel>()
                    .UseMongoDbReadModel<DutyReadModel>()
                    .UseMongoDbReadModel<UserReadModel>()
                    .CreateResolver();
            }
            catch (Exception e)
            {
                Logger.Error(e,"Couldn't resolve MongoDB.");
                throw;
            }


        }

    }
}
