﻿using System;
using System.Collections.Generic;
using System.Linq;
using WebPortal.CCMConnector.Data.EDMX;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.General.QueriesCCM
{
    public class GetCustomerIdsQuery : ACCMCommandAndQuery
    {
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();


        internal static List<IdsFromCCMandWP> GetMainFichesByClientId(int clientId)
        {
            try
            {
                using (var ent = new ProdtelesecEntities())
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;
                    var rtn = ent.Customer
                        .Where(x => x.ClientId == clientId
                                    && x.Mainfiche == null)
                        .Select(x => new IdsFromCCMandWP()
                        {
                            CustomerIdCCM = x.CustomerId,
                            MainIdCCM = x.CustomerId,
                            DossierId = x.DossierIdWP
                        })
                        .ToList();
                    return rtn;
                }

            }
            catch (Exception e)
            {
                Logger.Error(e, $"Could not get mainfiche Ids for Client {clientId}");
                throw;
            }
        }

        internal static List<IdsFromCCMandWP> GetCustomerIds(bool allowBothMainAndSub, List<int> clientIds = null,
            List<int> mainCustIds = null, List<int> subCustIds = null)
        {
            List<IdsFromCCMandWP> custIds = new List<IdsFromCCMandWP>();
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                if (clientIds != null && clientIds.Count > 0)
                {
                    var mainfiches = ent.Client.Where(x => clientIds.Contains(x.ClientId))
                        .SelectMany(x => x.Customer.Select(y => y.CustomerId)).ToList();
                    GetSubCustsFromMainCusts(mainfiches, ent, custIds, allowBothMainAndSub);
                }

                GetSubCustsFromMainCusts(mainCustIds, ent, custIds, allowBothMainAndSub);
                GetMainCustsFromSubcusts(subCustIds, ent, custIds);
            }

            return custIds;
        }

        private static void GetSubCustsFromMainCusts(List<int> mainCustIds,
            ProdtelesecEntities ent,
            List<IdsFromCCMandWP> custIds,
            bool allowBothMainAndSub)
        {
            if (mainCustIds == null || mainCustIds.Count == 0) return;

            var subfiches = ent.Customer
                .Where(x => mainCustIds.Contains(x.Mainfiche.Value))
                .Select(x => new IdsFromCCMandWP
                {
                    MainIdCCM = x.Mainfiche.Value,
                    CustomerIdCCM = x.CustomerId,
                    DossierId = x.DossierIdWP
                })
                .ToList();

            var mainFiches = ent.Customer
                .Where(x => mainCustIds.Contains(x.CustomerId))
                .Select(x => new IdsFromCCMandWP
                {
                    MainIdCCM = x.CustomerId,
                    CustomerIdCCM = x.CustomerId,
                    DossierId = x.DossierIdWP
                })
                .ToList();


            if (allowBothMainAndSub)
            {
                custIds.AddRange(subfiches);

                //List<IdsFromCCMandWP> mainFiches = mainCustIds
                //    .Select(x => new IdsFromCCMandWP(x))
                //    .ToList();
                custIds.AddRange(mainFiches);
                return;
            }

            foreach (var mcId in mainCustIds)
            {
                var sf = subfiches.Where(x => x.MainIdCCM == mcId).ToList();
                if (sf.Count > 0)
                    custIds.AddRange(sf);
                else
                {
                    custIds.Add(mainFiches.Single(x => x.CustomerIdCCM == mcId));
                }
            }

        }

        private static void GetMainCustsFromSubcusts(List<int> subCustIds,
            ProdtelesecEntities ent,
            List<IdsFromCCMandWP> custIds)
        {
            if (subCustIds == null || subCustIds.Count == 0) return;

            var mainfiches = ent.Customer
                .Where(x => subCustIds.Contains(x.CustomerId))
                .Select(x => new IdsFromCCMandWP
                {
                    MainIdCCM = x.Mainfiche.Value,
                    CustomerIdCCM = x.CustomerId,
                    DossierId = x.DossierIdWP
                })
                .ToList();

            custIds.AddRange(mainfiches);

        }
    }
}
