﻿using System;
using System.Collections.Generic;
using System.Threading;
using CRM.Domain.InteractionStatistic;
using CRM.Domain.InteractionStatistic.Enums;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using WebPortal.CCMConnector.Application.InteractionStats.Commands;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.InteractionStats.CommandsWP
{
    public class CreateInteractionStatsCommand
    {
        private readonly ICommandBus _commandBus;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public CreateInteractionStatsCommand(ICommandBus commandBus)
        {
            _commandBus = commandBus;
        }

        internal void UploadInteractionStats(List<TempInteractionStatItem> statItems)
        {
            foreach (var si in statItems)
            {
                UploadStatistic(si);

            }
        }

        private IExecutionResult UploadStatistic(TempInteractionStatItem stat)
        {
            try
            {
                var command = new CreateInteractionStatCommand(InteractionStatisticId.New, stat.InteractionIDKey,
                    stat.DossierCustomerCCMAndDNISList.DossierId, Convert.ToInt32(stat.DNIS), stat.Established,
                    stat.ConnectedDateTimeUTC, stat.Duration, stat.TerminatedDateTimeUTC, InteractionType.Call, stat.OriginalCcmMessageId, stat.TicketId);
                var result = _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                return result;
            }
            catch (Exception e)
            {
                if (stat.DossierCustomerCCMAndDNISList.DossierId == null)
                {
                    Logger.Error(e, $"CCM-fiche {stat.DossierCustomerCCMAndDNISList.CustomerIdCCM} has no dossierId! Provide fiche {stat.DossierCustomerCCMAndDNISList.CustomerIdCCM} with a DossierId so that the Connector can continue working!");
                }
                else
                {
                    Logger.Error(e, $"Couldn't upload statistic with InteractonIDKey '{stat.InteractionIDKey}' for dossier '{stat.DossierCustomerCCMAndDNISList.DossierId}' with CCM-CustomerId: {stat.DossierCustomerCCMAndDNISList.CustomerIdCCM}.");
                }
                throw;
            }

        }
    }
}
