﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.InteractionStatistic;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using WebPortal.CCMConnector.Application.InteractionStats.Commands;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.InteractionStats.CommandsWP
{
    public class ModifyInteractionStatAddTicketIdCommand
    {
        private readonly ICommandBus _commandBus;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public ModifyInteractionStatAddTicketIdCommand(ICommandBus commandBus)
        {
            _commandBus = commandBus;
        }

        internal async Task AddTicketIdsToStatistics(List<TicketMessageAndStatId> statAndTicketIds)
        {
            List<Task> taskList = new List<Task>();

            foreach (var statTicket in statAndTicketIds)
            {
                taskList.Add( AddTicketIdToStatistic(statTicket));
            }
            await Task.WhenAll(taskList);
        }

        private async Task<IExecutionResult> AddTicketIdToStatistic(TicketMessageAndStatId statTicket)
        {
            try
            {
                var command =
                    new ModifyInteractionStatTicketIdCommand(
                        InteractionStatisticId.With(statTicket.StatisticId), statTicket.TicketId);
                var result = await _commandBus.PublishAsync(command, new CancellationToken());
                return result;
            }
            catch (Exception e)
            {
               Logger.Error(e, $"Could not add ticketId '{statTicket.TicketId}' to statistic with id: '{statTicket.StatisticId}'");
                throw;
            }
        }
    }
}
