﻿using System;
using System.Collections.Generic;
using System.Linq;
using EventFlow;
using EventFlow.Configuration;
using EventFlow.Queries;
using WebPortal.CCMConnector.Data.ConnectorLogic.InteractionStats.CommandsWP;
using WebPortal.CCMConnector.Data.ConnectorLogic.InteractionStats.QueriesCCM;
using WebPortal.CCMConnector.Data.ConnectorLogic.InteractionStats.QueryININ;
using WebPortal.CCMConnector.Data.ConnectorLogic.Tickets.QueriesWP;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.InteractionStats
{
    public class InteractionStatLogic
    {
        private readonly ICommandBus _commandBus;
        private readonly IQueryProcessor _queryProcessor;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public InteractionStatLogic(IRootResolver serviceResolver)
        {
            _commandBus = serviceResolver.Resolve<ICommandBus>();
            _queryProcessor = serviceResolver.Resolve<IQueryProcessor>();
        }

        internal int UploadInteractionStats(List<WPModules> interactionStatKeys, List<IdsFromCCMandWP> custIdsForTickets, DateTime previousRun, DateTime currentRun)
        {
            var clientIds = interactionStatKeys
                .Where(x => x.CCMLevel == "CLIENT")
                .Select(x => x.CCMId).ToList();

            var mainCustIds = interactionStatKeys
                .Where(x => x.CCMLevel == "MAIN")
                .Select(x => x.CCMId).ToList();

            var allCustAndDnis = GetCustomerInfoForStatsQuery.GetIdsForStatistics(clientIds, mainCustIds);

            //new GetDossierIdsQuery(_queryProcessor).AddDossierIdsToCustomerInfoForStats(allCustAndDnis);


            var tempInteractionStats = GetInteractionStatisticsQuery.GetTempInteractionStatItems(allCustAndDnis, previousRun, currentRun, out var delay);

            GetCustomerInfoForStatsQuery.AddMessageIdCcmToInteractionStats(tempInteractionStats, custIdsForTickets, previousRun, delay);

            var messageIdList = tempInteractionStats
                .Where(x => x.OriginalCcmMessageId != null)
                .Select(x => x.OriginalCcmMessageId.Value)
                .ToList();

            var ticketsInWebPortal = new GetExistingTicketIdsQuery(_queryProcessor).GetUploadedTickets(messageIdList);

            tempInteractionStats
                .Where(x => ticketsInWebPortal
                    .Select(y => y.OriginalCCMMessageId)
                    .Contains(x.OriginalCcmMessageId))
                .ToList()
                .ForEach(z => z.TicketId = ticketsInWebPortal
                    .First(a => 
                        a.OriginalCCMMessageId == z.OriginalCcmMessageId).TicketId);

            if (tempInteractionStats.Count == 0) return tempInteractionStats.Count;

            Logger.Info($"{tempInteractionStats.Count} statistics to upload.");

            new CreateInteractionStatsCommand(_commandBus).UploadInteractionStats(tempInteractionStats);
            return tempInteractionStats.Count;
        }
    }
}
