﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.InteractionStats.QueriesCCM
{
    public class GetCustomerInfoForStatsQuery: ACCMCommandAndQuery
    {
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        internal static List<IdsWithININVars> GetIdsForStatistics(List<int> clientIds = null,
            List<int> mainCustIds = null)
        {
            var custIds = new List<int>();
            if (mainCustIds != null)
                custIds.AddRange(mainCustIds);
            List<IdsWithININVars> rtn = new List<IdsWithININVars>();
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                try
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;

                    if (clientIds != null)
                    {
                        var mainfiches = ent.Client.Where(x => clientIds.Contains(x.ClientId))
                            .SelectMany(x => x.Customer.Select(y => y.CustomerId)).ToList();
                        custIds.AddRange(mainfiches);
                    }

                    rtn = ent.Customer
                        .Where(x => custIds.Contains(x.CustomerId))
                        .Select(x => new IdsWithININVars()
                        {
                            AnouncementSeconds = x.AnnouncementTime ?? 0,
                            CustomerIdCCM = x.CustomerId,
                            DNISList = x.CustomerDNIS.Select(y => y.DNIS).ToList(),
                            ExtraSeconds = x.ExtraTime ?? 0,
                            MainIdCCM = x.CustomerId,
                            DossierId = x.DossierIdWP

                        }).ToList();
                    return rtn;
                }
                catch (Exception e)
                {
                    Logger.Error(e, "Couldn't get Ids for Statistics from ProdTelesec.");
                    throw;

                }
            }
        }

        internal static void AddMessageIdCcmToInteractionStats(List<TempInteractionStatItem> interactionStatItems, List<IdsFromCCMandWP> custIdsForTickets, DateTime previousRun, int delay)
        {
            if (interactionStatItems == null || interactionStatItems.Count == 0) return;
            DateTime previousStatsRun = previousRun.AddMinutes(delay).ToLocalTime();
            var customerTicketIds = custIdsForTickets
                .Select(x => x.CustomerIdCCM)
                .ToList();
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                try
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;
                    var interactionids = interactionStatItems.Select(x => x.InteractionIdString).ToList();

                    var ticketsAndInteractionIds = ent.Message
                        .Where(x => x.LastModifiedDate >= previousStatsRun
                                    && interactionids.Contains(x.InteractionId)
                                    && customerTicketIds.Contains(x.CustomerId))
                        .Select(x => new InteractionAndMessagIdModel()
                        {
                            InteractionId = x.InteractionId,
                            MessageId = x.Messageid
                        })
                        .ToList();

                    interactionStatItems.Where(x 
                        => ticketsAndInteractionIds.Select(x => x.InteractionId).Contains(x.InteractionIdString))
                        .ToList()
                        .ForEach(y => y.OriginalCcmMessageId = ticketsAndInteractionIds
                            .First(z => z.InteractionId == y.InteractionIdString)
                            .MessageId);


                }
                catch (Exception e)
                {
                    Logger.Error(e, "Couldn't attach TicketIds for Statistics from ProdTelesec.");
                    throw;
                }
            }
        }
    }
}
