﻿using System;
using System.Collections.Generic;
using System.Linq;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.InteractionStats.QueryININ
{
    public class GetInteractionStatisticsQuery: ACCMCommandAndQuery
    {
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        internal static List<TempInteractionStatItem> GetTempInteractionStatItems(List<IdsWithININVars> idList, DateTime previousRun, DateTime currentRun, out int delay)
        {
            delay = -14;
            DateTime previousStatsRun = previousRun.AddMinutes(delay);
            DateTime currentStatsRun = currentRun.AddMinutes(delay);
            //var previousStatsRun = DateTime.UtcNow.AddHours(-1);
            //var currentStatsRun = DateTime.UtcNow;
            var filteredIdList = idList.Where(x => x.DNISListString != null);
            var dnisList = filteredIdList
                .SelectMany(x => x.DNISListString).ToList();
            using (EDMX.I3_ICEntities1 ent = new EDMX.I3_ICEntities1())
            {
                try
                {
                    ent.Database.Connection.ConnectionString = _i3icConString;
                    var rtn = ent.InteractionSummary
                        .Join(ent.InteractionCustomAttributes,
                            su => su.InteractionIDKey,
                            cu => cu.InteractionIDKey,
                            (su,
                                cu) => new {su, cu})
                        .Where(x => dnisList.Contains(x.cu.CustomString1)
                                    && x.su.TerminatedDateTimeUTC >= previousStatsRun 
                                    && x.su.TerminatedDateTimeUTC < currentStatsRun
                                    && x.su.tConnected!= null && x.su.tConnected > 0)
                        .ToList()
                        .Select(x => new TempInteractionStatItem()
                        {
                            ConnectedDateTimeUTC = x.su.ConnectedDateTimeUTC,
                            DNIS = x.cu.CustomString1,
                            InteractionId = x.su.InteractionID,
                            InteractionIDKey = x.su.InteractionIDKey,
                            tAlert = x.su.tAlert ?? 0,
                            tConnected = x.su.tConnected ?? 0,
                            tHeld = x.su.tHeld ?? 0,
                            tIVRWait = x.su.tIVRWait ?? 0,
                            tPark = x.su.tPark ?? 0,
                            tQueueWait = x.su.tQueueWait ?? 0,
                            TerminatedDateTimeUTC = x.su.TerminatedDateTimeUTC,
                            DossierCustomerCCMAndDNISList = filteredIdList
                                .FirstOrDefault(y => y.DNISListString.Contains(x.cu.CustomString1)),
                        }).ToList();
                    return rtn;
                }
                catch (Exception e)
                {
                    Logger.Error(e, "Couldn't get statistics from I3_IC-database.");
                    throw;
                }
            }
        }

        //internal List<TempInteractionStatItem> GetStatistics (List<IdsWithININVars> idList, DateTime previousRun)
        //{
        //    var customerIds = idList.Select(x => x.MainIdCCM).ToList();
        //    using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
        //    {
        //        ent.Database.Connection.ConnectionString = _prodTelesecConString;

        //        var rtn = ent.CustomerDNIS
        //            .Where(x => customerIds.Contains(x.CustomerId))
        //            .Select(x => new
        //            {
        //                x.DNIS,
        //                x.CustomerId
        //            }).ToList();
        //        idList.ForEach(x => x.DNISList = rtn
        //            .Where(y => y.CustomerId == x.MainIdCCM)
        //            .Select(y => y.DNIS).ToList());
                    
        //    }

        //    return GetTempInteractionStatItems(idList, previousRun);
        //}
    }
}
