﻿using System;
using System.Collections.Generic;
using System.Threading;
using CRM.Domain.Ticket;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using WebPortal.CCMConnector.Application.Tickets.Commands;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Tickets.CommandsWP
{
    public class DeleteTicketsCommand
    {
        private readonly ICommandBus _commandBus;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public DeleteTicketsCommand(ICommandBus commandBus)
        {
            _commandBus = commandBus;
        }

        internal void DeleteTickets(List<string> ticketIds)
        {
            foreach (var id in ticketIds)
            {
                DeleteTicket(id);
            }
        }

        private IExecutionResult DeleteTicket(string ticketId)
        {
            try
            {
                var command = new DeleteTicketCommand(TicketId.With(ticketId));
                var result = _commandBus.PublishAsync(command, new CancellationToken())
                    .ConfigureAwait(false)
                    .GetAwaiter()
                    .GetResult();
                return result;
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't delete ticket with id: {ticketId}");
                throw;
            }
        }

    }
}
