﻿using System;
using EventFlow;
using EventFlow.Aggregates.ExecutionResults;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using CRM.Domain.Ticket;
using WebPortal.CCMConnector.Application.Tickets.Commands;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Tickets.CommandsWP
{
    public class UploadTicketsCommand
    {
        private readonly ICommandBus _commandBus;
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public UploadTicketsCommand(ICommandBus commandBus)
        {
            _commandBus = commandBus;
        }

        internal void UploadTickets(List<TempTicketItem> allTickets, List<Application.Tickets.Models.GetWPAndCCMTicketIdModel> ticketsToModify)
        {
            foreach (var ticket in allTickets)
            {
                if (ticket.DossierId == null)
                {
                    Logger.Error($"Can't upload ticket for CCM-fiche {ticket.CustomerId} by lack of a DossierId.  Provide fiche {ticket.CustomerId} with a DossierId so that the Connector can continue working!");
                    throw new Exception("No Dossier Id!");
                }

                try
                {
                    var ticketToModify = ticketsToModify.FirstOrDefault(x => x.OriginalCCMMessageId == ticket.OriginalCCMMessageId);
                    if (ticketToModify == null)
                    {
                        //insert
                        CreateNewTicket(ticket);
                    }
                    else
                    {
                        //modify
                        ModifyExistingTicket(ticketToModify.TicketId, ticket);
                    }
                }
                catch (Exception e)
                {
                    Logger.Error(e, $"Ticket with original CCMId '{ticket.OriginalCCMMessageId}' was more than once uploaded in WebPortal DB.");
                    throw;
                }
            }
        }

        private IExecutionResult CreateNewTicket(TempTicketItem ticket)
        {
            try
            {
                var ticketId = TicketId.New;
                ticket.TicketId = ticketId.Value;
                var command = new CreateTicketCommand(ticketId, ticket.DossierId, ticket.TicketText, ticket.OriginalCCMMessageId, ticket.CreatedInCRM, ticket.LastModifiedInCRM, ticket.CreatedCRMUser, ticket.LastModifiedCRMUser, ticket.Tags);
                var result = _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                return result;
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't create ticket. CCM-MessageId: '{ticket.OriginalCCMMessageId}', DossierId: '{ticket.DossierId}, CCM-CustomerId: {ticket.CustomerId}.");
                throw;
            }
        }

        private IExecutionResult ModifyExistingTicket(string ticketId, TempTicketItem ticket)
        {
            try
            {
                var aggregateId = TicketId.With(ticketId);
                var command = new ModifyTicketCommand(aggregateId, ticket.TicketText, ticket.LastModifiedInCRM, ticket.LastModifiedCRMUser);
                var result = _commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
                return result;
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't modify ticket '{ticketId}'. CCM-MessageId: '{ticket.OriginalCCMMessageId}', DossierId: '{ticket.DossierId}, CCM-CustomerId: {ticket.CustomerId}.");
                throw;
            }

        }
    }
}
