﻿using System;
using System.Collections.Generic;
using System.Linq;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.ConnectorLogic.Tickets.QueriesCCM
{
    public class GetTicketsAfterPreviousRun : ACCMCommandAndQuery
    {
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        internal static void GetMessageNames(List<TempMessageName> messageNames)
        {
            List<int> custIds = messageNames.Select(x => x.CustomerId).Distinct().ToList();

            if (custIds.Count == 0) return;

            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {

                try
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;
                    var rtn = ent.Structured_Message_Name
                        .Where(x => custIds.Contains(x.CustomerID))
                        .Select(x => new TempMessageName()
                            {
                                ControllerOk = x.WPControllerOK ?? true,
                                CustomerId = x.CustomerID,
                                IsRead = x.WPIsRead ?? true,
                                MessageName = x.MessageName,
                                Version = x.Version
                            }
                        )
                        .ToList();
                    messageNames.ForEach(x =>
                    {
                        var temp = rtn.Single(y =>
                            y.CustomerId == x.CustomerId && y.Version == x.Version && y.MessageName == x.MessageName);
                        x.ControllerOk = temp.ControllerOk;
                        x.IsRead = temp.IsRead;
                    });

                }
                catch (Exception e)
                {
                    Logger.Error(e, "Couldn't get messagenames from ProdTelesec.");
                    throw;
                }
            }
        }

        /// <summary>
            /// De lijst is gerangschikt op customerId
            /// </summary>
            /// <param name="mainAndSubficheIds"></param>
            /// <param name="previousRun"></param>
            /// <returns></returns>
            internal static List<TempMessageAndVersion> GetNewAndModifiedMessages(List<IdsFromCCMandWP> mainAndSubficheIds, DateTime previousRun)
        {
            string nullString = null;
            List<int> custIds = mainAndSubficheIds
                .Select(x => x.CustomerIdCCM)
                .Distinct().ToList();
            var localTime = previousRun.ToLocalTime();
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                try
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;
                    var messages = ent.Message

                        .Where(x => custIds.Contains(x.CustomerId)
                                    && x.LastModifiedDate >= localTime
                                    && x.ToTreat == false
                                    && x.DoNotUploadInWP != true
                        )
                        .Select(x => new TempMessageAndVersion()
                        {
                            MessageId = x.Messageid,
                            Version = x.Structured_Message.FirstOrDefault() == null
                                ? -1
                                : x.Structured_Message.FirstOrDefault().Version,
                            CustomerId =  x.CustomerId,
                            CreatedCRMUser = x.CreateUser.Trim(),
                            CreatedInCRM = x.MessageDateTime,
                            LastModifiedCRMUser = x.LastModifiedUser.Trim(),
                            LastModifiedInCRM = x.LastModifiedDate.HasValue
                                ? x.LastModifiedDate.Value
                                : x.MessageDateTime,
                            MainFicheId = x.Customer.Mainfiche,
                            MessageText = x.MessageText,
                            ControllerOK = x.ControllerOkDate.HasValue,
                            IsRead = !x.ToRead,
                            InteractionId = x.InteractionId,
                            SubficheTag = x.Customer.Mainfiche.HasValue ? x.Customer.SearchName: nullString,
                            StructuredMessages = x.Structured_Message
                                .Select(y => new TempStructuredMessage
                                {
                                    MessageName = y.MessageName,
                                    MessageValue = y.Value,
                                    OrderId = y.MSGID,
                                    LabelText = y.Structured_Messages_Parameters.Label,
                                    DynamicParameterId = y.DynamicParameterId,
                                })
                                .OrderBy(y => y.OrderId)
                                .ToList()

                        })
                        .OrderBy(x => x.CustomerId)
                        .ToList();
                    messages.ForEach(x =>
                    {
                        x.CreatedInCRM = x.CreatedInCRM.ToUniversalTime();
                        x.LastModifiedInCRM = x.LastModifiedInCRM.ToUniversalTime();
                        x.MainAndSubficheIdAndDossierId = mainAndSubficheIds
                            .FirstOrDefault(y => y.CustomerIdCCM == x.CustomerId);
                    });

                    return messages;
                }
                catch (Exception e)
                {
                    Logger.Error(e, "Couldn't get New And Modified Tickets from ProdTelesec.");
                    throw;
                }

            }
        }

        internal static List<TempDynamicParameter> GetDynamicParams(List<TempMessageAndVersion> messagesWithDynamicParamIds)
        {
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                try
                {
                    ent.Database.Connection.ConnectionString = _prodTelesecConString;

                    var allMessageIds = messagesWithDynamicParamIds.Select(x => x.MessageId).ToList();

                    var dynParams = ent.Structured_Message_Response
                        .Where(x => allMessageIds.Contains(x.MessageId))
                        .Select(x => new TempDynamicParameter
                        {
                            MessageId = x.MessageId,
                            OrderId = x.OrderId,
                            Answer = x.Response == null
                                ? x.Structured_Message_Answer_Name.Value
                                : x.Response,
                            Question = x.Structured_Message_Answer_Name.Structured_Message_Question.Value
                        }).ToList();
                    return dynParams;
                }
                catch (Exception e)
                {
                    Logger.Error(e, "Couldn't get Dynamic Parameters from ProdTelesec.");
                    throw;
                }

            }
        }


    }
}
