﻿using System.Collections.Generic;
using WebPortal.CCMConnector.Application.Customers.Models;
using WebPortal.CCMConnector.Application.Duties.Models;

namespace WebPortal.CCMConnector.Data.IO
{
    public class MailLogic
    {
        public static string MakeHtmlBody(DutyAlarmModel alarm,
            List<GetDutyToExpireModel> duties, string timeString)
        {
            var rtn = @"<p>Dear,</p>";
            rtn += alarm.OnFixedDate 
                ? $"<p>In the {alarm.Tenant}-WebPortal, there is no future duty planned for '{alarm.DossierName}'."
                : $"<p>In the {alarm.Tenant}-WebPortal, the following Duties for '{alarm.DossierName}' are about to expire in</p>";

            if (! alarm.OnFixedDate) rtn += $"<h3>{timeString}</h3>";

            if (duties != null && duties.Count > 0)
            {
                if (alarm.OnFixedDate)
                {
                    rtn += "<p>These are duties that are about to expire:</p>";
                }
                rtn += @"<table><tbody>";

                foreach (var d in duties)
                {
                    var noEmployees = d.AmountOfEmployees == 0 ?  
                        " and has no persons attached to it.": "";
                    rtn += "<tr>";
                    rtn += $"<td><strong>'{d.Category}'</strong></td>";
                    rtn += "<td><strong>will expire at</strong></td>";
                    rtn += $"<td><strong>{d.ExpireDateTime.ToString("yyyy-MM-dd HH:mm")}{noEmployees}</strong></td>";
                    rtn += "</tr>";
                }

                rtn += @"</tbody></table>";
            }
            return rtn;
        }

        public static string MakeTxtBody(DutyAlarmModel alarm,
            List<GetDutyToExpireModel> duties,
            string timeString)
        {
            var rtn = "Dear,";
            rtn += alarm.OnFixedDate
                ? $"\n\nIn the {alarm.Tenant}-WebPortal, there is no future duty planned for '{alarm.DossierName}'."
                :$"\n\nIn the {alarm.Tenant}-WebPortal, the following Duties for '{alarm.DossierName}' are about to expire in";

            if (!alarm.OnFixedDate) rtn += $"\n{timeString}";

            rtn += "\n\n";
            if (duties != null && duties.Count > 0)
            {
                if (alarm.OnFixedDate)
                {
                    rtn += "These are duties that are about to expire:\n";
                }

                foreach (var d in duties)
                {
                    var noEmployees = d.AmountOfEmployees == 0 ?
                        " and has no persons attached to it." : "";
                    rtn += $"'{d.Category}' will expire at {d.ExpireDateTime.ToString("yyyy-MM-dd HH:mm")}{noEmployees}";
                    rtn += "\n";

                }
            }

            return rtn;
        }
    }
}
