﻿using AutoMapper;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.Logic.Commands
{
    public class CommandsToCCM
    {
        private const string _prodTelesecConString = @"data source=aac-sql01.cisicc.local;initial catalog=Prodtelesec;user id=sa;Password=T@v3rn1er2015*;MultipleActiveResultSets=True;App=EntityFramework";

        public void AddNewSubscriptions(List<NewWPCompany> companiesToAddSubscriptionsFor, bool isActiphone, IMapper mapper)
        {
            CreateAndSaveSubscriptions(companiesToAddSubscriptionsFor, isActiphone, mapper);
        }

        public Dictionary<string, Guid> AddNewSubscriptionsRetunDictionary(List<NewWPCompany> companiesToAddSubscriptionsFor, bool isActiphone, IMapper mapper)
        {
            List<EDMX.WPSubscriptions> subs = CreateAndSaveSubscriptions(companiesToAddSubscriptionsFor, isActiphone, mapper);
            return subs.ToDictionary(x => x.CompanyIdWP, x => x.Id);
        }

        private static List<EDMX.WPSubscriptions> CreateAndSaveSubscriptions(List<NewWPCompany> companiesToAddSubscriptionsFor, bool isActiphone, IMapper mapper)
        {
            var subs = mapper.Map<List<NewWPCompany>, List<EDMX.WPSubscriptions>>(companiesToAddSubscriptionsFor);
            subs.ForEach(x =>
            {
                x.Id = Guid.NewGuid();
                x.StartDate = DateTime.UtcNow.Date;
                x.UserAmount = 0;
                x.MonthlySubscriptionPrice = 0;
                x.IsActiphoneCust = isActiphone;
            });

            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                ent.WPSubscriptions.AddRange(subs);
                ent.SaveChanges();
            }

            return subs;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="subscrList">Key: DossierId, Value: SubscriptionId</param>
        /// <param name="branches"></param>
        /// <param name="moduleId">Moet Id zijn van 'AutoTreeview'</param>
        public void AddNewTreeviewModuleSubscriptions(Dictionary<string, Guid> subscrList, List<NewWPCompanyBranch> branches, int moduleId)
        {
            var modulesToAdd = (from b in branches
                                join s in subscrList
                                on b.CompanyId equals s.Key
                                select new EDMX.WPCustomerModules()
                                {
                                    AllSubfiches = false,
                                    CreatedBy = b.CreatedBy,
                                    CreatedOn = DateTime.UtcNow,
                                    LastModifiedBy = b.LastModifiedBy,
                                    LastModifiedOn = DateTime.UtcNow,
                                    ModuleId = moduleId,
                                    StartDate = DateTime.UtcNow.Date,
                                    SubficheId = b.SubFicheId,
                                    UnitPrice = 0,
                                    WPSubscriptionId = s.Value
                                }).ToList();

            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                ent.WPCustomerModules.AddRange(modulesToAdd);
                ent.SaveChanges();
            }
        }

        public void UpdateSubscriptionsWithCompanyId(List<WPSubscriptionOverview> subscriptionsToUpdate)
        {
            var ids = subscriptionsToUpdate.Select(x => x.SubscriptionId).ToList();
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var toUpdate = ent.WPSubscriptions.Where(x => ids.Contains(x.Id)).ToList();
                toUpdate.ForEach(x =>
                {
                    x.CompanyIdWP = subscriptionsToUpdate.Single(y => y.SubscriptionId == x.Id).CompanyIdWP;
                    x.LastModifiedBy = "CCMConnector";
                    x.LastModifiedOn = DateTime.UtcNow;
                });
                ent.SaveChanges();
            }
        }

        public void MarkServicesAsUploadedInWP(List<string> uploadedServices)
        {
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var toMark = ent.Service_Organisation.Where(x => x.UploadedInWP != true
                && uploadedServices.Contains(x.SERVICE_NAME))
                    .ToList();
                toMark.ForEach(x => x.UploadedInWP = true);
                ent.SaveChanges();
            }
        }

        public void MarkTreeviewPersonsAsUploadedInWP(List<int> ansgateIds)
        {
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var toMark = ent.Treeview.Where(x => x.UploadedInWP != true
                && ansgateIds.Contains(x.AnsgateID))
                    .ToList();
                toMark.ForEach(x => x.UploadedInWP = true);
                ent.SaveChanges();
            }
        }

        public void MakeTestSubscriptions()
        {
            //Deze moet zowel voor mainfiche (ZTest Full Copy) als de twee subfiches (987730, 987720) een companyBranch genereren.
            EDMX.WPSubscriptions sub1 = new EDMX.WPSubscriptions()
            {
                CompanyIdWP = "",
                CreatedBy = "TestingSubscr",
                CreatedOn = DateTime.UtcNow,
                CustomerIdCCM = 9877, //ZTest Full Copy
                Id = new Guid("e45e99a6-1d57-4dd1-9da9-dc4eb64a9e5c"),
                IsActiphoneCust = false,
                LastModifiedBy = "TestingSubscr",
                MonthlySubscriptionPrice = 75,
                LastModifiedOn = DateTime.UtcNow,
                StartDate = DateTime.UtcNow.Date,
                UserAmount = 2,
            };
            EDMX.WPCustomerModules mod1 = new EDMX.WPCustomerModules()
            {
                AllSubfiches = true,
                CreatedBy = "TestingSubscr",
                LastModifiedBy = "TestingSubscr",
                CreatedOn = DateTime.UtcNow,
                LastModifiedOn = DateTime.UtcNow,
                ModuleId = 3, //tickets
                StartDate = DateTime.UtcNow.Date,
                SubficheId = null,
                UnitPrice = 60,
                WPSubscriptionId = new Guid("e45e99a6-1d57-4dd1-9da9-dc4eb64a9e5c")
            };
            EDMX.WPCustomerModules mod2 = new EDMX.WPCustomerModules()
            {
                AllSubfiches = true,
                CreatedBy = "TestingSubscr",
                LastModifiedBy = "TestingSubscr",
                CreatedOn = DateTime.UtcNow,
                LastModifiedOn = DateTime.UtcNow,
                ModuleId = 5, //script
                StartDate = DateTime.UtcNow.Date,
                SubficheId = null,
                UnitPrice = 60,
                WPSubscriptionId = new Guid("e45e99a6-1d57-4dd1-9da9-dc4eb64a9e5c")
            };

            //Voor Comfort Energy moet enkel Regio Geel en  Regio Mons worden ingeladen
            EDMX.WPSubscriptions sub2 = new EDMX.WPSubscriptions()
            {
                CompanyIdWP = "",
                CreatedBy = "TestingSubscr",
                CreatedOn = DateTime.UtcNow,
                CustomerIdCCM = 5212, //Comfort Energy
                Id = new Guid("04328972-3733-43f2-97a6-9a17190d58f2"),
                IsActiphoneCust = false,
                LastModifiedBy = "TestingSubscr",
                MonthlySubscriptionPrice = 60,
                LastModifiedOn = DateTime.UtcNow,
                StartDate = DateTime.UtcNow.Date,
                UserAmount = 3,
            };

            //Geel
            EDMX.WPCustomerModules mod3 = new EDMX.WPCustomerModules()
            {
                AllSubfiches = false,
                CreatedBy = "TestingSubscr",
                LastModifiedBy = "TestingSubscr",
                CreatedOn = DateTime.UtcNow,
                LastModifiedOn = DateTime.UtcNow,
                ModuleId = 3, //tickets
                StartDate = DateTime.UtcNow.Date,
                SubficheId = 521270,
                UnitPrice = 12,
                WPSubscriptionId = new Guid("04328972-3733-43f2-97a6-9a17190d58f2")
            };

            //Mons
            EDMX.WPCustomerModules mod4 = new EDMX.WPCustomerModules()
            {
                AllSubfiches = false,
                CreatedBy = "TestingSubscr",
                LastModifiedBy = "TestingSubscr",
                CreatedOn = DateTime.UtcNow,
                LastModifiedOn = DateTime.UtcNow,
                ModuleId = 5, //script
                StartDate = DateTime.UtcNow.Date,
                SubficheId = 521297,
                UnitPrice = 12,
                WPSubscriptionId = new Guid("04328972-3733-43f2-97a6-9a17190d58f2")
            };

            var lstSubs = new List<EDMX.WPSubscriptions>();
            lstSubs.Add(sub1);
            lstSubs.Add(sub2);

            var lstMods = new List<EDMX.WPCustomerModules>();
            lstMods.Add(mod1);
            lstMods.Add(mod2);
            lstMods.Add(mod3);
            lstMods.Add(mod4);

            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                ent.WPSubscriptions.AddRange(lstSubs);
                ent.WPCustomerModules.AddRange(lstMods);
                ent.SaveChanges();
            }
        }
    }
}
