﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WebPortal.CCMConnector.Data.Models;
using AutoMapper;
using CRM.Domain.Shared.Enums;

namespace WebPortal.CCMConnector.Data.Logic.Commands
{
    public class CommandsToWebPortal
    {
        private bool _isActiphone;
        private string _wpConString;

        public CommandsToWebPortal(bool isActiphone)
        {
            SetConnectionString(isActiphone);
        }

        public void ChangeConnectionString(bool isActiphone)
        {
            SetConnectionString(isActiphone);
        }

        private void SetConnectionString(bool isActiphone)
        {
            var initCat = isActiphone ? "WPActiphone" : "WPAac";
            _isActiphone = isActiphone;
            _wpConString = @"data source=(LocalDb)\MSSQLLocalDB;initial catalog=" + initCat + @";integrated security=True;MultipleActiveResultSets=True;App=EntityFramework";
        }

        public void UploadCompanies(List<NewWPCompany> newCompanies, IMapper mapper)
        {
            var companies = mapper.Map<List<NewWPCompany>, List<EDMX.Companies>>(newCompanies);
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                ent.Companies.AddRange(companies);
                ent.SaveChanges();
            }
        }

        public void UploadCompanyBranches(List<NewWPCompanyBranch> newWPCompanyBranches, IMapper mapper)
        {
            var companyBranches = mapper.Map<List<NewWPCompanyBranch>, List<EDMX.CompanyBranches>>(newWPCompanyBranches);
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                ent.CompanyBranches.AddRange(companyBranches);
                ent.SaveChanges();
            }
        }

        public void UploadAddresses(List<NewAddress> newAddresses, IMapper mapper)
        {
            var addresses = mapper.Map<List<NewAddress>, List<EDMX.Adresses>>(newAddresses);
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                ent.Adresses.AddRange(addresses);
                ent.SaveChanges();
            }
        }

        public void UploadContactDetails(List<WPContactDetail> newContactDetails, IMapper mapper)
        {
            var contactDetails = mapper.Map<List<WPContactDetail>, List<EDMX.ContactDetails>>(newContactDetails);
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                ent.ContactDetails.AddRange(contactDetails);
                ent.SaveChanges();
            }
        }

        public void ReEnableTags(List<string> disabledTagIds)
        {
            if (disabledTagIds == null || disabledTagIds.Count == 0)
                return;
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                var toUpdate = ent.Tags.Where(x => disabledTagIds.Contains(x.Id)).ToList();
                toUpdate.ForEach(x => x.IsDeleted = false);
                ent.SaveChanges();
            }
        }

        public void UploadTagsByTagNames(List<string> tagNames, TagType tagType, IMapper mapper)
        {
            //int tt = (int)tagType;
            List<WPTag> wpTags = new List<WPTag>();
            foreach (var t in tagNames)
            {
                var wpt = new WPTag(t, tagType);
                wpTags.Add(wpt);
            }
            UploadTags(wpTags, mapper);
            //List<EDMX.Tags> newTags = new List<EDMX.Tags>();

            //foreach (var t in tagNames)
            //{
            //    EDMX.Tags tag = new EDMX.Tags()
            //    {
            //        Id = Guid.NewGuid(),
            //        TagName = t,
            //        TypeOfTag = (int)tagType
            //    };
            //    newTags.Add(tag);
            //}

            //SaveTags(newTags);
        }

        public void UploadTags(List<WPTag> wpTags, IMapper mapper)
        {
            var newTags = mapper.Map<List<WPTag>, List<EDMX.Tags>>(wpTags);
            SaveTags(newTags);
        }

        private void SaveTags(List<EDMX.Tags> newTags)
        {
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                ent.Tags.AddRange(newTags);
                ent.SaveChanges();
            }
        }

        public void UploadEmployees(List<WPEmployee> newEmployees, IMapper mapper)
        {
            var employees = mapper.Map<List<WPEmployee>, List<EDMX.Employees>>(newEmployees);
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                ent.Employees.AddRange(employees);
                ent.SaveChanges();
            }
        }

        internal void UploadEmployeeTags(List<EDMX.TagsEmployee> tagsEmployee)
        {
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                ent.TagsEmployee.AddRange(tagsEmployee);
                ent.SaveChanges();
            }
        }
    }
}