﻿using AutoMapper;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CRM.Domain.Shared.Enums;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.Logic.Queries
{
    public class QueryFromCCM
    {
        private const string _prodTelesecConString = @"data source=aac-sql01.cisicc.local;initial catalog=Prodtelesec;user id=sa;Password=T@v3rn1er2015*;MultipleActiveResultSets=True;App=EntityFramework";

        /// <summary>
        /// Haalt alle huidige subscriptions op uit ProdTelesec
        /// </summary>
        /// <returns></returns>
        public List<WPSubscriptionOverview> GetSubscriptionOverviewFromCCM(bool isActiphone)
        {
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var today = DateTime.UtcNow.Date;
                var allSubscriptions = ent.WPSubscriptions.Where(x => x.StartDate <= today
                && (x.EndDate == null || x.EndDate > today)
                && x.IsActiphoneCust == isActiphone)
                    .Select(x => new WPSubscriptionOverview()
                    {
                        CompanyIdWP = x.CompanyIdWP,
                        CustomerIdCCM = x.CustomerIdCCM,
                        SubscriptionId = x.Id,
                        IsActiphoneCust = x.IsActiphoneCust
                    }).ToList();

                return allSubscriptions;
            }
        }

        public List<WPModules> GetAllTheKeysForGettingTheData(bool isActiphone)
        {
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var today = DateTime.UtcNow.Date;
                var rtn = ent.WPSubscriptions.Where(x => x.StartDate <= today
                && (x.EndDate == null || x.EndDate > today)
                && x.IsActiphoneCust == isActiphone)
                    .SelectMany(x => x.WPCustomerModules.Where(y => y.StartDate <= today && (y.EndDate == null || y.EndDate > today)), (x, modu) => new { sub = x, modu })
                    .Select(x => new WPModules()
                    {
                        CompanyIdWP = x.sub.CompanyIdWP,
                        CustomerIdCCM = x.sub.CustomerIdCCM,
                        SubscriptionId = x.sub.Id,
                        IsActiphoneCust = x.sub.IsActiphoneCust,
                        Module = (ModuleName)x.modu.ModuleId,
                        SubficheId = x.modu.SubficheId,
                        AllSubfiches = x.modu.AllSubfiches
                    }).ToList();

                var moreSubfiches = rtn.Where(x => x.AllSubfiches).ToList();
                if (moreSubfiches.Count == 0)
                    return rtn;

                //Je hebt alle mainfiches + subfiches met AllSubfiches == false
                foreach (var ms in moreSubfiches)
                {
                    var toAdd = ent.Customer.Where(x => x.Mainfiche == ms.CustomerIdCCM)
                        .Select(x => new WPModules()
                        {
                            AllSubfiches = true,
                            CompanyIdWP = ms.CompanyIdWP,
                            CustomerIdCCM = x.CustomerId,
                            IsActiphoneCust = isActiphone,
                            Module = ms.Module,
                            SubficheId = x.CustomerId,
                            SubscriptionId = ms.SubscriptionId
                        }).ToList();
                    if (toAdd.Count > 0)
                    {
                        rtn.AddRange(toAdd);
                    }
                }

                //var rtn2 = (from c in ent.Customer
                //            join s in rtn
                //            on c.Mainfiche equals s.CustomerIdCCM
                //            where s.AllSubfiches == true
                //            select new WPSubOverviewWithModules()
                //            {
                //                AllSubfiches = true,
                //                CompanyGuidWP = s.CompanyGuidWP,
                //                CustomerIdCCM = c.DossierId,
                //                IsActiphoneCust = s.IsActiphoneCust,
                //                Module = s.Module,
                //                SubficheId = c.DossierId,
                //                SubscriptionId = s.SubscriptionId
                //            }).ToList();

                //rtn.AddRange(rtn2);
                return rtn;
            }
        }

        public List<TempCompanyItem> GetTempCompanyItemsWithSubfiches(List<int> custIds)
        {
            int? noSubfiche = null;
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var rtn = ent.Customer
                .Where(x => custIds.Contains(x.CustomerId))
                .Select(x => new TempCompanyItem()
                {
                    TempId = Guid.NewGuid(),
                    MainFicheId = x.Mainfiche,
                    CustomerId = x.CustomerId,
                    DossierName = x.Name,
                    BranchName = x.SearchName,
                    TenantNumber = x.Company_ProfileID ?? 1,
                    SubFicheId = x.Mainfiche == null ? noSubfiche : x.CustomerId,
                    Street = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().Street.Trim(),
                    City = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().City.Trim(),
                    PostalCode = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().Zipcode.Trim(),
                    Country = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().Country.Trim(),
                    HouseNumber = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().Number.Trim(),
                    BoxNumber = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().Box.Trim(),
                    FaxNumber = x.FaxNumber.Trim(),
                    EmailAddress = x.EmailAddress.Trim(),
                    Fax = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().Fax.Trim(),
                    DefaultFax = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().DefaultFax.Trim(),
                    Email = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().Email.Trim(),
                    DefaultEmail = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().DefaultEmail.Trim(),
                    Phone1 = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().Phone1.Trim(),
                    Phone2 = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().Phone2.Trim(),
                    HomePage = x.Address.FirstOrDefault() == null
                        ? null : x.Address.FirstOrDefault().HomePage.Trim(),
                }).ToList();
                var res = JsonConvert.SerializeObject(rtn);
                return rtn;
            }
        }

        public List<WPSubscriptionOverview> GetAllNotYetUploadedSubscriptions(bool isActiphone, List<WPSubscriptionOverview> mainSubs)
        {
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;

                var today = DateTime.UtcNow.Date;

                var subscrIds = mainSubs.Select(x => x.SubscriptionId).ToList();

                var subsWithModules = ent.WPSubscriptions.Where(x => subscrIds.Contains(x.Id))
                    .SelectMany(x => x.WPCustomerModules.Where(y => y.StartDate <= today && (y.EndDate == null || y.EndDate > today)), (x, modu) => new { sub = x, modu })
                    .Select(x => new WPSubscriptionOverview()
                    {
                        CustomerIdCCM = x.modu.SubficheId.HasValue ? x.modu.SubficheId.Value : x.sub.CustomerIdCCM,
                        SubscriptionId = x.sub.Id,
                        IsActiphoneCust = x.sub.IsActiphoneCust,
                    })
                    .Distinct()
                    .ToList();

                var mainFicheIdsForAllSubs = ent.WPSubscriptions.Where(x => subscrIds.Contains(x.Id) && x.WPCustomerModules.Any(y => y.AllSubfiches))
                    .Select(x => new WPSubscriptionOverview()
                    {
                        CustomerIdCCM = x.CustomerIdCCM,
                        SubscriptionId = x.Id,
                        IsActiphoneCust = x.IsActiphoneCust,
                    }).ToList();

                var soSimple = mainFicheIdsForAllSubs.Select(y => y.CustomerIdCCM).ToList();

                List<MainFicheWithSubficheIds> temp2 = ent.Customer
                    .Where(x => soSimple.Contains(x.Mainfiche.Value) && x.EnableYN && x.PopupSubFicheList)
                    .GroupBy(x => x.Mainfiche.Value)
                    .Select(x => new MainFicheWithSubficheIds()
                    {
                        CustomerIds = x.Select(y => y.CustomerId).ToList(),
                        MainFicheId = x.Key
                    }).ToList();

                var temp = temp2.SelectMany(x => x.CustomerIds, (x, custId) => new { x, custId })
                    .Select(x => new WPSubscriptionOverview()
                    {
                        SubscriptionId = mainFicheIdsForAllSubs.SingleOrDefault(y => y.CustomerIdCCM == x.x.MainFicheId).SubscriptionId,
                        CustomerIdCCM = x.custId,
                        IsActiphoneCust = isActiphone
                    }).ToList();

                subsWithModules.AddRange(mainFicheIdsForAllSubs);
                subsWithModules.AddRange(temp);
                subsWithModules.AddRange(mainSubs);

                subsWithModules = subsWithModules
                    .GroupBy(x => x.CustomerIdCCM).Select(x => new WPSubscriptionOverview()
                    {
                        SubscriptionId = x.First().SubscriptionId,
                        CustomerIdCCM = x.Key,
                        IsActiphoneCust = isActiphone
                    }).ToList();

                return subsWithModules;
            }
        }

        public List<MainFicheWithSubficheIds> GetAllCustsWithTreeview(bool isActiphone)
        {
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                int profileId = isActiphone ? 2 : 1;
                var temp2 = ent.Customer.Where(x => x.Treeview.Count > 0 && x.EnableYN
                    && x.Company_ProfileID == profileId)
                    .GroupBy(x => x.Mainfiche)
                    .Select(x => new MainFicheWithSubficheIds()
                    {
                        CustomerIds = x.Select(y => y.CustomerId).ToList(),
                        MainFicheId = x.Key
                    }).ToList();
                var temp3 = temp2.Where(x => x.MainFicheId == null).FirstOrDefault();
                if (temp3 != null)
                {
                    var temp4 = temp3.CustomerIds.Where(x => !temp2.Select(y => y.MainFicheId).Contains(x))
                    .Select(z => new MainFicheWithSubficheIds()
                    {
                        CCMId = z,
                    }).ToList();
                    temp2.Remove(temp3);
                    temp2.AddRange(temp4);
                }
                return temp2;
            }
        }

        public List<string> GetTreeviewServiceChanges(List<int> customerIds)
        {
            //treeviews hebben een CreatedOn en LastModifiedOn die null is indien nog niet opgeladen in WP.
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var rtn = ent.Service_Organisation.Where(x =>
                   (x.UploadedInWP == null || x.UploadedInWP == false)
                    && customerIds.Contains(x.CustomerId))
                    .Select(x => x.SERVICE_NAME.Trim()).Distinct()
                    .ToList();
                return rtn;
            }
        }

        public List<TempEmployeeItem> GetTreeviewChanges(List<int> customerIds, IMapper mapper)
        {
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var rtn = ent.Treeview.Where(x =>
                (x.UploadedInWP == null || x.UploadedInWP == false)
                && customerIds.Contains(x.CustomerId)).ToList();
                var rtnTemp = mapper.Map<List<EDMX.Treeview>, List<TempEmployeeItem>>(rtn);

                var res = JsonConvert.SerializeObject(rtnTemp);
                return rtnTemp;
            }
        }

        public List<TempCallScriptItem> GetCallScripts(List<int> customerIds)
        {
            string sfNull = null;
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var rtn = ent.CallScriptsHTML.Join(ent.Customer, s => s.CustomerId, c => c.CustomerId, (s, c)
                    => new { script = s, cust = c })
                    .Where(x => customerIds.Contains(x.script.CustomerId))
                    .Select(x => new TempCallScriptItem()
                    {
                        CallScriptBody = x.script.NotesHTML,
                        CustomerId = x.script.CustomerId,
                        MainFicheId = x.cust.Mainfiche,
                        SubficheTagName = x.cust.Mainfiche == null ? sfNull : x.cust.CustomerId + "|" + x.cust.SearchName
                    }).ToList();
                var res = JsonConvert.SerializeObject(rtn);
                return rtn;
            }
        }

        public List<TempTicketItem> GetTickets(List<int> customerIds)
        {
            var start = new DateTime(2019, 6, 17, 9, 0, 0);
            var end = start.AddMinutes(1000);
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var rtn = ent.Message
                    .Where(x => customerIds.Contains(x.CustomerId)
                        && x.MessageDateTime >= start && x.MessageDateTime < end)
                    .Select(x => new TempTicketItem()
                    {
                        TicketText = x.MessageText,
                        CustomerId = x.CustomerId,
                        MainFicheId = x.Customer.Mainfiche,
                        CreatedCRMUser = x.CreateUser.Trim(),
                        CreatedInCRM = x.MessageDateTime,
                        LastModifiedCRMUser = x.LastModifiedUser.Trim(),
                        LastModifiedInCRM = x.LastModifiedDate ?? DateTime.Now,
                        OriginalCCMMessageId = x.Messageid
                    }).ToList();
                var res = JsonConvert.SerializeObject(rtn);
                return rtn;
            }
        }

        public List<TempDNIS> GetAllDNIS(Dictionary<int, string> companyAndCustomerIds)
        {
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var rtn = ent.CustomerDNIS.Where(x => companyAndCustomerIds.Keys.Contains(x.CustomerId))
                    .Select(x => new TempDNIS()
                    {
                        DNIS = x.DNIS,
                        CustomerIdCCM = x.CustomerId,
                        CompanyId = companyAndCustomerIds.Single(y => y.Key == x.CustomerId).Value
                    }).ToList();
                return rtn;
            }
        }

        public List<TempDNIS> GetDNisTest(List<int> customerIds)
        {
            using (EDMX.ProdtelesecEntities ent = new EDMX.ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var rtn = ent.CustomerDNIS.Where(x => customerIds.Contains(x.CustomerId))
                    .Select(x => new TempDNIS()
                    {
                        DNIS = x.DNIS,
                        CustomerIdCCM = x.CustomerId,
                        AnouncementSeconds = x.Customer.AnnouncementTime ?? 0,
                        ExtraSeconds = x.Customer.ExtraTime ?? 0
                    }).ToList();
                return rtn;
            }
        }
    }
}