﻿using AutoMapper;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CRM.Domain.Shared.Enums;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.Logic.Queries
{
    public class QueryFromWebPortal
    {
        private bool _isActiphone;
        private string _wpConString;

        public QueryFromWebPortal(bool isActiphone)
        {
            SetConnectionString(isActiphone);
        }

        public void ChangeConnectionString(bool isActiphone)
        {
            SetConnectionString(isActiphone);
        }

        private void SetConnectionString(bool isActiphone)
        {
            var initCat = isActiphone ? "WPActiphone" : "WPAac";
            _isActiphone = isActiphone;
            _wpConString = @"data source=(LocalDb)\MSSQLLocalDB;initial catalog=" + initCat + @";integrated security=True;MultipleActiveResultSets=True;App=EntityFramework";
        }

        /// <summary>
        /// Haalt alle actieve companies op uit webportal
        /// </summary>
        /// <returns></returns>
        public List<CompanyOverview> GetCompanyGuidsWP()
        {
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                var allCompanyGuids = ent.Companies.Where(x => x.IsDeleted == false).Select(x => new CompanyOverview()
                {
                    CompanyId = x.Id
                }).ToList();

                return allCompanyGuids;
            }
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="tagType"></param>
        /// <param name="mapper"></param>
        /// <param name="filter">De filter is een lijst van tags die je wil verifiëren zodat er geen duplicaten in de lijst kunnen komen te zitten.</param>
        /// <returns></returns>
        public List<WPTag> GetTagsByType(TagType tagType, IMapper mapper, List<string> filter = null)
        {
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                List<EDMX.Tags> tags;
                if (filter == null)
                {
                    tags = ent.Tags.Where(x => x.TypeOfTag == (int)tagType).ToList();
                }
                else
                {
                    tags = ent.Tags.Where(x => x.TypeOfTag == (int)tagType && filter.Contains(x.TagName)).ToList();
                }
                return mapper.Map<List<EDMX.Tags>, List<WPTag>>(tags);
            }
        }

        /// <summary>
        /// Zit een employee reeds in de webportal, dan kan hij niet opnieuw opgeladen worden. Deze methode haalt de reeds bestaande employees uit de lijst van potential Employees.
        /// </summary>
        /// <param name="companyInfo"></param>
        /// <param name="potentialEmployees"></param>
        /// <returns></returns>
        public List<TempEmployeeItem> FilterPotentialEmployees(List<WPModules> companyInfo, List<TempEmployeeItem> potentialEmployees)
        {
            var companyIds = companyInfo.Select(x => x.CompanyIdWP).Distinct().ToList();
            List<TempEmployeeItem> employeesInWP = null;
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                employeesInWP = ent.Employees.Where(x => companyIds.Contains(x.CompanyId) && x.Companies.OriginalCCMCustomerId.HasValue)
                    .Select(x => new TempEmployeeItem()
                    {
                        Firstname = x.FirstName,
                        LastName = x.LastName,
                        CustomerId = x.Companies.OriginalCCMCustomerId.Value,
                    })
                    .ToList();
            }
            if (employeesInWP == null || employeesInWP.Count == 0)
                return potentialEmployees;

            //We brengen geen wijzigingen aan. Zit een employee reeds in de webportal dan kan hij niet meer gewijzigd worden.
            var totallyNewEmployees = potentialEmployees.Where(x =>
            (!(employeesInWP.Select(y => y.CustomerId).Contains(x.CustomerId)
            || (x.MainFicheId.HasValue
            && employeesInWP.Select(y => y.CustomerId).Contains(x.MainFicheId.Value)))
            && (employeesInWP.Select(y => y.Firstname).Contains(x.Firstname)
            && employeesInWP.Select(y => y.LastName).Contains(x.LastName)))
            )
                .OrderBy(x => x.MainFicheId)
                .ToList();

            return totallyNewEmployees;
        }

        public void GetCallScriptsByCompanyId(Guid companyId)
        {
            using (EDMX.WPAacEntities ent = new EDMX.WPAacEntities())
            {
                ent.Database.Connection.ConnectionString = _wpConString;
                var callScripts = ent.CallScripts.Where(x => x.CompanyId == companyId)
                    .Select(x => new
                    {
                        Id = x.CallScriptId,
                        Body = x.CallScriptBody,
                        Tags = x.TagsCallScript.ToList(),
                        Comments = x.Comments.OrderBy(y => y.CreatedOn).Take(20).Skip(20).ToList()
                    }).ToList();
            }
        }
    }
}