﻿using System;
using AutoMapper;
using WebPortal.CCMConnector.Application.Customers.Models;

namespace WebPortal.CCMConnector.Data.Mapper
{

    public class ServiceMappingProfile : Profile
    {
        public ServiceMappingProfile()
        {
            CreateMap<Models.TempWPCustomerModule, EDMX.WPCustomerModules>()
                .ForMember(ent => ent.WPSubscriptionId, opt => opt.MapFrom(dto => dto.WPSubscriptionId))
                .ReverseMap()
                ;

            CreateMap<Models.TempWPSubscription, EDMX.WPSubscriptions>()
                   .ReverseMap();

            CreateMap<Models.WPSubscriptionOverview, EDMX.WPSubscriptions>()
                .ForMember(ent => ent.WPCustomerModules, opt => opt.MapFrom(dto => dto.Modules))
                .ReverseMap();
            CreateMap<Models.WPModules, EDMX.WPCustomerModules>()
                .ReverseMap()
                .ForMember(dto => dto.CustomerIdWP, opt => opt.MapFrom(ent => ent.WPSubscriptions.CustomerIdWP));

            CreateMap<EDMX.DutyAlarm, DutyAlarmModel>()
                .ForMember(m => m.OnFixedDate, opt => opt.MapFrom(d => d.OnFixedDate ?? false))
                .ForMember(m => m.UseEveryDay, opt => opt.MapFrom(d => d.UseEveryDay ?? false))
                .ForMember(m => m.UseWeekDay, opt => opt.MapFrom(d => d.UseWeekDay ?? false))
                .ForMember(m => m.UseMonthDay, opt => opt.MapFrom(d => d.UseMonthDay ?? false))

                .ReverseMap();

            CreateMap<EDMX.Message, EDMX.Histo_msg>()
                .ForMember(h => h.DateTime, opt => opt.MapFrom(m => m.LastModifiedDate ?? new DateTime()))
                .ForMember(h => h.Customerid, opt => opt.MapFrom(m => m.CustomerId))
                .ForMember(h => h.UserName, opt => opt.MapFrom(m => m.LastModifiedUser))
                .ReverseMap();


            #region comments

            //naar webPortal
            //CreateMap<Models.NewWPCompany, EDMX.Companies>()
            //    .ForMember(ent => ent.OriginalCCMCustomerId, opt => opt.MapFrom(dto => dto.CustomerId))
            //    .ReverseMap();
            //CreateMap<Models.NewWPCompanyBranch, EDMX.CompanyBranches>()
            //    .ForMember(ent => ent.OriginalCCMSubficheId, opt => opt.MapFrom(dto => dto.SubFicheId))
            //    .ReverseMap();
            //CreateMap<Models.NewAddress, EDMX.Adresses>().ReverseMap();
            //CreateMap<Models.WPContactDetail, EDMX.ContactDetails>().ReverseMap();

            //CreateMap<Models.WPTag, EDMX.Tags>().ReverseMap();
            //CreateMap<Models.WPEmployee, EDMX.Employees>().ReverseMap();
            //CreateMap<Models.WPTag, EDMX.TagsEmployee>()
            //    .ForMember(ent => ent.Tags, opt => opt.MapFrom(dto => dto))
            //    .ReverseMap();
            //CreateMap<Models.WPTag, EDMX.TagsTicket>()
            //    .ForMember(ent => ent.Tags, opt => opt.MapFrom(dto => dto))
            //    .ReverseMap();
            //CreateMap<Models.WPTag, EDMX.TagsCallScript>()
            //    .ForMember(ent => ent.Tags, opt => opt.MapFrom(dto => dto))
            //    .ReverseMap();

            //CreateMap(typeof(Source<>), typeof(Destination<>));


            //naar CCM
            //CreateMap<Models.NewWPCompany, EDMX.WPSubscriptions>()
            //    .ForMember(ent => ent.CompanyIdWP, opt => opt.MapFrom(dto => dto.Id))
            //    .ForMember(ent => ent.CustomerIdCCM, opt => opt.MapFrom(dto => dto.CustomerId))
            //    .ForMember(ent => ent.Id, opt => opt.Ignore());



            //CreateMap<EDMX.Treeview, Models.TempEmployeeItem>()
            //    .ForMember(dto => dto.MainFicheId, opt => opt.MapFrom(ent => ent.Customer.Mainfiche))
            //    .ForMember(dto => dto.TagServiceName, opt => opt.MapFrom(ent => ent.Service_Organisation.SERVICE_NAME.Trim()))
            //    .ForMember(dto => dto.TagSubficheName, opt => opt.MapFrom(ent => ent.Customer.Mainfiche.HasValue
            //        ? $"{ent.Customer.CustomerId.ToString()}|{ent.Customer.SearchName.Trim()}"
            //        : null))
            //    .ForMember(dto => dto.Firstname, opt => opt.MapFrom(ent => ent.Firstname.Trim()))
            //    .ForMember(dto => dto.LastName, opt => opt.MapFrom(ent => ent.LastName.Trim()))
            //    .ForMember(dto => dto.Position, opt => opt.MapFrom(ent => ent.Position.Trim()))
            //    .ForMember(dto => dto.OfficePhone, opt => opt.MapFrom(ent => ent.OfficePhone.Trim()))
            //    .ForMember(dto => dto.PrivatePhone, opt => opt.MapFrom(ent => ent.PrivatePhone.Trim()))
            //    .ForMember(dto => dto.GSM, opt => opt.MapFrom(ent => ent.GSM.Trim()))
            //    .ForMember(dto => dto.Comments, opt => opt.MapFrom(ent => ent.CommentText))
            //    .ReverseMap();

            #endregion
        }
    }

    //public class Source<T>
    //{
    //    public T Value { get; set; }
    //}

    //public class Destination<T>
    //{
    //    public T Value { get; set; }
    //}

}
