﻿
using System.Linq;
using CRM.Domain.Branch.ReadModels;
using WebPortal.CCMConnector.Data.Validation;

namespace WebPortal.CCMConnector.Data.Models.AbstractModels
{
    public abstract class ABranchModel
    {
        public string BranchName { get; set; }
        public string Street { get; set; }
        public string City { get; set; }
        public string PostalCode { get; set; }
        public string Country { get; set; }
        public string HouseNumber { get; set; }
        public string BoxNumber { get; set; }

        //ContactDetails
        public string FaxNumber { get; set; }
        public string Fax { get; set; }
        public string DefaultFax { get; set; }
        public string EmailAddress { get; set; }
        public string Email { get; set; }
        public string DefaultEmail { get; set; }
        public string Phone1 { get; set; }
        public string Phone2 { get; set; }
        public string HomePage { get; set; }

        public bool NeedToUpdateAddress(BranchReadModel brm)
        {
            if (brm.Address.City != City) return true;
            if (brm.Address.PostalCode != PostalCode) return true;
            if (brm.Address.Country != Country) return true;
            if (brm.Address.Street != Street) return true;
            if (brm.Address.Number != HouseNumber) return true;
            if (brm.Address.PostalBoxNr != BoxNumber) return true;

            return false;
        }

        public bool NeedToUpdateContactDetails(BranchReadModel brm)
        {

            var yup = NeedToUpdateByPropName("Office phone", "Phone1", brm);
            if (yup) return true;

            yup = NeedToUpdateByPropName("Office phone 2", "Phone2", brm);
            if (yup) return true;

            yup = NeedToUpdateByPropName("Website", "HomePage", brm);
            if (yup) return true;

            var allEmails = new TempEmail()
            {
                EmailAddress = $"{Email};{EmailAddress};{DefaultEmail}"
            };
            SplitAndValidate.ValidateAndSplitEmails(allEmails);
            int emailcounter = 0;
            foreach (var m in allEmails.EmailList)
            {
                string email = emailcounter > 0 ? $"Email {emailcounter}" : "Email";
                emailcounter++;
                yup = NeedToUpdate(email, m, brm);
                if (yup) return true;
            }
            return false;
        }

        private bool NeedToUpdateByPropName(string cdName,
            string propName, BranchReadModel brm)
        {
            var myProp = typeof(ABranchModel).GetProperty(propName);

            string myString = (string) myProp.GetValue(this);
            return NeedToUpdate(cdName, myString, brm);

        }

        private bool NeedToUpdate(string cdName,
            string myString, BranchReadModel brm)
        {
            if (string.IsNullOrWhiteSpace(myString)
                && brm.ContactDetails.SingleOrDefault(x
                    => x.ContactDetailName == cdName) == null)
                return false;

            if (string.IsNullOrWhiteSpace(myString) &&
                brm.ContactDetails.SingleOrDefault(x
                    => x.ContactDetailName == cdName) != null)
                return true;

            if (!string.IsNullOrWhiteSpace(myString) &&
                brm.ContactDetails.SingleOrDefault(x
                    => x.ContactDetailName == cdName) == null)
                return true;


            if (myString != brm.ContactDetails.SingleOrDefault(x
                        => x.ContactDetailName == cdName)
                    .ContactDetailValue)
                return true;

            return false;

        }

    }
}
