﻿using System;
using System.Collections.Generic;
using System.Linq;
using Newtonsoft.Json;
using WebPortal.CCMConnector.Data.Models.ForJsonOutput;

namespace WebPortal.CCMConnector.Data.Models
{
    public class TempTicketItem
    {
        private static readonly NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        public string TicketText { get; set; }
        public string TicketId { get; set; }
        public int CustomerId { get; set; }
        public int? MainFicheId { get; set; }
        public string CreatedCRMUser { get; set; }
        public DateTime CreatedInCRM { get; set; }
        public string LastModifiedCRMUser { get; set; }
        public DateTime LastModifiedInCRM { get; set; }
        public int OriginalCCMMessageId { get; set; }
        public string DossierId { get; set; }
        public List<string> Tags { get; set; }
        public string InteractionId { get; set; }

        public TempTicketItem()
        {
        }

        public TempTicketItem(TempMessageAndVersion message, List<TempDynamicParameter> dynamicParameters = null)
        {
            TicketText = SerializeToJson(message, dynamicParameters);
            //if (message.StructuredMessages != null && message.StructuredMessages.Count > 0)
            //{
            //    TicketText = MakeTicketJson(message, dynamicParameters);
            //}
            //else
            //{
            //    TicketText = AddMessageText(message.MessageText.Trim(), true);
            //}
            CustomerId = message.MainAndSubficheIdAndDossierId.CustomerIdCCM;
            MainFicheId = message.MainFicheId;
            CreatedCRMUser = message.CreatedCRMUser;
            CreatedInCRM = message.CreatedInCRM;
            LastModifiedCRMUser = message.LastModifiedCRMUser;
            LastModifiedInCRM = message.LastModifiedInCRM;
            OriginalCCMMessageId = message.MessageId;
            DossierId = message.MainAndSubficheIdAndDossierId.DossierId;
            InteractionId = message.InteractionId;
            Tags = string.IsNullOrWhiteSpace(message.SubficheTag)
                ? null : new List<string> { message.SubficheTag };
        }

        private static string SerializeToJson(TempMessageAndVersion message, List<TempDynamicParameter> dynamicParameters)
        {
            try
            {
                TicketText ticketText = new TicketText
                {
                    MessageText = message.MessageText.Trim()
                };
                if (message.StructuredMessages != null && message.StructuredMessages.Count > 0)
                {
                    SerializeParameters(message, dynamicParameters, ticketText);
                }

                return JsonConvert.SerializeObject(ticketText);
            }
            catch (Exception e)
            {
                Logger.Error(e, $"Couldn't serialize ticket with messageId '{message.MessageId}' for CCM-customer {message.CustomerId}");
                throw;
            }
        }

        private static void SerializeParameters(TempMessageAndVersion message, List<TempDynamicParameter> dynamicParameters, TicketText ticketText)
        {
            ticketText.TicketName = message.StructuredMessages.First().MessageName.Trim();
            ticketText.MessageId = message.MessageId.ToString();
            ticketText.TicketFields = new List<TicketField>();
            ticketText.InteractionId = message.InteractionId;
            foreach (var sm in message.StructuredMessages)
            {
                if (sm.DynamicParameterId.HasValue
                    && dynamicParameters != null
                    && dynamicParameters.Count > 0)
                {
                    dynamicParameters = dynamicParameters.OrderBy(x => x.OrderId).ToList();
                    foreach (var dp in dynamicParameters)
                    {
                        TicketField tField = new TicketField
                        {
                            FieldLabel = dp.Question.Trim(),
                            FieldValue = dp.Answer.Trim()
                        };
                        ticketText.TicketFields.Add(tField);
                    }
                }
                else //reguliere parameter
                {
                    TicketField rField = new TicketField
                    {
                        FieldLabel = sm.LabelText.Trim(),
                        FieldValue = sm.MessageValue.Trim()
                    };
                    ticketText.TicketFields.Add(rField);
                }
            }
        }

        private static string AddMessageText(string messageText, bool noStructuredMessages)
        {
            string rtn = noStructuredMessages
                ? "{\"messageText\":" + $"\"{messageText}\"" + "}"
                : "}," + $"\"messageText\":\"{messageText}\"" + "}";
            return rtn;
        }
    }
}