﻿using System;
using System.Linq;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.EDMX;

namespace WebPortal.CCMConnector.Data.TestGenerators
{
    public class ReplaceModificationsGenerator : ACCMCommandAndQuery
    {
        public static void GenerateReplaceModifications(string customerIdSource,
            string customerIdDestination, string dossierId = null)
        {
            using (EDMX.ProdtelesecEntities ent = new ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var subscription = ent.WPSubscriptions
                    .SingleOrDefault(x => !x.Deleted
                     && x.CustomerIdWP == customerIdDestination);
                if (subscription == null) return;

                var modif = AddModification(subscription.ClientIdCCM, subscription.Id, customerIdSource,
                    customerIdDestination, dossierId);

                ent.WPModificationUpload.Add(modif);
                ent.SaveChanges();

            }
        }

        private static WPModificationUpload AddModification( int ccmId, Guid subGuid,
            string customerIdSource, string customerIdDestination, 
             string dossierId = null)
        {
            WPModificationUpload modification = new WPModificationUpload()
            {
                CCMLevel = dossierId == null ? "CLIENT": "MAIN",
                CCMId = ccmId,
                CreatedBy = "Generator",
                CreatedOn = DateTime.UtcNow,
                LastModifiedOn = DateTime.UtcNow,
                LastModifiedBy = "Generator",
                SubscriptionId = subGuid,
                ModificationPinPoint = $"Customer|Replace|{customerIdSource}|{customerIdDestination}"
            };
            return modification;
        }

    }
}
