﻿using System;
using System.Collections.Generic;
using System.Linq;
using WebPortal.CCMConnector.Data.ConnectorLogic.General;
using WebPortal.CCMConnector.Data.EDMX;
using WebPortal.CCMConnector.Data.Models;

namespace WebPortal.CCMConnector.Data.TestGenerators
{
    public class SubscriptionGenerator : ACCMCommandAndQuery
    {

        public void GenerateNewSubscription(int clientId, int maxUsers, List<ModulesToUpload> modules)
        {
            List<WPCustomerModules> newModules = new List<WPCustomerModules>();
            var toRemove = new List<WPCustomerModules>();
            var modificationUploads = new List<WPModificationUpload>();

            //Kijk eerst of er al een subscription bestaat voor de clientId
            using (EDMX.ProdtelesecEntities ent = new ProdtelesecEntities())
            {
                ent.Database.Connection.ConnectionString = _prodTelesecConString;
                var subToUpdate = ent.WPSubscriptions.SingleOrDefault(x => x.ClientIdCCM == clientId);
                bool isActiphone = ent.Client.Single(x => x.ClientId == clientId).Tenant == "Actiphone";

                if (subToUpdate == null)
                {
                    Guid custId = Guid.NewGuid();
                    //Subscriptie en modules toevoegen
                    WPSubscriptions subscrip = new WPSubscriptions()
                    {
                        ClientIdCCM = clientId,
                        CreatedBy = "Generator",
                        CreatedOn = DateTime.UtcNow,
                        LastModifiedOn = DateTime.UtcNow,
                        Id = custId,
                        IsActiphoneCust = isActiphone,
                        StartDate = DateTime.UtcNow,
                        LastModifiedBy = "Generator",
                        UserAmount = maxUsers,
                        MonthlySubscriptionPrice = 0,
                    };

                    foreach (var m in modules)
                    {
                        AddModule1(m, custId, newModules);
                    }

                    ent.WPSubscriptions.Add(subscrip);
                }
                else
                {
                    //module nakijken
                    AddClientSubscriptionModification(subToUpdate.Id, subToUpdate.ClientIdCCM, modificationUploads, ent);
                    foreach (var m in modules)
                    {
                        var existing =
                            subToUpdate.WPCustomerModules.SingleOrDefault(x =>
                                x.CCMId == m.CCMId && x.CCMLevel == m.CCMLevel && x.ModuleId == (int)m.NameModule);
                        if (existing != null)
                        {
                            continue;
                        }


                        //Ga na: bestaat de ModuleId al?
                        var existingModuleId = subToUpdate.WPCustomerModules
                            .Where(x => x.ModuleId == (int)m.NameModule);
                        if (existingModuleId.Count() > 0)
                        {
                            //de moduleId is er reeds.
                            //is m van een lager niveau dan existing module, dan moet existing module vernietigd worden.
                            switch (m.CCMLevel)
                            {
                                case "CLIENT": break;
                                case "MAIN":
                                    var higherlevel = existingModuleId
                                        .Where(x => x.CCMLevel == "CLIENT");
                                    toRemove.AddRange(higherlevel);
                                    var dossierName = ent.Customer
                                        .Single(x => x.CustomerId == m.CCMId).Name;
                                    AddModification(m.CCMLevel, m.CCMId, subToUpdate.Id, dossierName, modificationUploads);

                                    break;
                                case "SUB":
                                    var higherlevel2 = existingModuleId
                                        .Where(x => x.CCMLevel == "CLIENT");
                                    toRemove.AddRange(higherlevel2);

                                    var higherLevelMain = existingModuleId
                                        .Where(x => x.CCMLevel == "MAIN").ToList();

                                    if (higherLevelMain.Count == 0)
                                    {
                                        break;
                                    }

                                    //behoort de m tot één van de mainfiches?
                                    var mainF = ent.Customer
                                        .Where(x => x.CustomerId == m.CCMId)
                                        .Select(x => new
                                        {
                                            x.Mainfiche.Value,
                                            x.Name
                                        })
                                        .Single();
                                    var mainToRemove = higherLevelMain.SingleOrDefault(x => x.CCMId == mainF.Value);
                                    toRemove.Add(mainToRemove);
                                    AddModification("MAIN", mainF.Value, subToUpdate.Id, mainF.Name, modificationUploads);


                                    break;

                            }

                        }

                        AddModule1(m, subToUpdate.Id, newModules);
                    }

                }
                ent.WPCustomerModules.AddRange(newModules);
                ent.WPCustomerModules.RemoveRange(toRemove);
                ent.WPModificationUpload.AddRange(modificationUploads);
                ent.SaveChanges();
            }
        }

        private static void AddModification(string ccmLevel, int ccmId, Guid subGuid,
            string dossierName,
            List<WPModificationUpload> modificationUploads)
        {
            WPModificationUpload modification = new WPModificationUpload()
            {
                CCMLevel = ccmLevel,
                CCMId = ccmId,
                CreatedBy = "Generator",
                CreatedOn = DateTime.UtcNow,
                LastModifiedOn = DateTime.UtcNow,
                LastModifiedBy = "Generator",
                SubscriptionId = subGuid,
                ModificationPinPoint = $"Dossier|Create|{dossierName}"
            };
            modificationUploads.Add(modification);
        }

        private static void AddClientSubscriptionModification(Guid subGuid, int ccmId, List<WPModificationUpload> modificationUploads, ProdtelesecEntities ent)
        {
            var existing = ent.WPModificationUpload.SingleOrDefault(x =>
                x.CCMId == ccmId && !x.IsUploadedInWP && x.SubscriptionId == subGuid);
            if (existing != null) return;

            WPModificationUpload modification = new WPModificationUpload()
            {
                CCMLevel = "CLIENT",
                CCMId = ccmId,
                CreatedBy = "Generator",
                CreatedOn = DateTime.UtcNow,
                LastModifiedOn = DateTime.UtcNow,
                LastModifiedBy = "Generator",
                SubscriptionId = subGuid,
                ModificationPinPoint = $"Customer|Subscription"
            };
            modificationUploads.Add(modification);
        }

        private static void AddModule1(ModulesToUpload m,
            Guid custId,
            List<WPCustomerModules> newModules)
        {
            WPCustomerModules module = new WPCustomerModules()
            {
                CCMLevel = m.CCMLevel,
                CCMId = m.CCMId,
                WPSubscriptionId = custId,
                StartDate = DateTime.UtcNow,
                CreatedBy = "Generator",
                CreatedOn = DateTime.UtcNow,
                LastModifiedOn = DateTime.UtcNow,
                LastModifiedBy = "Generator",
                ModuleId = (int)m.NameModule,
                UnitPrice = new decimal(0)
            };
            newModules.Add(module);
        }

    }
}
