﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Branch;
using CRM.Domain.Branch.Entities;
using CRM.Domain.Branch.Models;
using CRM.Domain.Branch.ReadModels;
using CRM.Domain.Customer;
using CRM.Domain.Shared.Enums;
using EventFlow;
using EventFlow.Configuration;
using EventFlow.Extensions;
using EventFlow.MongoDB.Extensions;
using EventFlow.Queries;
using WebPortal.Application.Branches.Commands;
using WebPortal.Application.Branches.Queries;
using Xunit;

namespace WebPortal.XUnitTest
{
    public class TestBranch
    {
        private static Guid _branchGuid = new Guid("458a1166-751d-41c7-9ae6-7b22ed5fc105");
        private BranchId _branchId = BranchId.With(_branchGuid);
        private static Guid _custGuid = new Guid("3807a41d-c74b-40ab-aaae-207913576a82");
        private CustomerId _customerId = CustomerId.With(_custGuid);

        private static Guid _contactDetailGuid1 = new Guid("ca5bdd82-0bec-48d1-a85d-50a200e9e317");
        private static Guid _contactDetailGuid2 = new Guid("856f78d4-0e4f-4c4a-98dd-22c7b4b69a21");
        private ContactDetailId _cdId1 = ContactDetailId.With(_contactDetailGuid1);
        private ContactDetailId _cdId2 = ContactDetailId.With(_contactDetailGuid2);

        private static IRootResolver Setup()
        {
            //var conString = "mongodb://aac-dev01.cisicc.local";

            var conString = "mongodb://localhost";

            return EventFlowOptions.New
                .AddDefaults(typeof(BranchAggregate).Assembly)
                .AddCommands(new Type[]{typeof(CreateBranchCommand),
                    typeof(ModifyBranchContactDetailsCommand),
                    typeof(ModifyBranchAddressCommand),
                    typeof(ModifyBranchCustomerIdCommand),
                    typeof(ModifyBranchNameCommand),
                    typeof(DeleteBranchCommand)})
                .AddCommandHandlers(new Type[] { typeof(CreateBranchCommandHandler),
                    typeof(ModifyBranchContactDetailsCommandHandler),
                    typeof(ModifyBranchAddressCommandHandler),
                    typeof(ModifyBranchCustomerIdCommandHandler),
                    typeof(ModifyBranchNameCommandHandler),
                    typeof(DeleteBranchCommandHandler)})
                .AddQueryHandlers(new Type[] { typeof(GetBranchByBranchIdQueryHandler),
                    typeof(GetBranchesByCustomerIdQueryHandler) })
                .ConfigureMongoDb(conString, "WPMongoTest")
                .UseMongoDbEventStore()
                .UseMongoDbSnapshotStore()
                .UseMongoDbReadModel<BranchReadModel>()

                //.AddAspNetCore()
                .CreateResolver();
        }

        [Fact]
        public void TestCreateBranch()
        {
            var collection = Setup();
            var commandBus = collection.Resolve<ICommandBus>();
            var queryProcessor = collection.Resolve<IQueryProcessor>();

            AddressModel am = new AddressModel
            {
                City = "Testcity",
                Country = "TestCountry",
                Number = "TestNr 5",
                PostalCode = "4567",
                PostalBoxNr = "56",
                Street = "TextStreet",
                State = "TestState"
            };

            ContactDetailModel c1 = new ContactDetailModel(_cdId1.Value, "Tel", "456", false, ContactDetailType.TelephoneNr);
            ContactDetailModel c2 = new ContactDetailModel(_cdId2.Value, "Email", "Testing@an.be", true, ContactDetailType.Email);

            List<ContactDetailModel> cdList = new List<ContactDetailModel> { c1, c2 };

            var command = new CreateBranchCommand(_branchId, "Test", am, _customerId.Value, cdList);
            var result = commandBus.PublishAsync(command, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);

            am.Street = "Aangepaste straat";
            am.State = "Ergens anders";
            var command2 = new ModifyBranchAddressCommand(_branchId, am);
            result = commandBus.PublishAsync(command2, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);

            ContactDetailModel c3 = new ContactDetailModel(ContactDetailId.New.Value, "Website", "somesite.com", false, ContactDetailType.Website);

            List<ContactDetailModel> cdList2 = new List<ContactDetailModel> { c1, c3 };
            var command3 = new ModifyBranchContactDetailsCommand(_branchId, cdList2);
            result = commandBus.PublishAsync(command3, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);

            var command4 = new ModifyBranchNameCommand(_branchId, "Aangepaste naam");
            result = commandBus.PublishAsync(command4, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);

            var q1 = new GetBranchByBranchIdQuery(_branchId.Value);
            var oneBranch = queryProcessor.Process(q1, new CancellationToken());

            Assert.True(q1 != null);

            var q2 = new GetBranchesByCustomerIdQuery(_customerId.Value);
            var branchList = queryProcessor.Process(q2, new CancellationToken());

            Assert.True(branchList.Count > 0);
        }
    }
}