﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using CRM.Domain.CallScript;
using CRM.Domain.CallScript.Entities;
using CRM.Domain.CallScript.ReadModels;
using CRM.Domain.Customer;
using CRM.Domain.User;
using EventFlow;
using EventFlow.Configuration;
using EventFlow.Extensions;
using EventFlow.MongoDB.Extensions;
using EventFlow.Queries;
using WebPortal.Application.CallScripts.Commands;
using WebPortal.Application.CallScripts.Queries;
using WebPortal.CCMConnector.Application.CallScripts.Commands;
using Xunit;

namespace WebPortal.XUnitTest
{
    public class TestCallScript
    {
        private static Guid _callScriptGuid = new Guid("1b3581cf-9c10-4b35-8836-b8a330bd05c1");
        private CallScriptId _callScriptId = CallScriptId.With(_callScriptGuid);

        private static Guid _custGuid = new Guid("3807a41d-c74b-40ab-aaae-207913576a82");
        private CustomerId _customerId = CustomerId.With(_custGuid);

        private static Guid _userGuid = new Guid("09591c98-b4e9-4641-94a1-07442138eb00");
        private UserId _userId = UserId.With(_userGuid);

        private static IRootResolver Setup()
        {
            //var conString = "mongodb://aac-dev01.cisicc.local";

            var conString = "mongodb://localhost";

            return EventFlowOptions.New
                .AddDefaults(typeof(CallScriptAggregate).Assembly)
                .AddCommands(new Type[]
                {
                    typeof(CreateCallScriptCommand),
                    typeof(ModifyCallScriptCommentCommand),
                    typeof(CreateCallScriptCommentCommand),
                    typeof(RemoveCallScriptCommentCommand),
                })
                .AddCommandHandlers(new Type[]
                {
                    typeof(CreateCallScriptCommandHandler),
                    typeof(ModifyCallScriptCommentCommandHandler),
                    typeof(CreateCallScriptCommentCommandHandler),
                    typeof(RemoveCallScriptCommentCommandHandler),
                })
                .AddQueryHandlers(new Type[]
                {
                    typeof(GetCallScriptsWithCommentsQueryHandler),
                    typeof(GetMoreCommentsForCallScriptQueryHandler)
                })
                .ConfigureMongoDb(conString, "WPMongoTest")
                .UseMongoDbEventStore()
                .UseMongoDbSnapshotStore()
                .UseMongoDbReadModel<CallScriptReadModel>()

                //.AddAspNetCore()
                .CreateResolver();
        }

        //[Fact]
        //public void TestCreateCallScript()
        //{
        //    var collection = Setup();
        //    var commandBus = collection.Resolve<ICommandBus>();
        //    var queryProcessor = collection.Resolve<IQueryProcessor>();
        //    List<string> tags = new List<string> {"I wonder", "how these tags", "will be combined"};
        //    var command = new CreateCallScriptCommand(_callScriptId, "Dit is de body van de callScript",
        //        _customerId.Value, tags);
        //    var result = commandBus.PublishAsync(command, new System.Threading.CancellationToken())
        //        .ConfigureAwait(false).GetAwaiter().GetResult();
        //    Assert.True(result.IsSuccess);

        //    var command2 = new CreateCallScriptCommentCommand(_callScriptId, "Deze comment staat in plain text.",
        //        "Geen quote beschikbaar", _userId.Value);
        //    result = commandBus.PublishAsync(command2, new System.Threading.CancellationToken()).ConfigureAwait(false)
        //        .GetAwaiter().GetResult();
        //    Assert.True(result.IsSuccess);

        //    var q1 = new GetCallScriptsWithCommentsQuery(_customerId.Value, 20, true);
        //    var callScripts = queryProcessor.Process(q1, new CancellationToken());

        //    Assert.True(callScripts.Count > 0);

        //    var commentId = callScripts.First().Comments.First().Id;

        //    var command3 = new ModifyCallScriptCommentCommand(_callScriptId, CallScriptCommentId.With(commentId),
        //        "Een aanpassing", "Nog steeds niets", _userId.Value);
        //    result = commandBus.PublishAsync(command3, new System.Threading.CancellationToken()).ConfigureAwait(false)
        //        .GetAwaiter().GetResult();
        //    Assert.True(result.IsSuccess);

        //    for (int i = 0; i < 7; i++)
        //    {
        //        var commandComment = new CreateCallScriptCommentCommand(_callScriptId, $"Dit is comment nummer {i + 1}",
        //            $"Dit is quote nr. {i + 1}", _userId.Value);
        //        result = commandBus.PublishAsync(commandComment, new System.Threading.CancellationToken())
        //            .ConfigureAwait(false).GetAwaiter().GetResult();
        //        Assert.True(result.IsSuccess);

        //    }

        //    var command4 = new RemoveCallScriptCommentCommand(_callScriptId, CallScriptCommentId.With(commentId));
        //    result = commandBus.PublishAsync(command4, new System.Threading.CancellationToken())
        //        .ConfigureAwait(false).GetAwaiter().GetResult();
        //    Assert.True(result.IsSuccess);

        //    var q2 = new GetMoreCommentsForCallScriptQuery(_callScriptId.Value, 3, 10, false);
        //    var moreComments = queryProcessor.Process(q2, new CancellationToken());
        //    Assert.Equal(4, moreComments.Count);
        //}
    }
}