﻿using Xunit;
using EventFlow;
using EventFlow.Extensions;
using EventFlow.Configuration;
using CRM.Domain.Customer;
using CRM.Domain.Customer.ReadModels;
using System;
using WebPortal.CCMConnector.Application.Customers.Commands;
using EventFlow.MongoDB.Extensions;
using WebPortal.Application.Customers.Queries;
using EventFlow.Queries;
using System.Threading;
using System.Threading.Tasks;
using CRM.Domain.Branch;
using CRM.Domain.Branch.ReadModels;
using CRM.Domain.CallScript;
using CRM.Domain.CallScript.ReadModels;
using CRM.Domain.Dossier;
using CRM.Domain.Dossier.ReadModels;
using CRM.Domain.Duty;
using CRM.Domain.Duty.ReadModels;
using CRM.Domain.Employee;
using CRM.Domain.Employee.ReadModels;
using CRM.Domain.InteractionStatistic;
using CRM.Domain.InteractionStatistic.ReadModels;
using CRM.Domain.Ticket;
using CRM.Domain.Ticket.ReadModels;
using CRM.Domain.User;
using CRM.Domain.User.ReadModels;
using Moq;
using WebPortal.Application.Branches.Commands;

namespace WebPortal.XUnitTest
{
    public class TestCustomer
    {


        private static Guid _custGuid = new Guid("3807a41d-c74b-40ab-aaae-207913576a82");
        private CustomerId _customerId = CustomerId.With(_custGuid);
        private static Guid _customerBranchGuid = new Guid("89d444bc-3157-442e-bb9e-bb161ac00113");
        string _customerBranchId = "CustomerBranch" + _customerBranchGuid;

        private static IRootResolver Setup()
        {
            //var conString = "mongodb://aac-dev01.cisicc.local";

            var conString = "mongodb://localhost";

            //return EventFlowOptions.New
            //    //.ConfigureMongoDb()
            //    .AddDefaults(typeof(CustomerAggregate).Assembly)
            //    .AddDefaults(typeof(UserAggregate).Assembly)
            //    .AddDefaults(typeof(CustomerAggregate).Assembly)
            //    .AddDefaults(typeof(BranchAggregate).Assembly)
            //    .AddDefaults(typeof(CallScriptAggregate).Assembly)
            //    .AddDefaults(typeof(DossierAggregate).Assembly)
            //    .AddDefaults(typeof(EmployeeAggregate).Assembly)
            //    .AddDefaults(typeof(InteractionStatisticAggregate).Assembly)
            //    .AddDefaults(typeof(TicketAggregate).Assembly)
            //    .AddDefaults(typeof(DutyAggregate).Assembly)
            //    .AddDefaults(typeof(CreateBranchCommand).Assembly)
            //            .UseMongoDbReadModel<CallScriptReadModel>()
            //            .UseMongoDbReadModel<CustomerReadModel>()
            //            .UseMongoDbReadModel<BranchReadModel>()
            //            .UseMongoDbReadModel<DossierReadModel>()
            //            .UseMongoDbReadModel<EmployeeReadModel>()
            //            .UseMongoDbReadModel<InteractionStatisticReadModel>()
            //            .UseMongoDbReadModel<TicketReadModel>()
            //            .UseMongoDbReadModel<UserReadModel>()
            //            .UseMongoDbReadModel<DutyReadModel>()
            //    .CreateResolver();


            //services.AddEventFlow(o =>
            //    o
            //        .ConfigureMongoDb(Configuration["Db:ConnectionString"], Configuration["Db:DbName"])
            //        .AddDefaults(typeof(UserAggregate).Assembly)
            //        .AddDefaults(typeof(CustomerAggregate).Assembly)
            //        .AddDefaults(typeof(BranchAggregate).Assembly)
            //        .AddDefaults(typeof(CallScriptAggregate).Assembly)
            //        .AddDefaults(typeof(DossierAggregate).Assembly)
            //        .AddDefaults(typeof(EmployeeAggregate).Assembly)
            //        .AddDefaults(typeof(InteractionStatisticAggregate).Assembly)
            //        .AddDefaults(typeof(TicketAggregate).Assembly)
            //        .AddDefaults(typeof(DutyAggregate).Assembly)
            //        .AddDefaults(typeof(CreateBranchCommand).Assembly)
            //        .UseMongoDbEventStore()
            //        .UseMongoDbSnapshotStore()
            //        .UseMongoDbReadModel<CallScriptReadModel>()
            //        .UseMongoDbReadModel<CustomerReadModel>()
            //        .UseMongoDbReadModel<BranchReadModel>()
            //        .UseMongoDbReadModel<DossierReadModel>()
            //        .UseMongoDbReadModel<EmployeeReadModel>()
            //        .UseMongoDbReadModel<InteractionStatisticReadModel>()
            //        .UseMongoDbReadModel<TicketReadModel>()
            //        .UseMongoDbReadModel<UserReadModel>()
            //        .UseMongoDbReadModel<DutyReadModel>()
            //        .AddAspNetCore()
            //);


            return EventFlowOptions.New
            .AddDefaults(typeof(CustomerAggregate).Assembly)
            .AddCommands(new Type[]{typeof(CreateCustomerCommand),
                typeof(ModifiyCustomerWebPortalLicenseAmountCommand),
                typeof(ModifyCustomerMainCustomerBranchCommand),
                typeof(ModifyCustomerNameCommand),
                typeof(DeleteCustomerCommand)})
            .AddCommandHandlers(new Type[] { typeof(CreateCustomerCommandHandler),
                typeof(ModifiyCustomerWebPortalLicenseAmountCommandHandler),
                typeof(ModifyCustomerMainCustomerBranchCommandHandler),
                typeof(ModifyCustomerNameCommandHandler),
                typeof(DeleteCustomerCommandHandler)})
            //.AddQueryHandlers(new Type[] { typeof(GetAllCustomersQueryHandler), 
            //    typeof(GetCustomerQueryHandler) })
            .ConfigureMongoDb(conString, "WPMongoTest")
            .UseMongoDbEventStore()
            .UseMongoDbSnapshotStore()
            .UseMongoDbReadModel<CustomerReadModel>()

            //.AddAspNetCore()
            .CreateResolver();
        }

        //[Fact]
        //public async Task CreateCustomer()
        //{
        //    var collection = Setup();
        //    var mongoMock = new Mock<EventFlowOptions>();
        //    collection =

        //    var commandBus = collection.Resolve<ICommandBus>();
        //    var queryProcessor = collection.Resolve<IQueryProcessor>();

        //    var command = new CreateCustomerCommand(_customerId, "TestCustomer", "AAC", 1, 5);
        //    var result = await commandBus.PublishAsync(command, new System.Threading.CancellationToken());

        //    Assert.True(result.IsSuccess);
            

        //}

        [Fact]
        public void TestCreateCustomer()
        {
            var collection = Setup();
            var commandBus = collection.Resolve<ICommandBus>();
            var queryProcessor = collection.Resolve<IQueryProcessor>();

            var query = new GetAllCustomersQuery(true, "CustomerName", 1, 20, "");
            var allCust = queryProcessor.Process(query, new CancellationToken());
            Assert.Equal(1, allCust.Results.Count);


            var command = new CreateCustomerCommand(_customerId, "TestCustomer", "AAC", 1, 5);
            var result = commandBus.PublishAsync(command, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);

            var command2 = new ModifiyCustomerWebPortalLicenseAmountCommand(_customerId, 3);
            result = commandBus.PublishAsync(command2, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);

            var command3 = new ModifyCustomerMainCustomerBranchCommand(_customerId, _customerBranchId);
            result = commandBus.PublishAsync(command3, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);

            var command4 = new ModifyCustomerNameCommand(_customerId, "Aanpassing");
            result = commandBus.PublishAsync(command4, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);

            command4 = new ModifyCustomerNameCommand(_customerId, "Aanpassing 2");
            result = commandBus.PublishAsync(command4, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);

            command2 = new ModifiyCustomerWebPortalLicenseAmountCommand(_customerId, 17);
            result = commandBus.PublishAsync(command2, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);

            //var query = new GetAllCustomersQuery(true, "CustomerName", 1, 20, "");
            //var allCust = queryProcessor.Process(query, new CancellationToken());
            //Assert.Equal(1, allCust.Results.Count);

            query = new GetAllCustomersQuery(false, "CustomerName", 1, 20, "pas");
            allCust = queryProcessor.Process(query, new CancellationToken());
            Assert.Equal(1, allCust.Results.Count);

            query = new GetAllCustomersQuery(true, "CustomerName", 4, 40, "");
            allCust = queryProcessor.Process(query, new CancellationToken());
            Assert.Equal(0, allCust.Results.Count);


            var command6 = new DeleteCustomerCommand(_customerId);
            result = commandBus.PublishAsync(command6, new System.Threading.CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();
            Assert.True(result.IsSuccess);


        }

        [Fact]
        public void TestGetOneCustomer()
        {
            //var collection = Setup();
            //var queryProcessor = collection.Resolve<IQueryProcessor>();

            //var q2 = new GetCustomerQuery(_customerId.Value);
            //var oneCust = queryProcessor.Process(q2, new CancellationToken());
            //Assert.Equal("Aanpassing 2", oneCust.CustomerName);

        }

    }
}
