﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using CRM.Domain.Branch;
using CRM.Domain.Branch.Entities;
using CRM.Domain.Branch.Models;
using CRM.Domain.Branch.ReadModels;
using CRM.Domain.CallScript;
using CRM.Domain.Customer;
using CRM.Domain.Customer.ReadModels;
using CRM.Domain.Dossier;
using CRM.Domain.Dossier.ReadModels;
using CRM.Domain.InteractionStatistic;
using CRM.Domain.InteractionStatistic.ReadModels;
using CRM.Domain.Ticket;
using CRM.Domain.Ticket.ReadModels;
using CRM.Domain.User;
using CRM.Domain.User.Entities;
using CRM.Domain.User.ReadModels;
using CRM.Domain.User.ValueObjects;
using EventFlow;
using EventFlow.Configuration;
using EventFlow.Extensions;
using EventFlow.MongoDB.Extensions;
using WebPortal.Application.Branches.Commands;
using WebPortal.Application.Dossiers.Commands;
using WebPortal.Application.Statistics.Queries;
using WebPortal.CCMConnector.Application.CallScripts.Commands;
using WebPortal.CCMConnector.Application.Customers.Commands;
using WebPortal.CCMConnector.Application.InteractionStats.Commands;
using WebPortal.CCMConnector.Application.Tickets.Commands;
using CRM.Domain.CallScript.ReadModels;
using CRM.Domain.Shared.Enums;
using WebPortal.XUnitTest.TestModels;
using Xunit;
using InteractionType = CRM.Domain.InteractionStatistic.Enums.InteractionType;

namespace WebPortal.XUnitTest
{
    public class TestSeedData
    {
        private static IRootResolver Setup()
        {
            var conString = "mongodb://aac-dev01.cisicc.local";

            //var conString = "mongodb://localhost";

            return EventFlowOptions.New
                //.AddDefaults(typeof(CreateCustomerCommand).Assembly)
                .AddDefaults(typeof(CustomerAggregate).Assembly)
                .AddDefaults(typeof(BranchAggregate).Assembly)
                .AddDefaults(typeof(DossierAggregate).Assembly)
                .AddDefaults(typeof(UserAggregate).Assembly)
                .AddDefaults(typeof(InteractionStatisticAggregate).Assembly)
                .AddDefaults(typeof(CallScriptAggregate).Assembly)
                .AddDefaults(typeof(TicketAggregate).Assembly)
                //.AddDefaults(typeof(BranchAggregate).Assembly)

                .AddCommands(new Type[]
                {
                    typeof(CreateCustomerCommand),
                    typeof(CreateBranchCommand),
                    typeof(ModifyCustomerMainCustomerBranchCommand),
                    typeof(CreateDossierCommand),
                    typeof(CreateInteractionStatCommand),
                    typeof(CreateCallScriptCommand),
                    typeof(CreateTicketCommand)
                })
                .AddCommandHandlers(new Type[]
                {
                    typeof(CreateCustomerCommandHandler),
                    typeof(CreateBranchCommandHandler),
                    typeof(ModifyCustomerMainCustomerBranchCommandHandler),
                    typeof(CreateDossierCommandHandler),
                    typeof(CreateInteractionStatCommandHandler),
                    typeof(CreateCallScriptCommandHandler),
                    typeof(CreateTicketCommandHandler)
                })
                //.AddQueryHandlers(new Type[] { typeof(GetStatisticsQueryHandler) })
                .ConfigureMongoDb(conString, "WPMongoTest")
                .UseMongoDbEventStore()
                .UseMongoDbSnapshotStore()
                .UseMongoDbReadModel<InteractionStatisticReadModel>()
                .UseMongoDbReadModel<CustomerReadModel>()
                .UseMongoDbReadModel<DossierReadModel>()
                .UseMongoDbReadModel<BranchReadModel>()
                .UseMongoDbReadModel<UserReadModel>()
                .UseMongoDbReadModel<CallScriptReadModel>()
                .UseMongoDbReadModel<TicketReadModel>()
                //.AddAspNetCore()
                .CreateResolver();
        }

        //[Fact]
        //public void a1ImportCustomers()
        //{
        //    var collection = Setup();
        //    var commandBus = collection.Resolve<ICommandBus>();
        //    var tempCompanyItems = TempCompanyItem.TestData.OrderBy(x => x.MainFicheId);
        //    var tempList2 = new List<TempCompanyItem>(tempCompanyItems);

        //    var tempStats = TempInteractionStatItem.TestData.OrderBy(x => x.CustomerId);

        //    var tempCallScripts = TempCallScriptItem.TestData;

        //    var tempTickets = TempTicketItem.TestData;

        //    int fakeClientId = 1;

        //    foreach (var tci in tempList2.Where(x => x.MainFicheId == null))
        //    {
        //        string tenant = tci.TenantNumber == 1 ? "AAC" : "Actiphone";
        //        CustomerId custId = CustomerId.New;
        //        var command = new CreateCustomerCommand(custId, tci.CompanyName, tenant, fakeClientId, 5);
        //        var result = commandBus.PublishAsync(command, new CancellationToken()).ConfigureAwait(false).GetAwaiter().GetResult();

        //        Assert.True(result.IsSuccess);

        //        #region MainBranch

        //        BranchId mainBranchId = BranchId.New;
        //        AddressModel address = new AddressModel()
        //        {
        //            City = tci.City,
        //            Country = tci.Country,
        //            Number = tci.HouseNumber,
        //            PostalBoxNr = tci.BoxNumber,
        //            PostalCode = tci.PostalCode,
        //        };
        //        ContactDetailModel det1 = new ContactDetailModel(ContactDetailId.New.Value, "Phone 1", tci.Phone1, false, ContactDetailType.TelephoneNr);
        //        ContactDetailModel det2 = new ContactDetailModel(ContactDetailId.New.Value, "Email", tci.Email, false, ContactDetailType.Email);
        //        var cdList = new List<ContactDetailModel> { det1, det2 };
        //        var commandCreateMb =
        //            new CreateBranchCommand(mainBranchId, tci.CompanyBranchName, address, custId.Value, cdList);
        //        result = commandBus.PublishAsync(commandCreateMb, new CancellationToken())
        //            .ConfigureAwait(false).GetAwaiter().GetResult();
        //        Assert.True(result.IsSuccess);

        //        var commandMb = new ModifyCustomerMainCustomerBranchCommand(custId, mainBranchId.Value);
        //        result = commandBus.PublishAsync(commandMb, new CancellationToken())
        //            .ConfigureAwait(false).GetAwaiter().GetResult();

        //        #endregion MainBranch

        //        #region Dossier

        //        var dossierId = DossierId.New;
        //        var commandDossier = new CreateDossierCommand(dossierId, tci.CompanyName, custId.Value, tci.CustomerId);

        //        result = commandBus.PublishAsync(commandDossier, new CancellationToken())
        //            .ConfigureAwait(false).GetAwaiter().GetResult();
        //        Assert.True(result.IsSuccess);

        //        #endregion Dossier

        //        #region Statistics

        //        foreach (var stat in tempStats.Where(x => x.CustomerId == tci.CustomerId))
        //        {
        //            var commandStat = new CreateInteractionStatCommand(InteractionStatisticId.New, stat.InteractionIDKey, dossierId.Value, stat.DNISInt, stat.Established, stat.Delivered, stat.Duration, stat.ClearedDate, InteractionType.Call);
        //            result = commandBus.PublishAsync(commandStat, new CancellationToken())
        //                .ConfigureAwait(false).GetAwaiter().GetResult();
        //            Assert.True(result.IsSuccess);
        //        }

        //        //#region  Users
        //        //var shortName = tci.CompanyName.Substring(0, 5);
        //        //var userId = UserId.New;
        //        //var commandUser = new CreateUserCommand(userId, "admin@" + shortName + ".be", shortName + "Admin",
        //        //    custId.Value, true);
        //        //result = commandBus.PublishAsync(commandUser, new CancellationToken())
        //        //    .ConfigureAwait(false).GetAwaiter().GetResult();
        //        //Assert.True(result.IsSuccess);

        //        //ModuleRight mRight = new ModuleRight(ModuleRightId.New);
        //        //mRight.CallScript = ModuleAccessLevel.ReadWrite;
        //        //mRight.Statistics = ModuleAccessLevel.ReadWrite;
        //        //mRight.UserManagement = ModuleAccessLevel.ReadWrite;
        //        //mRight.Tickets = ModuleAccessLevel.ReadWrite;
        //        //mRight.Duties = ModuleAccessLevel.ReadWrite;
        //        //var commandModuleRight = new AddModuleRightCustomerCommand(userId, mRight, custId.Value);
        //        //result = commandBus.PublishAsync(commandModuleRight, new CancellationToken())
        //        //    .ConfigureAwait(false).GetAwaiter().GetResult();
        //        //Assert.True(result.IsSuccess);

        //        #endregion Statistics

        //        #region CallScripts mainfiches

        //        var callScripts = tempCallScripts.Where(x => x.CustomerId == tci.CustomerId);
        //        foreach (var cScript in callScripts)
        //        {
        //            var commandCScript = new CreateCallScriptCommand(CallScriptId.New, cScript.CallScriptBody, dossierId.Value, null);

        //            result = commandBus.PublishAsync(commandCScript, new CancellationToken())
        //                .ConfigureAwait(false).GetAwaiter().GetResult();
        //            Assert.True(result.IsSuccess);
        //        }

        //        #endregion CallScripts mainfiches

        //        #region Tickets mainfiches

        //        var tickets = tempTickets.Where(x => x.CustomerId == tci.CustomerId);
        //        foreach (var tti in tempTickets)
        //        {
        //            var commandTicket = new CreateTicketCommand(TicketId.New, dossierId.Value, tti.TicketText, tti.OriginalCCMMessageId, tti.CreatedInCRM, tti.LastModifiedInCRM, tti.CreatedCRMUser, tti.LastModifiedCRMUser, null);
        //            result = commandBus.PublishAsync(commandTicket, new CancellationToken())
        //                .ConfigureAwait(false).GetAwaiter().GetResult();
        //            Assert.True(result.IsSuccess);
        //        }

        //        #endregion Tickets mainfiches

        //        #region ExtraBranches

        //        tci.CompanyBranches = tempCompanyItems.Where(x => x.MainFicheId == tci.CustomerId).ToList();

        //        if (tci.CompanyBranches != null && tci.CompanyBranches.Count > 0)
        //        {
        //            foreach (var subTci in tci.CompanyBranches)
        //            {
        //                BranchId subBranchId = BranchId.New;

        //                AddressModel address2 = new AddressModel()
        //                {
        //                    City = subTci.City,
        //                    Country = subTci.Country,
        //                    Number = subTci.HouseNumber,
        //                    PostalBoxNr = subTci.BoxNumber,
        //                    PostalCode = subTci.PostalCode,
        //                };
        //                ContactDetailModel det1Sub = new ContactDetailModel(ContactDetailId.New.Value, "Phone 1", subTci.Phone1, false, ContactDetailType.TelephoneNr);
        //                ContactDetailModel det2Sub = new ContactDetailModel(ContactDetailId.New.Value, "Email", subTci.Email, false, ContactDetailType.Email);

        //                var cdSublist = new List<ContactDetailModel> { det1Sub, det2Sub };

        //                commandCreateMb =
        //                    new CreateBranchCommand(subBranchId, subTci.CompanyBranchName, address2, custId.Value, cdSublist);
        //                result = commandBus.PublishAsync(commandCreateMb, new CancellationToken())
        //                    .ConfigureAwait(false).GetAwaiter().GetResult();
        //                Assert.True(result.IsSuccess);

        //                #region CallScript subfiche

        //                var tags = new List<string>
        //                    {subTci.SubFicheId.ToString() + "|" + subTci.CompanyBranchName};

        //                var callScript = tempCallScripts.SingleOrDefault(x =>
        //                    x.MainFicheId != null && x.CustomerId == subTci.SubFicheId);
        //                if (callScript != null)
        //                {
        //                    var commandCallScript = new CreateCallScriptCommand(CallScriptId.New, callScript.CallScriptBody, dossierId.Value, tags);
        //                    result = commandBus.PublishAsync(commandCallScript, new CancellationToken())
        //                        .ConfigureAwait(false).GetAwaiter().GetResult();
        //                    Assert.True(result.IsSuccess);
        //                }

        //                #endregion CallScript subfiche

        //                #region Tickets Subfiches

        //                var ticketsSub =
        //                    tempTickets.Where(x => x.MainFicheId != null && x.CustomerId == subTci.SubFicheId);
        //                foreach (var tti in ticketsSub)
        //                {
        //                    var commandTicketSub = new CreateTicketCommand(TicketId.New, dossierId.Value, tti.TicketText, tti.OriginalCCMMessageId, tti.CreatedInCRM, tti.LastModifiedInCRM, tti.CreatedCRMUser, tti.LastModifiedCRMUser, tags);
        //                    result = commandBus.PublishAsync(commandTicketSub, new CancellationToken())
        //                        .ConfigureAwait(false).GetAwaiter().GetResult();
        //                    Assert.True(result.IsSuccess);
        //                }

        //                #endregion Tickets Subfiches
        //            }
        //        }

        //        #endregion ExtraBranches

        //        fakeClientId++;
        //    }
        //}

        //public void a2ImportDossiers()
        //{
        //    var collection = Setup();
        //    var commandBus = collection.Resolve<ICommandBus>();
        //    var tempCompanyItems = TempCompanyItem.TestData.OrderBy(x => x.MainFicheId);
        //    var tempList2 = new List<TempCompanyItem>(tempCompanyItems);

        //    foreach (var tci in tempList2.Where(x => x.MainFicheId == null))
        //    {
        //    }
        //}
    }
}