﻿using System;
using System.Collections.Generic;
using System.Linq;
using CRM.Domain.Employee.Models;
using Newtonsoft.Json;

namespace WebPortal.CCMConnector.Data.Models
{
    public class TempEmployeeItem
    {
        public int CustomerId { get; set; }
        public int? MainFicheId { get; set; }
        public string TagServiceName { get; set; }
        public string LastName { get; set; }
        public string Firstname { get; set; }
        public string Position { get; set; }
        public string OfficePhone { get; set; }
        public string PrivatePhone { get; set; }
        public string GSM { get; set; }
        public string Email { get; set; }
        public string Comments { get; set; }
        public bool IsOfficePhonePrivate { get; set; }
        public bool IsPrivatePhonePrivate { get; set; }
        public bool IsGSMPrivate { get; set; }
        public bool IsEmailPrivate { get; set; }
        public bool IsServicePrivate { get; set; }
        public bool IsFirstNamePrivate { get; set; }
        public bool IsLastNamePrivate { get; set; }
        public bool IsPositionPrivate { get; set; }
        public string CC { get; set; }
        public bool? SetMessageToRead { get; set; }
        public bool? SendMail { get; set; }
        public IdsFromCCMandWP MainAndSubficheIdAndCustomerId { get; set; }

        public List<string> TagList
        {
            get
            {
                string theService = string.IsNullOrWhiteSpace(TagServiceName) ? null : TagServiceName;
                string thePosition = string.IsNullOrWhiteSpace(Position) ? null : Position;

                List<string> rtn = new List<string>();

                SplitTagsAndAddToList(rtn, theService);
                SplitTagsAndAddToList(rtn, thePosition);
                rtn = rtn.Where(x => !string.IsNullOrWhiteSpace(x)).ToList();
                if (ExtraTags.Count > 0)
                    rtn.AddRange(ExtraTags);
                if (rtn.Count > 0)
                    return rtn;
                return null;
            }
        }

        private void SplitTagsAndAddToList(List<string> rtn,
            string multipleTags)
        {
            if (multipleTags == null) return;
            var sa = multipleTags.Split(';');
            var tmp = new List<string>();
            tmp.Add(sa[0]);

            if (sa.Length > 1)
            {
                var precursor = multipleTags.Substring(0, 1);
                for (int i = 1; i < sa.Length; i++)
                {
                    tmp.Add($"{precursor}{sa[i].Trim()}");
                }
            }

            rtn.AddRange(tmp);
        }

        public List<string> ExtraTags { get; set; } = new List<string>();

        public List<EmployeeContactDetailModel> ContactDetails { get; set; }

        public override bool Equals(object obj)
        {
            if (obj == null)
                return false;
            var otherEmp = (TempEmployeeItem)obj;
            if (otherEmp.LastName == LastName && otherEmp.Firstname == Firstname &&
                IdsFromCCMandWP.IsEqualByCustomer(MainAndSubficheIdAndCustomerId,
                    otherEmp.MainAndSubficheIdAndCustomerId))
                return true;
            return false;
        }

        public void CombineWithOther(TempEmployeeItem otherEmp)
        {
            CombineContactDetails(otherEmp);
            CombineTags(otherEmp);
            CombineComments(otherEmp.Comments);
            CombineCC(otherEmp.CC);
        }

        private void CombineCC(string otherCC)
        {
            if (string.IsNullOrWhiteSpace(otherCC)) return;

            if (CC != otherCC)
            {
                if (string.IsNullOrWhiteSpace(CC))
                    CC = otherCC;
                else
                {
                    CC += "; " + otherCC;
                }
            }
        }

        private void CombineComments(string otherText)
        {
            if (string.IsNullOrWhiteSpace(otherText)) return;

            if (Comments != otherText)
            {
                if (string.IsNullOrWhiteSpace(Comments))
                    Comments = otherText;
                else
                {
                    Comments += "\r\n" + otherText;
                }
            }
        }

        private void CombineContactDetails(TempEmployeeItem otherEmp)
        {
            foreach (var contactDetail in otherEmp.ContactDetails)
            {
                var mycontactDetail =
                    ContactDetails.FirstOrDefault(x => x.Equals(contactDetail));
                if (mycontactDetail != null)
                {
                    //De overeenkomstige tempEmployee heeft exact dezelfde contactDetail, er moet dus niets gebeuren.
                    continue;
                }

                //We hebben geen volledig overeenkomstig contactDetail
                //Hebben we misschien wel een contactDetail dat slechts op private niveau verschilt
                mycontactDetail = ContactDetails.FirstOrDefault(x =>
                    x.ContactDetailValue == contactDetail.ContactDetailValue
                    && x.ContactDetailName == contactDetail.ContactDetailName
                    && x.ContactType == contactDetail.ContactType);
                if (mycontactDetail == null)
                {
                    ContactDetails.Add(contactDetail);
                }
                else if (!mycontactDetail.IsPrivate)
                {
                    mycontactDetail.IsPrivate = true;
                }
            }
        }

        private void CombineTags(TempEmployeeItem oterEmp)
        {
            if (oterEmp.TagList == null)
                return;
            string nullString = null;
            foreach (var tag in oterEmp.TagList)
            {
                var myTag = TagList == null ? nullString : TagList.FirstOrDefault(x => x == tag);
                if (myTag == null)
                {
                    ExtraTags.Add(tag);
                }
            }
        }

        //public List<WPTag> TagList { get; set; }

        public static List<TempEmployeeItem> TestData => JsonConvert.DeserializeObject<List<TempEmployeeItem>>("[{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"WOLLANTS Dennis\",\"Firstname\":\"Dennis\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.01\",\"PrivatePhone\":\"\",\"GSM\":\"0495/13.47.25\",\"Email\":\"dennis.wollants@altius.com\",\"Comments\":\"SECRETARESSE = VAN WANGHE An\",\"AnsgateID\":23693,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"an.vanwanghe@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"LE BRIS Leslie\",\"Firstname\":\"Leslie\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.90\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"leslie.lebris@altius.com\",\"Comments\":\"ALGEMEEN SECRETARESSE\",\"AnsgateID\":23720,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"HR MANAGER\",\"LastName\":\"BEUTELS Nadia\",\"Firstname\":\"Nadia\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.71\",\"PrivatePhone\":\"\",\"GSM\":\"0477/99.61.23\",\"Email\":\"nadia.beutels@altius.com\",\"Comments\":\"\",\"AnsgateID\":25136,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"LAKHOUIT Majdoline\",\"Firstname\":\"Majdoline\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.60\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"majdoline.lakhouit@altius.com\",\"Comments\":\"\",\"AnsgateID\":25133,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"BIBLIOTHEEK\",\"LastName\":\"DE GROOT Domien\",\"Firstname\":\"Domien\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.14\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"domien.degroot@altius.com\",\"Comments\":\"\",\"AnsgateID\":25450,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"EXTERNAL CONSULTANT\",\"LastName\":\"BARJOT Julien\",\"Firstname\":\"Julien\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.77\",\"PrivatePhone\":\"\",\"GSM\":\"+33/631.05.70.29\",\"Email\":\"julien.barjot@altius.com\",\"Comments\":\"\",\"AnsgateID\":25014,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VAN CAENEGEM Emma\",\"Firstname\":\"Emma\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.08\",\"PrivatePhone\":\"\",\"GSM\":\"0491/53.45.58\",\"Email\":\"emma.vancaenegem@altius.com\",\"Comments\":\"SECRETARESSE = SYMONS Els\",\"AnsgateID\":24360,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"els.symons@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-29T18:29:42.497\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"VAN LEUVENHAEGE Martine\",\"Firstname\":\"Martine\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.44\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"martine.vanleuvenhaege@altius.com\",\"Comments\":\"\",\"AnsgateID\":25717,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"CAPORALI Astrid\",\"Firstname\":\"Astrid\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.67\",\"PrivatePhone\":\"\",\"GSM\":\"0477/46.47.03\",\"Email\":\"astrid.caporali@altius.com\",\"Comments\":\"SECRETARESSE = Majdoline Lakhouit\",\"AnsgateID\":24434,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"majdoline.lakhouit@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"HUISHOUDSTER\",\"LastName\":\"BOUDAOUDI Ikrame\",\"Firstname\":\"Ikrame\",\"Position\":\"\",\"OfficePhone\":\"GEEN\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"GEEN\",\"Comments\":\"\",\"AnsgateID\":25951,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VANTYGHEM Alexander\",\"Firstname\":\"Alexander\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.47\",\"PrivatePhone\":\"\",\"GSM\":\"0479/76.79.31\",\"Email\":\"alexander.vantyghem@altius.com\",\"Comments\":\"SECRETARESSE = Majdoline Lakhouit\\n\\n\",\"AnsgateID\":13081,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"majdoline.lakhouit@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"VAN CUTSEM Valerie\",\"Firstname\":\"Valerie\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.64\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"valerie.vancutsem@altius.com\",\"Comments\":\"\",\"AnsgateID\":25879,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VERBEEREN Manon\",\"Firstname\":\"Manon\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.05\",\"PrivatePhone\":\"\",\"GSM\":\"0497/70.34.73\",\"Email\":\"manon.verbeeren@altius.com\",\"Comments\":\"SECRETARESSE = VAN DER HAEGHEN Saskia\",\"AnsgateID\":25880,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"saskia.vanderhaeghen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ICT HELPDESK\",\"LastName\":\"COENEGRACHTS Jimmy\",\"Firstname\":\"Jimmy\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.23\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"jimmy.coenegrachts@altius.com\",\"Comments\":\"\",\"AnsgateID\":25883,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"PRIGOGINE Davina\",\"Firstname\":\"Davina\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.02\",\"PrivatePhone\":\"\",\"GSM\":\"0486/39.48.07\",\"Email\":\"davina.prigogine@altius.com\",\"Comments\":\"SECRETARESSE = VAN CUTSEM Valerie\",\"AnsgateID\":25953,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"valerie.vancutstem@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"SYMONS Els\",\"Firstname\":\"Els\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.42\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"els.symons@altius.com\",\"Comments\":\"\",\"AnsgateID\":26601,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-29T18:28:32.877\",\"LastModifiedOn\":\"2019-10-29T18:28:42.363\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"ANTOINNE Pierre\",\"Firstname\":\"Pierre\",\"Position\":\"\",\"OfficePhone\":\"02/423.28.26\",\"PrivatePhone\":\"\",\"GSM\":\"0494/49.50.46\",\"Email\":\"pierre.antoine@altius.com\",\"Comments\":null,\"AnsgateID\":26656,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-11-18T14:11:30.63\",\"LastModifiedOn\":\"2019-11-18T14:15:21.707\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"HERREMANS Thijs\",\"Firstname\":\"Thijs\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.79\",\"PrivatePhone\":\"\",\"GSM\":\"0468/26.47.18\",\"Email\":\"thijs.herremans@altius.com\",\"Comments\":\"SECRETARESSE = DE KLERK Ingrid\",\"AnsgateID\":25130,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"ingrid.deklerk@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"DE LANDSHEERE Jonathan\",\"Firstname\":\"Jonathan\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.50\",\"PrivatePhone\":\"\",\"GSM\":\"0495/71.28.27\",\"Email\":\"jonathan.delandsheere@altius.com\",\"Comments\":\"SECRETARESSE = VRANCKEN Nicole\",\"AnsgateID\":25131,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"nicole.vrancken@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"ROUX Lore\",\"Firstname\":\"Lore\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.39\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"Lore.roux@altius.com\",\"Comments\":\"HEEFT KANTOOR VERLATEN \",\"AnsgateID\":25132,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-11-18T15:32:08.01\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"PEETERS Charlotte\",\"Firstname\":\"Charlotte\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.63\",\"PrivatePhone\":\"\",\"GSM\":\"0495/03.12.10\",\"Email\":\"charlotte.peeters@altius.com\",\"Comments\":\"SECRETARESSE = DE KLERK Ingrid\",\"AnsgateID\":24737,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"ingrid.deklerk@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"MANIET Mathieu\",\"Firstname\":\"Mathieu\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.31\",\"PrivatePhone\":\"\",\"GSM\":\"0475/39.58.01\",\"Email\":\"mathieu.maniet@altius.com\",\"Comments\":\"SECRETARESSE = DE MOERLOOZE Anne\",\"AnsgateID\":25708,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"anne.demoerlooze@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"CLINCK Jan\",\"Firstname\":\"Jan\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.55\",\"PrivatePhone\":\"\",\"GSM\":\"0494/28.56.98\",\"Email\":\"jan.clinck@altius.com\",\"Comments\":\"SECRETARESSE = VERTONGEN Joske\",\"AnsgateID\":24647,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"joske.vertongen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"VERTONGEN Joske\",\"Firstname\":\"Joske\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.07\",\"PrivatePhone\":\"\",\"GSM\":\"-\",\"Email\":\"joske.vertongen@altius.com\",\"Comments\":\"NIET AANWEZIG op WOENSDAGNAMIDDAG EN VRIJDAGNAMIDDAG\",\"AnsgateID\":12519,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-11-18T16:02:57.19\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"BROSENS Yves\",\"Firstname\":\"Yves\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.20\",\"PrivatePhone\":\"\",\"GSM\":\"0472 58 20 00\",\"Email\":\"yves.brosens@altius.com\",\"Comments\":\"SECRETARESSE = ANGILLIS Lucette\",\"AnsgateID\":23772,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"lucette.angillis@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"HUISHOUDSTER\",\"LastName\":\"EL MAELI Assia\",\"Firstname\":\"Assia\",\"Position\":\"\",\"OfficePhone\":\"GEEN\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"GEEN\",\"Comments\":\"\",\"AnsgateID\":25952,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VANQUATHEM Lies\",\"Firstname\":\"Lies\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.70\",\"PrivatePhone\":\"\",\"GSM\":\"0478/57.89.29\",\"Email\":\"lies.vanquathem@altius.com\",\"Comments\":\"SECRETARESSE = VANDAMME Anne-Marie\",\"AnsgateID\":25728,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"anne-marie.vandamme@altius.com\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VAN THUYNE Vera\",\"Firstname\":\"Vera\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.96\",\"PrivatePhone\":\"\",\"GSM\":\"0476/20.15.51\",\"Email\":\"vera.vanthuyne@altius.com\",\"Comments\":\"SECRETARESSE = VAN WANGHE An\",\"AnsgateID\":25585,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"an.vanwanghe@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"HUISHOUDSTER\",\"LastName\":\"ABOU Meriyem\",\"Firstname\":\"Meriyem\",\"Position\":\"\",\"OfficePhone\":\"ZIE COMMENT\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"GEEN\",\"Comments\":\"HEEFT KANTOOR VERLATEN \",\"AnsgateID\":25586,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"ANGILLIS Lucette\",\"Firstname\":\"Lucette\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.51\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"lucette.angillis@altius.com\",\"Comments\":\"\",\"AnsgateID\":12239,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"BRASSEUR Marie\",\"Firstname\":\"Marie\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.45\",\"PrivatePhone\":\"\",\"GSM\":\"0479/72.80.89\",\"Email\":\"marie.brasseur@altius.com\",\"Comments\":\"SECRETARESSE = VAN CUTSEM Valerie\",\"AnsgateID\":12247,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"valerie.vancutsem@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"MARKETING COMMUNICATION OFFICER\",\"LastName\":\"VANDERIJST Melissa\",\"Firstname\":\"Melissa\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.39\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"melissa.vanderijst@altius.com\",\"Comments\":null,\"AnsgateID\":26658,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-11-18T15:56:22.583\",\"LastModifiedOn\":\"2019-11-18T15:56:22.583\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"VAN REGENMORTEL Carine\",\"Firstname\":\"Carine\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.61\",\"PrivatePhone\":\"\",\"GSM\":\"0477/50.14.46\",\"Email\":\"carine.vanregenmortel@altius.com\",\"Comments\":\"SECRETARESSE = VRANCKEN Nicole\",\"AnsgateID\":12321,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"nicole.vrancken@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VAN SCHEL Stijn\",\"Firstname\":\"Stijn\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.83\",\"PrivatePhone\":\"\",\"GSM\":\"0473/40.36.60\",\"Email\":\"stijn.vanschel@altius.com\",\"Comments\":\"SECRETARESSE = VANDERHAEGHEN Saskia\",\"AnsgateID\":12323,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"saskia.vanderhaeghen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"VAN WANGHE An\",\"Firstname\":\"An\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.73\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"an.vanwanghe@altius.com\",\"Comments\":\"\",\"AnsgateID\":12325,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"VANDAMME Anne-Marie\",\"Firstname\":\"Anne-Marie\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.58\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"anne-marie.vandamme@altius.com\",\"Comments\":\"\\n\",\"AnsgateID\":12326,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"VANDENDRIESSCHE Gerrit\",\"Firstname\":\"Gerrit\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.82\",\"PrivatePhone\":\"\",\"GSM\":\"0476/99.68.76\",\"Email\":\"gerrit.vandendriessche@altius.com\",\"Comments\":\"SECRETARESSE = VERTONGEN Joske\",\"AnsgateID\":12327,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"joske.vertongen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADMINISTRATIEF BEDIENDE\",\"LastName\":\"VANDEVOORT Martine\",\"Firstname\":\"Martine\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.14\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"martine.vandevoordt@altius.com\",\"Comments\":\"LANDURIG AFWEZIG - TIJDELIJKE VERVANGING: DOMIEN DE GROOT\\nNIET AANWEZIG op MAANDAG EN DINSDAG\",\"AnsgateID\":12330,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-11-18T15:58:33.207\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"VANTROYEN Tom\",\"Firstname\":\"Tom\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.40\",\"PrivatePhone\":\"\",\"GSM\":\"0476/82.93.63\",\"Email\":\"tom.vantroyen@altius.com\",\"Comments\":\"SECRETARESSE = DE KLERCK Ingrid\",\"AnsgateID\":12332,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"ingrid.deklerk@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"HUISHOUDSTER\",\"LastName\":\"VERA BRIONES Juliana\",\"Firstname\":\"Juliana\",\"Position\":\"\",\"OfficePhone\":\"GEEN\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"GEEN\",\"Comments\":\"\",\"AnsgateID\":12333,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"VERDONCK Carmen\",\"Firstname\":\"Carmen\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.57\",\"PrivatePhone\":\"\",\"GSM\":\"0476/32.42.66\",\"Email\":\"carmen.verdonck@altius.com\",\"Comments\":\"SECRETARESSE = DE MOERLOOZE Anne\",\"AnsgateID\":12335,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"anne.demoerlooze@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VERMEERSCH Charlotte\",\"Firstname\":\"Charlotte\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.28\",\"PrivatePhone\":\"\",\"GSM\":\"0479/45.16.41\",\"Email\":\"charlotte.vermeersch@altius.com\",\"Comments\":\"SECRETARESSE = GELDERS Brigitte\",\"AnsgateID\":12336,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"brigitte.gelders@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"VERMEYLEN Jerome\",\"Firstname\":\"Jérôme\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.76\",\"PrivatePhone\":\"\",\"GSM\":\"0494/23.03.43\",\"Email\":\"jerome.vermeylen@altius.com\",\"Comments\":\"SECRETARESSE = VRANCKEN Nicole\",\"AnsgateID\":12337,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"nicole.vrancken@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"VRANCKEN Nicole\",\"Firstname\":\"Nicole\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.56\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"nicole.vrancken@altius.com\",\"Comments\":\"\",\"AnsgateID\":12340,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"VRINS Olivier\",\"Firstname\":\"Olivier\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.66\",\"PrivatePhone\":\"\",\"GSM\":\"0494/37.31.18\",\"Email\":\"olivier.vrins@altius.com\",\"Comments\":\"SECRETARESSE = VAN DER HAEGHEN Saskia\",\"AnsgateID\":12342,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"saskia.vanderhaeghen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"ARNAUT Diederik\",\"Firstname\":\"Diederik\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.58\",\"PrivatePhone\":\"\",\"GSM\":\"0479/73.63.41\",\"Email\":\"diederik.arnaut@altius.com\",\"Comments\":\"SECRETARESSE = CASSE Christelle\",\"AnsgateID\":12240,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"christelle.casse@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"BELMANS Sheena\",\"Firstname\":\"Sheena\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.24\",\"PrivatePhone\":\"\",\"GSM\":\"0499/61.65.69\",\"Email\":\"sheena.belmans@altius.com\",\"Comments\":\"SECRETARESSE = VRANCKEN Nicole\",\"AnsgateID\":12242,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"nicole.vrancken@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"DE BLEECKERE Alexander\",\"Firstname\":\"Alexander\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.09\",\"PrivatePhone\":\"\",\"GSM\":\"0472/63.52.46\",\"Email\":\"alexander.debleeckere@altius.com\",\"Comments\":\"SECRETARESSE = GELDERS Brigitte\",\"AnsgateID\":12250,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"brigitte.gelders@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"JUNIOR ACCOUNTANT\",\"LastName\":\"DE DEKEN Ria\",\"Firstname\":\"Ria\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.38\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"ria.dedeken@altius.com\",\"Comments\":\"\",\"AnsgateID\":12253,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"DE JONG Philippe\",\"Firstname\":\"Philippe\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.25\",\"PrivatePhone\":\"\",\"GSM\":\"0473/57.92.21\",\"Email\":\"philippe.dejong@altius.com\",\"Comments\":\"SECRETARESSE = CASSE Christelle\",\"AnsgateID\":12254,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"christelle.casse@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"DE KLERK Ingrid\",\"Firstname\":\"Ingrid\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.72\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"ingrid.deklerk@altius.com\",\"Comments\":\"WERKT NIET OP WOENSDAG\\n> DOORGEVEN AAN ANGILLIS Luxette of VRANCKEN Nicole\\n\",\"AnsgateID\":12255,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"DE MOERLOOZE Anne\",\"Firstname\":\"Anne\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.99\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"anne.demoerlooze@altius.com\",\"Comments\":\"\",\"AnsgateID\":12256,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"DUBOIS Sylvie\",\"Firstname\":\"Sylvie\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.74\",\"PrivatePhone\":\"\",\"GSM\":\"0475/71.67.57\",\"Email\":\"sylvie.dubois@altius.com\",\"Comments\":\"SECRETARESSE = VAN LEUVENHAEGE Martine\",\"AnsgateID\":12262,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"martine.vanleuvenhaege@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"ERNES Gregory\",\"Firstname\":\"Gregory\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.37\",\"PrivatePhone\":\"\",\"GSM\":\"0498/41.45.34\",\"Email\":\"gregory.ernes@altius.com\",\"Comments\":\"SECRETARESSE = Majdoline Lakhouit\\n\",\"AnsgateID\":12264,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"majdoline.lakhouit@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"CLAEYE Kirian\",\"Firstname\":\"Kirian\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.94\",\"PrivatePhone\":\"\",\"GSM\":\"0472/59.39.28\",\"Email\":\"kirian.claeye@altius.com\",\"Comments\":\"SECRETARESSE = EYCKMANS Wendy\\n\",\"AnsgateID\":12248,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"wendy.eyckmans@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"DE WULF Philippe\",\"Firstname\":\"Philippe\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.51\",\"PrivatePhone\":\"\",\"GSM\":\"0497/43.28.20\",\"Email\":\"philippe.dewulf@altius.com\",\"Comments\":\"SECRETARESSE = SYMONS Els\\n\",\"AnsgateID\":12259,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"els.symons@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-29T18:29:03.21\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"DEMEULEMEESTER Sven\",\"Firstname\":\"Sven\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.75\",\"PrivatePhone\":\"\",\"GSM\":\"0499/32.23.27\",\"Email\":\"sven.demeulemeester@altius.com\",\"Comments\":\"SECRETARESSE = Majdoline Lakhouit\",\"AnsgateID\":12260,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"majdoline.lakhouit@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VANHOLSBEKE Pierre\",\"Firstname\":\"Pierre\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.87\",\"PrivatePhone\":\"\",\"GSM\":\"0472/79.54.24\",\"Email\":\"pierre.vanholsbeke@altius.com\",\"Comments\":null,\"AnsgateID\":26659,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-11-18T16:00:16.857\",\"LastModifiedOn\":\"2019-11-18T16:00:16.857\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"FROIDBISE Geoffrey\",\"Firstname\":\"Geoffrey\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.49\",\"PrivatePhone\":\"\",\"GSM\":\"0496/23.92.15\",\"Email\":\"geoffrey.froidbise@altius.com\",\"Comments\":\"SECRETARESSE = EYCKMANS Wendy\",\"AnsgateID\":12269,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"wendy.eyckmans@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"GALOT Adrien\",\"Firstname\":\"Adrien\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.03\",\"PrivatePhone\":\"\",\"GSM\":\"0484/67.34.74\",\"Email\":\"adrien.galot@altius.com\",\"Comments\":\"SECRETARESSE = VAN CUTSEM Valerie\",\"AnsgateID\":12270,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"valerie.vancutsem@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"GELDERS Brigitte\",\"Firstname\":\"Brigitte\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.86\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"brigitte.gelders@altius.com\",\"Comments\":\"\",\"AnsgateID\":12273,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"GIELEN Beatrijs\",\"Firstname\":\"Beatrijs\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.62\",\"PrivatePhone\":\"\",\"GSM\":\"0472/65.74.39\",\"Email\":\"\",\"Comments\":\"SECRETARESSE = VERTONGEN Joske\\n\",\"AnsgateID\":12281,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"joske.vertongen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"HEYNINCKX Bart\",\"Firstname\":\"Bart\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.27\",\"PrivatePhone\":\"\",\"GSM\":\"0498/74.50.66\",\"Email\":\"bart.heyninckx@altius.com\",\"Comments\":\"SECRETARESSE = GELDERS Brigitte\",\"AnsgateID\":12288,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"brigitte.gelders@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"JOLLY Louis-Dorsan\",\"Firstname\":\"Louis-Dorsan\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.97\",\"PrivatePhone\":\"\",\"GSM\":\"0477/19.51.36\",\"Email\":\"louis-dorsan.jolly@altius.com\",\"Comments\":\"SECRETARESSE = DE MOERLOOZE Anne\",\"AnsgateID\":12290,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"anne.demoerlooze@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"LAMBRECHT Wouter\",\"Firstname\":\"Wouter\",\"Position\":\"\",\"OfficePhone\":\"GEEN\",\"PrivatePhone\":\"\",\"GSM\":\"+41/794.755.443\",\"Email\":\"wouter.lambrecht@altius.com\",\"Comments\":\"SECRETARESSE = Majdoline Lakhouit\\n\",\"AnsgateID\":12291,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"majdoline.lakhouit@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"HUISHOUDSTER\",\"LastName\":\"LOZANO Maite\",\"Firstname\":\"Maite\",\"Position\":\"\",\"OfficePhone\":\"GEEN\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"GEEN\",\"Comments\":\"\",\"AnsgateID\":12296,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"GAUL Julien\",\"Firstname\":\"Julien\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.81\",\"PrivatePhone\":\"\",\"GSM\":\"0494/82.87.88\",\"Email\":\"julien.gaul@altius.com\",\"Comments\":\"SECRETARESSE = VAN WANGHE An\",\"AnsgateID\":12271,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"an.vanwanghe@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"HANSEBOUT Alexander\",\"Firstname\":\"Alexander\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.80\",\"PrivatePhone\":\"\",\"GSM\":\"0494/74.65.51\",\"Email\":\"alexander.hansebout@altius.com\",\"Comments\":\"SECRETARESSE = VAN DER HAEGHEN Saskia\",\"AnsgateID\":12287,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"saskia.vanderhaeghen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"IT MANAGER\",\"LastName\":\"LEFEVER Yves\",\"Firstname\":\"Yves\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.21\",\"PrivatePhone\":\"\",\"GSM\":\"0486/17.49.98\",\"Email\":\"yves.lefever@altius.com\",\"Comments\":\"\",\"AnsgateID\":12293,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"LENS Sophie\",\"Firstname\":\"Sophie\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.68\",\"PrivatePhone\":\"\",\"GSM\":\"0477/64.89.06\",\"Email\":\"sophie.lens@altius.com\",\"Comments\":\"SECRETARESSE = CASSE Christelle\",\"AnsgateID\":12295,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"christelle.casse@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"MAEYAERT Paul\",\"Firstname\":\"Paul\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.69\",\"PrivatePhone\":\"\",\"GSM\":\"0475/76.88.47\",\"Email\":\"paul.maeyaert@altius.com\",\"Comments\":\"SECRETARESSE = LE BRIS Leslie\",\"AnsgateID\":12297,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"leslie.lebris@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"MUYLDERMANS Jeroen\",\"Firstname\":\"Jeroen\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.46\",\"PrivatePhone\":\"\",\"GSM\":\"0498/72.29.66\",\"Email\":\"jeroen.muyldermans@altius.com\",\"Comments\":\"SECRETARESSE = GELDERS Brigitte\",\"AnsgateID\":12301,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"brigitte.gelders@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"BOEKHOUDER\",\"LastName\":\"OPITEK Patrick\",\"Firstname\":\"Patrick\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.53\",\"PrivatePhone\":\"\",\"GSM\":\"0471/72.52.96\",\"Email\":\"patrick.opitek@altius.com\",\"Comments\":\"\",\"AnsgateID\":12303,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"PEETERS Lieven\",\"Firstname\":\"Lieven\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.89\",\"PrivatePhone\":\"\",\"GSM\":\"0478/40.15.60\",\"Email\":\"lieven.peeters@altius.com\",\"Comments\":\"SECRETARESSE = VANDAMME Anne-Marie\",\"AnsgateID\":12304,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"anne-marie.vandamme@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"PROOFREADER\",\"LastName\":\"RATTAN Arun\",\"Firstname\":\"Arun\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.65\",\"PrivatePhone\":\"\",\"GSM\":\"0486/68.77.75\",\"Email\":\"arun.rattan@altius.com\",\"Comments\":\"\",\"AnsgateID\":12305,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"SILVESTRE Quentin\",\"Firstname\":\"Quentin\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.22\",\"PrivatePhone\":\"\",\"GSM\":\"0472/20.38.83\",\"Email\":\"quentin.silvestre@altius.com\",\"Comments\":\"SECRETARESSE = DE MOERLOOZE Anne\",\"AnsgateID\":12309,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"anne.demoerlooze@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"TIMMERMAN Isabelle\",\"Firstname\":\"Isabelle\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.04\",\"PrivatePhone\":\"\",\"GSM\":\"0486.28.06.47\",\"Email\":\"isabelle.timmerman@altius.com\",\"Comments\":\"SECRETARESSE = SYMONS Els\",\"AnsgateID\":12314,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"els.symons@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-29T18:29:59.747\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"TIMMERMANS William\",\"Firstname\":\"William\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.41\",\"PrivatePhone\":\"\",\"GSM\":\"0473/97.10.36\",\"Email\":\"william.timmermans@altius.com\",\"Comments\":\"SECRETARESSE = VAN WANGHE An\",\"AnsgateID\":12315,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"an.vanwanghe@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"TRUYENS Lauranne\",\"Firstname\":\"Lauranne\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.19\",\"PrivatePhone\":\"\",\"GSM\":\"0472/61.02.84\",\"Email\":\"lauranne.truyens@altius.com\",\"Comments\":\"SECRETARESSE = GELDERS Brigitte\",\"AnsgateID\":12316,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"brigitte.gelders@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"OFFICE MANAGER\",\"LastName\":\"VAN BIESBROECK Liselotte\",\"Firstname\":\"Liselotte\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.54\",\"PrivatePhone\":\"\",\"GSM\":\"0476/71.36.74\",\"Email\":\"liselotte.vanbiesbroeck@altius.com\",\"Comments\":\"NIET AANWEZIG op VRIJDAG\\n\\nte contacteren voor alle praktische problemen (huisvesting, personeel, recepties,...)\",\"AnsgateID\":12317,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"TRAEST Laura\",\"Firstname\":\"Laura\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.43\",\"PrivatePhone\":\"\",\"GSM\":\"0473/64.25.73\",\"Email\":\"laura.traest@altius.com\",\"Comments\":\"SECRETARESSE = CASSE Christelle\",\"AnsgateID\":24679,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"christelle.casse@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"REYGAERT Andreas\",\"Firstname\":\"Andreas\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.85\",\"PrivatePhone\":\"\",\"GSM\":\"0484/62.79.05\",\"Email\":\"andreas.reygaert@altius.com\",\"Comments\":\"SECRETARESSE = EYCKMANS Wendy\",\"AnsgateID\":24680,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"wendy.eyckmans@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"BAEYENS Hanne\",\"Firstname\":\"Hanne\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.33\",\"PrivatePhone\":\"\",\"GSM\":\"0478/40.98.88\",\"Email\":\"hanne.baeyens@altius.com\",\"Comments\":\"SECRETARESSE = VERTONGEN Joske\",\"AnsgateID\":26556,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"joske.vertongen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"SOETENS Esther\",\"Firstname\":\"Esther\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.93\",\"PrivatePhone\":\"\",\"GSM\":\"0495/90.98.02\",\"Email\":\"esther.soetens@altius.com\",\"Comments\":\"SECRETARESSE = SYMONS Els\",\"AnsgateID\":23731,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"els.symons@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-29T18:29:18.18\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"CASSE Christelle\",\"Firstname\":\"Christelle\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.84\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"christelle.casse@altius.com\",\"Comments\":\"\",\"AnsgateID\":25274,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"christelle.casse@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VAN WICHELEN Caro\",\"Firstname\":\"Caro\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.18\",\"PrivatePhone\":\"\",\"GSM\":\"0474/51.69.02\",\"Email\":\"caro.vanwichelen@altius.com\",\"Comments\":\"SECRETARESSE = VAN DER HAEGHEN Saskia\",\"AnsgateID\":12470,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"saskia.vanderhaeghen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"BIBLIOTHEEK\",\"LastName\":\"VERGAUWEN Laurent\",\"Firstname\":\"Laurent\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.17\",\"PrivatePhone\":\"\",\"GSM\":\"0474/42.35.77\",\"Email\":\"laurent.vergauwen@altius.com\",\"Comments\":\"\",\"AnsgateID\":12514,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"CEBBAR Aylin\",\"Firstname\":\"Aylin\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.59\",\"PrivatePhone\":\"\",\"GSM\":\"0479/53.72.09\",\"Email\":\"aylin.cebbar@altius.com\",\"Comments\":\"SECRETARESSE = Ingrid de Klerk\",\"AnsgateID\":25635,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"ingrid.deklerk@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"METHENS Nina\",\"Firstname\":\"Nina\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.48\",\"PrivatePhone\":\"\",\"GSM\":\"0479/73.19.76\",\"Email\":\"nina.methens@altius.com\",\"Comments\":\"SECRETARESSE = DE MOERLOOZE Anne\",\"AnsgateID\":25605,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"anne.demoerlooze@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VANDERSCHRICK Britt\",\"Firstname\":\"Britt\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.34\",\"PrivatePhone\":\"\",\"GSM\":\"0499/24.30.35\",\"Email\":\"britt.vanderschrick@altius.com\",\"Comments\":\"SECRETARESSE = ANGILLIS Lucette\",\"AnsgateID\":23979,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"lucette.angillis@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"SWENNEN Joris\",\"Firstname\":\"Joris\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.06\",\"PrivatePhone\":\"\",\"GSM\":\"0496/23.61.86\",\"Email\":\"joris.swennen@altius.com\",\"Comments\":\"SECRETARESSE = CASSE Christelle\",\"AnsgateID\":24738,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"christelle.casse@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"MAES Ruth\",\"Firstname\":\"Ruth\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.95\",\"PrivatePhone\":\"\",\"GSM\":\"0474/61.88.27\",\"Email\":\"ruth.maes@altius.com\",\"Comments\":\"SECRETARESSE = SYMONS Els\",\"AnsgateID\":24008,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"els.symons@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-29T18:29:29.62\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VERBORGH Niels\",\"Firstname\":\"Niels\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.57\",\"PrivatePhone\":\"\",\"GSM\":\"0479/40.67.95\",\"Email\":\"niels.verborgh@altius.com\",\"Comments\":\"SECRETARESSE = LAKHOUIT Majdoline\",\"AnsgateID\":25149,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"majdoline.lakhouit@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"EYCKMANS Wendy\",\"Firstname\":\"Wendy\",\"Position\":\"\",\"OfficePhone\":\"02 423 58 13\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"wendy.eyckmans@altius.com\",\"Comments\":\"\",\"AnsgateID\":24869,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT/VENNOOT\",\"LastName\":\"RONSE Christophe\",\"Firstname\":\"Christophe\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.78\",\"PrivatePhone\":\"\",\"GSM\":\"0475/22.42.12\",\"Email\":\"christophe.ronse@altius.com\",\"Comments\":\"SECRETARESSE = EYCKMANS, Wendy\",\"AnsgateID\":24870,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"wendy.eyckmans@altius.com\\r\\n\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"BILLING OFFICER\",\"LastName\":\"KAZARIAN Ani\",\"Firstname\":\"Ani\",\"Position\":\"\",\"OfficePhone\":\"02 423 58 29\",\"PrivatePhone\":\"\",\"GSM\":\"GEEN\",\"Email\":\"ani.kazarian@altius.com\",\"Comments\":\"\",\"AnsgateID\":24909,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"BOUCHAT Gauthier\",\"Firstname\":\"Gauthier\",\"Position\":\"\",\"OfficePhone\":\"02 423 58 50\",\"PrivatePhone\":\"\",\"GSM\":\"0497 69 65 49\",\"Email\":\"majdoline.lakhouit@altius.com\",\"Comments\":\"SECRETARESSE = Majdoline Lakhouit\",\"AnsgateID\":24910,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"majdoline.lakhouit@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"RAXHON Anne-Sophie\",\"Firstname\":\"Anne-Sophie\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.32\",\"PrivatePhone\":\"\",\"GSM\":\"0495/73.73.37\",\"Email\":\"anne-sophie.raxhon@altius.com\",\"Comments\":\"SECRETARESSE = VERTONGEN Joske\",\"AnsgateID\":25608,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"joske.vertongen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"RAUS Bregt\",\"Firstname\":\"Bregt\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.35\",\"PrivatePhone\":\"\",\"GSM\":\"0473/51.16.80\",\"Email\":\"bregt.raus@altius.com\",\"Comments\":\"SECRETARESSE = CASSE Christelle\",\"AnsgateID\":26551,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"christelle.casse@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"VERSTREPEN Hans\",\"Firstname\":\"Hans\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.88\",\"PrivatePhone\":\"\",\"GSM\":\"0474/26.34.32\",\"Email\":\"hans.verstrepen@altius.com\",\"Comments\":\"SECRETARESSE =EYCKMANS Wendy\",\"AnsgateID\":26552,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"wendy.eyckmans@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"COOPMAN Annabel\",\"Firstname\":\"Annabel\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.11\",\"PrivatePhone\":\"\",\"GSM\":\"0479/02.46.16\",\"Email\":\"annabel.coopman@altius.com\",\"Comments\":\"SECRETARESSE = SYMONS Els\",\"AnsgateID\":26553,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"els.symons@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-29T18:30:12.797\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"XIONG Jessica\",\"Firstname\":\"Jessica\",\"Position\":\"\",\"OfficePhone\":\"02/421.08.00\",\"PrivatePhone\":\"\",\"GSM\":\"0488/21.41.33\",\"Email\":\"jessica.xiong@altius.com\",\"Comments\":\"SECRETARESSE = VANDAMME Anne-Marie\",\"AnsgateID\":26557,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"anne-marie.vandamme@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"JUNIOR COMMUNICATION & EVENTS OFFICER\",\"LastName\":\"DE MEULDER Marie\",\"Firstname\":\"Marie\",\"Position\":\"\",\"OfficePhone\":\"02/421.50.91\",\"PrivatePhone\":\"\",\"GSM\":\"0498/51.94.48\",\"Email\":\"marie.demeulder@altius.com\",\"Comments\":\"\",\"AnsgateID\":26548,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":true,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"MARKETING COMMUNICATION & BUSINESS DEVELOPMENT MANAGER\",\"LastName\":\"VAN MELKEBEKE Gert\",\"Firstname\":\"Gert\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.16\",\"PrivatePhone\":\"\",\"GSM\":\"0468/01.55.15\",\"Email\":\"gert.vanmelkebeke@altius.com\",\"Comments\":\"\",\"AnsgateID\":26549,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"SECRETARESSE\",\"LastName\":\"VAN DER HAEGHEN Saskia\",\"Firstname\":\"Saskia\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.30\",\"PrivatePhone\":\"\",\"GSM\":\"\",\"Email\":\"saskia.vanderhaeghen@altius.com\",\"Comments\":null,\"AnsgateID\":26657,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":false,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"\",\"SetMessageToRead\":false,\"SendMail\":false,\"CreatedOn\":\"2019-11-18T15:46:48.923\",\"LastModifiedOn\":\"2019-11-18T15:46:48.923\"},{\"DossierId\":5363,\"MainFicheId\":null,\"SubficheTagName\":null,\"ServiceName\":\"ADVOCAAT\",\"LastName\":\"BEX Fauve\",\"Firstname\":\"Fauve\",\"Position\":\"\",\"OfficePhone\":\"02/423.58.10\",\"PrivatePhone\":\"\",\"GSM\":\"0497/70.34.73\",\"Email\":\"fauve.bex@altius.com\",\"Comments\":\"SECRETARESSE = VANDERHAEGHEN Saskia\",\"AnsgateID\":26554,\"IsOfficePhonePrivate\":true,\"IsPrivatePhonePrivate\":false,\"IsGSMPrivate\":true,\"IsEmailPrivate\":false,\"IsServicePrivate\":false,\"IsFirstNamePrivate\":false,\"IsLastNamePrivate\":false,\"IsPositionPrivate\":false,\"CC\":\"saskia.vanderhaeghen@altius.com\",\"SetMessageToRead\":false,\"SendMail\":true,\"CreatedOn\":\"2019-10-28T12:46:22.447\",\"LastModifiedOn\":\"2019-10-28T12:46:22.447\"}]");
    }
}
