﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WebPortal.XUnitTest.TestModels
{
    public class TempInteractionStatItem
    {
        public int CustomerId { get; set; }
        public string InteractionIDKey { get; set; }
        public long InteractionId { get; set; }
        public string DNIS { get; set; }

        public int DNISInt
        {
            get
            {
                return Convert.ToInt32(DNIS);
            }
        }
        public decimal tConnected { get; set; }
        public decimal tHeld { get; set; }
        public decimal tPark { get; set; }
        public decimal tQueueWait { get; set; }

        public int tIVRWait { get; set; }
        public int tAlert { get; set; }

        public DateTime ConnectedDateTimeUTC { get; set; }
        public DateTime TerminatedDateTimeUTC { get; set; }


        public int AnouncementSeconds { get; set; }
        public int ExtraSeconds { get; set; }

        public TimeSpan Duration
        {
            get
            {
                return tConnected > 0 ?

                    new TimeSpan(0, 0, 0, 0, Convert.ToInt32(tConnected + tHeld + tPark) + ExtraSeconds * 1000)
                    : new TimeSpan();
            }
        }

        public DateTime Delivered
        {
            get
            {
                return ConnectedDateTimeUTC.AddSeconds(AnouncementSeconds);
            }
        }

        public DateTime? Established
        {
            get
            {
                DateTime? nullDT = null;
                return tConnected > 0
                    ? Delivered.AddMilliseconds(Convert.ToInt32(tIVRWait + tQueueWait + tAlert))
                        .AddSeconds(ExtraSeconds)
                    : nullDT;
            }
        }

        public DateTime ClearedDate
        {
            get
            {
                return tConnected > 0
                    ? TerminatedDateTimeUTC.AddSeconds(ExtraSeconds)
                    : TerminatedDateTimeUTC;
            }
        }

        public static List<TempInteractionStatItem> TestData => JsonConvert.DeserializeObject<List<TempInteractionStatItem>>("[{\"CustomerId\":9112,\"InteractionIDKey\":\"300148001620190617\",\"InteractionId\":3001480016,\"DNIS\":\"4112\",\"tConnected\":31213.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":37045.0,\"tIVRWait\":953,\"tAlert\":1922,\"ConnectedDateTimeUTC\":\"2019-06-17T13:02:27.798\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:03:38.934\",\"AnouncementSeconds\":0,\"ExtraSeconds\":5,\"Duration\":\"00:00:36.2130000\",\"Delivered\":\"2019-06-17T13:02:27.798\",\"Established\":\"2019-06-17T13:03:12.718\",\"ClearedDate\":\"2019-06-17T13:03:43.934\"},{\"CustomerId\":9198,\"InteractionIDKey\":\"300148003720190617\",\"InteractionId\":3001480037,\"DNIS\":\"4198\",\"tConnected\":294900.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":14067.0,\"tIVRWait\":10675,\"tAlert\":2844,\"ConnectedDateTimeUTC\":\"2019-06-17T13:03:14.441\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:08:36.929\",\"AnouncementSeconds\":0,\"ExtraSeconds\":10,\"Duration\":\"00:05:04.9000000\",\"Delivered\":\"2019-06-17T13:03:14.441\",\"Established\":\"2019-06-17T13:03:52.027\",\"ClearedDate\":\"2019-06-17T13:08:46.929\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148005320190617\",\"InteractionId\":3001480053,\"DNIS\":\"5704\",\"tConnected\":268762.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":25852,\"tAlert\":6736,\"ConnectedDateTimeUTC\":\"2019-06-17T13:03:51.454\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:08:52.837\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:04:28.7620000\",\"Delivered\":\"2019-06-17T13:03:51.454\",\"Established\":\"2019-06-17T13:04:24.073\",\"ClearedDate\":\"2019-06-17T13:08:52.837\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148008520190617\",\"InteractionId\":3001480085,\"DNIS\":\"5701\",\"tConnected\":234843.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":98814.0,\"tIVRWait\":25869,\"tAlert\":1515,\"ConnectedDateTimeUTC\":\"2019-06-17T13:06:14.897\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:14:31.26\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:03:54.8430000\",\"Delivered\":\"2019-06-17T13:06:14.897\",\"Established\":\"2019-06-17T13:08:21.095\",\"ClearedDate\":\"2019-06-17T13:14:31.26\"},{\"CustomerId\":5363,\"InteractionIDKey\":\"300148010520190617\",\"InteractionId\":3001480105,\"DNIS\":\"5363\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":117036.0,\"tIVRWait\":1015,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T13:07:48.747\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:09:46.799\",\"AnouncementSeconds\":0,\"ExtraSeconds\":10,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T13:07:48.747\",\"Established\":null,\"ClearedDate\":\"2019-06-17T13:09:46.799\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148011020190617\",\"InteractionId\":3001480110,\"DNIS\":\"4211\",\"tConnected\":264677.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":71888.0,\"tIVRWait\":937,\"tAlert\":2359,\"ConnectedDateTimeUTC\":\"2019-06-17T13:08:11.423\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:13:51.286\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:04:44.6770000\",\"Delivered\":\"2019-06-17T13:08:11.423\",\"Established\":\"2019-06-17T13:09:46.607\",\"ClearedDate\":\"2019-06-17T13:14:11.286\"},{\"CustomerId\":9198,\"InteractionIDKey\":\"300148016620190617\",\"InteractionId\":3001480166,\"DNIS\":\"4198\",\"tConnected\":14876.0,\"tHeld\":12250.0,\"tPark\":0.0,\"tQueueWait\":46.0,\"tIVRWait\":10690,\"tAlert\":2687,\"ConnectedDateTimeUTC\":\"2019-06-17T13:11:38.88\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:12:19.432\",\"AnouncementSeconds\":0,\"ExtraSeconds\":10,\"Duration\":\"00:00:37.1260000\",\"Delivered\":\"2019-06-17T13:11:38.88\",\"Established\":\"2019-06-17T13:12:02.303\",\"ClearedDate\":\"2019-06-17T13:12:29.432\"},{\"CustomerId\":9198,\"InteractionIDKey\":\"300148019220190617\",\"InteractionId\":3001480192,\"DNIS\":\"4198\",\"tConnected\":187015.0,\"tHeld\":60101.0,\"tPark\":5859.0,\"tQueueWait\":31.0,\"tIVRWait\":10657,\"tAlert\":4422,\"ConnectedDateTimeUTC\":\"2019-06-17T13:13:11.703\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:17:39.792\",\"AnouncementSeconds\":0,\"ExtraSeconds\":10,\"Duration\":\"00:04:22.9750000\",\"Delivered\":\"2019-06-17T13:13:11.703\",\"Established\":\"2019-06-17T13:13:36.813\",\"ClearedDate\":\"2019-06-17T13:17:49.792\"},{\"CustomerId\":9198,\"InteractionIDKey\":\"300148021320190617\",\"InteractionId\":3001480213,\"DNIS\":\"4198\",\"tConnected\":320321.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":17188.0,\"tIVRWait\":10672,\"tAlert\":531,\"ConnectedDateTimeUTC\":\"2019-06-17T13:14:15.931\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:20:04.645\",\"AnouncementSeconds\":0,\"ExtraSeconds\":10,\"Duration\":\"00:05:30.3210000\",\"Delivered\":\"2019-06-17T13:14:15.931\",\"Established\":\"2019-06-17T13:14:54.322\",\"ClearedDate\":\"2019-06-17T13:20:14.645\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148021720190617\",\"InteractionId\":3001480217,\"DNIS\":\"5710\",\"tConnected\":315571.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":28814,\"tAlert\":6250,\"ConnectedDateTimeUTC\":\"2019-06-17T13:14:24.713\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:20:15.38\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:05:15.5710000\",\"Delivered\":\"2019-06-17T13:14:24.713\",\"Established\":\"2019-06-17T13:14:59.808\",\"ClearedDate\":\"2019-06-17T13:20:15.38\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148022020190617\",\"InteractionId\":3001480220,\"DNIS\":\"4211\",\"tConnected\":500002.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":953,\"tAlert\":4906,\"ConnectedDateTimeUTC\":\"2019-06-17T13:14:40.527\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:23:06.405\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:08:40.0020000\",\"Delivered\":\"2019-06-17T13:14:40.527\",\"Established\":\"2019-06-17T13:15:06.401\",\"ClearedDate\":\"2019-06-17T13:23:26.405\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148023320190617\",\"InteractionId\":3001480233,\"DNIS\":\"4211\",\"tConnected\":236096.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":6922.0,\"tIVRWait\":937,\"tAlert\":984,\"ConnectedDateTimeUTC\":\"2019-06-17T13:15:06.294\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:19:11.235\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:04:16.0960000\",\"Delivered\":\"2019-06-17T13:15:06.294\",\"Established\":\"2019-06-17T13:15:35.137\",\"ClearedDate\":\"2019-06-17T13:19:31.235\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148024720190617\",\"InteractionId\":3001480247,\"DNIS\":\"5701\",\"tConnected\":281688.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":25878,\"tAlert\":10094,\"ConnectedDateTimeUTC\":\"2019-06-17T13:16:17.02\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:21:34.712\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:04:41.6880000\",\"Delivered\":\"2019-06-17T13:16:17.02\",\"Established\":\"2019-06-17T13:16:53.023\",\"ClearedDate\":\"2019-06-17T13:21:34.712\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148024820190617\",\"InteractionId\":3001480248,\"DNIS\":\"5701\",\"tConnected\":289673.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15547.0,\"tIVRWait\":25862,\"tAlert\":1140,\"ConnectedDateTimeUTC\":\"2019-06-17T13:16:17.895\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:21:50.12\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:04:49.6730000\",\"Delivered\":\"2019-06-17T13:16:17.895\",\"Established\":\"2019-06-17T13:17:00.444\",\"ClearedDate\":\"2019-06-17T13:21:50.12\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148026520190617\",\"InteractionId\":3001480265,\"DNIS\":\"4211\",\"tConnected\":118460.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":937,\"tAlert\":1859,\"ConnectedDateTimeUTC\":\"2019-06-17T13:17:09.556\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:19:10.845\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:02:18.4600000\",\"Delivered\":\"2019-06-17T13:17:09.556\",\"Established\":\"2019-06-17T13:17:32.383\",\"ClearedDate\":\"2019-06-17T13:19:30.845\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148026820190617\",\"InteractionId\":3001480268,\"DNIS\":\"4211\",\"tConnected\":465583.0,\"tHeld\":0.0,\"tPark\":153821.0,\"tQueueWait\":163557.0,\"tIVRWait\":953,\"tAlert\":1015,\"ConnectedDateTimeUTC\":\"2019-06-17T13:17:28.495\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:33:25.218\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:10:39.4040000\",\"Delivered\":\"2019-06-17T13:17:28.495\",\"Established\":\"2019-06-17T13:20:34.02\",\"ClearedDate\":\"2019-06-17T13:33:45.218\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148027920190617\",\"InteractionId\":3001480279,\"DNIS\":\"5711\",\"tConnected\":356285.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":29189,\"tAlert\":6937,\"ConnectedDateTimeUTC\":\"2019-06-17T13:18:08.373\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:24:40.817\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:05:56.2850000\",\"Delivered\":\"2019-06-17T13:18:08.373\",\"Established\":\"2019-06-17T13:18:44.53\",\"ClearedDate\":\"2019-06-17T13:24:40.817\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148029220190617\",\"InteractionId\":3001480292,\"DNIS\":\"5710\",\"tConnected\":307719.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":42033.0,\"tIVRWait\":29174,\"tAlert\":6359,\"ConnectedDateTimeUTC\":\"2019-06-17T13:19:04.423\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:25:29.709\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:05:07.7190000\",\"Delivered\":\"2019-06-17T13:19:04.423\",\"Established\":\"2019-06-17T13:20:21.989\",\"ClearedDate\":\"2019-06-17T13:25:29.709\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148031120190617\",\"InteractionId\":3001480311,\"DNIS\":\"4211\",\"tConnected\":308610.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":87770.0,\"tIVRWait\":937,\"tAlert\":15345,\"ConnectedDateTimeUTC\":\"2019-06-17T13:20:06.223\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:26:58.887\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:05:28.6100000\",\"Delivered\":\"2019-06-17T13:20:06.223\",\"Established\":\"2019-06-17T13:22:10.275\",\"ClearedDate\":\"2019-06-17T13:27:18.887\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148033020190617\",\"InteractionId\":3001480330,\"DNIS\":\"5711\",\"tConnected\":376409.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":171462.0,\"tIVRWait\":29158,\"tAlert\":1093,\"ConnectedDateTimeUTC\":\"2019-06-17T13:21:20.414\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:30:58.538\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:06:16.4090000\",\"Delivered\":\"2019-06-17T13:21:20.414\",\"Established\":\"2019-06-17T13:24:42.127\",\"ClearedDate\":\"2019-06-17T13:30:58.538\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148035820190617\",\"InteractionId\":3001480358,\"DNIS\":\"5701\",\"tConnected\":300985.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":25861,\"tAlert\":15188,\"ConnectedDateTimeUTC\":\"2019-06-17T13:22:41.075\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:28:23.125\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:05:00.9850000\",\"Delivered\":\"2019-06-17T13:22:41.075\",\"Established\":\"2019-06-17T13:23:22.139\",\"ClearedDate\":\"2019-06-17T13:28:23.125\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148037720190617\",\"InteractionId\":3001480377,\"DNIS\":\"5704\",\"tConnected\":65394.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":27830.0,\"tIVRWait\":25876,\"tAlert\":1078,\"ConnectedDateTimeUTC\":\"2019-06-17T13:23:37.844\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:25:38.023\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:01:05.3940000\",\"Delivered\":\"2019-06-17T13:23:37.844\",\"Established\":\"2019-06-17T13:24:32.628\",\"ClearedDate\":\"2019-06-17T13:25:38.023\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148038520190617\",\"InteractionId\":3001480385,\"DNIS\":\"4211\",\"tConnected\":142712.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":1000,\"tAlert\":1687,\"ConnectedDateTimeUTC\":\"2019-06-17T13:24:01.72\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:26:27.135\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:02:42.7120000\",\"Delivered\":\"2019-06-17T13:24:01.72\",\"Established\":\"2019-06-17T13:24:24.422\",\"ClearedDate\":\"2019-06-17T13:26:47.135\"},{\"CustomerId\":9112,\"InteractionIDKey\":\"300148039320190617\",\"InteractionId\":3001480393,\"DNIS\":\"4112\",\"tConnected\":157524.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":10907.0,\"tIVRWait\":953,\"tAlert\":1921,\"ConnectedDateTimeUTC\":\"2019-06-17T13:24:33.331\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:27:24.638\",\"AnouncementSeconds\":0,\"ExtraSeconds\":5,\"Duration\":\"00:02:42.5240000\",\"Delivered\":\"2019-06-17T13:24:33.331\",\"Established\":\"2019-06-17T13:24:52.112\",\"ClearedDate\":\"2019-06-17T13:27:29.638\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148040820190617\",\"InteractionId\":3001480408,\"DNIS\":\"5710\",\"tConnected\":622214.0,\"tHeld\":304184.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":29159,\"tAlert\":5531,\"ConnectedDateTimeUTC\":\"2019-06-17T13:25:28.381\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:41:29.487\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:15:26.3980000\",\"Delivered\":\"2019-06-17T13:25:28.381\",\"Established\":\"2019-06-17T13:26:03.086\",\"ClearedDate\":\"2019-06-17T13:41:29.487\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148042020190617\",\"InteractionId\":3001480420,\"DNIS\":\"5704\",\"tConnected\":477693.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":25844,\"tAlert\":4093,\"ConnectedDateTimeUTC\":\"2019-06-17T13:26:34.199\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:35:01.846\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:07:57.6930000\",\"Delivered\":\"2019-06-17T13:26:34.199\",\"Established\":\"2019-06-17T13:27:04.151\",\"ClearedDate\":\"2019-06-17T13:35:01.846\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148045520190617\",\"InteractionId\":3001480455,\"DNIS\":\"5710\",\"tConnected\":183181.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":29125,\"tAlert\":5203,\"ConnectedDateTimeUTC\":\"2019-06-17T13:28:46.204\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:32:23.73\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:03:03.1810000\",\"Delivered\":\"2019-06-17T13:28:46.204\",\"Established\":\"2019-06-17T13:29:20.547\",\"ClearedDate\":\"2019-06-17T13:32:23.73\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148047920190617\",\"InteractionId\":3001480479,\"DNIS\":\"5704\",\"tConnected\":165100.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":25891,\"tAlert\":9203,\"ConnectedDateTimeUTC\":\"2019-06-17T13:29:50.926\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:33:11.138\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:02:45.1000000\",\"Delivered\":\"2019-06-17T13:29:50.926\",\"Established\":\"2019-06-17T13:30:26.035\",\"ClearedDate\":\"2019-06-17T13:33:11.138\"},{\"CustomerId\":9112,\"InteractionIDKey\":\"300148048520190617\",\"InteractionId\":3001480485,\"DNIS\":\"4112\",\"tConnected\":148443.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":890,\"tAlert\":12500,\"ConnectedDateTimeUTC\":\"2019-06-17T13:30:00.552\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:32:42.418\",\"AnouncementSeconds\":0,\"ExtraSeconds\":5,\"Duration\":\"00:02:33.4430000\",\"Delivered\":\"2019-06-17T13:30:00.552\",\"Established\":\"2019-06-17T13:30:18.973\",\"ClearedDate\":\"2019-06-17T13:32:47.418\"},{\"CustomerId\":9112,\"InteractionIDKey\":\"300148050120190617\",\"InteractionId\":3001480501,\"DNIS\":\"4112\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":108160.0,\"tIVRWait\":953,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T13:31:24.321\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:33:13.435\",\"AnouncementSeconds\":0,\"ExtraSeconds\":5,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T13:31:24.321\",\"Established\":null,\"ClearedDate\":\"2019-06-17T13:33:13.435\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148055920190617\",\"InteractionId\":3001480559,\"DNIS\":\"5704\",\"tConnected\":417829.0,\"tHeld\":77674.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":25906,\"tAlert\":6687,\"ConnectedDateTimeUTC\":\"2019-06-17T13:35:54.536\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:44:42.665\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:08:15.5030000\",\"Delivered\":\"2019-06-17T13:35:54.536\",\"Established\":\"2019-06-17T13:36:27.16\",\"ClearedDate\":\"2019-06-17T13:44:42.665\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148056320190617\",\"InteractionId\":3001480563,\"DNIS\":\"4211\",\"tConnected\":47626.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":953,\"tAlert\":15735,\"ConnectedDateTimeUTC\":\"2019-06-17T13:36:26.896\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:37:31.227\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:01:07.6260000\",\"Delivered\":\"2019-06-17T13:36:26.896\",\"Established\":\"2019-06-17T13:37:03.599\",\"ClearedDate\":\"2019-06-17T13:37:51.227\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148057720190617\",\"InteractionId\":3001480577,\"DNIS\":\"5711\",\"tConnected\":285932.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":29157,\"tAlert\":11531,\"ConnectedDateTimeUTC\":\"2019-06-17T13:37:37.477\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:43:04.131\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:04:45.9320000\",\"Delivered\":\"2019-06-17T13:37:37.477\",\"Established\":\"2019-06-17T13:38:18.196\",\"ClearedDate\":\"2019-06-17T13:43:04.131\"},{\"CustomerId\":9112,\"InteractionIDKey\":\"300148059020190617\",\"InteractionId\":3001480590,\"DNIS\":\"4112\",\"tConnected\":279432.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":953,\"tAlert\":5343,\"ConnectedDateTimeUTC\":\"2019-06-17T13:38:25.589\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:43:11.334\",\"AnouncementSeconds\":0,\"ExtraSeconds\":5,\"Duration\":\"00:04:44.4320000\",\"Delivered\":\"2019-06-17T13:38:25.589\",\"Established\":\"2019-06-17T13:38:36.9\",\"ClearedDate\":\"2019-06-17T13:43:16.334\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148059720190617\",\"InteractionId\":3001480597,\"DNIS\":\"5710\",\"tConnected\":378328.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":29172,\"tAlert\":5890,\"ConnectedDateTimeUTC\":\"2019-06-17T13:38:45.59\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:45:38.998\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:06:18.3280000\",\"Delivered\":\"2019-06-17T13:38:45.59\",\"Established\":\"2019-06-17T13:39:20.667\",\"ClearedDate\":\"2019-06-17T13:45:38.998\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148060020190617\",\"InteractionId\":3001480600,\"DNIS\":\"5704\",\"tConnected\":1167042.0,\"tHeld\":547567.0,\"tPark\":189537.0,\"tQueueWait\":11172.0,\"tIVRWait\":25844,\"tAlert\":937,\"ConnectedDateTimeUTC\":\"2019-06-17T13:38:48.153\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:11:10.26\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:31:44.1460000\",\"Delivered\":\"2019-06-17T13:38:48.153\",\"Established\":\"2019-06-17T13:39:26.106\",\"ClearedDate\":\"2019-06-17T14:11:10.26\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148061720190617\",\"InteractionId\":3001480617,\"DNIS\":\"5701\",\"tConnected\":263292.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":87409.0,\"tIVRWait\":25861,\"tAlert\":1968,\"ConnectedDateTimeUTC\":\"2019-06-17T13:39:44.514\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:46:03.046\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:04:23.2920000\",\"Delivered\":\"2019-06-17T13:39:44.514\",\"Established\":\"2019-06-17T13:41:39.752\",\"ClearedDate\":\"2019-06-17T13:46:03.046\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148064220190617\",\"InteractionId\":3001480642,\"DNIS\":\"5711\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":29172,\"tAlert\":4687,\"ConnectedDateTimeUTC\":\"2019-06-17T13:42:40.896\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:43:14.772\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T13:42:40.896\",\"Established\":null,\"ClearedDate\":\"2019-06-17T13:43:14.772\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148065420190617\",\"InteractionId\":3001480654,\"DNIS\":\"5710\",\"tConnected\":157179.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":29157,\"tAlert\":3828,\"ConnectedDateTimeUTC\":\"2019-06-17T13:43:37.179\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:46:47.375\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:02:37.1790000\",\"Delivered\":\"2019-06-17T13:43:37.179\",\"Established\":\"2019-06-17T13:44:10.195\",\"ClearedDate\":\"2019-06-17T13:46:47.375\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148066320190617\",\"InteractionId\":3001480663,\"DNIS\":\"5701\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":0.0,\"tIVRWait\":22297,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T13:44:27.54\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:44:49.837\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T13:44:27.54\",\"Established\":null,\"ClearedDate\":\"2019-06-17T13:44:49.837\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148068320190617\",\"InteractionId\":3001480683,\"DNIS\":\"5704\",\"tConnected\":99882.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":25844,\"tAlert\":10923,\"ConnectedDateTimeUTC\":\"2019-06-17T13:46:26.219\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:48:42.9\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:01:39.8820000\",\"Delivered\":\"2019-06-17T13:46:26.219\",\"Established\":\"2019-06-17T13:47:03.017\",\"ClearedDate\":\"2019-06-17T13:48:42.9\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148069020190617\",\"InteractionId\":3001480690,\"DNIS\":\"5704\",\"tConnected\":18860.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":25658,\"tAlert\":6547,\"ConnectedDateTimeUTC\":\"2019-06-17T13:46:57.205\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:47:48.302\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:00:18.8600000\",\"Delivered\":\"2019-06-17T13:46:57.205\",\"Established\":\"2019-06-17T13:47:29.441\",\"ClearedDate\":\"2019-06-17T13:47:48.302\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148070620190617\",\"InteractionId\":3001480706,\"DNIS\":\"4211\",\"tConnected\":462773.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":146367.0,\"tIVRWait\":937,\"tAlert\":11406,\"ConnectedDateTimeUTC\":\"2019-06-17T13:48:07.115\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:58:28.601\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:08:02.7730000\",\"Delivered\":\"2019-06-17T13:48:07.115\",\"Established\":\"2019-06-17T13:51:05.825\",\"ClearedDate\":\"2019-06-17T13:58:48.601\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148098220190617\",\"InteractionId\":3001480982,\"DNIS\":\"5710\",\"tConnected\":86425.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":29173,\"tAlert\":4171,\"ConnectedDateTimeUTC\":\"2019-06-17T13:51:27.625\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:53:27.427\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:01:26.4250000\",\"Delivered\":\"2019-06-17T13:51:27.625\",\"Established\":\"2019-06-17T13:52:01\",\"ClearedDate\":\"2019-06-17T13:53:27.427\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148098420190617\",\"InteractionId\":3001480984,\"DNIS\":\"5701\",\"tConnected\":148165.0,\"tHeld\":0.0,\"tPark\":267466.0,\"tQueueWait\":39735.0,\"tIVRWait\":25860,\"tAlert\":7187,\"ConnectedDateTimeUTC\":\"2019-06-17T13:51:39.969\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:59:48.386\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:06:55.6310000\",\"Delivered\":\"2019-06-17T13:51:39.969\",\"Established\":\"2019-06-17T13:52:52.751\",\"ClearedDate\":\"2019-06-17T13:59:48.386\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148101420190617\",\"InteractionId\":3001481014,\"DNIS\":\"5710\",\"tConnected\":324502.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":22361.0,\"tIVRWait\":29188,\"tAlert\":1000,\"ConnectedDateTimeUTC\":\"2019-06-17T13:53:02.176\",\"TerminatedDateTimeUTC\":\"2019-06-17T13:59:19.229\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:05:24.5020000\",\"Delivered\":\"2019-06-17T13:53:02.176\",\"Established\":\"2019-06-17T13:53:54.725\",\"ClearedDate\":\"2019-06-17T13:59:19.229\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148103620190617\",\"InteractionId\":3001481036,\"DNIS\":\"5711\",\"tConnected\":346283.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":29188,\"tAlert\":5500,\"ConnectedDateTimeUTC\":\"2019-06-17T13:55:22.997\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:01:43.985\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:05:46.2830000\",\"Delivered\":\"2019-06-17T13:55:22.997\",\"Established\":\"2019-06-17T13:55:57.7\",\"ClearedDate\":\"2019-06-17T14:01:43.985\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148105020190617\",\"InteractionId\":3001481050,\"DNIS\":\"5711\",\"tConnected\":277075.0,\"tHeld\":119848.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":29159,\"tAlert\":6922,\"ConnectedDateTimeUTC\":\"2019-06-17T13:56:07.249\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:03:20.286\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:06:36.9230000\",\"Delivered\":\"2019-06-17T13:56:07.249\",\"Established\":\"2019-06-17T13:56:43.361\",\"ClearedDate\":\"2019-06-17T14:03:20.286\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148105220190617\",\"InteractionId\":3001481052,\"DNIS\":\"5710\",\"tConnected\":488834.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":29064,\"tAlert\":2578,\"ConnectedDateTimeUTC\":\"2019-06-17T13:56:24.58\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:05:05.072\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:08:08.8340000\",\"Delivered\":\"2019-06-17T13:56:24.58\",\"Established\":\"2019-06-17T13:56:56.237\",\"ClearedDate\":\"2019-06-17T14:05:05.072\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148107720190617\",\"InteractionId\":3001481077,\"DNIS\":\"5704\",\"tConnected\":302748.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":12797.0,\"tIVRWait\":25891,\"tAlert\":10266,\"ConnectedDateTimeUTC\":\"2019-06-17T13:57:50.131\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:03:41.834\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:05:02.7480000\",\"Delivered\":\"2019-06-17T13:57:50.131\",\"Established\":\"2019-06-17T13:58:39.085\",\"ClearedDate\":\"2019-06-17T14:03:41.834\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148110820190617\",\"InteractionId\":3001481108,\"DNIS\":\"5701\",\"tConnected\":184618.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":25845,\"tAlert\":4296,\"ConnectedDateTimeUTC\":\"2019-06-17T14:00:21.247\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:03:56.039\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:03:04.6180000\",\"Delivered\":\"2019-06-17T14:00:21.247\",\"Established\":\"2019-06-17T14:00:51.419\",\"ClearedDate\":\"2019-06-17T14:03:56.039\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148112020190617\",\"InteractionId\":3001481120,\"DNIS\":\"5663\",\"tConnected\":463379.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":250341.0,\"tIVRWait\":29172,\"tAlert\":7250,\"ConnectedDateTimeUTC\":\"2019-06-17T14:01:34.798\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:14:04.941\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:07:43.3790000\",\"Delivered\":\"2019-06-17T14:01:34.798\",\"Established\":\"2019-06-17T14:06:21.561\",\"ClearedDate\":\"2019-06-17T14:14:04.941\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148112420190617\",\"InteractionId\":3001481124,\"DNIS\":\"5714\",\"tConnected\":404282.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":60753.0,\"tIVRWait\":29172,\"tAlert\":11735,\"ConnectedDateTimeUTC\":\"2019-06-17T14:02:12.142\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:10:38.087\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:06:44.2820000\",\"Delivered\":\"2019-06-17T14:02:12.142\",\"Established\":\"2019-06-17T14:03:53.802\",\"ClearedDate\":\"2019-06-17T14:10:38.087\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148112520190617\",\"InteractionId\":3001481125,\"DNIS\":\"5710\",\"tConnected\":423862.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":196745.0,\"tIVRWait\":29188,\"tAlert\":10031,\"ConnectedDateTimeUTC\":\"2019-06-17T14:02:25.643\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:13:25.472\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:07:03.8620000\",\"Delivered\":\"2019-06-17T14:02:25.643\",\"Established\":\"2019-06-17T14:06:21.607\",\"ClearedDate\":\"2019-06-17T14:13:25.472\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148118420190617\",\"InteractionId\":3001481184,\"DNIS\":\"5701\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":0.0,\"tIVRWait\":22172,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T14:05:50.358\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:06:12.531\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T14:05:50.358\",\"Established\":null,\"ClearedDate\":\"2019-06-17T14:06:12.531\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148118620190617\",\"InteractionId\":3001481186,\"DNIS\":\"5704\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":10078.0,\"tIVRWait\":25907,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T14:06:02.39\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:06:38.376\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T14:06:02.39\",\"Established\":null,\"ClearedDate\":\"2019-06-17T14:06:38.376\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148120620190617\",\"InteractionId\":3001481206,\"DNIS\":\"5704\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":0.0,\"tIVRWait\":19360,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T14:07:18.221\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:07:37.581\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T14:07:18.221\",\"Established\":null,\"ClearedDate\":\"2019-06-17T14:07:37.581\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148120920190617\",\"InteractionId\":3001481209,\"DNIS\":\"5710\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":0.0,\"tIVRWait\":27673,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T14:08:05.035\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:08:32.708\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T14:08:05.035\",\"Established\":null,\"ClearedDate\":\"2019-06-17T14:08:32.708\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148122820190617\",\"InteractionId\":3001481228,\"DNIS\":\"4211\",\"tConnected\":246778.0,\"tHeld\":115099.0,\"tPark\":0.0,\"tQueueWait\":77580.0,\"tIVRWait\":953,\"tAlert\":4640,\"ConnectedDateTimeUTC\":\"2019-06-17T14:09:44.476\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:17:09.528\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:06:21.8770000\",\"Delivered\":\"2019-06-17T14:09:44.476\",\"Established\":\"2019-06-17T14:11:27.649\",\"ClearedDate\":\"2019-06-17T14:17:29.528\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148124020190617\",\"InteractionId\":3001481240,\"DNIS\":\"4211\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":26563.0,\"tIVRWait\":968,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T14:10:40.774\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:11:08.306\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T14:10:40.774\",\"Established\":null,\"ClearedDate\":\"2019-06-17T14:11:08.306\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148124220190617\",\"InteractionId\":3001481242,\"DNIS\":\"5701\",\"tConnected\":229855.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":27049.0,\"tIVRWait\":25875,\"tAlert\":640,\"ConnectedDateTimeUTC\":\"2019-06-17T14:10:52.259\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:15:35.68\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:03:49.8550000\",\"Delivered\":\"2019-06-17T14:10:52.259\",\"Established\":\"2019-06-17T14:11:45.823\",\"ClearedDate\":\"2019-06-17T14:15:35.68\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148124320190617\",\"InteractionId\":3001481243,\"DNIS\":\"4211\",\"tConnected\":267731.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":293046.0,\"tIVRWait\":968,\"tAlert\":1437,\"ConnectedDateTimeUTC\":\"2019-06-17T14:10:58.181\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:20:21.366\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:04:47.7310000\",\"Delivered\":\"2019-06-17T14:10:58.181\",\"Established\":\"2019-06-17T14:16:13.632\",\"ClearedDate\":\"2019-06-17T14:20:41.366\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148124720190617\",\"InteractionId\":3001481247,\"DNIS\":\"5701\",\"tConnected\":228181.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":140507.0,\"tIVRWait\":25861,\"tAlert\":1125,\"ConnectedDateTimeUTC\":\"2019-06-17T14:11:18.791\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:17:54.467\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:03:48.1810000\",\"Delivered\":\"2019-06-17T14:11:18.791\",\"Established\":\"2019-06-17T14:14:06.284\",\"ClearedDate\":\"2019-06-17T14:17:54.467\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148125620190617\",\"InteractionId\":3001481256,\"DNIS\":\"4211\",\"tConnected\":138227.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":436613.0,\"tIVRWait\":937,\"tAlert\":3062,\"ConnectedDateTimeUTC\":\"2019-06-17T14:12:07.076\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:21:45.918\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:02:38.2270000\",\"Delivered\":\"2019-06-17T14:12:07.076\",\"Established\":\"2019-06-17T14:19:47.688\",\"ClearedDate\":\"2019-06-17T14:22:05.918\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148125720190617\",\"InteractionId\":3001481257,\"DNIS\":\"4211\",\"tConnected\":137537.0,\"tHeld\":48127.0,\"tPark\":0.0,\"tQueueWait\":712519.0,\"tIVRWait\":953,\"tAlert\":4578,\"ConnectedDateTimeUTC\":\"2019-06-17T14:12:19.546\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:27:23.261\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:03:25.6640000\",\"Delivered\":\"2019-06-17T14:12:19.546\",\"Established\":\"2019-06-17T14:24:37.596\",\"ClearedDate\":\"2019-06-17T14:27:43.261\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148128720190617\",\"InteractionId\":3001481287,\"DNIS\":\"5704\",\"tConnected\":62861.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":138505.0,\"tIVRWait\":25877,\"tAlert\":4125,\"ConnectedDateTimeUTC\":\"2019-06-17T14:15:10.319\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:19:01.688\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:01:02.8610000\",\"Delivered\":\"2019-06-17T14:15:10.319\",\"Established\":\"2019-06-17T14:17:58.826\",\"ClearedDate\":\"2019-06-17T14:19:01.688\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148130020190617\",\"InteractionId\":3001481300,\"DNIS\":\"5704\",\"tConnected\":111007.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":127332.0,\"tIVRWait\":25845,\"tAlert\":1234,\"ConnectedDateTimeUTC\":\"2019-06-17T14:16:28.729\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:20:54.149\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:01:51.0070000\",\"Delivered\":\"2019-06-17T14:16:28.729\",\"Established\":\"2019-06-17T14:19:03.14\",\"ClearedDate\":\"2019-06-17T14:20:54.149\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148130120190617\",\"InteractionId\":3001481301,\"DNIS\":\"5714\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":0.0,\"tIVRWait\":24939,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T14:16:37.807\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:17:02.747\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T14:16:37.807\",\"Established\":null,\"ClearedDate\":\"2019-06-17T14:17:02.747\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148133520190617\",\"InteractionId\":3001481335,\"DNIS\":\"4211\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":544793.0,\"tIVRWait\":953,\"tAlert\":17813,\"ConnectedDateTimeUTC\":\"2019-06-17T14:18:17.733\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:27:41.294\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T14:18:17.733\",\"Established\":null,\"ClearedDate\":\"2019-06-17T14:27:41.294\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148135320190617\",\"InteractionId\":3001481353,\"DNIS\":\"5706\",\"tConnected\":84833.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":25876,\"tAlert\":10719,\"ConnectedDateTimeUTC\":\"2019-06-17T14:19:53.301\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:21:54.762\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:01:24.8330000\",\"Delivered\":\"2019-06-17T14:19:53.301\",\"Established\":\"2019-06-17T14:20:29.927\",\"ClearedDate\":\"2019-06-17T14:21:54.762\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148136020190617\",\"InteractionId\":3001481360,\"DNIS\":\"5701\",\"tConnected\":128114.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":109285.0,\"tIVRWait\":25877,\"tAlert\":9438,\"ConnectedDateTimeUTC\":\"2019-06-17T14:20:16.209\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:24:48.925\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:02:08.1140000\",\"Delivered\":\"2019-06-17T14:20:16.209\",\"Established\":\"2019-06-17T14:22:40.809\",\"ClearedDate\":\"2019-06-17T14:24:48.925\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148137920190617\",\"InteractionId\":3001481379,\"DNIS\":\"4211\",\"tConnected\":265608.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":683498.0,\"tIVRWait\":953,\"tAlert\":3218,\"ConnectedDateTimeUTC\":\"2019-06-17T14:20:58.696\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:36:51.976\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:04:45.6080000\",\"Delivered\":\"2019-06-17T14:20:58.696\",\"Established\":\"2019-06-17T14:32:46.365\",\"ClearedDate\":\"2019-06-17T14:37:11.976\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148139420190617\",\"InteractionId\":3001481394,\"DNIS\":\"4211\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":561962.0,\"tIVRWait\":937,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T14:21:53.465\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:31:16.365\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T14:21:53.465\",\"Established\":null,\"ClearedDate\":\"2019-06-17T14:31:16.365\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148139520190617\",\"InteractionId\":3001481395,\"DNIS\":\"4211\",\"tConnected\":0.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":76644.0,\"tIVRWait\":937,\"tAlert\":0,\"ConnectedDateTimeUTC\":\"2019-06-17T14:21:57.699\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:23:15.281\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:00:00\",\"Delivered\":\"2019-06-17T14:21:57.699\",\"Established\":null,\"ClearedDate\":\"2019-06-17T14:23:15.281\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148141220190617\",\"InteractionId\":3001481412,\"DNIS\":\"5701\",\"tConnected\":452144.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":51580.0,\"tIVRWait\":25876,\"tAlert\":859,\"ConnectedDateTimeUTC\":\"2019-06-17T14:23:03.702\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:31:54.163\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:07:32.1440000\",\"Delivered\":\"2019-06-17T14:23:03.702\",\"Established\":\"2019-06-17T14:24:22.017\",\"ClearedDate\":\"2019-06-17T14:31:54.163\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148142220190617\",\"InteractionId\":3001481422,\"DNIS\":\"5711\",\"tConnected\":124068.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15297.0,\"tIVRWait\":29157,\"tAlert\":2109,\"ConnectedDateTimeUTC\":\"2019-06-17T14:24:04.705\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:26:55.338\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:02:04.0680000\",\"Delivered\":\"2019-06-17T14:24:04.705\",\"Established\":\"2019-06-17T14:24:51.268\",\"ClearedDate\":\"2019-06-17T14:26:55.338\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148143020190617\",\"InteractionId\":3001481430,\"DNIS\":\"4211\",\"tConnected\":194977.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":706486.0,\"tIVRWait\":937,\"tAlert\":7297,\"ConnectedDateTimeUTC\":\"2019-06-17T14:24:27.534\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:40:14.079\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:03:34.9770000\",\"Delivered\":\"2019-06-17T14:24:27.534\",\"Established\":\"2019-06-17T14:36:42.254\",\"ClearedDate\":\"2019-06-17T14:40:34.079\"},{\"CustomerId\":9112,\"InteractionIDKey\":\"300148145120190617\",\"InteractionId\":3001481451,\"DNIS\":\"4112\",\"tConnected\":212836.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":73831.0,\"tIVRWait\":953,\"tAlert\":3859,\"ConnectedDateTimeUTC\":\"2019-06-17T14:26:28.368\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:31:19.849\",\"AnouncementSeconds\":0,\"ExtraSeconds\":5,\"Duration\":\"00:03:37.8360000\",\"Delivered\":\"2019-06-17T14:26:28.368\",\"Established\":\"2019-06-17T14:27:52.011\",\"ClearedDate\":\"2019-06-17T14:31:24.849\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148149620190617\",\"InteractionId\":3001481496,\"DNIS\":\"4211\",\"tConnected\":232573.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":466679.0,\"tIVRWait\":953,\"tAlert\":1125,\"ConnectedDateTimeUTC\":\"2019-06-17T14:29:56.706\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:41:38.037\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:04:12.5730000\",\"Delivered\":\"2019-06-17T14:29:56.706\",\"Established\":\"2019-06-17T14:38:05.463\",\"ClearedDate\":\"2019-06-17T14:41:58.037\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148152620190617\",\"InteractionId\":3001481526,\"DNIS\":\"4211\",\"tConnected\":145381.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":471540.0,\"tIVRWait\":937,\"tAlert\":3406,\"ConnectedDateTimeUTC\":\"2019-06-17T14:32:21.82\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:42:43.086\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:02:45.3810000\",\"Delivered\":\"2019-06-17T14:32:21.82\",\"Established\":\"2019-06-17T14:40:37.703\",\"ClearedDate\":\"2019-06-17T14:43:03.086\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148153720190617\",\"InteractionId\":3001481537,\"DNIS\":\"5701\",\"tConnected\":770195.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":25860,\"tAlert\":9407,\"ConnectedDateTimeUTC\":\"2019-06-17T14:32:52.4\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:46:17.894\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:12:50.1950000\",\"Delivered\":\"2019-06-17T14:32:52.4\",\"Established\":\"2019-06-17T14:33:27.698\",\"ClearedDate\":\"2019-06-17T14:46:17.894\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148155420190617\",\"InteractionId\":3001481554,\"DNIS\":\"5701\",\"tConnected\":166384.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":15.0,\"tIVRWait\":25845,\"tAlert\":4015,\"ConnectedDateTimeUTC\":\"2019-06-17T14:33:55.106\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:37:11.367\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:02:46.3840000\",\"Delivered\":\"2019-06-17T14:33:55.106\",\"Established\":\"2019-06-17T14:34:24.981\",\"ClearedDate\":\"2019-06-17T14:37:11.367\"},{\"CustomerId\":9112,\"InteractionIDKey\":\"300148156520190617\",\"InteractionId\":3001481565,\"DNIS\":\"4112\",\"tConnected\":46580.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":937,\"tAlert\":3656,\"ConnectedDateTimeUTC\":\"2019-06-17T14:35:21.096\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:36:12.302\",\"AnouncementSeconds\":0,\"ExtraSeconds\":5,\"Duration\":\"00:00:51.5800000\",\"Delivered\":\"2019-06-17T14:35:21.096\",\"Established\":\"2019-06-17T14:35:30.72\",\"ClearedDate\":\"2019-06-17T14:36:17.302\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148159420190617\",\"InteractionId\":3001481594,\"DNIS\":\"5704\",\"tConnected\":54939.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":31.0,\"tIVRWait\":25861,\"tAlert\":1671,\"ConnectedDateTimeUTC\":\"2019-06-17T14:38:34.527\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:39:57.031\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:00:54.9390000\",\"Delivered\":\"2019-06-17T14:38:34.527\",\"Established\":\"2019-06-17T14:39:02.09\",\"ClearedDate\":\"2019-06-17T14:39:57.031\"},{\"CustomerId\":5692,\"InteractionIDKey\":\"300148159720190617\",\"InteractionId\":3001481597,\"DNIS\":\"5711\",\"tConnected\":145101.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":210041.0,\"tIVRWait\":29173,\"tAlert\":2421,\"ConnectedDateTimeUTC\":\"2019-06-17T14:38:45.074\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:45:11.813\",\"AnouncementSeconds\":0,\"ExtraSeconds\":0,\"Duration\":\"00:02:25.1010000\",\"Delivered\":\"2019-06-17T14:38:45.074\",\"Established\":\"2019-06-17T14:42:46.709\",\"ClearedDate\":\"2019-06-17T14:45:11.813\"},{\"CustomerId\":5211,\"InteractionIDKey\":\"300148161920190617\",\"InteractionId\":3001481619,\"DNIS\":\"4211\",\"tConnected\":2140.0,\"tHeld\":0.0,\"tPark\":0.0,\"tQueueWait\":130444.0,\"tIVRWait\":937,\"tAlert\":12563,\"ConnectedDateTimeUTC\":\"2019-06-17T14:39:26.889\",\"TerminatedDateTimeUTC\":\"2019-06-17T14:41:52.975\",\"AnouncementSeconds\":0,\"ExtraSeconds\":20,\"Duration\":\"00:00:22.1400000\",\"Delivered\":\"2019-06-17T14:39:26.889\",\"Established\":\"2019-06-17T14:42:10.833\",\"ClearedDate\":\"2019-06-17T14:42:12.975\"}]");
    }
}
